@@ -32,22 +32,18 @@
 #include <QtGui/QPushButton>
 #include <QtGui/QAbstractScrollArea>
 #include <QtGui/QScrollBar>
-#include <QtCore/QMutex>
 #include <QtCore/QScopedPointer>
 
 #include <kicon.h>
 #include <kdialog.h>
 
-#include <ktexteditor/smartcursornotifier.h>
 #include <ktexteditor/codecompletionmodelcontrollerinterface.h>
 
 #include "kateview.h"
-#include "katesmartmanager.h"
 #include "katerenderer.h"
 #include "kateconfig.h"
 #include "katedocument.h"
-#include "katesmartrange.h"
-#include "kateedit.h"
+#include "katebuffer.h"
 
 #include "katecompletionmodel.h"
 #include "katecompletiontree.h"
@@ -63,17 +59,6 @@ const bool hideAutomaticCompletionOnExactMatch = true;
 const bool shellLikeTabCompletion = false;
 
 #define CALLCI(WHAT,WHATELSE,WHAT2,model,FUNC) \
-{\
-  /*kDebug()<<"CALLCI";*/ \
-  KTextEditor::CodeCompletionModelControllerInterface* ret =\
-    qobject_cast<KTextEditor::CodeCompletionModelControllerInterface*>(model);\
-  if (ret) {\
-        kDebug()<<"You are using a deprecated API, this will break in KDE 4.6, please port your module to the new KTextEditor::CodeCompletionModelControllerInterface3";\
-    WHAT ret->FUNC;\
-    WHATELSE;\
-  }\
-}\
-\
 {\
   static KTextEditor::CodeCompletionModelControllerInterface3 defaultIf;\
   KTextEditor::CodeCompletionModelControllerInterface3* ret =\
@@ -89,15 +74,15 @@ static KTextEditor::Range _completionRange(KTextEditor::CodeCompletionModel *mod
   CALLCI(return,,return, model,completionRange(view, cursor));
 }
 
-static KTextEditor::Range _updateRange(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, KTextEditor::SmartRange& range) {
+static KTextEditor::Range _updateRange(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, KTextEditor::Range& range) {
   CALLCI(, return range,return, model,updateCompletionRange(view, range));
 }
 
-static QString _filterString(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, const KTextEditor::SmartRange& range, const KTextEditor::Cursor& cursor) {
+static QString _filterString(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, const KTextEditor::Range& range, const KTextEditor::Cursor& cursor) {
   CALLCI(return,,return, model,filterString(view, range, cursor));
 }
 
-static bool _shouldAbortCompletion(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, const KTextEditor::SmartRange& range, const QString& currentCompletion) {
+static bool _shouldAbortCompletion(KTextEditor::CodeCompletionModel *model,KTextEditor::View *view, const KTextEditor::Range& range, const QString& currentCompletion) {
       CALLCI(return,,return, model,shouldAbortCompletion(view, range, currentCompletion));
 }
 
@@ -125,6 +110,7 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
   , m_needShow(false)
   , m_hadCompletionNavigation(false)
   , m_noAutoHide(false)
+  , m_completionEditRunning (false)
   , m_expandedAddedHeightBase(0)
   , m_lastInvocationType(KTextEditor::CodeCompletionModel::AutomaticInvocation)
 {
@@ -159,19 +145,22 @@ KateCompletionWidget::KateCompletionWidget(KateView* parent)
   m_automaticInvocationTimer->setSingleShot(true);
   connect(m_automaticInvocationTimer, SIGNAL(timeout()), this, SLOT(automaticInvocation()));
 
-//   QVBoxLayout* vl = new QVBoxLayout(this);
-//   vl->addWidget(m_entryList);
-//   vl->setMargin(0);
-
   // Keep branches expanded
   connect(m_presentationModel, SIGNAL(modelReset()), this, SLOT(modelReset()));
   connect(m_presentationModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SLOT(rowsInserted(const QModelIndex&, int, int)));
   connect(m_argumentHintModel, SIGNAL(contentStateChanged(bool)), this, SLOT(argumentHintsChanged(bool)));
 
-  // These must be queued connections so that we're not holding the smart lock when we ask for the model to update.
-  connect(view(), SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(cursorPositionChanged()), Qt::QueuedConnection);
-  connect(view()->doc()->history(), SIGNAL(editDone(KateEditInfo*)), SLOT(editDone(KateEditInfo*)));
-  connect(view(), SIGNAL(verticalScrollPositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(updatePositionSlot()), Qt::QueuedConnection);
+  // No smart lock, no queued connects
+  connect(view(), SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(cursorPositionChanged()));
+  connect(view(), SIGNAL(verticalScrollPositionChanged (KTextEditor::View*, const KTextEditor::Cursor&)), this, SLOT(updatePositionSlot()));
+  
+  /**
+   * connect to all possible editing primitives
+   */
+  connect(&view()->doc()->buffer(), SIGNAL(lineWrapped(const KTextEditor::Cursor&)), this, SLOT(wrapLine(const KTextEditor::Cursor&)));
+  connect(&view()->doc()->buffer(), SIGNAL(lineUnwrapped(int)), this, SLOT(unwrapLine(int)));
+  connect(&view()->doc()->buffer(), SIGNAL(textInserted(const KTextEditor::Cursor &, const QString &)), this, SLOT(insertText(const KTextEditor::Cursor &, const QString &)));
+  connect(&view()->doc()->buffer(), SIGNAL(textRemoved(const KTextEditor::Range &, const QString &)), this, SLOT(removeText(const KTextEditor::Range &)));
 
   // This is a non-focus widget, it is passed keyboard input from the view
 
@@ -302,7 +291,7 @@ void KateCompletionWidget::startCompletion(KTextEditor::CodeCompletionModel::Inv
 
 void KateCompletionWidget::deleteCompletionRanges()
 {
-  kDebug();
+  //kDebug();
   foreach(const CompletionRange &r, m_completionRanges)
     delete r.range;
   m_completionRanges.clear();
@@ -356,7 +345,6 @@ void KateCompletionWidget::startCompletion(const KTextEditor::Range& word, const
   m_presentationModel->clearCompletionModels();
 
   if(invocationType == KTextEditor::CodeCompletionModel::UserInvocation) {
-    QMutexLocker lock(view()->doc()->smartMutex());
     deleteCompletionRanges();
   }
 
@@ -372,8 +360,7 @@ void KateCompletionWidget::startCompletion(const KTextEditor::Range& word, const
     kDebug()<<"range is"<<range;
     if(!range.isValid()) {
       if(m_completionRanges.contains(model)) {
-        QMutexLocker lock(view()->doc()->smartMutex());
-        KTextEditor::SmartRange *oldRange = m_completionRanges[model].range;
+        KTextEditor::MovingRange *oldRange = m_completionRanges[model].range;
         kDebug()<<"removing completion range 1";
         m_completionRanges.remove(model);
         delete oldRange;
@@ -386,8 +373,7 @@ void KateCompletionWidget::startCompletion(const KTextEditor::Range& word, const
         continue; //Leave it running as it is
       }
       else { // delete the range that was used previously
-        QMutexLocker lock(view()->doc()->smartMutex());
-        KTextEditor::SmartRange *oldRange = m_completionRanges[model].range;
+        KTextEditor::MovingRange *oldRange = m_completionRanges[model].range;
         kDebug()<<"removing completion range 2";
         m_completionRanges.remove(model);
         delete oldRange;
@@ -401,10 +387,7 @@ void KateCompletionWidget::startCompletion(const KTextEditor::Range& word, const
 
     disconnect(model, SIGNAL(waitForReset()), this, SLOT(waitForModelReset()));
 
-    QMutexLocker lock(view()->doc()->smartMutex());
-
-    m_completionRanges[model] = view()->doc()->smartManager()->newSmartRange(range);
-    m_completionRanges[model].range->setInsertBehavior(KTextEditor::SmartRange::ExpandRight | KTextEditor::SmartRange::ExpandLeft);
+    m_completionRanges[model] = view()->doc()->newMovingRange(range, KTextEditor::MovingRange::ExpandRight | KTextEditor::MovingRange::ExpandLeft);
 
     //In automatic invocation mode, hide the completion widget as soon as the position where the completion was started is passed to the left
     m_completionRanges[model].leftBoundary = view()->cursorPosition();
@@ -414,14 +397,11 @@ void KateCompletionWidget::startCompletion(const KTextEditor::Range& word, const
       if(range.start() < m_completionRanges[model].leftBoundary)
         m_completionRanges[model].leftBoundary = range.start();
 
-    if(!m_completionRanges[model].range->isValid()) {
+    if(!m_completionRanges[model].range->toRange().isValid()) {
       kWarning(13035) << "Could not construct valid smart-range from" << range << "instead got" << *m_completionRanges[model].range;
-      lock.unlock();
       abortCompletion();
       return;
     }
-
-    lock.unlock();
   }
 
   m_presentationModel->setCompletionModels(models);
@@ -677,7 +657,7 @@ void KateCompletionWidget::updateHeight()
 
 void KateCompletionWidget::cursorPositionChanged( )
 {
-  kDebug();
+  //kDebug();
   if (m_completionRanges.isEmpty())
     return;
 
@@ -689,9 +669,6 @@ void KateCompletionWidget::cursorPositionChanged( )
 
   QList<KTextEditor::CodeCompletionModel*> checkCompletionRanges = m_completionRanges.keys();
 
-  QMutexLocker lock(view()->doc()->smartMutex());
-  lock.unlock();
-
   //Check the models and eventuall abort some
   for(QList<KTextEditor::CodeCompletionModel*>::iterator it = checkCompletionRanges.begin();
       it != checkCompletionRanges.end(); ++it) {
@@ -699,10 +676,11 @@ void KateCompletionWidget::cursorPositionChanged( )
         continue;
 
       KTextEditor::CodeCompletionModel *model = *it;
-      KateSmartRange* range = m_completionRanges[*it].range;
+      KTextEditor::MovingRange *range = m_completionRanges[*it].range;
 
       kDebug()<<"range before _updateRange:"<< *range;
-      *range=_updateRange(model, view(),*range);
+      KTextEditor::Range rangeTE = range->toRange();
+      range->setRange (_updateRange(model, view(),rangeTE));
       kDebug()<<"range after _updateRange:"<< *range;
       QString currentCompletion = _filterString(model,view(), *range, view()->cursorPosition());
       kDebug()<<"after _filterString, currentCompletion="<< currentCompletion;
@@ -723,11 +701,9 @@ void KateCompletionWidget::cursorPositionChanged( )
           return;
         } else {
           {
-            lock.relock();
             delete m_completionRanges[*it].range;
             kDebug()<<"removing completion range 3";
             m_completionRanges.remove(*it);
-            lock.unlock();
           }
 
           _aborted(model,view());
@@ -785,8 +761,6 @@ void KateCompletionWidget::clear() {
   foreach(KTextEditor::CodeCompletionModel* model, m_completionRanges.keys())
     _aborted(model,view());
 
-  QMutexLocker lock(view()->doc()->smartMutex());
-
   deleteCompletionRanges();
 }
 
@@ -806,7 +780,7 @@ bool KateCompletionWidget::navigateAccept() {
 
 void KateCompletionWidget::execute()
 {
-  kDebug(13035) ;
+  //kDebug(13035) ;
 
   if (!isCompletionActive())
     return;
@@ -829,7 +803,8 @@ void KateCompletionWidget::execute()
   }
 
   // encapsulate all editing as being from the code completion, and undo-able in one step.
-  view()->doc()->editStart(Kate::CodeCompletionEdit);
+  view()->doc()->editStart();
+  m_completionEditRunning = true;
 
   // create scoped pointer, to ensure deletion of cursor
   QScopedPointer<KTextEditor::MovingCursor> oldPos (view()->doc()->newMovingCursor(view()->cursorPosition(), KTextEditor::MovingCursor::StayOnInsert));
@@ -842,10 +817,6 @@ void KateCompletionWidget::execute()
   Q_ASSERT(m_completionRanges.contains(model));
   KTextEditor::Cursor start = m_completionRanges[model].range->start();
 
-  //editStart locks the smart-mutex, but it must not be locked when calling external functions,
-  //else we may get deadlock-issues.
-  view()->doc()->smartMutex()->unlock();
-
   if(model2)
     model2->executeCompletionItem2(view()->document(), *m_completionRanges[model].range, toExecute);
   else if(toExecute.parent().isValid())
@@ -854,10 +825,8 @@ void KateCompletionWidget::execute()
   else
     model->executeCompletionItem(view()->document(), *m_completionRanges[model].range, toExecute.row());
 
-  //Relock, because editEnd expects it to be locked
-  view()->doc()->smartMutex()->lock();
-
   view()->doc()->editEnd();
+  m_completionEditRunning = false;
 
   abortCompletion();
 
@@ -889,14 +858,14 @@ void KateCompletionWidget::showEvent ( QShowEvent * event )
     m_argumentHintTree->show();
 }
 
-KateSmartRange * KateCompletionWidget::completionRange(KTextEditor::CodeCompletionModel* model) const
+KTextEditor::MovingRange * KateCompletionWidget::completionRange(KTextEditor::CodeCompletionModel* model) const
 {
   if (!model) {
     if (m_completionRanges.isEmpty()) return 0;
 
-    KateSmartRange* ret = m_completionRanges.begin()->range;
+    KTextEditor::MovingRange* ret = m_completionRanges.begin()->range;
     
-    foreach(CompletionRange range, m_completionRanges)
+    foreach(const CompletionRange &range, m_completionRanges)
       if(range.range->start() > ret->start())
         ret = range.range;
     return ret;
@@ -1216,6 +1185,10 @@ void KateCompletionWidget::modelDestroyed(QObject* model) {
 
 void KateCompletionWidget::registerCompletionModel(KTextEditor::CodeCompletionModel* model)
 {
+  if (m_sourceModels.contains(model)) {
+    return;
+  }
+
   connect(model, SIGNAL(destroyed(QObject*)), SLOT(modelDestroyed(QObject*)));
   //This connection must not be queued
   connect(model, SIGNAL(modelReset()), SLOT(completionModelReset()));
@@ -1244,8 +1217,66 @@ void KateCompletionWidget::setAutomaticInvocationDelay(int delay) {
   m_automaticInvocationDelay = delay;
 }
 
+void KateCompletionWidget::wrapLine (const KTextEditor::Cursor &)
+{
+  m_lastInsertionByUser = !m_completionEditRunning;
+  
+  // wrap line, be done
+  m_automaticInvocationLine.clear();
+  m_automaticInvocationTimer->stop();
+}
+
+void KateCompletionWidget::unwrapLine (int)
+{
+  m_lastInsertionByUser = !m_completionEditRunning;
+  
+  // just removal
+  m_automaticInvocationLine.clear();
+  m_automaticInvocationTimer->stop();
+}
+
+void KateCompletionWidget::insertText (const KTextEditor::Cursor &position, const QString &text)
+{
+  m_lastInsertionByUser = !m_completionEditRunning;
+
+  // no invoke?
+  if (!view()->config()->automaticCompletionInvocation()) {
+    m_automaticInvocationLine.clear();
+    m_automaticInvocationTimer->stop();
+    return;
+  }
+  
+  if(m_automaticInvocationAt != position) {
+    m_automaticInvocationLine.clear();
+    m_lastInsertionByUser = !m_completionEditRunning;
+  }
+
+  m_automaticInvocationLine += text;
+  m_automaticInvocationAt = position;
+  m_automaticInvocationAt.setColumn (position.column() + text.length());
+
+  if (m_automaticInvocationLine.isEmpty()) {
+    m_automaticInvocationTimer->stop();
+    return;
+  }
+
+  m_automaticInvocationTimer->start(m_automaticInvocationDelay);
+}
+
+void KateCompletionWidget::removeText (const KTextEditor::Range &)
+{
+  m_lastInsertionByUser = !m_completionEditRunning;
+
+  // just removal
+  m_automaticInvocationLine.clear();
+  m_automaticInvocationTimer->stop();
+}
+
+#if 0
 void KateCompletionWidget::editDone(KateEditInfo * edit)
 {
+  return;
+  
   if(!edit->newText().join("\n").trimmed().isEmpty())
   m_lastInsertionByUser = edit->editSource() == Kate::UserInputEdit;
 
@@ -1275,6 +1306,7 @@ void KateCompletionWidget::editDone(KateEditInfo * edit)
 
   m_automaticInvocationTimer->start(m_automaticInvocationDelay);
 }
+#endif
 
 void KateCompletionWidget::automaticInvocation()
 {
