@@ -27,6 +27,7 @@
 #include "katescript.h"
 
 #include <ktexteditor/highlightinterface.h>
+#include <ktexteditor/movingcursor.h>
 
 #include <QtScript/QScriptEngine>
 
@@ -47,9 +48,8 @@ KateDocument *KateScriptDocument::document()
 
 int KateScriptDocument::defStyleNum(int line, int column)
 {
-  KateDocCursor cursor(line, column, m_document);
   QList<KTextEditor::Attribute::Ptr> attributes = m_document->highlight()->attributes(((KateView*) m_document->activeView())->renderer()->config()->schema());
-  KTextEditor::Attribute::Ptr a = attributes[cursor.currentAttrib()];
+  KTextEditor::Attribute::Ptr a = attributes[document()->plainKateTextLine(line)->attribute(column)];
   return a->property(KateExtendedAttribute::AttributeDefaultStyleIndex).toInt();
 }
 
@@ -128,7 +128,7 @@ bool KateScriptDocument::isOthers(const KTextEditor::Cursor& cursor)
 int KateScriptDocument::firstVirtualColumn(int line)
 {
   const int tabWidth = m_document->config()->tabWidth();
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   const int firstPos = textLine ? textLine->firstChar() : -1;
   if(!textLine || firstPos == -1)
     return -1;
@@ -138,7 +138,7 @@ int KateScriptDocument::firstVirtualColumn(int line)
 int KateScriptDocument::lastVirtualColumn(int line)
 {
   const int tabWidth = m_document->config()->tabWidth();
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   const int lastPos = textLine ? textLine->lastChar() : -1;
   if(!textLine || lastPos == -1)
     return -1;
@@ -148,7 +148,7 @@ int KateScriptDocument::lastVirtualColumn(int line)
 int KateScriptDocument::toVirtualColumn(int line, int column)
 {
   const int tabWidth = m_document->config()->tabWidth();
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if (!textLine || column < 0 || column > textLine->length()) return -1;
   return textLine->toVirtualColumn(column, tabWidth);
 }
@@ -167,7 +167,7 @@ KTextEditor::Cursor KateScriptDocument::toVirtualCursor(const KTextEditor::Curso
 int KateScriptDocument::fromVirtualColumn(int line, int virtualColumn)
 {
   const int tabWidth = m_document->config()->tabWidth();
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if(!textLine || virtualColumn < 0 || virtualColumn > textLine->virtualLength(tabWidth))
     return -1;
   return textLine->fromVirtualColumn(virtualColumn, tabWidth);
@@ -186,27 +186,24 @@ KTextEditor::Cursor KateScriptDocument::fromVirtualCursor(const KTextEditor::Cur
 
 KTextEditor::Cursor KateScriptDocument::rfind(int line, int column, const QString& text, int attribute)
 {
-  KateDocCursor cursor(line, column, m_document);
-  const int start = cursor.line();
+  QScopedPointer<KTextEditor::MovingCursor> cursor(document()->newMovingCursor(KTextEditor::Cursor(line, column)));
+  const int start = cursor->line();
   QList<KTextEditor::Attribute::Ptr> attributes =
       m_document->highlight()->attributes(((KateView*)m_document->activeView())->renderer()->config()->schema());
 
   do {
-    KateTextLine::Ptr textLine = m_document->plainKateTextLine(cursor.line());
+    Kate::TextLine textLine = m_document->plainKateTextLine(cursor->line());
     if (!textLine)
       break;
 
-    if (cursor.line() != start) {
-      cursor.setColumn(textLine->length());
+    if (cursor->line() != start) {
+      cursor->setColumn(textLine->length());
     } else if (column >= textLine->length()) {
-      cursor.setColumn(qMax(textLine->length(), 0));
+      cursor->setColumn(qMax(textLine->length(), 0));
     }
 
-    bool found = true;
-    while (found) {
-      uint foundAt;
-      found = textLine->searchText(0, cursor.column(), text, &foundAt, 0, true, true);
-      if (found) {
+    int foundAt;
+    while ((foundAt = textLine->string().left(cursor->column()).lastIndexOf(text, -1, Qt::CaseSensitive)) >= 0) {
         bool hasStyle = true;
         if (attribute != -1) {
           KTextEditor::Attribute::Ptr a = attributes[textLine->attribute(foundAt)];
@@ -215,13 +212,12 @@ KTextEditor::Cursor KateScriptDocument::rfind(int line, int column, const QStrin
         }
 
         if (hasStyle) {
-          return KTextEditor::Cursor(cursor.line(), foundAt);
+          return KTextEditor::Cursor(cursor->line(), foundAt);
         } else {
-          cursor.setColumn(foundAt);
+          cursor->setColumn(foundAt);
         }
-      }
     }
-  } while (cursor.gotoPreviousLine());
+  } while (cursor->gotoPreviousLine());
 
   return KTextEditor::Cursor::invalid();
 }
@@ -233,29 +229,39 @@ KTextEditor::Cursor KateScriptDocument::rfind(const KTextEditor::Cursor& cursor,
 
 KTextEditor::Cursor KateScriptDocument::anchor(int line, int column, QChar character)
 {
-  KateDocCursor cursor(line, column, m_document);
   QList<KTextEditor::Attribute::Ptr> attributes =
       m_document->highlight()->attributes(((KateView*) m_document->activeView())->renderer()->config()->schema());
   int count = 1;
-  QChar lc = character;
+  QChar lc;
   QChar rc;
-  if (lc == '(') rc = ')';
-  else if (lc == '{') rc = '}';
-  else if (lc == '[') rc = ']';
-  else return KTextEditor::Cursor::invalid ();
+  if (character == '(' || character == ')') {
+    lc = '(';
+    rc = ')';
+  } else if (character == '{' || character == '}') {
+    lc = '{';
+    rc = '}';
+  } else if (character == '[' || character == ']') {
+    lc = '[';
+    rc = ']';
+  } else {
+    kDebug(13060) << "invalid anchor character:" << character << " allowed are: (){}[]";
+    return KTextEditor::Cursor::invalid();
+  }
+
+  QScopedPointer<KTextEditor::MovingCursor> cursor(document()->newMovingCursor(KTextEditor::Cursor(line, column)));
 
   // Move backwards char by char and find the opening character
-  while (cursor.moveBackward(1)) {
-    QChar ch = cursor.currentChar();
+  while (cursor->move(-1)) {
+    QChar ch = document()->character(cursor->toCursor());
     if (ch == lc) {
-      KTextEditor::Attribute::Ptr a = attributes[cursor.currentAttrib()];
+      KTextEditor::Attribute::Ptr a = attributes[document()->plainKateTextLine(cursor->line())->attribute(cursor->column())];
       const int ds = a->property(KateExtendedAttribute::AttributeDefaultStyleIndex).toInt();
       if (_isCode(ds)) {
         --count;
       }
     }
     else if (ch == rc) {
-      KTextEditor::Attribute::Ptr a = attributes[cursor.currentAttrib()];
+      KTextEditor::Attribute::Ptr a = attributes[document()->plainKateTextLine(cursor->line())->attribute(cursor->column())];
       const int ds = a->property(KateExtendedAttribute::AttributeDefaultStyleIndex).toInt();
       if (_isCode(ds)) {
         ++count;
@@ -263,7 +269,7 @@ KTextEditor::Cursor KateScriptDocument::anchor(int line, int column, QChar chara
     }
 
     if (count == 0) {
-      return cursor;
+      return cursor->toCursor();
     }
   }
   return KTextEditor::Cursor::invalid ();
@@ -276,7 +282,7 @@ KTextEditor::Cursor KateScriptDocument::anchor(const KTextEditor::Cursor& cursor
 
 bool KateScriptDocument::startsWith (int line, const QString &pattern, bool skipWhiteSpaces)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
 
   if (!textLine)
     return false;
@@ -289,7 +295,7 @@ bool KateScriptDocument::startsWith (int line, const QString &pattern, bool skip
 
 bool KateScriptDocument::endsWith (int line, const QString &pattern, bool skipWhiteSpaces)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
 
   if (!textLine)
     return false;
@@ -322,6 +328,21 @@ QString KateScriptDocument::encoding()
   return m_document->encoding();
 }
 
+QString KateScriptDocument::highlightingMode()
+{
+  return m_document->highlightingMode();
+}
+
+QStringList KateScriptDocument::embeddedHighlightingModes()
+{
+  return m_document->embeddedHighlightingModes();
+}
+
+QString KateScriptDocument::highlightingModeAt(const KTextEditor::Cursor& pos)
+{
+  return m_document->highlightingModeAt(pos);
+}
+
 bool KateScriptDocument::isModified()
 {
   return m_document->isModified();
@@ -375,7 +396,7 @@ QString KateScriptDocument::charAt(const KTextEditor::Cursor& cursor)
 
 QString KateScriptDocument::firstChar(int line)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if(!textLine) return "";
   // check for isNull(), as the returned character then would be "\0"
   const QChar c = textLine->at(textLine->firstChar());
@@ -384,7 +405,7 @@ QString KateScriptDocument::firstChar(int line)
 
 QString KateScriptDocument::lastChar(int line)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if(!textLine) return "";
   // check for isNull(), as the returned character then would be "\0"
   const QChar c = textLine->at(textLine->lastChar());
@@ -403,7 +424,7 @@ bool KateScriptDocument::isSpace(const KTextEditor::Cursor& cursor)
 
 bool KateScriptDocument::matchesAt(int line, int column, const QString &s)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   return textLine ? textLine->matchesAt(column, s) : false;
 }
 
@@ -424,9 +445,12 @@ bool KateScriptDocument::clear()
 
 bool KateScriptDocument::truncate(int line, int column)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
-  if(textLine) textLine->truncate(column);
-  return static_cast<bool>(textLine);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
+  if (!textLine || textLine->text().size() < column)
+    return false;
+
+  KTextEditor::Cursor from (line, column), to (line, textLine->text().size() - column);
+  return removeText(KTextEditor::Range(from, to));
 }
 
 bool KateScriptDocument::truncate(const KTextEditor::Cursor& cursor)
@@ -501,19 +525,19 @@ void KateScriptDocument::editEnd()
 
 int KateScriptDocument::firstColumn(int line)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   return textLine ? textLine->firstChar() : -1;
 }
 
 int KateScriptDocument::lastColumn(int line)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   return textLine ? textLine->lastChar() : -1;
 }
 
 int KateScriptDocument::prevNonSpaceColumn(int line, int column)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if(!textLine) return -1;
   return textLine->previousNonSpaceChar(column);
 }
@@ -525,7 +549,7 @@ int KateScriptDocument::prevNonSpaceColumn(const KTextEditor::Cursor& cursor)
 
 int KateScriptDocument::nextNonSpaceColumn(int line, int column)
 {
-  KateTextLine::Ptr textLine = m_document->plainKateTextLine(line);
+  Kate::TextLine textLine = m_document->plainKateTextLine(line);
   if(!textLine) return -1;
   return textLine->nextNonSpaceChar(column);
 }
@@ -539,7 +563,7 @@ int KateScriptDocument::prevNonEmptyLine(int line)
 {
   const int startLine = line;
   for (int currentLine = startLine; currentLine >= 0; --currentLine) {
-    KateTextLine::Ptr textLine = m_document->plainKateTextLine(currentLine);
+    Kate::TextLine textLine = m_document->plainKateTextLine(currentLine);
     if(!textLine)
       return -1;
     if(textLine->firstChar() != -1)
@@ -552,7 +576,7 @@ int KateScriptDocument::nextNonEmptyLine(int line)
 {
   const int startLine = line;
   for (int currentLine = startLine; currentLine < m_document->lines(); ++currentLine) {
-    KateTextLine::Ptr textLine = m_document->plainKateTextLine(currentLine);
+    Kate::TextLine textLine = m_document->plainKateTextLine(currentLine);
     if(!textLine)
       return -1;
     if(textLine->firstChar() != -1)
@@ -591,9 +615,19 @@ QString KateScriptDocument::commentEnd(int attribute)
   return m_document->highlight()->getCommentEnd(attribute);
 }
 
+KTextEditor::Range KateScriptDocument::documentRange()
+{
+  return m_document->documentRange();
+}
+
+KTextEditor::Cursor KateScriptDocument::documentEnd()
+{
+  return m_document->documentEnd();
+}
+
 int KateScriptDocument::attribute(int line, int column)
 {
-  KateTextLine::Ptr textLine = m_document->kateTextLine(line);
+  Kate::TextLine textLine = m_document->kateTextLine(line);
   if(!textLine) return 0;
   return textLine->attribute(column);
 }
@@ -615,9 +649,8 @@ bool KateScriptDocument::isAttribute(const KTextEditor::Cursor& cursor, int attr
 
 QString KateScriptDocument::attributeName(int line, int column)
 {
-  KateDocCursor cursor(line, column, m_document);
   QList<KTextEditor::Attribute::Ptr> attributes = m_document->highlight()->attributes(((KateView*) m_document->activeView())->renderer()->config()->schema());
-  KTextEditor::Attribute::Ptr a = attributes[cursor.currentAttrib()];
+  KTextEditor::Attribute::Ptr a = attributes[document()->plainKateTextLine(line)->attribute(column)];
   return a->property(KateExtendedAttribute::AttributeName).toString();
 }
 
@@ -652,5 +685,17 @@ bool KateScriptDocument::_isCode(int defaultStyle)
        && defaultStyle != KTextEditor::HighlightInterface::dsOthers);
 }
 
+void KateScriptDocument::indent(KTextEditor::Range range, int change)
+{
+  m_document->indent(range, change);
+}
+
+void KateScriptDocument::align(const KTextEditor::Range& range)
+{
+  if (m_document->activeKateView()) {
+    m_document->align(m_document->activeKateView(), range);
+  }
+}
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
 
