@@ -244,9 +244,11 @@ void KateSearchBar::setSearchMode(KateSearchBar::SearchMode mode) {
 void KateSearchBar::findNext() {
     const bool found = find();
 
-    if (found && m_powerUi != NULL) {
+    if (found) {
+        QComboBox *combo = m_powerUi != 0 ? m_powerUi->pattern : m_incUi->pattern;
+
         // Add to search history
-        addCurrentTextToHistory(m_powerUi->pattern);
+        addCurrentTextToHistory(combo);
     }
 }
 
@@ -255,9 +257,11 @@ void KateSearchBar::findNext() {
 void KateSearchBar::findPrevious() {
     const bool found = find(SearchBackward);
 
-    if (found && m_powerUi != NULL) {
+    if (found) {
+        QComboBox *combo = m_powerUi != 0 ? m_powerUi->pattern : m_incUi->pattern;
+
         // Add to search history
-        addCurrentTextToHistory(m_powerUi->pattern);
+        addCurrentTextToHistory(combo);
     }
 }
 
@@ -656,11 +660,13 @@ void KateSearchBar::givePatternFeedback() {
 void KateSearchBar::addCurrentTextToHistory(QComboBox * combo) {
     const QString text = combo->currentText();
     const int index = combo->findText(text);
-    if (index != -1) {
+
+    if (index > 0)
         combo->removeItem(index);
+    if (index != 0) {
+        combo->insertItem(0, text);
+        combo->setCurrentIndex(0);
     }
-    combo->insertItem(0, text);
-    combo->setCurrentIndex(0);
 }
 
 
@@ -791,6 +797,8 @@ int KateSearchBar::findAll(Range inputRange, const QString * replacement) {
             }
 
             // Continue after match
+            if (highlightRanges.last().end() >= workingRange->end())
+                break;
             KTextEditor::MovingCursor* workingStart =
                 static_cast<KateDocument*>(m_view->document())->newMovingCursor(highlightRanges.last().end());
             if (originalMatchEmpty) {
@@ -1255,9 +1263,11 @@ void KateSearchBar::enterPowerMode() {
         m_layout->addWidget(m_widget);
 
         // Bind to shared history models
+        m_powerUi->pattern->setDuplicatesEnabled(false);
         m_powerUi->pattern->setInsertPolicy(QComboBox::InsertAtTop);
         m_powerUi->pattern->setMaxCount(m_config->maxHistorySize());
         m_powerUi->pattern->setModel(m_config->patternHistoryModel());
+        m_powerUi->replacement->setDuplicatesEnabled(false);
         m_powerUi->replacement->setInsertPolicy(QComboBox::InsertAtTop);
         m_powerUi->replacement->setMaxCount(m_config->maxHistorySize());
         m_powerUi->replacement->setModel(m_config->replacementHistoryModel());
@@ -1418,9 +1428,11 @@ void KateSearchBar::enterIncrementalMode() {
         // Focus proxy
         centralWidget()->setFocusProxy(m_incUi->pattern);
 
+        m_incUi->pattern->setDuplicatesEnabled(false);
         m_incUi->pattern->setInsertPolicy(QComboBox::InsertAtTop);
         m_incUi->pattern->setMaxCount(m_config->maxHistorySize());
         m_incUi->pattern->setModel(m_config->patternHistoryModel());
+        m_incUi->pattern->setAutoCompletion(false);
     }
 
     // Restore previous settings
@@ -1481,13 +1493,6 @@ void KateSearchBar::showEvent(QShowEvent * event) {
 }
 
 
-
-void KateSearchBar::closed() {
-    // empty, can probably be removed
-}
-
-
-
 void KateSearchBar::updateSelectionOnly() {
     if (m_powerUi == NULL) {
         return;
