@@ -1,21 +1,23 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2001-2005 Christoph Cullmann <cullmann@kde.org>
-   Copyright (C) 2009 Erlend Hamberg <ehamberg@gmail.com>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
-*/
+/*  This file is part of the KDE libraries and the Kate part.
+ *
+ *  Copyright (C) 2001-2010 Christoph Cullmann <cullmann@kde.org>
+ *  Copyright (C) 2009 Erlend Hamberg <ehamberg@gmail.com>
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
+ */
 
 #include "kateglobal.h"
 #include "kateglobal.moc"
@@ -54,7 +56,7 @@ int KateGlobal::s_ref = 0;
 
 QString KateGlobal::katePartVersion()
 {
-  return QString("%1.%2").arg(KDE::versionMajor() - 1).arg(KDE::versionMinor());
+  return QString("3.6");
 }
 
 KateGlobal::KateGlobal ()
@@ -153,13 +155,13 @@ KateGlobal::KateGlobal ()
   m_spellCheckManager = new KateSpellCheckManager ();
 
   // config objects
+  m_globalConfig = new KateGlobalConfig ();
   m_documentConfig = new KateDocumentConfig ();
   m_viewConfig = new KateViewConfig ();
   m_rendererConfig = new KateRendererConfig ();
 
-  // create script manager (search scripts) + register commands
+  // create script manager (search scripts)
   m_scriptManager = new KateScriptManager ();
-  KateCmd::self()->registerCommand (m_scriptManager);
 
   //
   // plugin manager
@@ -171,6 +173,7 @@ KateGlobal::KateGlobal ()
   //
   m_cmds.push_back (new KateCommands::CoreCommands());
   m_cmds.push_back (new KateCommands::ViCommands());
+  m_cmds.push_back (new KateCommands::AppCommands());
   m_cmds.push_back (new KateCommands::SedReplace ());
   m_cmds.push_back (new KateCommands::Character ());
   m_cmds.push_back (new KateCommands::Date ());
@@ -186,6 +189,7 @@ KateGlobal::~KateGlobal()
 {
   delete m_pluginManager;
 
+  delete m_globalConfig;
   delete m_documentConfig;
   delete m_viewConfig;
   delete m_rendererConfig;
@@ -205,7 +209,7 @@ KateGlobal::~KateGlobal()
 
   delete m_viInputModeGlobal;
   delete m_spellCheckManager;
-  
+
   // cu model
   delete m_wordCompletionModel;
 
@@ -232,6 +236,8 @@ void KateGlobal::readConfig(KConfig *config)
   if( !config )
     config = KGlobal::config().data();
 
+  KateGlobalConfig::global()->readConfig (KConfigGroup(config, "Kate Part Defaults"));
+
   KateDocumentConfig::global()->readConfig (KConfigGroup(config, "Kate Document Defaults"));
 
   KateViewConfig::global()->readConfig (KConfigGroup(config, "Kate View Defaults"));
@@ -246,6 +252,9 @@ void KateGlobal::writeConfig(KConfig *config)
   if( !config )
     config = KGlobal::config().data();
 
+  KConfigGroup cgGlobal(config, "Kate Part Defaults");
+  KateGlobalConfig::global()->writeConfig (cgGlobal);
+
   KConfigGroup cg(config, "Kate Document Defaults");
   KateDocumentConfig::global()->writeConfig (cg);
 
@@ -297,6 +306,7 @@ void KateGlobal::configDialog(QWidget *parent)
 
   if (kd->exec() && kd)
   {
+    KateGlobalConfig::global()->configStart ();
     KateDocumentConfig::global()->configStart ();
     KateViewConfig::global()->configStart ();
     KateRendererConfig::global()->configStart ();
@@ -306,6 +316,7 @@ void KateGlobal::configDialog(QWidget *parent)
       editorPages.at(i)->apply();
     }
 
+    KateGlobalConfig::global()->configEnd ();
     KateDocumentConfig::global()->configEnd ();
     KateViewConfig::global()->configEnd ();
     KateRendererConfig::global()->configEnd ();
@@ -329,7 +340,7 @@ KTextEditor::ConfigPage *KateGlobal::configPage (int number, QWidget *parent)
       return new KateViewDefaultsConfig (parent);
 
     case 1:
-      return new KateSchemaConfigPage (parent, 0);
+      return new KateSchemaConfigPage (parent);
 
     case 2:
       return new KateEditConfigTab (parent);
@@ -487,4 +498,15 @@ void KateGlobal::setContainer( QObject * container )
 //END container interface
 
 
+
+KTextEditor::TemplateScript* KateGlobal::registerTemplateScript (QObject* owner, const QString& script)
+{
+  return scriptManager()->registerTemplateScript(owner, script);
+}
+
+void KateGlobal::unregisterTemplateScript(KTextEditor::TemplateScript* templateScript)
+{
+  scriptManager()->unregisterTemplateScript(templateScript);
+}
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
