@@ -92,7 +92,6 @@ KateTemplateHandler::KateTemplateHandler(KateView *view,
 
   ifDebug(kDebug() << initial_Values;)
 
-
   connect(doc(), SIGNAL(aboutToReload(KTextEditor::Document*)),
           this, SLOT(cleanupAndExit()));
 
@@ -119,13 +118,12 @@ KateTemplateHandler::KateTemplateHandler(KateView *view,
   /// - create view => ranges are added
   /// for now simply "just insert" the code when we have no active view
 
+  handleTemplateString(initial_Values);
+  m_undoManager->undoSafePoint();
+  doc()->editEnd();
+
   if (!initialValues.isEmpty() && m_view) {
     // only do complex stuff when required
-
-    handleTemplateString(initial_Values);
-    m_undoManager->undoSafePoint();
-    doc()->editEnd();
-
     if (!m_templateRanges.isEmpty()) {
       foreach(View* view, doc()->views()) {
         setupEventHandler(view);
@@ -150,9 +148,6 @@ KateTemplateHandler::KateTemplateHandler(KateView *view,
     }
 
   } else {
-    m_undoManager->undoSafePoint();
-    doc()->editEnd();
-    // simple templates just need to be (which gets done in handleTemplateString())
     cleanupAndExit();
   }
 }
@@ -268,6 +263,7 @@ bool KateTemplateHandler::eventFilter(QObject* object, QEvent* event)
       // terminate
       jumpToFinalCursorPosition();
       cleanupAndExit();
+      keyEvent->accept();
       return true;
 
     } else if (keyEvent->key() == Qt::Key_Escape) {
@@ -275,6 +271,7 @@ bool KateTemplateHandler::eventFilter(QObject* object, QEvent* event)
         // terminate
         jumpToFinalCursorPosition();
         cleanupAndExit();
+        keyEvent->accept();
         return true;
       }
 
@@ -286,10 +283,12 @@ bool KateTemplateHandler::eventFilter(QObject* object, QEvent* event)
         jumpToNextRange();
       }
 
+      keyEvent->accept();
       return true;
 
     } else if (keyEvent->key() == Qt::Key_Backtab && !m_view->isCompletionActive()) {
       jumpToPreviousRange();
+      keyEvent->accept();
       return true;
     }
   }
@@ -378,10 +377,11 @@ void KateTemplateHandler::setCurrentRange(MovingRange* range)
         found = true;
         break;
       }
-
-      if (!found) range = m_templateRangesChildren[range].first();
     }
 
+    if (!found) {
+      range = m_templateRangesChildren[range].first();
+    }
   }
 
   if (m_view) {
@@ -586,21 +586,19 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
         // do nothing
       } else if ((pos_slash != -1) && (pos_colon == -1)) {
         check_slash = true;
-
       } else if ((pos_slash == -1) && (pos_colon != -1)) {
         check_colon = true;
-
       } else {
         if (pos_colon < pos_slash) {
           check_colon = true;
-
         } else {
           check_slash = true;
         }
       }
 
-      if ((!check_slash) && (!check_colon) && (pos_backtick >= 0))
+      if (!check_slash && !check_colon && pos_backtick >= 0) {
         check_backtick = true;
+      }
 
       QString functionName;
 
@@ -608,11 +606,9 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
         searchReplace = key.mid(pos_slash + 1);
         key = key.left(pos_slash);
         ifDebug(kDebug() << "search_replace" << searchReplace;)
-
       } else if (check_colon) {
         key = key.left(pos_colon);
         ifDebug(kDebug() << "real key found:" << key;)
-
       } else if (check_backtick) {
         functionName = key.mid(pos_backtick + 1);
         functionName = functionName.left(functionName.indexOf("`"));
@@ -629,7 +625,6 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
 
       if (!initialValues.contains(key)) {
         kWarning() << "unknown variable key:" << key;
-
       } else if (key == "cursor") {
         finalCursorPosition = Cursor(line, column - keyLength - 2);
         // don't insert anything, just remove the placeholder
@@ -638,7 +633,6 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
         i -= 3 + keyLength;
         column -= 2 + keyLength;
         startPos = -1;
-
       } else {
         MirrorBehaviour behaviour;
 
@@ -655,15 +649,15 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
             int regescapes = 0;
             int pos = searchReplace.indexOf("/");
 
-            for (int epos = pos - 1;(epos >= 0) && (searchReplace.at(epos) == '\\');epos--)
+            for (int epos = pos - 1; epos >= 0 && searchReplace.at(epos) == '\\'; epos--) {
               regescapes++;
+            }
 
             ifDebug(kDebug() << "regescapes=" << regescapes;)
             if ((regescapes % 2) == 1) {
               search += searchReplace.left(pos + 1);
               searchReplace = searchReplace.mid(pos + 1);
               ifDebug(kDebug() << "intermediate search string is=" << search;)
-
             } else {
               search += searchReplace.left(pos);
               searchReplace = searchReplace.mid(pos + 1);
@@ -686,8 +680,9 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
             }
           }
 
-          if (searchValid && replaceValid)
+          if (searchValid && replaceValid) {
             behaviour = MirrorBehaviour(search, replace, flags);
+          }
 
         } else if (!functionName.isEmpty()) {
           behaviour = MirrorBehaviour(m_templateScript, functionName, this);
@@ -706,7 +701,6 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
 
         // correct column to point at end of range, taking replacement width diff into account
         // 2 == % + {
-
         column -= 2 + keyLength - initialVal.length();
 
         // always add ${...} to the editable ranges
@@ -736,9 +730,9 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
             range_list.append(tmp);
             ranges.remove(key);
 
-            while (!range_list.isEmpty())
+            while (!range_list.isEmpty()) {
               ranges.insert(key, range_list.takeLast());
-
+            }
           } else {
             ranges.insert(key, tmp);
           }
@@ -750,7 +744,6 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
       startPos = -1;
 
       ifDebug(kDebug() << "i=" << i << " template size=" << templateString.size();)
-
     } else {
       ++column;
       lastWasBrace = false;
@@ -777,7 +770,6 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
 
   if (finalCursorPosition.isValid()) {
     m_finalCursorPosition = doc()->newMovingCursor(finalCursorPosition);
-
   } else {
     m_finalCursorPosition = doc()->newMovingCursor(Cursor(line, column));
   }
@@ -817,18 +809,15 @@ void KateTemplateHandler::handleTemplateString(const QMap< QString, QString >& i
         m_templateRangesChildToParent[range] = parent;
 
         // the last item will be our real first range (see multimap docs)
-
         if (i == values.size() - 1) {
           range->setAttribute(editableAttribute);
           m_uneditedRanges.append(range);
           m_masterRanges.append(range);
-
         } else {
           range->setAttribute(mirroredAttribute);
           m_mirrorBehaviour.insert(range, mirrorBehaviourBuildHelper[values[i]]);
         }
       }
-
     } else {
       // just a single range
       parent->setAttribute(editableAttribute);
@@ -881,7 +870,7 @@ void KateTemplateHandler::slotTextChanged(Document* document, const Range& range
 
   if (!m_initialRemodify) {
     if ((!m_editWithUndo && doc()->isEditRunning()) || range.isEmpty()) {
-      ifDebug(kDebug(13020) << "slotTextChanged returning prematurely";)
+      ifDebug(kDebug() << "slotTextChanged returning prematurely";)
       return;
     }
   }
@@ -901,6 +890,7 @@ void KateTemplateHandler::slotTextChanged(Document* document, const Range& range
   if (!m_wholeTemplateRange->toRange().contains(range.start())) {
     // outside our template or one of our own changes
     ifDebug(kDebug() << range << "not contained in" << m_wholeTemplateRange->toRange();)
+    cleanupAndExit();
     return;
   }
 
@@ -922,8 +912,8 @@ void KateTemplateHandler::slotTextChanged(Document* document, const Range& range
     if (customContains(parent, range.start())) {
       if (m_templateRangesChildren[parent].isEmpty()) {
         // simple, not-mirrored range got changed
-        m_uneditedRanges.removeOne(parent);
-
+        if (!m_initialRemodify)
+          m_uneditedRanges.removeOne(parent);
       } else {
         // handle mirrored ranges
         if (baseRange) {
@@ -940,7 +930,6 @@ void KateTemplateHandler::slotTextChanged(Document* document, const Range& range
 
           // adjacent mirror handled, we can finish
           break;
-
         } else {
           // find mirrored range that got edited
           foreach(MovingRange* child, m_templateRangesChildren[parent]) {
@@ -953,7 +942,6 @@ void KateTemplateHandler::slotTextChanged(Document* document, const Range& range
           if (baseRange && baseRange->end() != range.end()) {
             // finish, don't look for adjacent mirror as we are not at the end of this range
             break;
-
           } else if (baseRange && (baseRange->isEmpty() || range == *baseRange)) {
             // always add to empty ranges first. else ${foo}${bar} will make foo unusable when
             // it gets selected and an edit takes place.
@@ -1072,29 +1060,26 @@ QString KateTemplateHandler::MirrorBehaviour::getMirrorString(const QString &sou
         ahead = source;
 
         while (ahead.length() > 0) {
-          if ((pos = m_expr.indexIn(ahead)) == -1) return finalOutput + ahead;
+          if ((pos = m_expr.indexIn(ahead)) == -1) {
+            return finalOutput + ahead;
+          }
 
           QStringList results = m_expr.capturedTexts();
-
           output = KateRegExpSearch::buildReplacement(m_replace, results, ++matchCounter);
 
           finalOutput = finalOutput + ahead.left(pos) + output;
-
           ahead = ahead.mid(pos + m_expr.matchedLength());
         }
 
         return finalOutput;
-
       } else {
-        if ((pos = m_expr.indexIn(source)) == -1) return source;
+        if ((pos = m_expr.indexIn(source)) == -1) {
+          return source;
+        }
 
         QStringList results = m_expr.capturedTexts();
-
         output = KateRegExpSearch::buildReplacement(m_replace, results, 1);
-
         return source.left(pos) + output + source.mid(pos + m_expr.matchedLength());
-
-        break;
       }
 
       break;
@@ -1112,8 +1097,6 @@ QString KateTemplateHandler::MirrorBehaviour::getMirrorString(const QString &sou
       }
 
       return source;
-
-      break;
     }
 
     default:
