@@ -64,6 +64,7 @@ KateViNormalMode::KateViNormalMode( KateViInputModeManager *viInputModeManager,
   connect(m_mappingTimer, SIGNAL(timeout()), this, SLOT(mappingTimerTimeOut()));
 
   initializeCommands();
+  m_ignoreMapping = false;
   resetParser(); // initialise with start configuration
 }
 
@@ -108,7 +109,7 @@ bool KateViNormalMode::handleKeypress( const QKeyEvent *e )
   QChar key = KateViKeyParser::getInstance()->KeyEventToQChar( keyCode, text, e->modifiers(), e->nativeScanCode() );
 
   // check for matching mappings
-  if ( !m_mappingKeyPress && m_matchingCommands.size() == 0 ) {
+  if ( !m_mappingKeyPress && !m_ignoreMapping && m_matchingCommands.size() == 0 ) {
     m_mappingKeys.append( key );
 
     foreach ( const QString &str, getMappings() ) {
@@ -130,6 +131,14 @@ bool KateViNormalMode::handleKeypress( const QKeyEvent *e )
     //m_mappingKeyPress = false; // key press ignored wrt mappings, re-set m_mappingKeyPress
   }
 
+  if ( m_ignoreMapping ) m_ignoreMapping = false;
+
+  if ( key == 'f' || key == 'F' || key == 't' || key == 'T' ) {
+      // don't translate next character, we need the actual character so that
+      // 'ab' is translated to 'fb' if the mapping 'a' -> 'f' exists
+      m_ignoreMapping = true;
+  }
+
   m_keysVerbatim.append( KateViKeyParser::getInstance()->decodeKeySequence( key ) );
 
   QChar c = QChar::Null;
@@ -659,7 +668,7 @@ bool KateViNormalMode::commandMakeLowercaseLine()
   m_commandRange.startLine = c.line();
   m_commandRange.endLine = c.line();
   m_commandRange.startColumn = 0;
-  m_commandRange.endColumn = doc()->lineLength( c.line() );
+  m_commandRange.endColumn = doc()->lineLength( c.line() )-1;
 
   return commandMakeLowercase();
 }
@@ -688,7 +697,7 @@ bool KateViNormalMode::commandMakeUppercaseLine()
   m_commandRange.startLine = c.line();
   m_commandRange.endLine = c.line();
   m_commandRange.startColumn = 0;
-  m_commandRange.endColumn = doc()->lineLength( c.line() );
+  m_commandRange.endColumn = doc()->lineLength( c.line() )-1;
 
   return commandMakeUppercase();
 }
@@ -1254,6 +1263,7 @@ bool KateViNormalMode::commandPrintCharacterCode()
     QString hex = QString::number( code, 16 );
     QString oct = QString::number( code, 8 );
     if ( oct.length() < 3 ) { oct.prepend( '0' ); }
+    if ( code > 0x80 && code < 0x1000 ) { hex.prepend( ( code < 0x100 ? "00" : "0" ) ); }
     message( i18n("'%1' %2,  Hex %3,  Octal %4", ch, dec, hex, oct ) );
   }
 
