@@ -59,7 +59,7 @@ KConfigPrivate::KConfigPrivate(const KComponentData &componentData_, KConfig::Op
       bFileImmutable(false), bForceGlobal(false), bSuppressGlobal(false),
       componentData(componentData_), configState(KConfigBase::NoAccess)
 {
-    sGlobalFileName = componentData.dirs()->saveLocation("config") + QLatin1String("kdeglobals");
+    sGlobalFileName = componentData.dirs()->saveLocation("config", QString(), false) + QLatin1String("kdeglobals");
 
     static int use_etc_kderc = -1;
     if (use_etc_kderc < 0)
@@ -70,7 +70,7 @@ KConfigPrivate::KConfigPrivate(const KComponentData &componentData_, KConfig::Op
 #ifdef Q_WS_WIN
             QFile::decodeName( qgetenv("WINDIR") + "/kde4rc" );
 #else
-        QLatin1String("/etc/kde4rc");
+            QLatin1String("/etc/kde4rc");
 #endif
         if (!KStandardDirs::checkAccess(etc_kderc, R_OK)) {
             etc_kderc.clear();
@@ -207,10 +207,11 @@ QStringList KConfig::groupList() const
     QSet<QString> groups;
 
     for (KEntryMap::ConstIterator entryMapIt( d->entryMap.constBegin() ); entryMapIt != d->entryMap.constEnd(); ++entryMapIt) {
-        const QByteArray group = entryMapIt.key().mGroup;
-        if (entryMapIt.key().mKey.isNull() && !group.isEmpty() && group != "<default>" && group != "$Version") {
-            QString groupname = QString::fromUtf8(group);
-            groups << groupname.left(groupname.indexOf('\x1d'));
+        const KEntryKey& key = entryMapIt.key();	
+        const QByteArray group = key.mGroup;
+        if (key.mKey.isNull() && !group.isEmpty() && group != "<default>" && group != "$Version") {
+            const QString groupname = QString::fromUtf8(group);
+            groups << groupname.left(groupname.indexOf(QLatin1Char('\x1d')));
         }
     }
 
@@ -222,23 +223,53 @@ QStringList KConfigPrivate::groupList(const QByteArray& group) const
     QByteArray theGroup = group + '\x1d';
     QSet<QString> groups;
 
-    for (KEntryMap::ConstIterator entryMapIt( entryMap.constBegin() ); entryMapIt != entryMap.constEnd(); ++entryMapIt)
-        if (entryMapIt.key().mKey.isNull() && entryMapIt.key().mGroup.startsWith(theGroup)) {
-            QString groupname = QString::fromUtf8(entryMapIt.key().mGroup.mid(theGroup.length()));
-            groups << groupname.left(groupname.indexOf('\x1d'));
+    for (KEntryMap::ConstIterator entryMapIt( entryMap.constBegin() ); entryMapIt != entryMap.constEnd(); ++entryMapIt) {
+        const KEntryKey& key = entryMapIt.key();	
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+            const QString groupname = QString::fromUtf8(key.mGroup.mid(theGroup.length()));
+            groups << groupname.left(groupname.indexOf(QLatin1Char('\x1d')));
         }
+    }
 
     return groups.toList();
 }
 
-QStringList KConfig::keyList(const QString& aGroup) const
+// List all sub groups, including subsubgroups
+QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray& parentGroup) const
 {
-    Q_D(const KConfig);
-    QStringList keys;
-    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
+    QSet<QByteArray> groups;
+    QByteArray theGroup = parentGroup + '\x1d';
+    groups << parentGroup;
 
-    const KEntryMapConstIterator theEnd = d->entryMap.constEnd();
-    KEntryMapConstIterator it = d->entryMap.findEntry(theGroup);
+    for (KEntryMap::const_iterator entryMapIt = entryMap.begin(); entryMapIt != entryMap.end(); ++entryMapIt) {
+        const KEntryKey& key = entryMapIt.key();
+        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+            groups << key.mGroup;
+        }
+    }
+    return groups;
+}
+
+bool KConfigPrivate::hasNonDeletedEntries(const QByteArray& group) const
+{
+    const QSet<QByteArray> allGroups = allSubGroups(group);
+
+    Q_FOREACH(const QByteArray& aGroup, allGroups) {
+        // Could be optimized, let's use the slow way for now
+        // Check for any non-deleted entry
+        if (!keyListImpl(aGroup).isEmpty())
+            return true;
+    }
+    return false;
+}
+
+
+QStringList KConfigPrivate::keyListImpl(const QByteArray& theGroup) const
+{
+    QStringList keys;
+
+    const KEntryMapConstIterator theEnd = entryMap.constEnd();
+    KEntryMapConstIterator it = entryMap.findEntry(theGroup);
     if (it != theEnd) {
         ++it; // advance past the special group entry marker
 
@@ -254,6 +285,13 @@ QStringList KConfig::keyList(const QString& aGroup) const
     return keys;
 }
 
+QStringList KConfig::keyList(const QString& aGroup) const
+{
+    Q_D(const KConfig);
+    const QByteArray theGroup(aGroup.isEmpty() ? "<default>" : aGroup.toUtf8());
+    return d->keyListImpl(theGroup);
+}
+
 QMap<QString,QString> KConfig::entryMap(const QString& aGroup) const
 {
     Q_D(const KConfig);
@@ -364,10 +402,10 @@ void KConfig::markAsClean()
 void KConfig::checkUpdate(const QString &id, const QString &updateFile)
 {
     const KConfigGroup cg(this, "$Version");
-    const QString cfg_id = updateFile+':'+id;
+    const QString cfg_id = updateFile+QLatin1Char(':')+id;
     const QStringList ids = cg.readEntry("update_info", QStringList());
     if (!ids.contains(cfg_id)) {
-        KToolInvocation::kdeinitExecWait("kconf_update", QStringList() << "--check" << updateFile);
+        KToolInvocation::kdeinitExecWait(QString::fromLatin1("kconf_update"), QStringList() << QString::fromLatin1("--check") << updateFile);
         reparseConfiguration();
     }
 }
@@ -415,9 +453,10 @@ void KConfigPrivate::changeFileName(const QString& name, const char* type)
             file = sGlobalFileName;
         } // else anonymous config.
         // KDE5: remove these magic overloads
-    } else if (QDir::isAbsolutePath(fileName))
+    } else if (QDir::isAbsolutePath(fileName)) {
+        fileName = KStandardDirs::realFilePath(fileName);
         file = fileName;
-    else {
+    } else {
         if (type && *type)
             resourceType = type; // only change it if it's not empty
         file = KStandardDirs::locateLocal(resourceType, fileName, false, componentData);
@@ -612,17 +651,21 @@ bool KConfig::isGroupImmutableImpl(const QByteArray& aGroup) const
     return isImmutable() || d->entryMap.getEntryOption(aGroup, 0, 0, KEntryMap::EntryImmutable);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KConfig::setForceGlobal(bool b)
 {
     Q_D(KConfig);
     d->bForceGlobal = b;
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KConfig::forceGlobal() const
 {
     Q_D(const KConfig);
     return d->bForceGlobal;
 }
+#endif
 
 KConfigGroup KConfig::groupImpl(const QByteArray &group)
 {
@@ -652,18 +695,9 @@ void KConfig::deleteGroupImpl(const QByteArray &aGroup, WriteConfigFlags flags)
     Q_D(KConfig);
     KEntryMap::EntryOptions options = convertToOptions(flags)|KEntryMap::EntryDeleted;
 
-    QByteArray theGroup = aGroup + '\x1d';
-    QSet<QByteArray> groups;
-    groups << aGroup;
-
-    for (KEntryMap::ConstIterator entryMapIt( d->entryMap.constBegin() ); entryMapIt != d->entryMap.constEnd(); ++entryMapIt) {
-        if (entryMapIt.key().mKey.isNull() && entryMapIt.key().mGroup.startsWith(theGroup)) {
-            groups << entryMapIt.key().mGroup;
-        }
-    }
-
+    const QSet<QByteArray> groups = d->allSubGroups(aGroup);
     foreach (const QByteArray& group, groups) {
-        const QStringList keys = keyList(QString::fromUtf8(group));
+        const QStringList keys = d->keyListImpl(group);
         foreach (const QString& _key, keys) {
             const QByteArray &key = _key.toUtf8();
             if (d->canWriteEntry(group, key.constData())) {
@@ -686,11 +720,12 @@ bool KConfig::isConfigWritable(bool warnUser)
 
         // Note: We don't ask the user if we should not ask this question again because we can't save the answer.
         errorMsg += i18n("Please contact your system administrator.");
-        QString cmdToExec = KStandardDirs::findExe(QString("kdialog"));
+        QString cmdToExec = KStandardDirs::findExe(QString::fromLatin1("kdialog"));
         if (!cmdToExec.isEmpty() && componentData().isValid())
         {
-            QProcess::execute(cmdToExec,QStringList() << "--title" << componentData().componentName()
-                    << "--msgbox" << errorMsg);
+            QProcess::execute(cmdToExec, QStringList()
+                              << QString::fromLatin1("--title") << componentData().componentName()
+                              << QString::fromLatin1("--msgbox") << errorMsg);
         }
     }
 
@@ -703,17 +738,10 @@ bool KConfig::hasGroupImpl(const QByteArray& aGroup) const
 {
     Q_D(const KConfig);
 
-    if (d->entryMap.hasEntry(aGroup)) return true;
+    // No need to look for the actual group entry anymore, or for subgroups:
+    // a group exists if it contains any non-deleted entry.
 
-    QByteArray subGroupMarker = aGroup + '\x1d';
-    // Because this group could have only subgroups but no entries we have to
-    // search for group markers
-    for (KEntryMap::ConstIterator entryMapIt( d->entryMap.constBegin() ); entryMapIt != d->entryMap.constEnd(); ++entryMapIt) {
-        if (entryMapIt.key().mGroup.startsWith(subGroupMarker)) {
-            return true;
-        }
-    }
-    return false;
+    return d->hasNonDeletedEntries(aGroup);
 }
 
 bool KConfigPrivate::canWriteEntry(const QByteArray& group, const char* key, bool isDefault) const
