@@ -2,17 +2,17 @@
     Copyright (c) 2002 Carlos Moro <cfmoro@correo.uniovi.es>
     Copyright (c) 2002-2003 Hans Petter Bieker <bieker@kde.org>
     Copyright 2007, 2010 John Layt <john@layt.net>
- 
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
- 
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
- 
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
@@ -41,7 +41,8 @@ public:
     virtual ~KCalendarSystemGregorianPrivate();
 
     // Virtual methods each calendar system must re-implement
-    virtual void initDefaultEraList();
+    virtual KLocale::CalendarSystem calendarSystem() const;
+    virtual void loadDefaultEraList();
     virtual int monthsInYear( int year ) const;
     virtual int daysInMonth( int year, int month ) const;
     virtual int daysInYear( int year ) const;
@@ -70,11 +71,17 @@ KCalendarSystemGregorianPrivate::~KCalendarSystemGregorianPrivate()
 {
 }
 
-void KCalendarSystemGregorianPrivate::initDefaultEraList()
+KLocale::CalendarSystem KCalendarSystemGregorianPrivate::calendarSystem() const
+{
+    return KLocale::QDateCalendar;
+}
+
+void KCalendarSystemGregorianPrivate::loadDefaultEraList()
 {
     QString name, shortName, format;
 
-    KConfigGroup cg( config(), QString( "KCalendarSystem %1" ).arg( q->calendarType() ) );
+    KConfigGroup lcg( config(), QString::fromLatin1( "Locale" ) );
+    KConfigGroup cg = lcg.group( QString::fromLatin1( "KCalendarSystem %1" ).arg( q->calendarType() ) );
     m_useCommonEra = cg.readEntry( "UseCommonEra", false );
 
     if ( m_useCommonEra ) {
@@ -161,21 +168,21 @@ KCalendarSystemGregorian::KCalendarSystemGregorian( const KLocale *locale )
                         : KCalendarSystem( *new KCalendarSystemGregorianPrivate( this ), KSharedConfig::Ptr(), locale ),
                           dont_use( 0 )
 {
-    d_ptr->initialiseEraList( calendarType() );
+    d_ptr->loadConfig( calendarType() );
 }
 
 KCalendarSystemGregorian::KCalendarSystemGregorian( const KSharedConfig::Ptr config, const KLocale *locale )
                         : KCalendarSystem( *new KCalendarSystemGregorianPrivate( this ), config, locale ),
                           dont_use( 0 )
 {
-    d_ptr->initialiseEraList( calendarType() );
+    d_ptr->loadConfig( calendarType() );
 }
 
 KCalendarSystemGregorian::KCalendarSystemGregorian( KCalendarSystemGregorianPrivate &dd, const KSharedConfig::Ptr config, const KLocale *locale )
                         : KCalendarSystem( dd, config, locale ),
                           dont_use( 0 )
 {
-    d_ptr->initialiseEraList( calendarType() );
+    d_ptr->loadConfig( calendarType() );
 }
 
 KCalendarSystemGregorian::~KCalendarSystemGregorian()
