@@ -29,7 +29,7 @@
 #include <kdecore_export.h>
 #include <ktimezone.h>
 
-#include <QMetaType>
+#include <QtCore/QMetaType>
 #include <QtCore/QSharedDataPointer>
 
 class QDataStream;
@@ -124,7 +124,7 @@ class KDateTimeSpecPrivate;
  * conversions even after the copy is performed. A separate copy of the data is
  * created whenever a non-const method is called. If you want to force the
  * creation of a separate copy of the data (e.g. if you want two copies to
- * cache different time zone conversions), call detach(). 
+ * cache different time zone conversions), call detach().
  *
  * @section compatibility QDateTime Considerations
  *
@@ -1582,6 +1582,7 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
      *
      * @see currentDateTime(), currentLocalDateTime(), currentUtcDateTime(),
      *      currentLocalDate(), currentLocalTime()
+     * @since 4.3
      */
     static void setSimulatedSystemTime(const KDateTime& newTime);
 
@@ -1599,6 +1600,8 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
      *             dt = KDateTime::realCurrentLocalDateTime();
      *          #endif
      *          \endcode
+     *
+     * @since 4.3
      */
     static KDateTime realCurrentLocalDateTime();
 
