@@ -36,7 +36,9 @@
 #ifdef Q_WS_WIN
 #include <fcntl.h>
 #include <windows.h>
+#ifndef _WIN32_WCE
 #include <wincon.h>
+#endif
 #else
 #include <unistd.h>
 #include <stdio.h>
@@ -215,7 +217,7 @@ struct KDebugPrivate
     typedef QHash<unsigned int, Area> Cache;
 
     KDebugPrivate()
-        : config(0), kDebugDBusIface(0), m_disableAll(false)
+        : config(0), kDebugDBusIface(0), m_disableAll(false), m_seenMainComponent(false)
     {
         Q_ASSERT(int(QtDebugMsg) == 0);
         Q_ASSERT(int(QtFatalMsg) == 3);
@@ -250,14 +252,20 @@ struct KDebugPrivate
 
     void loadAreaNames()
     {
-        cache.clear();
+        // Don't clear the cache here, that would lose previously registered dynamic areas
+        //cache.clear();
 
         Area &areaData = cache[0];
         areaData.clear();
 
-        //AB: this is necessary here, otherwise all output with area 0 won't be
-        //prefixed with anything, unless something with area != 0 is called before
-        areaData.name = KGlobal::mainComponent().componentName().toUtf8();
+        if (KGlobal::hasMainComponent()) {
+            areaData.name = KGlobal::mainComponent().componentName().toUtf8();
+            m_seenMainComponent = true;
+        } else {
+            areaData.name = qApp ? qAppName().toUtf8() : QByteArray("unnamed app");
+            m_seenMainComponent = false;
+        }
+        //qDebug() << "loadAreaNames: area 0 has name" << areaData.name;
 
         for (int i = 0; i < 8; i++) {
             m_nullOutputYesNoCache[i] = -1;
@@ -323,7 +331,7 @@ struct KDebugPrivate
 
     OutputMode areaOutputMode(QtMsgType type, unsigned int area, bool enableByDefault)
     {
-        if (!config)
+        if (!configObject())
             return QtOutput;
 
         QString key;
@@ -353,7 +361,7 @@ struct KDebugPrivate
 
     QString logFileName(QtMsgType type, unsigned int area)
     {
-        if (!config)
+        if (!configObject())
             return QString();
 
         const char* aKey;
@@ -381,10 +389,6 @@ struct KDebugPrivate
     KConfig* configObject()
     {
         if (!config) {
-            if (!KGlobal::hasMainComponent()) {
-                return NULL;
-            }
-
             config = new KConfig(QLatin1String("kdebugrc"), KConfig::NoGlobals);
             m_disableAll = config->group(QString()).readEntry("DisableAll", false);
         }
@@ -393,28 +397,29 @@ struct KDebugPrivate
 
     Cache::Iterator areaData(QtMsgType type, unsigned int num, bool enableByDefault = true)
     {
-        if (!configObject()) {
-            // we don't have a config and we can't create one...
-            Area &area = cache[0]; // create a dummy entry
-            if (KGlobal::hasMainComponent())
-                area.name = KGlobal::mainComponent().componentName().toUtf8();
-            return cache.find(0);
-        }
-
-        if (cache.count() <= 1) { // empty or containing only entry "0"
+        if (!cache.contains(0)) {
+            //qDebug() << "cache size=" << cache.count() << "loading area names";
             loadAreaNames(); // fills 'cache'
+            Q_ASSERT(cache.contains(0));
+        } else if (!m_seenMainComponent && KGlobal::hasMainComponent()) {
+            // Update the name for area 0 once a main component exists
+            cache[0].name = KGlobal::mainComponent().componentName().toUtf8();
+            m_seenMainComponent = true;
         }
 
         Cache::Iterator it = cache.find(num);
         if (it == cache.end()) {
             // unknown area
-            return cache.find(0);
+            Q_ASSERT(cache.contains(0));
+            it = cache.find(0);
+            num = 0;
         }
 
         if (num == 0 && type == QtDebugMsg) { // area 0 is special, it becomes the named area "appname"
             static bool s_firstDebugFromApplication = true;
             if (s_firstDebugFromApplication && !m_disableAll) {
                 s_firstDebugFromApplication = false;
+                //qDebug() << "First debug output from" << it->name << "writing out with default" << enableByDefault;
                 writeGroupForNamedArea(it->name, enableByDefault);
             }
         }
@@ -426,6 +431,8 @@ struct KDebugPrivate
         if (it->mode[lev] == FileOutput && it->logFileName[lev].isEmpty())
             it->logFileName[lev] = logFileName(type, num);
 
+        Q_ASSERT(it->mode[lev] != Unknown);
+
         return it;
     }
 
@@ -507,19 +514,29 @@ struct KDebugPrivate
         return QDebug(&lineendstrippingwriter);
     }
 
-    QDebug printHeader(QDebug s, const QByteArray &areaName, const char *, int, const char *funcinfo, QtMsgType type, bool colored)
+    QDebug printHeader(QDebug s, const QByteArray &areaName, const char * file, int line, const char *funcinfo, QtMsgType type, bool colored)
     {
 #ifdef KDE_EXTENDED_DEBUG_OUTPUT
         static bool printProcessInfo = (qgetenv("KDE_DEBUG_NOPROCESSINFO").isEmpty());
         static bool printAreaName = (qgetenv("KDE_DEBUG_NOAREANAME").isEmpty());
         static bool printMethodName = (qgetenv("KDE_DEBUG_NOMETHODNAME").isEmpty());
-        static bool printTimeStamp = !(qgetenv("KDE_DEBUG_TIMESTAMP").isEmpty());
+        static bool printFileLine = (!qgetenv("KDE_DEBUG_FILELINE").isEmpty());
+
+        static int printTimeStamp = qgetenv("KDE_DEBUG_TIMESTAMP").toInt();
         QByteArray programName;
         s = s.nospace();
-        if (printTimeStamp) {
-            s << qPrintable(QDateTime::currentDateTime().time().toString());
+        if (printTimeStamp > 0) {
+            if (printTimeStamp >= 2) {
+                // the extended print: 17:03:24.123
+                const QString sformat = QString::fromLatin1("hh:mm:ss.zzz");
+                s << qPrintable(QDateTime::currentDateTime().time().toString(sformat));
+            } else {
+                // the default print: 17:03:24
+                s << qPrintable(QDateTime::currentDateTime().time().toString());
+            }
             s << ' ';
         }
+
         if (printProcessInfo) {
             programName = cache.value(0).name;
             if (programName.isEmpty()) {
@@ -536,6 +553,14 @@ struct KDebugPrivate
             s << areaName.constData();
         }
 
+        if (m_indentString.hasLocalData()) {
+            s << m_indentString.localData()->toLatin1().constData();
+        }
+
+        if (printFileLine) {
+            s << ' ' << file << ':' << line << ' ';
+        }
+
         if (funcinfo && printMethodName) {
             if (colored) {
                 if (type <= QtDebugMsg)
@@ -605,6 +630,7 @@ struct KDebugPrivate
         static bool env_colored = (!qgetenv("KDE_COLOR_DEBUG").isEmpty());
         Cache::Iterator it = areaData(type, area);
         OutputMode mode = it->mode[level(type)];
+        Q_ASSERT(mode != Unknown);
         QString file = it->logFileName[level(type)];
         QByteArray areaName = it->name;
 
@@ -628,6 +654,7 @@ struct KDebugPrivate
             s = QDebug(&devnull);
             return s; //no need to take the time to "print header" if we don't want to output anyway
             break;
+        case Unknown: // should not happen
         default:                // QtOutput
             s = setupQtWriter(type);
 #ifndef Q_OS_WIN
@@ -660,9 +687,11 @@ struct KDebugPrivate
     KDebugDBusIface *kDebugDBusIface;
     Cache cache;
     bool m_disableAll;
+    bool m_seenMainComponent; // false: area zero still contains qAppName
     int m_nullOutputYesNoCache[8];
 
     KNoDebugStream devnull;
+    QThreadStorage<QString*> m_indentString;
     QThreadStorage<KSyslogDebugStream*> syslogwriter;
     QThreadStorage<KFileDebugStream*> filewriter;
     QThreadStorage<KMessageBoxDebugStream*> messageboxwriter;
@@ -834,7 +863,48 @@ int KDebug::registerArea(const QByteArray& areaName, bool enabled)
     }
     KDebugPrivate::Area areaData;
     areaData.name = areaName;
+    //qDebug() << "Assigning area number" << areaNumber << "for name" << areaName;
     d->cache.insert(areaNumber, areaData);
     d->writeGroupForNamedArea(areaName, enabled);
     return areaNumber;
 }
+
+KDebug::Block::Block(const char* label, int area)
+    : m_label(label), m_area(area), d(0)
+{
+    if (hasNullOutputQtDebugMsg(area)) {
+        m_label = 0; // remember, for the dtor
+    } else {
+        m_startTime.start();
+        kDebug(area) << "BEGIN:" << label;
+
+        // The indent string is per thread
+        QThreadStorage<QString*> & indentString = kDebug_data->m_indentString;
+        if (!indentString.hasLocalData()) {
+            indentString.setLocalData(new QString);
+        }
+        *(indentString.localData()) += QLatin1String("  ");
+    }
+}
+
+KDebug::Block::~Block()
+{
+    if (m_label) {
+        const double duration = m_startTime.elapsed() / 1000.0;
+        QThreadStorage<QString*> & indentString = kDebug_data->m_indentString;
+        indentString.localData()->chop(2);
+
+        // Print timing information, and a special message (DELAY) if the method took longer than 5s
+        if (duration < 5.0) {
+            kDebug(m_area)
+                << "END__:"
+                << m_label
+                << qPrintable(QString::fromLatin1("[Took: %3s]").arg(QString::number(duration, 'g', 2)));
+        } else {
+            kDebug(m_area)
+                << "END__:"
+                << m_label
+                << qPrintable(QString::fromLatin1("[DELAY Took (quite long) %3s]").arg(QString::number(duration, 'g', 2)));
+        }
+    }
+}
