@@ -28,6 +28,7 @@
 
 #include "kglobal.h"
 #include "kglobal_p.h"
+#include <QThread>
 
 #include <config.h>
 
@@ -47,6 +48,7 @@
 #undef QT_NO_TRANSLATION
 #include <QtCore/QCoreApplication>
 #define QT_NO_TRANSLATION
+#include <QtCore/QDebug>
 #include <QtCore/QTextCodec>
 #include "kcmdlineargs.h"
 #include <unistd.h> // umask
@@ -73,7 +75,8 @@ class KGlobalPrivate
         inline KGlobalPrivate()
             : stringDict(0),
             locale(0),
-            charsets(0)
+            charsets(0),
+            localeIsFromFakeComponent(false)
         {
             // the umask is read here before any threads are created to avoid race conditions
             mode_t tmp = 0;
@@ -96,6 +99,8 @@ class KGlobalPrivate
         KStringDict *stringDict;
         KLocale *locale;
         KCharsets *charsets;
+        bool localeIsFromFakeComponent;
+        QStringList catalogsToInsert;
 
         /**
          * This component may be used in applications that doesn't have a
@@ -104,7 +109,7 @@ class KGlobalPrivate
         static KComponentData initFakeComponent()
         {
             QString name = QCoreApplication::applicationName();
-            if(name.isEmpty())
+            if(name.isEmpty() && QCoreApplication::instance())
                 name = qAppName();
             if(name.isEmpty())
                 name = QString::fromLatin1("kde");
@@ -113,6 +118,12 @@ class KGlobalPrivate
         }
 };
 
+KCatalogLoader::KCatalogLoader(const QString &catalogName)
+{
+    KGlobal::insertCatalog(catalogName);
+}
+
+
 K_GLOBAL_STATIC(KGlobalPrivate, globalData)
 K_GLOBAL_STATIC_WITH_ARGS(KComponentData, fakeComponent, (KGlobalPrivate::initFakeComponent()))
 
@@ -145,20 +156,38 @@ bool KGlobal::hasMainComponent()
     return d->mainComponent.isValid();
 }
 
+void KGlobal::insertCatalog(const QString& catalog)
+{
+    PRIVATE_DATA;
+    if (d->locale) {
+        d->locale->insertCatalog(catalog);
+    } else {
+        d->catalogsToInsert.append(catalog);
+    }
+}
+
 KLocale *KGlobal::locale()
 {
     PRIVATE_DATA;
-    if (d->locale == 0) {
-        if (!d->mainComponent.isValid()) {
-            return 0;
-        }
-
-        d->locale = new KLocale(d->mainComponent.catalogName());
+    if (d->locale == 0 || (d->localeIsFromFakeComponent && d->mainComponent.isValid() && d->mainComponent.config())) {
+        if (d->locale != 0) qDebug() << "KGlobal::locale::Warning your global KLocale is being recreated with a valid main component instead of a fake component, this usually means you tried to call i18n related functions before your main component was created. You should not do that since it most likely will not work";
+        delete d->locale;
+        d->locale = 0;
+        d->locale = new KLocale(mainComponent().catalogName());
+        d->localeIsFromFakeComponent = !d->mainComponent.isValid();
         QTextCodec::setCodecForLocale(d->locale->codecForEncoding());
-        d->mainComponent.aboutData()->translateInternalProgramName();
+        mainComponent().aboutData()->translateInternalProgramName();
         QCoreApplication* coreApp = QCoreApplication::instance();
-        if (coreApp) // testcase: kwrite --help: no qcore app
-            QCoreApplication::installTranslator(new KDETranslator(coreApp));
+        if (coreApp) { // testcase: kwrite --help: no qcore app
+            if (coreApp->thread() != QThread::currentThread()) {
+                qFatal("KGlobal::locale() must be called from the main thread before using i18n() in threads. KApplication takes care of this. If not using KApplication, call KGlobal::locale() during initialization.");
+            } else {
+                QCoreApplication::installTranslator(new KDETranslator(coreApp));
+            }
+        }
+        foreach(const QString &catalog, d->catalogsToInsert)
+            d->locale->insertCatalog(catalog);
+        d->catalogsToInsert.clear();
     }
     return d->locale;
 }
@@ -300,3 +329,19 @@ void KGlobal::setAllowQuit(bool allowQuit)
 }
 
 #undef PRIVATE_DATA
+
+QObject* KGlobal::findDirectChild_helper(const QObject* parent, const QMetaObject& mo)
+{
+    if (!parent)
+        return 0;
+
+    const QObjectList &children = parent->children();
+    for (int i = 0; i < children.size(); ++i) {
+        QObject* obj = children.at(i);
+        if (mo.cast(obj)) {
+            return obj;
+        }
+    }
+    return 0;
+
+}
