@@ -27,13 +27,12 @@
 #include <kstandarddirs.h>
 #include <klocale.h>
 
-#include <QtCore/QMutex>
 #include <QtCore/QStringList>
 #include <QtNetwork/QSslKey>
 #include <QtNetwork/QSslCipher>
+#include <QtNetwork/QHostAddress>
 #include <QtNetwork/QNetworkProxy>
 
-
 static KTcpSocket::SslVersion kSslVersionFromQ(QSsl::SslProtocol protocol)
 {
     switch (protocol) {
@@ -358,7 +357,7 @@ public:
     void maybeLoadCertificates()
     {
         if (!certificatesLoaded) {
-            sock.setCaCertificates(KSslCertificateManager::self()->rootCertificates());
+            sock.setCaCertificates(KSslCertificateManager::self()->caCertificates());
             certificatesLoaded = true;
         }
     }
@@ -385,8 +384,10 @@ KTcpSocket::KTcpSocket(QObject *parent)
     connect(&d->sock, SIGNAL(connected()), this, SIGNAL(connected()));
     connect(&d->sock, SIGNAL(encrypted()), this, SIGNAL(encrypted()));
     connect(&d->sock, SIGNAL(disconnected()), this, SIGNAL(disconnected()));
+#ifndef QT_NO_NETWORKPROXY
     connect(&d->sock, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)),
             this, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)));
+#endif
     connect(&d->sock, SIGNAL(error(QAbstractSocket::SocketError)),
             this, SLOT(reemitSocketError(QAbstractSocket::SocketError)));
     connect(&d->sock, SIGNAL(sslErrors(const QList<QSslError> &)),
@@ -579,11 +580,12 @@ quint16 KTcpSocket::peerPort() const
 }
 
 
+#ifndef QT_NO_NETWORKPROXY
 QNetworkProxy KTcpSocket::proxy() const
 {
     return d->sock.proxy();
 }
-
+#endif
 
 qint64 KTcpSocket::readBufferSize() const
 {
@@ -591,11 +593,12 @@ qint64 KTcpSocket::readBufferSize() const
 }
 
 
+#ifndef QT_NO_NETWORKPROXY
 void KTcpSocket::setProxy(const QNetworkProxy &proxy)
 {
     d->sock.setProxy(proxy);
 }
-
+#endif
 
 void KTcpSocket::setReadBufferSize(qint64 size)
 {
@@ -632,6 +635,7 @@ bool KTcpSocket::waitForDisconnected(int msecs)
 
 void KTcpSocket::addCaCertificate(const QSslCertificate &certificate)
 {
+    d->maybeLoadCertificates();
     d->sock.addCaCertificate(certificate);
 }
 
@@ -640,6 +644,7 @@ void KTcpSocket::addCaCertificate(const QSslCertificate &certificate)
 bool KTcpSocket::addCaCertificates(const QString &path, QSsl::EncodingFormat format,
                                    QRegExp::PatternSyntax syntax)
 {
+    d->maybeLoadCertificates();
     return d->sock.addCaCertificates(path, format, syntax);
 }
 */
@@ -647,6 +652,7 @@ bool KTcpSocket::addCaCertificates(const QString &path, QSsl::EncodingFormat for
 
 void KTcpSocket::addCaCertificates(const QList<QSslCertificate> &certificates)
 {
+    d->maybeLoadCertificates();
     d->sock.addCaCertificates(certificates);
 }
 
@@ -706,6 +712,7 @@ void KTcpSocket::setCaCertificates(const QList<QSslCertificate> &certificates)
 
 void KTcpSocket::setCiphers(const QList<KSslCipher> &ciphers)
 {
+    d->ciphers = ciphers;
     QList<QSslCipher> cl;
     foreach (const KSslCipher &c, d->ciphers) {
         cl.append(d->ccc.converted(c));
@@ -729,6 +736,7 @@ void KTcpSocket::setLocalCertificate(const QString &fileName, QSsl::EncodingForm
 //TODO
 void KTcpSocket::setPrivateKey(const KSslKey &key)
 {
+    Q_UNUSED(key)
 }
 
 
@@ -736,6 +744,10 @@ void KTcpSocket::setPrivateKey(const KSslKey &key)
 void KTcpSocket::setPrivateKey(const QString &fileName, KSslKey::Algorithm algorithm,
                                QSsl::EncodingFormat format, const QByteArray &passPhrase)
 {
+    Q_UNUSED(fileName)
+    Q_UNUSED(algorithm)
+    Q_UNUSED(format)
+    Q_UNUSED(passPhrase)
 }
 
 
@@ -750,6 +762,15 @@ KTcpSocket::EncryptionMode KTcpSocket::encryptionMode() const
     return d->encryptionMode(d->sock.mode());
 }
 
+QVariant KTcpSocket::socketOption(QAbstractSocket::SocketOption options) const
+{
+    return d->sock.socketOption(options);
+}
+
+void KTcpSocket::setSocketOption(QAbstractSocket::SocketOption options, const QVariant &value)
+{
+    return d->sock.setSocketOption(options, value);
+}
 
 //slot
 void KTcpSocket::ignoreSslErrors()
@@ -929,7 +950,7 @@ KSslCipher::KSslCipher(const QSslCipher &qsc)
     d->encryptionMethod = qsc.encryptionMethod();
     //Qt likes to append the number of bits (usedBits?) to the algorithm,
     //for example "AES(256)". We only want the pure algorithm name, though.
-    int parenIdx = d->encryptionMethod.indexOf('(');
+    int parenIdx = d->encryptionMethod.indexOf(QLatin1Char('('));
     if (parenIdx > 0)
         d->encryptionMethod.truncate(parenIdx);
     d->keyExchangeMethod = qsc.keyExchangeMethod();
@@ -981,12 +1002,12 @@ QString KSslCipher::digestMethod() const
 {
     //### This is not really backend neutral. It works for OpenSSL and
     //    for RFC compliant names, though.
-    if (d->name.endsWith("SHA"))
-        return "SHA-1";
-    else if (d->name.endsWith("MD5"))
-        return "MD5";
+    if (d->name.endsWith(QLatin1String("SHA")))
+        return QString::fromLatin1("SHA-1");
+    else if (d->name.endsWith(QLatin1String("MD5")))
+        return QString::fromLatin1("MD5");
     else
-        return "";
+        return QString::fromLatin1(""); // ## probably QString() is enough
 }
 
 
