@@ -26,11 +26,11 @@ class KMimeTypePrivate: public KServiceTypePrivate
 public:
   K_SYCOCATYPE( KST_KMimeType, KServiceTypePrivate )
 
-  KMimeTypePrivate(const QString &path) : KServiceTypePrivate(path) {}
+  KMimeTypePrivate(const QString &path) : KServiceTypePrivate(path), m_xmlDataLoaded(false)
+    {}
   KMimeTypePrivate(QDataStream &_str, int offset)
-      : KServiceTypePrivate(_str, offset)
+      : KServiceTypePrivate(_str, offset), m_xmlDataLoaded(false)
   {
-    loadInternal(_str);
   }
 
   virtual void save(QDataStream &s);
@@ -39,25 +39,36 @@ public:
 
   virtual QStringList propertyNames() const;
 
-  virtual QString iconName(const KUrl &) const
-  {
-    QString icon = name();
-    int slashindex = icon.indexOf( QLatin1Char( '/' ) );
-    if ( slashindex != -1 ) {
-        icon[ slashindex ] = QLatin1Char( '-' );
+    virtual QString comment(const KUrl & = KUrl()) const
+    {
+        ensureXmlDataLoaded();
+        return m_strComment;
+    }
+
+    // virtual because reimplemented in KFolderMimeType
+    virtual QString iconName(const KUrl &) const
+    {
+        if (!m_iconName.isEmpty())
+            return m_iconName;
+
+        // Make default icon name from the mimetype name
+        // Don't store this in m_iconName, it would make the filetype editor
+        // write out icon names in every local mimetype definition file.
+        QString icon = name();
+        const int slashindex = icon.indexOf(QLatin1Char('/'));
+        if (slashindex != -1) {
+            icon[slashindex] = QLatin1Char('-');
+        }
+        return icon;
     }
-    return icon;
-  }
 
     bool inherits(const QString& mime) const;
-    QString fallbackParent() const;
-    QStringList parentMimeTypes() const;
-    void collectParentMimeTypes(QStringList&) const;
+    void ensureXmlDataLoaded() const;
+    virtual int serviceOffersOffset() const;
 
-  QStringList m_lstPatterns;
-  QStringList m_parentMimeTypes; // shared-mime-info supports multiple inheritance
-
-  void loadInternal( QDataStream& _str );
+    mutable QStringList m_lstPatterns;
+    mutable QString m_iconName; // user-specified
+    mutable bool m_xmlDataLoaded;
 };
 
 #endif // __kmimetype_p_h__
