@@ -36,7 +36,7 @@
 #include <kprotocolinfo.h>
 #include <kmimetypetrader.h>
 #include <kservicetypetrader.h>
-#include <kmimetypefactory.h>
+#include <kmimetyperepository_p.h>
 #include <ktemporaryfile.h>
 #include <ktempdir.h>
 #include <kdesktopfile.h>
@@ -93,6 +93,7 @@ void KMimeTypeTest::initTestCase()
         group.writeEntry("NotShowIn", "KDE;FVWM;");
         group.writeEntry("MimeType", "text/plain;");
         group.writeEntry("InitialPreference", "50");
+        group.writeEntry("Categories", "Qt;KDE;");
     }
 
     if ( mustUpdateKSycoca ) {
@@ -103,6 +104,8 @@ void KMimeTypeTest::initTestCase()
     KService::Ptr fakeApp = KService::serviceByStorageId("fake_nonkde_application.desktop");
     QVERIFY(fakeApp); // it should be found.
     QVERIFY(KService::serviceByDesktopPath(m_nonKdeApp)); // the desktoppath is the full path nowadays
+
+    KGlobal::locale();
 }
 
 void KMimeTypeTest::cleanupTestCase()
@@ -130,13 +133,10 @@ QTEST_KDEMAIN_CORE( KMimeTypeTest )
 
 void KMimeTypeTest::testByName()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     KMimeType::Ptr s0 = KMimeType::mimeType("application/x-zerosize");
     QVERIFY( s0 );
     QCOMPARE( s0->name(), QString::fromLatin1("application/x-zerosize") );
-    //qDebug("Comment is %s", qPrintable(s0->comment()) );
+    QCOMPARE( s0->comment(), QString::fromLatin1("empty document") );
 
     KMimeType::Ptr s0Again = KMimeType::mimeType("application/x-zerosize");
     QCOMPARE(s0Again->name(), s0->name());
@@ -150,8 +150,14 @@ void KMimeTypeTest::testByName()
     KMimeType::Ptr krita = KMimeType::mimeType("application/x-krita");
     QVERIFY( krita );
 
-    KMimeType::Ptr bzip2 = KMimeType::mimeType("application/x-bzip2", KMimeType::ResolveAliases);
+    // Test <comment> parsing with application/rdf+xml which has the english comment after the other ones
+    KMimeType::Ptr rdf = KMimeType::mimeType("application/rdf+xml");
+    QVERIFY(rdf);
+    QCOMPARE(rdf->comment(), QString::fromLatin1("RDF file"));
+
+    KMimeType::Ptr bzip2 = KMimeType::mimeType("application/x-bzip2");
     QVERIFY( bzip2 );
+    QCOMPARE(bzip2->comment(), QString::fromLatin1("Bzip archive"));
 
     KMimeType::Ptr defaultMime = KMimeType::mimeType("application/octet-stream");
     QVERIFY(defaultMime);
@@ -160,9 +166,6 @@ void KMimeTypeTest::testByName()
 
 void KMimeTypeTest::testIcons()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     checkIcon( KUrl( "file:/tmp/" ), "inode-directory" );
 
     if ( !KUser().isSuperUser() ) // Can't test this one if running as root
@@ -179,8 +182,6 @@ void KMimeTypeTest::testIcons()
 
 void KMimeTypeTest::testFindByPathUsingFileName_data()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
     QTest::addColumn<QString>("fileName");
     QTest::addColumn<QString>("expectedMimeType");
     // Maybe we could also add a expectedAccuracy column...
@@ -248,6 +249,9 @@ void KMimeTypeTest::testAdditionalGlobs_data()
     QTest::addColumn<bool>("expected");
 
     QTest::newRow("one star, match") << "foo.txt" << "*.txt" << true;
+    QTest::newRow("README*, match") << "README.foo" << "README*" << true;
+    QTest::newRow("README.*, match") << "README.foo" << "README.*" << true;
+    QTest::newRow("README.*, no match") << "README" << "README.*" << false;
     QTest::newRow("two stars, match") << "andre.ts.001" << "*.ts.0*" << true;
     QTest::newRow("two stars, no match") << "andre.ts" << "*.ts.0*" << false;
 }
@@ -258,17 +262,13 @@ void KMimeTypeTest::testAdditionalGlobs()
     QFETCH(QString, pattern);
     QFETCH(bool, expected);
 
-    QCOMPARE(KMimeTypeFactory::matchFileName(filename, pattern), expected);
+    QCOMPARE(KMimeTypeRepository::matchFileName(filename, pattern), expected);
 }
 
 // All the simple tests for findByPath are in testFindByPathUsingFileName_data.
 // In here we do the tests that need some content in a temporary file.
 void KMimeTypeTest::testFindByPathWithContent()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
-
     KMimeType::Ptr mime;
 
     // Test a real PDF file.
@@ -344,7 +344,6 @@ void KMimeTypeTest::testFindByUrl()
     mime = KMimeType::findByUrl(KUrl("fish://host/test1")); // like fish does, to test for known extensions
     QVERIFY(mime);
     QCOMPARE(mime->name(), QString::fromLatin1("application/octet-stream"));
-
 }
 
 void KMimeTypeTest::testFindByNameAndContent()
@@ -458,9 +457,6 @@ void KMimeTypeTest::testFindByFileContent()
 
 void KMimeTypeTest::testAllMimeTypes()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     const KMimeType::List lst = KMimeType::allMimeTypes(); // does NOT include aliases
     QVERIFY( !lst.isEmpty() );
 
@@ -481,9 +477,6 @@ void KMimeTypeTest::testAllMimeTypes()
 
 void KMimeTypeTest::testAlias()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     const KMimeType::Ptr canonical = KMimeType::mimeType( "application/xml" );
     QVERIFY( canonical );
     KMimeType::Ptr alias = KMimeType::mimeType("text/xml", KMimeType::DontResolveAlias);
@@ -502,9 +495,6 @@ void KMimeTypeTest::testAlias()
 
 void KMimeTypeTest::testMimeTypeParent()
 {
-    if ( !KSycoca::isAvailable() )
-        QSKIP( "ksycoca not available", SkipAll );
-
     // All file-like mimetypes inherit from octet-stream
     const KMimeType::Ptr wordperfect = KMimeType::mimeType("application/vnd.wordperfect");
     QVERIFY(wordperfect);
@@ -591,6 +581,8 @@ void KMimeTypeTest::testMimeTypeInheritancePerformance()
     // 0.57 msec / 1,115,000 ticks / 938,356 instr. loads per iteration
     // After converting the QMap for aliases into a QHash too:
     // 0.48 msec / 960,000 ticks / 791,404 instr. loads per iteration
+    // July 2010: After moving KMimeType out of ksycoca:
+    // 0.21 msec / 494,000 ticks / 568,345 instr. loads per iteration
 }
 
 // Helper method for all the trader tests
@@ -688,9 +680,9 @@ void KMimeTypeTest::testHasServiceType1() // with services constructed with a fu
     QString katepartPath = KStandardDirs::locate( "services", "katepart.desktop" );
     QVERIFY( !katepartPath.isEmpty() );
     KService katepart( katepartPath );
-    QVERIFY( katepart.hasMimeType( KMimeType::mimeType( "text/plain" ).data() ) );
-    //QVERIFY( katepart.hasMimeType( KMimeType::mimeType( "text/x-patch" ).data() ) ); // inherited mimetype; fails
-    QVERIFY( !katepart.hasMimeType( KMimeType::mimeType( "image/png" ).data() ) );
+    QVERIFY( katepart.hasMimeType( "text/plain" ) );
+    //QVERIFY( katepart.hasMimeType( "text/x-patch" ) ); // inherited mimetype; fails
+    QVERIFY( !katepart.hasMimeType( "image/png" ) );
     QVERIFY( katepart.hasServiceType( "KParts/ReadOnlyPart" ) );
     QVERIFY( katepart.hasServiceType( "KParts/ReadWritePart" ) );
     QVERIFY( !katepart.hasServiceType( "KTextEditor/Plugin" ) );
@@ -706,9 +698,9 @@ void KMimeTypeTest::testHasServiceType2() // with services coming from ksycoca
 {
     KService::Ptr katepart = KService::serviceByDesktopPath( "katepart.desktop" );
     QVERIFY( !katepart.isNull() );
-    QVERIFY( katepart->hasMimeType( KMimeType::mimeType( "text/plain" ).data() ) );
-    QVERIFY( katepart->hasMimeType( KMimeType::mimeType( "text/x-patch" ).data() ) ); // due to inheritance
-    QVERIFY( !katepart->hasMimeType( KMimeType::mimeType( "image/png" ).data() ) );
+    QVERIFY( katepart->hasMimeType( "text/plain" ) );
+    QVERIFY( katepart->hasMimeType( "text/x-patch" ) ); // due to inheritance
+    QVERIFY( !katepart->hasMimeType( "image/png" ) );
     QVERIFY( katepart->hasServiceType( "KParts/ReadOnlyPart" ) );
     QVERIFY( katepart->hasServiceType( "KParts/ReadWritePart" ) );
     QVERIFY( !katepart->hasServiceType( "KTextEditor/Plugin" ) );
@@ -747,6 +739,15 @@ void KMimeTypeTest::testPatterns()
     QStringList expectedPatterns = patterns.split(';');
     expectedPatterns.sort();
     QStringList mimePatterns = mime->patterns();
+
+    if (mimeType == "application/vnd.oasis.opendocument.text" && mimePatterns.contains("*.fodt")) {
+        QSKIP("Skipping test which would fail due to an upstream bug, see https://bugs.freedesktop.org/show_bug.cgi?id=31242", SkipSingle);
+    }
+
+    if (mimeType == "application/vnd.oasis.opendocument.presentation" && mimePatterns.contains("*.fodp")) {
+        QSKIP("Skipping test which would fail due to an upstream bug, see https://bugs.freedesktop.org/show_bug.cgi?id=31242", SkipSingle);
+    }
+
     // shared-mime-info 0.30 adds *,v to text/plain, let's add it from this test so that it works
     // with older versions too.
     if (mimeType == "text/plain" && !mimePatterns.contains("*,v"))
@@ -838,7 +839,7 @@ void KMimeTypeTest::testParseMagicFile_data()
 
     QBuffer magicBuffer(&magicData);
     magicBuffer.open(QIODevice::ReadOnly);
-    m_rules = KMimeTypeFactory::self()->parseMagicFile(&magicBuffer, "magicData");
+    m_rules = KMimeTypeRepository::self()->parseMagicFile(&magicBuffer, "magicData");
     QCOMPARE(m_rules.count(), 7);
 
     const KMimeMagicRule rule = m_rules[0];
@@ -885,4 +886,50 @@ void KMimeTypeTest::testParseMagicFile()
     testBuffer.close();
 }
 
+void KMimeTypeTest::testHelperProtocols()
+{
+    QVERIFY(KProtocolInfo::isKnownProtocol("mailto"));
+    QVERIFY(KProtocolInfo::isHelperProtocol("mailto"));
+    QVERIFY(KProtocolInfo::isHelperProtocol(KUrl("mailto:faure@kde.org")));
+    QCOMPARE(KProtocolInfo::exec("mailto"), QString::fromLatin1("kmailservice %u"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("http"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("ftp"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("file"));
+    QVERIFY(!KProtocolInfo::isHelperProtocol("unknown"));
+    QVERIFY(KProtocolInfo::isHelperProtocol("telnet"));
+
+    // To test that compat still works
+    if (KProtocolInfo::isKnownProtocol("tel")) {
+        QVERIFY(KProtocolInfo::isHelperProtocol("tel"));
+    }
+}
+
+void KMimeTypeTest::testFromThread()
+{
+    // Some simple tests to test more API from testThreads without using _data()
+    KMimeType::Ptr mime = KMimeType::mimeType("application/pdf");
+    QVERIFY(mime);
+    QCOMPARE(mime->mainExtension(), QString::fromLatin1(".pdf"));
+}
+
+void KMimeTypeTest::testThreads()
+{
+    QThreadPool::globalInstance()->setMaxThreadCount(20);
+    // Note that data-based tests cannot be used here (QTest::fetchData asserts).
+    QList<QFuture<void> > futures;
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByUrl);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByFileContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByNameAndContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFindByPathWithContent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testAllMimeTypes);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testAlias);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testMimeTypeParent);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testPreferredService);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testFromThread);
+    futures << QtConcurrent::run(this, &KMimeTypeTest::testHelperProtocols);
+    kDebug() << "Joining all threads";
+    Q_FOREACH(QFuture<void> f, futures)
+        f.waitForFinished();
+}
+
 #include "kmimetypetest.moc"
