@@ -46,19 +46,19 @@ QString findLibrary(const QString &name, const KComponentData &cData)
       libname = name;
       QString file, path;
 
-      int pos = libname.lastIndexOf( '/' );
+      int pos = libname.lastIndexOf( QLatin1Char('/') );
       if ( pos >= 0 )
       {
         file = libname.mid( pos + 1 );
         path = libname.left( pos );
-        libname = path + '/' + file.mid( 3 );
+        libname = path + QLatin1Char('/') + file.mid( 3 );
       }
       else
       {
         file = libname;
         libname = file.mid( 3 );
       }
-      if( !file.startsWith( "lib" ) )
+      if( !file.startsWith( QLatin1String("lib") ) )
           return file;
 
       libname = findLibraryInternal(libname, cData);
@@ -99,10 +99,10 @@ static KPluginFactory* kde3Factory(KLibrary *lib, const QByteArray &factoryname)
     if(!factoryname.isEmpty()) {
         symname += factoryname;
     } else {
-        symname += QFileInfo(lib->fileName()).fileName().split('.').first().toLatin1();
+        symname += QFileInfo(lib->fileName()).fileName().split(QLatin1Char('.')).first().toLatin1();
     }
 
-    const QString hashKey = lib->fileName() + QLatin1Char(':') + QString::fromAscii(symname);
+    const QString hashKey = lib->fileName() + QLatin1Char(':') + QString::fromLatin1(symname);
     KPluginFactory *factory = s_createdKde3Factories->value(hashKey);
     if (factory) {
         return factory;
