@@ -24,6 +24,7 @@
 #include "vfolder_menu.h"
 
 #include <config.h>
+#include <config-kded.h>
 
 #include <kservice.h>
 #include <kmimetype.h>
@@ -52,16 +53,10 @@
 #include <klocale.h>
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
+#ifndef KBUILDSYCOCA_NO_KCRASH
 #include <kcrash.h>
-#include <kmemfile.h>
-
-#ifdef KBUILDSYCOCA_GUI // KBUILDSYCOCA_GUI is used on win32 to build
-                        // GUI version of kbuildsycoca, so-called "kbuildsycocaw".
-# include <qlabel.h>
-# include <kmessagebox.h>
-  bool silent;
-  bool showprogress;
 #endif
+#include <kmemfile.h>
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -73,16 +68,15 @@ typedef QList<KSycocaEntry::List> KSycocaEntryListList;
 
 static quint32 newTimestamp = 0;
 
-static KBuildServiceFactory *g_bsf = 0;
-static KBuildServiceGroupFactory *g_bsgf = 0;
-static KSycocaFactory *g_factory = 0;
-static KCTimeInfo *g_ctimeInfo = 0;
-static QHash<QString, quint32> *g_ctimeDict = 0;
+static KBuildServiceFactory *g_serviceFactory = 0;
+static KBuildServiceGroupFactory *g_buildServiceGroupFactory = 0;
+static KSycocaFactory *g_currentFactory = 0;
+static KCTimeInfo *g_ctimeInfo = 0; // factory
+static KCTimeDict *g_ctimeDict = 0; // old timestamps
 static QByteArray g_resource = 0;
-static KBSEntryDict *g_entryDict = 0;
+static KBSEntryDict *g_currentEntryDict = 0;
 static KBSEntryDict *g_serviceGroupEntryDict = 0;
-static KSycocaEntryListList *g_allEntries = 0;
-static QStringList *g_changeList = 0;
+static KSycocaEntryListList *g_allEntries = 0; // entries from existing ksycoca
 static QStringList *g_allResourceDirs = 0;
 static bool g_changed = false;
 static KSycocaEntry::List g_tempStorage;
@@ -118,7 +112,7 @@ KBuildSycoca::~KBuildSycoca()
 
 KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFactory)
 {
-   quint32 timeStamp = g_ctimeInfo->ctime(file);
+   quint32 timeStamp = g_ctimeInfo->dict()->ctime(file, g_resource);
    if (!timeStamp)
    {
       timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, file,
@@ -128,43 +122,46 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
    if (g_allEntries)
    {
       assert(g_ctimeDict);
-      quint32 oldTimestamp = g_ctimeDict->value( file, 0 );
+      quint32 oldTimestamp = g_ctimeDict->ctime(file, g_resource);
 
       if (timeStamp && (timeStamp == oldTimestamp))
       {
          // Re-use old entry
-         if (g_factory == g_bsgf) // Strip .directory from service-group entries
+         if (g_currentFactory == g_buildServiceGroupFactory) // Strip .directory from service-group entries
          {
-            entry = g_entryDict->value(file.left(file.length()-10));
+            entry = g_currentEntryDict->value(file.left(file.length()-10));
          } else {
-            entry = g_entryDict->value(file);
+            entry = g_currentEntryDict->value(file);
          }
          // remove from g_ctimeDict; if g_ctimeDict is not empty
          // after all files have been processed, it means
          // some files were removed since last time
-         g_ctimeDict->remove( file );
+         if (file.contains("fake"))
+            kDebug(7021) << g_resource << "reusing (and removing) old entry [\"fake\"] for:" << file;
+         g_ctimeDict->remove(file, g_resource);
       }
       else if (oldTimestamp)
       {
          g_changed = true;
-         kDebug(7021) << "modified:" << file;
+         g_ctimeDict->remove(file, g_resource);
+         kDebug(7021) << "modified:" << file << "in" << g_resource.constData();
       }
       else
       {
          g_changed = true;
-         kDebug(7021) << "new:" << file;
+         kDebug(7021) << "new:" << file << "in" << g_resource.constData();
       }
    }
-   g_ctimeInfo->addCTime(file, timeStamp );
+   g_ctimeInfo->dict()->addCTime(file, g_resource, timeStamp);
    if (!entry)
    {
       // Create a new entry
-      entry = g_factory->createEntry( file, g_resource );
+      entry = g_currentFactory->createEntry( file, g_resource );
    }
    if ( entry && entry->isValid() )
    {
       if (addToFactory)
-         g_factory->addEntry(entry);
+         g_currentFactory->addEntry(entry);
       else
          g_tempStorage.append(entry);
       return entry;
@@ -172,14 +169,13 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
    return KSycocaEntry::Ptr();
 }
 
-// Callback for VFolderMenu
-void KBuildSycoca::slotCreateEntry(const QString &file, KService::Ptr *service)
+KService::Ptr KBuildSycoca::createService(const QString &path)
 {
-   KSycocaEntry::Ptr entry = createEntry(file, false);
-   *service = KService::Ptr::staticCast( entry );
+   KSycocaEntry::Ptr entry = createEntry(path, false);
+   return KService::Ptr::staticCast(entry);
 }
 
-// returns false if the database is up to date
+// returns false if the database is up to date, true if it needs to be saved
 bool KBuildSycoca::build()
 {
   typedef QLinkedList<KBSEntryDict *> KBSEntryDictList;
@@ -204,9 +200,9 @@ bool KBuildSycoca::build()
             entryDict->insert( (*it)->entryPath(), *it );
          }
      }
-     if ((*factory) == g_bsf)
+     if ((*factory) == g_serviceFactory)
         serviceEntryDict = entryDict;
-     else if ((*factory) == g_bsgf)
+     else if ((*factory) == g_buildServiceGroupFactory)
         g_serviceGroupEntryDict = entryDict;
      entryDictList.append(entryDict);
   }
@@ -258,11 +254,11 @@ bool KBuildSycoca::build()
      const KSycocaFactoryList::const_iterator end = factories()->constEnd();
      for ( ; it != end; ++it, ++ed_it )
      {
-        g_factory = (*it);
+        g_currentFactory = (*it);
         // g_ctimeInfo gets created after the initial loop, so it has no entryDict.
-        g_entryDict = ed_it == ed_end ? 0 : *ed_it;
+        g_currentEntryDict = ed_it == ed_end ? 0 : *ed_it;
 	// For each resource the factory deals with
-        const KSycocaResourceList *list = g_factory->resourceList();
+        const KSycocaResourceList *list = g_currentFactory->resourceList();
         if (!list) continue;
 
         for( KSycocaResourceList::ConstIterator it2 = list->constBegin();
@@ -286,42 +282,49 @@ bool KBuildSycoca::build()
      if (g_changed || !g_allEntries)
      {
         uptodate = false;
-        g_changeList->append(g_resource);
+        //kDebug() << "CHANGED:" << g_resource;
+        m_changedResources.append(g_resource);
      }
   }
 
   bool result = !uptodate || (g_ctimeDict && !g_ctimeDict->isEmpty());
+  if (g_ctimeDict && !g_ctimeDict->isEmpty()) {
+      //kDebug() << "Still in time dict:";
+      //g_ctimeDict->dump();
+      // ## It seems entries filtered out by vfolder are still in there,
+      // so we end up always saving ksycoca, i.e. this method never returns false
+
+      // Get the list of resources from which some files were deleted
+      const QStringList resources = g_ctimeDict->resourceList();
+      kDebug() << "Still in the time dict (i.e. deleted files)" << resources;
+      m_changedResources += resources;
+  }
 
   if (result || bMenuTest)
   {
      g_resource = "apps";
-     g_factory = g_bsf;
-     g_entryDict = serviceEntryDict;
+     g_currentFactory = g_serviceFactory;
+     g_currentEntryDict = serviceEntryDict;
      g_changed = false;
 
-     g_vfolder = new VFolderMenu(g_bsf);
+     g_vfolder = new VFolderMenu(g_serviceFactory, this);
      if (!m_trackId.isEmpty())
         g_vfolder->setTrackId(m_trackId);
 
-     connect(g_vfolder, SIGNAL(newService(const QString &, KService::Ptr *)),
-             this, SLOT(slotCreateEntry(const QString &, KService::Ptr *)));
-
      VFolderMenu::SubMenu *kdeMenu = g_vfolder->parseMenu("applications.menu", true);
 
-     KServiceGroup::Ptr entry = g_bsgf->addNew("/", kdeMenu->directoryFile, KServiceGroup::Ptr(), false);
+     KServiceGroup::Ptr entry = g_buildServiceGroupFactory->addNew("/", kdeMenu->directoryFile, KServiceGroup::Ptr(), false);
      entry->setLayoutInfo(kdeMenu->layoutList);
      createMenu(QString(), QString(), kdeMenu);
 
      (void) existingResourceDirs();
      *g_allResourceDirs += g_vfolder->allDirectories();
 
-     disconnect(g_vfolder, SIGNAL(newService(const QString &, KService::Ptr *)),
-                this, SLOT(slotCreateEntry(const QString &, KService::Ptr *)));
-
      if (g_changed || !g_allEntries)
      {
         uptodate = false;
-        g_changeList->append(g_resource);
+        //kDebug() << "CHANGED:" << g_resource;
+        m_changedResources.append(g_resource);
      }
      if (bMenuTest) {
          result = false;
@@ -343,7 +346,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
      QString directoryFile = subMenu->directoryFile;
      if (directoryFile.isEmpty())
         directoryFile = subName+".directory";
-     quint32 timeStamp = g_ctimeInfo->ctime(directoryFile);
+     quint32 timeStamp = g_ctimeInfo->dict()->ctime(directoryFile, g_resource);
      if (!timeStamp) {
         timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, directoryFile,
                                                        KStandardDirs::Recursive );
@@ -352,7 +355,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
      KServiceGroup::Ptr entry;
      if (g_allEntries)
      {
-        quint32 oldTimestamp = g_ctimeDict->value( directoryFile, 0 );
+        const quint32 oldTimestamp = g_ctimeDict->ctime(directoryFile, g_resource);
 
         if (timeStamp && (timeStamp == oldTimestamp))
         {
@@ -365,9 +368,9 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
             }
         }
      }
-     g_ctimeInfo->addCTime(directoryFile, timeStamp);
+     g_ctimeInfo->dict()->addCTime(directoryFile, g_resource, timeStamp);
 
-     entry = g_bsgf->addNew(subName, subMenu->directoryFile, entry, subMenu->isDeleted);
+     entry = g_buildServiceGroupFactory->addNew(subName, subMenu->directoryFile, entry, subMenu->isDeleted);
      entry->setLayoutInfo(subMenu->layoutList);
      if (! (bMenuTest && entry->noDisplay()) )
         createMenu(caption + entry->caption() + '/', subName, subMenu);
@@ -385,7 +388,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
      }
      else
      {
-        g_bsgf->addNewEntryTo( name, p );
+        g_buildServiceGroupFactory->addNewEntryTo( name, p );
      }
   }
 }
@@ -407,11 +410,6 @@ bool KBuildSycoca::recreate()
   {
     fprintf(stderr, "kbuildsycoca4: ERROR creating database '%s'! %s\n",
       path.toLocal8Bit().data(), database.errorString().toLocal8Bit().data());
-#ifdef KBUILDSYCOCA_GUI // KBUILDSYCOCA_GUI is used on win32 to build
-                        // GUI version of kbuildsycoca, so-called "kbuildsycocaw".
-    if (!silent)
-      KMessageBox::error(0, i18n("Error creating database '%1'.\nCheck that the permissions are correct on the directory and the disk is not full.\n", path.toLocal8Bit().data()), i18n("KBuildSycoca"));
-#endif
     return false;
   }
 
@@ -423,9 +421,9 @@ bool KBuildSycoca::recreate()
   // It is very important to build the servicetype one first
   // Both are registered in KSycoca, no need to keep the pointers
   KSycocaFactory *stf = new KBuildServiceTypeFactory;
-  KBuildMimeTypeFactory *mtf = new KBuildMimeTypeFactory;
-  g_bsgf = new KBuildServiceGroupFactory();
-  g_bsf = new KBuildServiceFactory(stf, mtf, g_bsgf);
+  KBuildMimeTypeFactory* mimeTypeFactory = new KBuildMimeTypeFactory;
+  g_buildServiceGroupFactory = new KBuildServiceGroupFactory();
+  g_serviceFactory = new KBuildServiceFactory(stf, mimeTypeFactory, g_buildServiceGroupFactory);
   (void) new KBuildProtocolInfoFactory();
 
   if( build()) // Parse dirs
@@ -439,10 +437,6 @@ bool KBuildSycoca::recreate()
     {
       fprintf(stderr, "kbuildsycoca4: ERROR writing database '%s'!\n", database.fileName().toLocal8Bit().data());
       fprintf(stderr, "kbuildsycoca4: Disk full?\n");
-#ifdef KBUILDSYCOCA_GUI
-      if (!silent)
-        KMessageBox::error(0, i18n("Error writing database '%1'.\nCheck that the permissions are correct on the directory and the disk is not full.\n", path.toLocal8Bit().data()), i18n("KBuildSycoca"));
-#endif
       return false;
     }
   }
@@ -511,7 +505,6 @@ void KBuildSycoca::save(QDataStream* str)
    (*str) << (*g_allResourceDirs);
 
    // Calculate per-servicetype/mimetype data
-   mimeTypeFactory->parseSubclasses();
    serviceFactory->postProcessServices();
 
    // Here so that it's the last debug message
@@ -653,10 +646,6 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    options.add("global", ki18n("Create global database"));
    options.add("menutest", ki18n("Perform menu generation test run only"));
    options.add("track <menu-id>", ki18n("Track menu id for debug purposes"));
-#ifdef KBUILDSYCOCA_GUI
-   options.add("silent", ki18n("Silent - work without windows and stderr"));
-   options.add("showprogress", ki18n("Show progress information (even if 'silent' mode is on)"));
-#endif
 
    KCmdLineArgs::init(argc, argv, &d);
    KCmdLineArgs::addCmdLineOptions(options);
@@ -670,31 +659,14 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
      setenv("KDEROOTHOME", "-", 1);
    }
 
-#ifdef KBUILDSYCOCA_GUI
-   KApplication k;
-#else
-   KApplication k(false);
-#endif
-   k.disableSessionManagement();
-
-#ifdef KBUILDSYCOCA_GUI
-   silent = args->isSet("silent");
-   showprogress = args->isSet("showprogress");
-   QLabel progress( QString("<p><br><nobr>    %1    </nobr><br>").arg( i18n("Reloading KDE configuration, please wait...") ), 0, "", Qt::WType_Dialog | Qt::WStyle_DialogBorder  | Qt::WStyle_Customize| Qt::WStyle_Title );
-   QString capt = i18n("KDE Configuration Manager");
-   if (!silent) {
-     if (KMessageBox::No == KMessageBox::questionYesNo(0, i18n("Do you want to reload KDE configuration?"), capt, i18nc("Reload KDE configuration messagebox", "Reload"), i18n("Do Not Reload")))
-       return 0;
-   }
-   if (!silent || showprogress) {
-     progress.setCaption( capt );
-     progress.show();
-   }
-#endif
+   QCoreApplication k(argc, argv);
+   KComponentData mainComponent(d);
 
+#ifndef KBUILDSYCOCA_NO_KCRASH
    KCrash::setCrashHandler(KCrash::defaultCrashHandler);
    KCrash::setEmergencySaveFunction(crashHandler);
    KCrash::setApplicationName(QString(appName));
+#endif
 
    // force generating of KLocale object. if not, the database will get
    // be translated
@@ -744,8 +716,6 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
      }
    }
 
-   g_changeList = new QStringList;
-
    bool checkstamps = incremental && args->isSet("checkstamps") && checkfiles;
    quint32 filestamp = 0;
    QStringList oldresourcedirs;
@@ -787,6 +757,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    }
 
    newTimestamp = (quint32) time(0);
+   QStringList changedResources;
 
    if( checkfiles && ( !checkstamps || !KBuildSycoca::checkTimestamps( filestamp, oldresourcedirs )))
    {
@@ -801,7 +772,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
          KSycoca::self();
          KSycocaFactoryList *factories = new KSycocaFactoryList;
          g_allEntries = new KSycocaEntryListList;
-         g_ctimeDict = new QHash<QString, quint32>;
+         g_ctimeDict = new KCTimeDict;
 
          // Must be in same order as in KBuildSycoca::recreate()!
          factories->append( new KServiceTypeFactory );
@@ -819,7 +790,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
          }
          delete factories; factories = 0;
          KCTimeInfo *ctimeInfo = new KCTimeInfo;
-         ctimeInfo->fillCTimeDict(*g_ctimeDict);
+         *g_ctimeDict = ctimeInfo->loadDict();
       }
       cSycocaPath = 0;
 
@@ -827,12 +798,9 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
       if (args->isSet("track"))
          sycoca->setTrackId(args->getOption("track"));
       if (!sycoca->recreate()) {
-#ifdef KBUILDSYCOCA_GUI
-        if (!silent || showprogress)
-          progress.close();
-#endif
         return -1;
       }
+      changedResources = sycoca->changedResources();
 
       if (bGlobalDatabase)
       {
@@ -849,21 +817,15 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    {
      // Notify ALL applications that have a ksycoca object, using a signal
      QDBusMessage signal = QDBusMessage::createSignal("/", "org.kde.KSycoca", "notifyDatabaseChanged" );
-     signal << *g_changeList;
+     signal << changedResources;
 
      if (QDBusConnection::sessionBus().isConnected()) {
-        kDebug() << "Emitting notifyDatabaseChanged" << *g_changeList;
+         kDebug() << "Emitting notifyDatabaseChanged" << changedResources;
        QDBusConnection::sessionBus().send(signal);
        qApp->processEvents(); // make sure the dbus signal is sent before we quit.
      }
    }
 
-#ifdef KBUILDSYCOCA_GUI
-   if (!silent) {
-     progress.close();
-     KMessageBox::information(0, i18n("Configuration information reloaded successfully."), capt);
-   }
-#endif
    return 0;
 }
 
