@@ -18,6 +18,7 @@
 
 #include "vfolder_menu.h"
 #include "kbuildservicefactory.h"
+#include "kbuildsycocainterface.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -299,9 +300,10 @@ VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService::P
 }
 
 
-VFolderMenu::VFolderMenu(KBuildServiceFactory* serviceFactory) :
-    m_track(false),
-    m_serviceFactory(serviceFactory)
+VFolderMenu::VFolderMenu(KBuildServiceFactory* serviceFactory, KBuildSycocaInterface* kbuildsycocaInterface)
+    : m_track(false),
+      m_serviceFactory(serviceFactory),
+      m_kbuildsycocaInterface(kbuildsycocaInterface)
 {
    m_usedAppsDict.reserve(797);
    m_rootMenu = 0;
@@ -371,11 +373,7 @@ VFolderMenu::buildApplicationIndex(bool unusedOnly)
             continue;
          }
 
-         const QStringList cats = s->categories();
-         for(QStringList::ConstIterator it2 = cats.begin();
-             it2 != cats.end(); ++it2)
-         {
-            const QString &cat = *it2;
+         Q_FOREACH(const QString& cat, s->categories()) {
             info->dictCategories[cat].append(s); // find or insert entry in hash
          }
       }
@@ -433,12 +431,14 @@ VFolderMenu::absoluteDir(const QString &_dir, const QString &baseDir, bool keepR
    if (!dir.endsWith('/'))
       dir += '/';
 
-   if (QDir::isRelativePath(dir) && !keepRelativeToCfg)
-   {
+   bool relative = QDir::isRelativePath(dir);
+   if (relative && !keepRelativeToCfg) {
+      relative = false;
       dir = KGlobal::dirs()->findResource("xdgconf-menu", dir);
    }
 
-   dir = KGlobal::dirs()->realPath(dir);
+   if (!relative)
+      dir = KGlobal::dirs()->realPath(dir);
 
    return dir;
 }
@@ -835,7 +835,7 @@ VFolderMenu::loadMenu(const QString &fileName)
 {
    m_defaultMergeDirs.clear();
 
-   if (!fileName.endsWith(".menu"))
+   if (!fileName.endsWith(QLatin1String(".menu")))
       return;
 
    pushDocInfo(fileName);
@@ -991,8 +991,7 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
       if (fi.isFile()) {
          if (!fn.endsWith(QLatin1String(".desktop")))
             continue;
-         KService::Ptr service;
-         emit newService(fi.absoluteFilePath(), &service); // calls KBuildSycoca::slotCreateEntry
+         KService::Ptr service = m_kbuildsycocaInterface->createService(fi.absoluteFilePath());
          if (service)
             addApplication(prefix + fn, service);
       }
@@ -1020,7 +1019,7 @@ VFolderMenu::processKDELegacyDirs()
       if (!m_forcedLegacyLoad && (*it).endsWith(QLatin1String(".directory")))
       {
          QString name = *it;
-         if (!name.endsWith("/.directory"))
+         if (!name.endsWith(QLatin1String("/.directory")))
             continue; // Probably ".directory", skip it.
 
          name = name.left(name.length()-11);
@@ -1035,8 +1034,7 @@ VFolderMenu::processKDELegacyDirs()
       if ((*it).endsWith(QLatin1String(".desktop")))
       {
          QString name = *it;
-         KService::Ptr service;
-         emit newService(name, &service);
+         KService::Ptr service = m_kbuildsycocaInterface->createService(name);
 
          if (service && !m_forcedLegacyLoad)
          {
@@ -1090,8 +1088,7 @@ VFolderMenu::processLegacyDir(const QString &dir, const QString &relDir, const Q
       if (fi.isFile() /*&& !fi.isSymLink() ?? */) {
          if (!fn.endsWith(QLatin1String(".desktop")))
             continue;
-         KService::Ptr service;
-         emit newService(fi.absoluteFilePath(), &service);
+         KService::Ptr service = m_kbuildsycocaInterface->createService(fi.absoluteFilePath());
          if (service)
          {
             const QString id = prefix + fn;
@@ -1459,7 +1456,7 @@ static QString parseAttribute( const QDomElement &e)
         else if ( str=="false" )
             option+="NI ";
         else
-            kDebug()<<" Error in parsing inlibe attribute :"<<str;
+            kDebug()<<" Error in parsing inline attribute :"<<str;
     }
     if ( e.hasAttribute( "inline_limit" ) )
     {
@@ -1507,6 +1504,7 @@ static QStringList parseLayoutNode(const QDomElement &docElem)
    if ( !optionDefaultLayout.isEmpty() )
        layout.append( optionDefaultLayout );
 
+   bool mergeTagExists = false;
    QDomNode n = docElem.firstChild();
    while( !n.isNull() ) {
       QDomElement e = n.toElement(); // try to convert the node to an element.
@@ -1534,10 +1532,17 @@ static QStringList parseLayoutNode(const QDomElement &docElem)
             layout.append(":M");
          else if (type == "all")
             layout.append(":A");
+         mergeTagExists = true;
       }
 
       n = n.nextSibling();
    }
+
+   if ( !mergeTagExists ) {
+       layout.append(":M");
+       layout.append(":F");
+       kWarning() << "The menu spec file contains a Layout or DefaultLayout tag without the mandatory Merge tag inside. Please fix your file.";
+   }
    return layout;
 }
 
