@@ -36,7 +36,9 @@
 #include <QEvent>
 #include <QToolButton>
 #include <QToolBar>
+#include <QStandardItem>
 #include <kicon.h>
+#include <klocale.h>
 #include <kdebug.h>
 
 #include "kcombobox.h"
@@ -47,17 +49,7 @@
 // display ampersands literally.
 static QString DropAmpersands(const QString &text)
 {
-    QString result;
-    for (int i = 0; i < text.count(); ++i) {
-        if (text.at(i) == '&') {
-            if (i + 1 < text.count() && text.at(i + 1) == '&')
-                result.append(text.at(i++)); // "&&" -> '&'
-            // else eat the ampersand
-        }
-        else
-            result.append(text.at(i));
-    }
-    return result;
+    return KGlobal::locale()->removeAcceleratorMarker(text);
 }
 
 
@@ -241,7 +233,7 @@ void KSelectAction::addAction(QAction* action)
   //kDebug (129) << "KSelectAction::addAction(" << action << ")";
 
   action->setActionGroup(selectableActionGroup());
-  
+
   // Re-Enable when an action is added
   setEnabled(true);
 
@@ -310,7 +302,7 @@ QAction* KSelectAction::removeAction(QAction* action)
 
   menu()->removeAction(action);
 
-  
+
   return action;
 }
 
@@ -529,6 +521,9 @@ QWidget * KSelectAction::createWidget( QWidget * parent )
     switch (mode) {
     case MenuMode: {
       QToolButton* button = new QToolButton(toolBar);
+      button->setToolTip(toolTip());
+      button->setWhatsThis(whatsThis());
+      button->setStatusTip(statusTip());
       button->setAutoRaise(true);
       button->setFocusPolicy(Qt::NoFocus);
       button->setIconSize(toolBar->iconSize());
@@ -549,11 +544,8 @@ QWidget * KSelectAction::createWidget( QWidget * parent )
     }
 
     case ComboBoxMode: {
-        KComboBox* comboBox = new KComboBox(parent);
+      KComboBox* comboBox = new KComboBox(parent);
       comboBox->installEventFilter (this);
-      // hack for the fact that QWidgetAction does not sync all its created widgets
-      // to its enabled state, just QToolButtons (Qt 4.4.3)
-      installEventFilter( comboBox );
 
       if ( d->m_maxComboViewCount != -1 )
         comboBox->setMaxVisibleItems( d->m_maxComboViewCount );
@@ -562,10 +554,16 @@ QWidget * KSelectAction::createWidget( QWidget * parent )
         comboBox->setMaximumWidth( d->m_comboWidth );
 
       comboBox->setEditable(isEditable());
+      comboBox->setToolTip(toolTip());
+      comboBox->setWhatsThis(whatsThis());
+      comboBox->setStatusTip(statusTip());
 
       foreach (QAction* action, selectableActionGroup()->actions())
         comboBox->addAction(action);
 
+      if (selectableActionGroup()->actions().isEmpty())
+          comboBox->setEnabled(false);
+
       connect(comboBox, SIGNAL(destroyed(QObject*)), SLOT(_k_comboBoxDeleted(QObject*)));
       connect(comboBox, SIGNAL(currentIndexChanged(int)), SLOT(_k_comboBoxCurrentIndexChanged(int)));
       d->m_comboBoxes.append(comboBox);
@@ -587,6 +585,24 @@ void KSelectAction::deleteWidget(QWidget *widget)
     KAction::deleteWidget(widget);
 }
 
+bool KSelectAction::event(QEvent *event)
+{
+    Q_D(KSelectAction);
+    if (event->type() == QEvent::ActionChanged) {
+        Q_FOREACH(KComboBox* comboBox, d->m_comboBoxes) {
+            comboBox->setToolTip(toolTip());
+            comboBox->setWhatsThis(whatsThis());
+            comboBox->setStatusTip(statusTip());
+        }
+        Q_FOREACH(QToolButton* toolButton, d->m_buttons) {
+            toolButton->setToolTip(toolTip());
+            toolButton->setWhatsThis(whatsThis());
+            toolButton->setStatusTip(statusTip());
+        }
+    }
+    return KAction::event(event);
+}
+
 // KSelectAction::eventFilter() is called before action->setChecked()
 // invokes the signal to update QActionGroup so KSelectAction::currentItem()
 // returns an old value.  There are 3 possibilities, where n actions will
@@ -624,14 +640,8 @@ static int TrueCurrentItem (KSelectAction *sa)
   return (curAction && curAction->isChecked ()) ? sa->actions ().indexOf (curAction) : -1;
 }
 
-// A plain "QVariant (action)" results in a bool being stored.  So all
-// QAction pointers become stored as "true".  This hacks around it.
-static QVariant QVariantFromQAction (QAction *action)
-{
-    // "(void *)" or "long" would be smaller than "qlonglong"
-    // but neither is supported by QVariant :(
-    return qVariantFromValue ((qlonglong) action);
-}
+// We store the QAction* as the userData of each combobox item
+Q_DECLARE_METATYPE(QAction*)
 
 bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
 {
@@ -660,7 +670,6 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
     return false/*propagate event*/;
   }
 
-
   bool blocked = comboBox->blockSignals (true);
 
   if (event->type () == QEvent::ActionAdded)
@@ -668,7 +677,7 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
     QActionEvent * const e = static_cast <QActionEvent *> (event);
 
     const int index = e->before () ?
-      comboBox->findData (QVariantFromQAction (e->before ())) :
+      comboBox->findData (QVariant::fromValue (e->before ())) :
       comboBox->count ();
     const int newItem = ::TrueCurrentItem (this);
     //kDebug (129) << "KSelectAction::eventFilter(ActionAdded)"
@@ -684,7 +693,11 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
     comboBox->insertItem (index,
       e->action()->icon(),
       ::DropAmpersands (e->action()->text()),
-      QVariantFromQAction (e->action ()));
+      QVariant::fromValue (e->action ()));
+    if (QStandardItemModel *model = qobject_cast<QStandardItemModel *>(comboBox->model())) {
+        QStandardItem *item = model->item(index);
+        item->setEnabled(e->action()->isEnabled());
+    }
 
     // Inserting an item into a combobox can change the current item so
     // make sure the item corresponding to the checked action is selected.
@@ -694,7 +707,7 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
   {
     QActionEvent * const e = static_cast <QActionEvent *> (event);
 
-    const int index = comboBox->findData (QVariantFromQAction (e->action ()));
+    const int index = comboBox->findData (QVariant::fromValue (e->action ()));
     const int newItem = ::TrueCurrentItem (this);
     //kDebug (129) << "KSelectAction::eventFilter(ActionChanged)"
     //          << "    comboBox: ptr=" << comboBox
@@ -707,6 +720,10 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
     //          << endl;
     comboBox->setItemIcon (index, e->action ()->icon ());
     comboBox->setItemText (index, ::DropAmpersands (e->action ()->text ()));
+    if (QStandardItemModel *model = qobject_cast<QStandardItemModel *>(comboBox->model())) {
+        QStandardItem *item = model->item(index);
+        item->setEnabled(e->action()->isEnabled());
+    }
 
     // The checked action may have become unchecked so
     // make sure the item corresponding to the checked action is selected.
@@ -716,7 +733,7 @@ bool KSelectAction::eventFilter (QObject *watched, QEvent *event)
   {
     QActionEvent * const e = static_cast <QActionEvent *> (event);
 
-    const int index = comboBox->findData (QVariantFromQAction (e->action ()));
+    const int index = comboBox->findData (QVariant::fromValue (e->action ()));
     const int newItem = ::TrueCurrentItem (this);
     //kDebug (129) << "KSelectAction::eventFilter(ActionRemoved)"
     //          << "    comboBox: ptr=" << comboBox
