@@ -22,6 +22,7 @@
 
 #include <config.h>
 
+#include <QtCore/QPointer>
 #include <QtGui/QPainter>
 #include <QtGui/qdrawutil.h>
 #include <QtGui/QApplication>
@@ -32,37 +33,44 @@
 #include <QMouseEvent>
 #include <QStyleOptionButton>
 #include "kcolordialog.h"
+#include "kcolorhelpers_p.h"
 #include "kcolormimedata.h"
 #include "kdebug.h"
 
+using KDEPrivate::fillOpaqueRect;
+
 class KColorButton::KColorButtonPrivate
 {
 public:
-    KColorButtonPrivate(KColorButton *q): q(q) {}
+    KColorButtonPrivate(KColorButton *q);
 
     void _k_chooseColor();
 
     KColorButton *q;
     QColor m_defaultColor;
     bool m_bdefaultColor : 1;
+    bool m_alphaChannel : 1;
 
-    bool dragFlag : 1;
     QColor col;
     QPoint mPos;
 
     void initStyleOption(QStyleOptionButton* opt) const;    
 };
 
+KColorButton::KColorButtonPrivate::KColorButtonPrivate(KColorButton *q)
+    : q(q)
+{
+  m_bdefaultColor = false;
+  m_alphaChannel = false;
+  q->setAcceptDrops(true);
+
+  connect(q, SIGNAL(clicked()), q, SLOT(_k_chooseColor()));
+}
+
 KColorButton::KColorButton( QWidget *parent )
   : QPushButton( parent )
   , d( new KColorButtonPrivate(this) )
 {
-  d->m_bdefaultColor = false;
-  d->m_defaultColor = QColor();
-  setAcceptDrops( true);
-
-  // 2000-10-15 (putzer): fixes broken keyboard usage
-  connect (this, SIGNAL(clicked()), this, SLOT(_k_chooseColor()));
 }
 
 KColorButton::KColorButton( const QColor &c, QWidget *parent )
@@ -70,12 +78,6 @@ KColorButton::KColorButton( const QColor &c, QWidget *parent )
   , d( new KColorButtonPrivate(this) )
 {
   d->col = c;
-  d->m_bdefaultColor = false;
-  d->m_defaultColor = QColor();
-  setAcceptDrops( true);
-
-  // 2000-10-15 (putzer): fixes broken keyboard usage
-  connect (this, SIGNAL(clicked()), this, SLOT(_k_chooseColor()));
 }
 
 KColorButton::KColorButton( const QColor &c, const QColor &defaultColor, QWidget *parent )
@@ -83,12 +85,7 @@ KColorButton::KColorButton( const QColor &c, const QColor &defaultColor, QWidget
   , d( new KColorButtonPrivate(this) )
 {
   d->col = c;
-  d->m_bdefaultColor = true;
-  d->m_defaultColor = defaultColor;
-  setAcceptDrops( true);
-
-  // 2000-10-15 (putzer): fixes broken keyboard usage
-  connect (this, SIGNAL(clicked()), this, SLOT(_k_chooseColor()));
+  setDefaultColor(defaultColor);
 }
 
 KColorButton::~KColorButton()
@@ -105,11 +102,21 @@ void KColorButton::setColor( const QColor &c )
 {
   if ( d->col != c ) {
     d->col = c;
-    repaint();
+    update();
     emit changed( d->col );
   }
 }
 
+void KColorButton::setAlphaChannelEnabled( bool alpha )
+{
+    d->m_alphaChannel = alpha;
+}
+
+bool KColorButton::isAlphaChannelEnabled() const
+{
+    return d->m_alphaChannel;
+}
+
 QColor KColorButton::defaultColor() const
 {
   return d->m_defaultColor;
@@ -135,38 +142,40 @@ void KColorButton::KColorButtonPrivate::initStyleOption(QStyleOptionButton* opt)
 void KColorButton::paintEvent( QPaintEvent* )
 {
   QPainter painter(this);
+  QStyle *style = QWidget::style();
 
   //First, we need to draw the bevel.
   QStyleOptionButton butOpt;
   d->initStyleOption(&butOpt);
-  style()->drawControl( QStyle::CE_PushButtonBevel, &butOpt, &painter, this );
+  style->drawControl( QStyle::CE_PushButtonBevel, &butOpt, &painter, this );
 
   //OK, now we can muck around with drawing out pretty little color box
   //First, sort out where it goes
-  QRect labelRect = style()->subElementRect( QStyle::SE_PushButtonContents,
+  QRect labelRect = style->subElementRect( QStyle::SE_PushButtonContents,
       &butOpt, this );
-  int shift = style()->pixelMetric( QStyle::PM_ButtonMargin ) / 2;
+  int shift = style->pixelMetric( QStyle::PM_ButtonMargin, &butOpt, this ) / 2;
   labelRect.adjust(shift, shift, -shift, -shift);
   int x, y, w, h;
   labelRect.getRect(&x, &y, &w, &h);
 
   if (isChecked() || isDown()) {
-    x += style()->pixelMetric( QStyle::PM_ButtonShiftHorizontal );
-    y += style()->pixelMetric( QStyle::PM_ButtonShiftVertical   );
+    x += style->pixelMetric( QStyle::PM_ButtonShiftHorizontal, &butOpt, this );
+    y += style->pixelMetric( QStyle::PM_ButtonShiftVertical, &butOpt, this );
   }
 
   QColor fillCol = isEnabled() ? d->col : palette().color(backgroundRole());
   qDrawShadePanel( &painter, x, y, w, h, palette(), true, 1, NULL);
-  if ( fillCol.isValid() )
-    painter.fillRect( x+1, y+1, w-2, h-2, fillCol );
+  if ( fillCol.isValid() ) {
+    fillOpaqueRect(&painter, QRect( x+1, y+1, w-2, h-2), fillCol );
+  }
 
   if ( hasFocus() ) {
-    QRect focusRect = style()->subElementRect( QStyle::SE_PushButtonFocusRect, &butOpt, this );
+    QRect focusRect = style->subElementRect( QStyle::SE_PushButtonFocusRect, &butOpt, this );
     QStyleOptionFocusRect focusOpt;
     focusOpt.init(this);
     focusOpt.rect            = focusRect;
     focusOpt.backgroundColor = palette().background().color();
-    style()->drawPrimitive( QStyle::PE_FrameFocusRect, &focusOpt, &painter, this );
+    style->drawPrimitive( QStyle::PE_FrameFocusRect, &focusOpt, &painter, this );
   }
 }
 
@@ -234,22 +243,20 @@ void KColorButton::mouseMoveEvent( QMouseEvent *e)
 
 void KColorButton::KColorButtonPrivate::_k_chooseColor()
 {
-  QColor c = q->color();
-  if ( m_bdefaultColor )
-  {
-      if( KColorDialog::getColor( c, m_defaultColor, q ) != QDialog::Rejected ) {
-          if ( c.isValid() )
-              q->setColor( c );
-          else
-              q->setColor( m_defaultColor );
-      }
-  }
-  else
-  {
-      if( KColorDialog::getColor( c, q ) != QDialog::Rejected ) {
-          q->setColor( c );
-      }
-  }
+    QPointer<KColorDialog> dialog = new KColorDialog(q, true);
+    dialog->setColor(q->color());
+    if (m_bdefaultColor) {
+        dialog->setDefaultColor(m_defaultColor);
+    }
+    dialog->setAlphaChannelEnabled(m_alphaChannel);
+    if (dialog->exec() != QDialog::Rejected) {
+        if (dialog->color().isValid()) {
+            q->setColor(dialog->color());
+        } else if (m_bdefaultColor) {
+            q->setColor(m_defaultColor);
+        }
+    }
+    delete dialog;
 }
 
 
