@@ -53,7 +53,7 @@ public:
   /**
   * KConfigSkeleton object used to store settings
    */
-  KConfigSkeleton *m_conf;
+  KCoreConfigSkeleton *m_conf;
 
   /**
   * Dialog being managed
@@ -66,6 +66,14 @@ public:
   bool trackChanges : 1;
 };
 
+KConfigDialogManager::KConfigDialogManager(QWidget *parent, KCoreConfigSkeleton *conf)
+ : QObject(parent), d(new Private(this))
+{
+  d->m_conf = conf;
+  d->m_dialog = parent;
+  init(true);
+}
+
 KConfigDialogManager::KConfigDialogManager(QWidget *parent, KConfigSkeleton *conf)
  : QObject(parent), d(new Private(this))
 {
@@ -117,7 +125,7 @@ void KConfigDialogManager::initMaps()
 
     // KDE
     s_changedMap->insert( "KComboBox", SIGNAL(activated (int)));
-    s_changedMap->insert( "KFontCombo", SIGNAL(activated (int)));
+    s_changedMap->insert( "KFontComboBox", SIGNAL(activated (int)));
     s_changedMap->insert( "KFontRequester", SIGNAL(fontSelected(const QFont &)));
     s_changedMap->insert( "KFontChooser",  SIGNAL(fontSelected(const QFont &)));
     s_changedMap->insert( "KHistoryCombo", SIGNAL(activated (int)));
