@@ -44,13 +44,14 @@
 #include <kstandardguiitem.h>
 #include <ktoolinvocation.h>
 #include <kurllabel.h>
-#include <kwhatsthismanager_p.h>
 
 #ifdef Q_WS_X11
 #include <qx11info_x11.h>
 #include <netwm.h>
 #endif
 
+static bool sAllowEmbeddingInGraphicsView = false;
+
 void KDialogPrivate::setupLayout()
 {
     Q_Q(KDialog);
@@ -107,15 +108,10 @@ void KDialogPrivate::queuedLayoutUpdate()
     }
 }
 
-void KDialogPrivate::setButtonFocus(QPushButton *button, bool isDefault, bool isFocus)
-{
-  button->setDefault( isDefault );
-  if ( isFocus )
-      button->setFocus();
-}
-
 void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
 {
+    Q_Q(KDialog);
+
   QDialogButtonBox::ButtonRole role = QDialogButtonBox::InvalidRole;
   switch ( key ) {
     case KDialog::Help:
@@ -148,7 +144,6 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
     case KDialog::User3:
       role = QDialogButtonBox::ActionRole;
       break;
-    case KDialog::NoDefault:
     default:
       role = QDialogButtonBox::InvalidRole;
       break;
@@ -165,12 +160,16 @@ void KDialogPrivate::appendButton(KDialog::ButtonCode key, const KGuiItem &item)
 
     QObject::connect(button, SIGNAL(clicked()),
            &mButtonSignalMapper, SLOT( map() ) );
+
+    if (key == mDefaultButton) {
+        // Now that it exists, set it as default
+        q->setDefaultButton(mDefaultButton);
+    }
 }
 
 void KDialogPrivate::init(KDialog *q)
 {
     q_ptr = q;
-    KWhatsThisManager::init();
 
     dirty = false;
 
@@ -188,13 +187,13 @@ void KDialogPrivate::helpLinkClicked()
 }
 
 KDialog::KDialog( QWidget *parent, Qt::WFlags flags )
-  : QDialog(parent, flags), d_ptr(new KDialogPrivate)
+  : QDialog(parent, sAllowEmbeddingInGraphicsView ? flags : flags | Qt::BypassGraphicsProxyWidget ), d_ptr(new KDialogPrivate)
 {
     d_ptr->init(this);
 }
 
 KDialog::KDialog(KDialogPrivate &dd, QWidget *parent, Qt::WFlags flags)
-    : QDialog(parent, flags), d_ptr(&dd)
+    : QDialog(parent, sAllowEmbeddingInGraphicsView ? flags : flags | Qt::BypassGraphicsProxyWidget), d_ptr(&dd)
 {
     d_ptr->init(this);
 }
@@ -288,18 +287,38 @@ void KDialog::setEscapeButton( ButtonCode id )
 void KDialog::setDefaultButton( ButtonCode newDefaultButton )
 {
     Q_D(KDialog);
-    bool makeDefault = true;
-    if (newDefaultButton == NoDefault) {
-        // if we already have a default, let's be sure to reset
-        // it to not be the default!
-        newDefaultButton = defaultButton();
-        makeDefault = false;
+
+    if (newDefaultButton == None)
+        newDefaultButton = NoDefault; // #148969
+
+    const KDialog::ButtonCode oldDefault = defaultButton();
+
+    bool oldDefaultHadFocus = false;
+
+    if (oldDefault != NoDefault) {
+        KPushButton *old = button(oldDefault);
+        if (old) {
+            oldDefaultHadFocus = (focusWidget() == old);
+            old->setDefault(false);
+        }
     }
 
-    KPushButton *b = button(newDefaultButton);
-    if (b) {
-        d->setButtonFocus(b, makeDefault, false);
+    if (newDefaultButton != NoDefault) {
+        KPushButton *b = button(newDefaultButton);
+        if (b) {
+            b->setDefault(true);
+            if (focusWidget() == 0 || oldDefaultHadFocus) {
+                // No widget had focus yet, or the old default button had
+                // -> ok to give focus to the new default button, so that it's
+                // really default (Enter triggers it).
+                // But we don't do this if the kdialog user gave focus to a
+                // specific widget in the dialog.
+                b->setFocus();
+            }
+        }
     }
+    d->mDefaultButton = newDefaultButton;
+    Q_ASSERT(defaultButton() == newDefaultButton);
 }
 
 KDialog::ButtonCode KDialog::defaultButton() const
@@ -308,11 +327,12 @@ KDialog::ButtonCode KDialog::defaultButton() const
   QHashIterator<int, KPushButton*> it( d->mButtonList );
   while ( it.hasNext() ) {
     it.next();
-    if ( it.value()->isDefault() )
+    if (it.value()->isDefault()) {
       return (ButtonCode)it.key();
+    }
   }
 
-  return NoDefault;
+    return d->mDefaultButton;
 }
 
 void KDialog::setMainWidget( QWidget *widget )
@@ -778,8 +798,9 @@ QString KDialog::buttonWhatsThis( ButtonCode id ) const
 void KDialog::setButtonFocus( ButtonCode id )
 {
   KPushButton *button = this->button( id );
-  if ( button )
-        d_func()->setButtonFocus(button, button->isDefault(), true);
+  if ( button ) {
+      button->setFocus();
+  }
 }
 
 void KDialog::setDetailsWidget( QWidget *detailsWidget )
@@ -891,7 +912,7 @@ void KDialog::slotButtonClicked( int button )
       break;
     case Close:
       emit closeClicked();
-      close();
+      done(Close); // KDE5: call reject() instead; more QDialog-like.
       break;
     case Help:
       emit helpClicked();
@@ -908,6 +929,12 @@ void KDialog::slotButtonClicked( int button )
       setDetailsWidgetVisible( !d->mDetailsVisible );
       break;
   }
+
+    // If we're here from the closeEvent, and auto-delete is on, well, auto-delete now.
+    if (d->mDeferredDelete) {
+        d->mDeferredDelete = false;
+        delayedDestruct();
+    }
 }
 
 void KDialog::enableLinkedHelp( bool state )
@@ -973,11 +1000,18 @@ void KDialog::hideEvent( QHideEvent *event )
 void KDialog::closeEvent( QCloseEvent *event )
 {
     Q_D(KDialog);
-  KPushButton *button = this->button( d->mEscapeButton );
-  if ( button && !isHidden() )
-    button->animateClick();
-  else
-    QDialog::closeEvent( event );
+    KPushButton *button = this->button(d->mEscapeButton);
+    if (button && !isHidden()) {
+        button->animateClick();
+
+        if (testAttribute(Qt::WA_DeleteOnClose)) {
+            // Don't let QWidget::close do a deferred delete just yet, wait for the click first
+            d->mDeferredDelete = true;
+            setAttribute(Qt::WA_DeleteOnClose, false);
+        }
+    } else {
+        QDialog::closeEvent(event);
+    }
 }
 
 void KDialog::restoreDialogSize( const KConfigGroup& cfg )
@@ -1006,6 +1040,11 @@ void KDialog::saveDialogSize( KConfigGroup& config, KConfigGroup::WriteConfigFla
    config.writeEntry( QString::fromLatin1("Height %1").arg( desk.height() ), sizeToSave.height(), options );
 }
 
+void KDialog::setAllowEmbeddingInGraphicsView( bool allowEmbedding )
+{
+  sAllowEmbeddingInGraphicsView = allowEmbedding;
+}
+
 
 class KDialogQueue::Private
 {
