@@ -30,9 +30,12 @@
 #include <QtGui/QToolButton>
 #include <QtGui/QLabel>
 #include <QtGui/QApplication>
+#include <QtGui/QGridLayout>
+#include <QtGui/QCheckBox>
 #include <QMimeData>
 
 #include <kstandarddirs.h>
+#include <klistwidgetsearchline.h>
 #include <klocale.h>
 #include <kicon.h>
 #include <kiconloader.h>
@@ -47,6 +50,7 @@
 #include <ktoolbar.h>
 #include <kdeversion.h>
 #include <kcombobox.h>
+#include <klineedit.h>
 
 #include "kaction.h"
 #include "kactioncollection.h"
@@ -173,7 +177,8 @@ public:
           m_internalTag(tag),
           m_internalName(name),
           m_statusText(statusText),
-          m_isSeparator(false)
+          m_isSeparator(false),
+          m_isTextAlongsideIconHidden(false)
     {
         // Drop between items, not onto items
         setFlags((flags() | Qt::ItemIsDragEnabled) & ~Qt::ItemIsDropEnabled);
@@ -183,10 +188,12 @@ public:
     void setInternalName(const QString &name) { m_internalName = name; }
     void setStatusText(const QString &text) { m_statusText = text; }
     void setSeparator(bool sep) { m_isSeparator = sep; }
+    void setTextAlongsideIconHidden(bool hidden) { m_isTextAlongsideIconHidden = hidden; }
     QString internalTag() const { return m_internalTag; }
     QString internalName() const { return m_internalName; }
     QString statusText() const { return m_statusText; }
     bool isSeparator() const { return m_isSeparator; }
+    bool isTextAlongsideIconHidden() const { return m_isTextAlongsideIconHidden; }
 
     int index() const { return listWidget()->row(const_cast<ToolBarItem*>(this)); }
 
@@ -195,6 +202,7 @@ private:
     QString m_internalName;
     QString m_statusText;
     bool m_isSeparator;
+    bool m_isTextAlongsideIconHidden;
 };
 
 static QDataStream & operator<< ( QDataStream & s, const ToolBarItem & item ) {
@@ -202,6 +210,7 @@ static QDataStream & operator<< ( QDataStream & s, const ToolBarItem & item ) {
     s << item.internalName();
     s << item.statusText();
     s << item.isSeparator();
+    s << item.isTextAlongsideIconHidden();
     return s;
 }
 static QDataStream & operator>> ( QDataStream & s, ToolBarItem & item ) {
@@ -217,6 +226,9 @@ static QDataStream & operator>> ( QDataStream & s, ToolBarItem & item ) {
     bool sep;
     s >> sep;
     item.setSeparator(sep);
+    bool hidden;
+    s >> hidden;
+    item.setTextAlongsideIconHidden(hidden);
     return s;
 }
 
@@ -268,20 +280,78 @@ ToolBarItem* ToolBarListWidget::currentItem() const
     return static_cast<ToolBarItem*>(QListWidget::currentItem());
 }
 
+
+IconTextEditDialog::IconTextEditDialog(QWidget *parent)
+    : KDialog(parent)
+{
+    setCaption(i18n("Change Text"));
+    setButtons(Ok | Cancel);
+    setDefaultButton(Ok);
+    setModal(true);
+
+    QWidget *mainWidget = new QWidget(this);
+    QGridLayout *layout = new QGridLayout(mainWidget);
+    layout->setMargin(0);
+
+    m_lineEdit = new KLineEdit(mainWidget);
+    m_lineEdit->setClearButtonShown(true);
+    QLabel *label = new QLabel(i18n("Icon te&xt:"), this);
+    label->setBuddy(m_lineEdit);
+    layout->addWidget(label, 0, 0);
+    layout->addWidget(m_lineEdit, 0, 1);
+
+    m_cbHidden = new QCheckBox(i18n("&Hide text when toolbar shows text alongside icons"), mainWidget);
+    layout->addWidget(m_cbHidden, 1, 1);
+
+    connect(m_lineEdit, SIGNAL(textChanged(const QString &)), SLOT(slotTextChanged(const QString &)));
+
+    m_lineEdit->setFocus();
+    setMainWidget(mainWidget);
+    setFixedHeight(sizeHint().height());
+}
+
+void IconTextEditDialog::setIconText(const QString &text)
+{
+    m_lineEdit->setText(text);
+}
+
+QString IconTextEditDialog::iconText() const
+{
+    return m_lineEdit->text().trimmed();
+}
+
+void IconTextEditDialog::setTextAlongsideIconHidden(bool hidden)
+{
+    m_cbHidden->setChecked(hidden);
+}
+
+bool IconTextEditDialog::textAlongsideIconHidden() const
+{
+    return m_cbHidden->isChecked();
+}
+
+void IconTextEditDialog::slotTextChanged(const QString &text)
+{
+    // Do not allow empty icon text
+    enableButton(Ok, !text.trimmed().isEmpty());
+}
+
+
 class KEditToolBarWidgetPrivate
 {
 public:
     /**
      *
-     * @param collection In a non-KParts application, this is the collection passed
+     * @param collection In the old-style constructor, this is the collection passed
      * to the KEditToolBar constructor.
-     * In a KParts application we let create a KXMLGUIClient create a dummy one,
+     * In the xmlguifactory-based constructor, we let KXMLGUIClient create a dummy one,
      * but it probably isn't used.
      */
     KEditToolBarWidgetPrivate(KEditToolBarWidget* widget,
                               const KComponentData &cData, KActionCollection* collection)
         : m_collection( collection ),
-          m_widget (widget),
+          m_widget(widget),
+          m_factory(NULL),
           m_loadedOnce( false )
     {
         m_componentData = cData;
@@ -310,8 +380,9 @@ public:
     void slotDownButton();
 
     void selectActiveItem(const QString&);
-    
+
     void slotChangeIcon();
+    void slotChangeIconText();
 
     void slotProcessExited();
 
@@ -320,8 +391,8 @@ public:
 
     void setupLayout();
 
-    void initNonKPart( const QString& file, bool global, const QString& defaultToolbar );
-    void initKPart( KXMLGUIFactory* factory, const QString& defaultToolbar );
+    void initOldStyle( const QString& file, bool global, const QString& defaultToolbar );
+    void initFromFactory( KXMLGUIFactory* factory, const QString& defaultToolbar );
     void loadToolBarCombo( const QString& defaultToolbar );
     void loadActions(const QDomElement& elem);
 
@@ -391,6 +462,7 @@ public:
     //QValueList<KAction*> m_actionList;
     KActionCollection* m_collection;
     KEditToolBarWidget* m_widget;
+    KXMLGUIFactory* m_factory;
     KComponentData m_componentData;
 
     QPixmap m_emptyIcon;
@@ -413,6 +485,7 @@ public:
     KSeparator *m_comboSeparator;
     QLabel * m_helpArea;
     KPushButton* m_changeIcon;
+    KPushButton* m_changeIconText;
     KProcess* m_kdialogProcess;
     bool m_isPart : 1;
     bool m_hasKDialog : 1;
@@ -529,39 +602,29 @@ void KEditToolBarPrivate::_k_slotDefault()
     if ( KMessageBox::warningContinueCancel(q, i18n("Do you really want to reset all toolbars of this application to their default? The changes will be applied immediately."), i18n("Reset Toolbars"),KGuiItem(i18n("Reset")))!=KMessageBox::Continue )
         return;
 
-    delete m_widget;
+    KEditToolBarWidget * oldWidget = m_widget;
     m_widget = 0;
     m_accept = false;
 
     if ( m_factory )
     {
-        const QString localPrefix = KStandardDirs::locateLocal("data", "");
         foreach (KXMLGUIClient* client, m_factory->clients())
         {
-            QString file = client->xmlFile();
-
-            if (file.isNull()) // ##### should be isEmpty?
+            const QString file = client->localXMLFile();
+            if (file.isEmpty())
                 continue;
-
-            if (QDir::isRelativePath(file))
-            {
-                const KComponentData cData = client->componentData().isValid() ? client->componentData() : KGlobal::mainComponent();
-                file = KStandardDirs::locateLocal("data", cData.componentName() + '/' + file);
-            }
-            else
-            {
-                if (!file.startsWith(localPrefix))
-                    continue;
-            }
-
+            kDebug(240) << "Deleting local xml file" << file;
+            // << "for client" << client << typeid(*client).name();
             if ( QFile::exists( file ) )
                 if ( !QFile::remove( file ) )
-                    kWarning() << "Could not delete " << file;
+                    kWarning() << "Could not delete" << file;
         }
 
+        // Reload the xml files in all clients, now that the local files are gone
+        oldWidget->rebuildKXMLGUIClients();
+
         m_widget = new KEditToolBarWidget( q );
         m_widget->load( m_factory, m_defaultToolBar );
-        m_widget->rebuildKXMLGUIClients();
     }
     else
     {
@@ -578,13 +641,16 @@ void KEditToolBarPrivate::_k_slotDefault()
         q->setResourceFile( m_file, m_global );
     }
 
+    // Copy the geometry to minimize UI flicker
+    m_widget->setGeometry( oldWidget->geometry() );
     q->setMainWidget(m_widget);
-    m_widget->show();
+    delete oldWidget;
 
     q->connect(m_widget, SIGNAL(enableOk(bool)), SLOT(_k_acceptOK(bool)));
     q->connect(m_widget, SIGNAL(enableOk(bool)), SLOT(enableButtonApply(bool)));
 
     q->enableButtonApply(false);
+
     emit q->newToolBarConfig();
     emit q->newToolbarConfig(); // compat
 }
@@ -643,15 +709,15 @@ KEditToolBarWidget::~KEditToolBarWidget()
 
 void KEditToolBarWidget::load( const QString& file, bool global, const QString& defaultToolBar )
 {
-    d->initNonKPart( file, global, defaultToolBar );
+    d->initOldStyle( file, global, defaultToolBar );
 }
 
 void KEditToolBarWidget::load( KXMLGUIFactory* factory, const QString& defaultToolBar )
 {
-    d->initKPart( factory, defaultToolBar );
+    d->initFromFactory( factory, defaultToolBar );
 }
 
-void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
+void KEditToolBarWidgetPrivate::initOldStyle( const QString& resourceFile,
                                               bool global,
                                               const QString& defaultToolBar )
 {
@@ -665,7 +731,7 @@ void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
 
     // handle the merging
     if (global)
-        m_widget->setXMLFile(KStandardDirs::locate("config", "ui/ui_standards.rc"));
+        m_widget->loadStandardsXmlFile(); // ui_standards.rc
     const QString localXML = loadXMLFile( resourceFile );
     m_widget->setXML(localXML, global ? true /*merge*/ : false);
 
@@ -691,8 +757,8 @@ void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
     m_widget->setMinimumSize( m_widget->sizeHint() );
 }
 
-void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
-                                           const QString& defaultToolBar )
+void KEditToolBarWidgetPrivate::initFromFactory(KXMLGUIFactory* factory,
+                                                const QString& defaultToolBar)
 {
     //TODO: make sure we can call this multiple times?
     if ( m_loadedOnce ) {
@@ -701,10 +767,7 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
 
     m_loadedOnce = true;
 
-  // reusable vars
-  QDomElement elem;
-
-  m_widget->setFactory( factory );
+    m_factory = factory;
 
   // add all of the client data
   bool first = true;
@@ -717,11 +780,11 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
     if ( first ) {
       type = XmlData::Shell;
       first = false;
+      Q_ASSERT(!client->localXMLFile().isEmpty()); // where would we save changes??
     }
 
     XmlData data(type, client->localXMLFile(), client->actionCollection());
-    QDomDocument domDoc;
-    domDoc.setContent( KXMLGUIFactory::readConfigFile( client->xmlFile(), client->componentData() ) );
+    QDomDocument domDoc = client->domDocument();
     data.setDomDocument(domDoc);
     m_xmlFiles.append(data);
 
@@ -763,8 +826,8 @@ bool KEditToolBarWidget::save()
     KXMLGUIFactory::saveConfigFile((*it).domDocument(), (*it).xmlFile());
   }
 
-  if ( !factory() )
-    return true;
+    if (!d->m_factory)
+        return true;
 
   rebuildKXMLGUIClients();
 
@@ -773,23 +836,23 @@ bool KEditToolBarWidget::save()
 
 void KEditToolBarWidget::rebuildKXMLGUIClients()
 {
-  if ( !factory() )
-    return;
+    if (!d->m_factory)
+        return;
 
-  QList<KXMLGUIClient*> clients = factory()->clients();
-  //kDebug(240) << "factory: " << clients.count() << " clients";
+    const QList<KXMLGUIClient*> clients = d->m_factory->clients();
+    //kDebug(240) << "factory: " << clients.count() << " clients";
 
-  if (!clients.count())
-    return;
+    // remove the elements starting from the last going to the first
+    if (!clients.count())
+        return;
 
-  // remove the elements starting from the last going to the first
-  QListIterator<KXMLGUIClient*> clientIterator = clients;
-  clientIterator.toBack();
-  while ( clientIterator.hasPrevious() )
-  {
-    //kDebug(240) << "factory->removeClient " << client;
-    factory()->removeClient( clientIterator.previous() );
-  }
+    QListIterator<KXMLGUIClient*> clientIterator = clients;
+    clientIterator.toBack();
+    while (clientIterator.hasPrevious()) {
+        KXMLGUIClient* client = clientIterator.previous();
+        //kDebug(240) << "factory->removeClient " << client;
+        d->m_factory->removeClient(client);
+    }
 
   KXMLGUIClient *firstClient = clients.first();
 
@@ -806,18 +869,21 @@ void KEditToolBarWidget::rebuildKXMLGUIClients()
 
         // for the shell, merge in ui_standards.rc
         if ( client == firstClient ) // same assumption as in the ctor: first==shell
-            client->setXMLFile(KStandardDirs::locate("config", "ui/ui_standards.rc"));
+            client->loadStandardsXmlFile();
 
         // and this forces it to use the *new* XML file
         client->setXMLFile( file, client == firstClient /* merge if shell */ );
+
+        // [we can't use reloadXML, it doesn't load ui_standards.rc]
     }
   }
 
-  // Now we can add the clients to the factory
-  // We don't do it in the loop above because adding a part automatically
-  // adds its plugins, so we must make sure the plugins were updated first.
-  foreach (KXMLGUIClient* client, clients)
-    factory()->addClient( client );
+    // Now we can add the clients to the factory
+    // We don't do it in the loop above because adding a part automatically
+    // adds its plugins, so we must make sure the plugins were updated first.
+    foreach(KXMLGUIClient* client, clients) {
+        d->m_factory->addClient(client);
+    }
 }
 
 void KEditToolBarWidgetPrivate::setupLayout()
@@ -852,6 +918,9 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect(m_inactiveList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
                    m_widget,       SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
 
+  KListWidgetSearchLine *inactiveListSearchLine = new KListWidgetSearchLine(m_widget, m_inactiveList);
+  inactiveListSearchLine->setClickMessage(i18n("Filter"));
+
   // our list of active actions
   QLabel *active_label = new QLabel(i18n("Curr&ent actions:"), m_widget);
   m_activeList = new ToolBarListWidget(m_widget);
@@ -868,8 +937,12 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect(m_activeList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
                    m_widget,     SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
 
+  KListWidgetSearchLine *activeListSearchLine = new KListWidgetSearchLine(m_widget, m_activeList);
+  activeListSearchLine->setClickMessage(i18n("Filter"));
+
   // "change icon" button
   m_changeIcon = new KPushButton(i18n( "Change &Icon..." ), m_widget);
+  m_changeIcon->setIcon(KIcon("preferences-desktop-icons"));
   QString kdialogExe = KStandardDirs::findExe(QLatin1String("kdialog"));
   m_hasKDialog = !kdialogExe.isEmpty();
   m_changeIcon->setEnabled(m_hasKDialog && m_activeList->currentItem());
@@ -877,6 +950,14 @@ void KEditToolBarWidgetPrivate::setupLayout()
   QObject::connect( m_changeIcon, SIGNAL( clicked() ),
                     m_widget, SLOT( slotChangeIcon() ) );
 
+  // "change icon text" button
+  m_changeIconText = new KPushButton(i18n( "Change Te&xt..." ), m_widget);
+  m_changeIconText->setIcon(KIcon("edit-rename"));
+  m_changeIconText->setEnabled(m_activeList->currentItem() != 0);
+
+  QObject::connect( m_changeIconText, SIGNAL( clicked() ),
+                    m_widget, SLOT( slotChangeIconText() ) );
+
   // The buttons in the middle
 
   m_upAction     = new QToolButton(m_widget);
@@ -931,15 +1012,17 @@ void KEditToolBarWidgetPrivate::setupLayout()
   button_layout->setRowStretch( 4, 10 );
 
   inactive_layout->addWidget(inactive_label);
+  inactive_layout->addWidget(inactiveListSearchLine);
   inactive_layout->addWidget(m_inactiveList, 1);
 
   active_layout->addWidget(active_label);
+  active_layout->addWidget(activeListSearchLine);
   active_layout->addWidget(m_activeList, 1);
   active_layout->addLayout(changeIcon_layout);
 
-  changeIcon_layout->addStretch( 1 );
   changeIcon_layout->addWidget(m_changeIcon);
   changeIcon_layout->addStretch( 1 );
+  changeIcon_layout->addWidget(m_changeIconText);
 
   list_layout->addLayout(inactive_layout);
   list_layout->addLayout(button_layout);
@@ -1065,8 +1148,9 @@ void KEditToolBarWidgetPrivate::loadActions(const QDomElement& elem)
       {
         // we have a match!
         ToolBarItem *act = new ToolBarItem(m_activeList, it.tagName(), action->objectName(), action->toolTip());
-        act->setText(nameFilter.subs(KGlobal::locale()->removeAcceleratorMarker(action->text())).toString());
+        act->setText(nameFilter.subs(KGlobal::locale()->removeAcceleratorMarker(action->iconText())).toString());
         act->setIcon(!action->icon().isNull() ? action->icon() : m_emptyIcon);
+        act->setTextAlongsideIconHidden(action->priority() < QAction::NormalPriority);
 
         active_list.insert(action->objectName());
         break;
@@ -1172,6 +1256,9 @@ void KEditToolBarWidgetPrivate::slotActiveSelectionChanged()
                             m_hasKDialog &&
                             toolitem->internalTag() == "Action" );
 
+  m_changeIconText->setEnabled( toolitem &&
+                                toolitem->internalTag() == "Action" );
+
   if (toolitem)
   {
     m_upAction->setEnabled(toolitem->index() != 0);
@@ -1191,19 +1278,17 @@ void KEditToolBarWidgetPrivate::slotActiveSelectionChanged()
 void KEditToolBarWidgetPrivate::slotInsertButton()
 {
   QString internalName = static_cast<ToolBarItem *>(m_inactiveList->currentItem())->internalName();
-  
+
   insertActive(m_inactiveList->currentItem(), m_activeList->currentItem(), false);
   // we're modified, so let this change
   emit m_widget->enableOk(true);
 
-  // TODO: #### this causes #97572.
-  // It would be better to just "delete item; loadActions( ... , ActiveListOnly );" or something.
   slotToolBarSelected( m_toolbarCombo->currentIndex() );
-  
+
   selectActiveItem( internalName );
 }
 
-void KEditToolBarWidgetPrivate::selectActiveItem(const QString& internalName) 
+void KEditToolBarWidgetPrivate::selectActiveItem(const QString& internalName)
 {
   int activeItemCount = m_activeList->count();
   for(int i = 0; i < activeItemCount; i++)
@@ -1435,7 +1520,7 @@ void KEditToolBarWidgetPrivate::slotChangeIcon()
   (*m_kdialogProcess) << "--caption";
   (*m_kdialogProcess) << i18n( "Change Icon" );
   (*m_kdialogProcess) << "--embed";
-  (*m_kdialogProcess) << QString::number( (ulong)m_widget->window()->winId() );
+  (*m_kdialogProcess) << QString::number( (quintptr)m_widget->window()->winId() );
   (*m_kdialogProcess) << "--geticon";
   (*m_kdialogProcess) << "Toolbar";
   (*m_kdialogProcess) << "Actions";
@@ -1456,6 +1541,51 @@ void KEditToolBarWidgetPrivate::slotChangeIcon()
                     m_widget, SLOT( slotProcessExited() ) );
 }
 
+void KEditToolBarWidgetPrivate::slotChangeIconText()
+{
+  m_currentXmlData->dump();
+  ToolBarItem *item = m_activeList->currentItem();
+
+  if(item){
+    QString iconText = item->text();
+    bool hidden = item->isTextAlongsideIconHidden();
+
+    IconTextEditDialog dialog(m_widget);
+    dialog.setIconText(iconText);
+    dialog.setTextAlongsideIconHidden(hidden);
+
+    bool ok = dialog.exec() == KDialog::Accepted;
+    iconText = dialog.iconText();
+    hidden = dialog.textAlongsideIconHidden();
+
+    bool hiddenChanged = hidden != item->isTextAlongsideIconHidden();
+    bool iconTextChanged = iconText != item->text();
+
+    if (!ok || (!hiddenChanged && !iconTextChanged))
+      return;
+
+    item->setText(iconText);
+    item->setTextAlongsideIconHidden(hidden);
+
+    Q_ASSERT( m_currentXmlData->type() != XmlData::Merged );
+
+    m_currentXmlData->m_isModified = true;
+
+    // Get hold of ActionProperties tag
+    QDomElement elem = KXMLGUIFactory::actionPropertiesElement( m_currentXmlData->domDocument() );
+    // Find or create an element for this action
+    QDomElement act_elem = KXMLGUIFactory::findActionByName( elem, item->internalName(), true /*create*/ );
+    Q_ASSERT( !act_elem.isNull() );
+    if (iconTextChanged)
+        act_elem.setAttribute( "iconText", iconText );
+    if (hiddenChanged)
+        act_elem.setAttribute( "priority", hidden ? QAction::LowPriority : QAction::NormalPriority );
+
+    // we're modified, so let this change
+    emit m_widget->enableOk(true);
+  }
+}
+
 void KEditToolBarWidgetPrivate::slotProcessExited()
 {
   m_activeList->setEnabled( true );
@@ -1535,7 +1665,7 @@ void KEditToolBar::showEvent( QShowEvent * event )
     if (!event->spontaneous()) {
         // The dialog has been shown, enable toolbar editing
         if ( d->m_factory ) {
-            // call the kpart version
+            // call the xmlgui-factory version
             d->m_widget->load( d->m_factory, d->m_defaultToolBar );
         } else {
             // call the action collection version
