@@ -364,7 +364,7 @@ int KCategorizedView::Private::highestElementInLastRow(const Block &block) const
 {
     //Find the highest element in the last row
     const QModelIndex lastIndex = proxyModel->index(block.firstIndex.row() + block.items.count() - 1, q->modelColumn(), q->rootIndex());
-    QRect prevRect = q->visualRect(lastIndex);
+    const QRect prevRect = q->visualRect(lastIndex);
     int res = prevRect.height();
     QModelIndex prevIndex = proxyModel->index(lastIndex.row() - 1, q->modelColumn(), q->rootIndex());
     if (!prevIndex.isValid()) {
@@ -1365,12 +1365,11 @@ void KCategorizedView::updateGeometries()
         verticalScrollBar()->setSingleStep(lastItemRect.height());
         const int rowsPerPage = qMax(viewport()->height() / lastItemRect.height(), 1);
         verticalScrollBar()->setPageStep(rowsPerPage * lastItemRect.height());
-        verticalScrollBar()->setRange(0, bottomRange);
-    } else {
-        verticalScrollBar()->setRange(0, bottomRange);
-        verticalScrollBar()->setValue(oldVerticalOffset);
     }
 
+    verticalScrollBar()->setRange(0, bottomRange);
+    verticalScrollBar()->setValue(oldVerticalOffset);
+
     //TODO: also consider working with the horizontal scroll bar. since at this level I am not still
     //      supporting "top to bottom" flow, there is no real problem. If I support that someday
     //      (think how to draw categories), we would have to take care of the horizontal scroll bar too.
@@ -1430,6 +1429,7 @@ void KCategorizedView::rowsInserted(const QModelIndex &parent,
     d->rowsInserted(parent, start, end);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KCategorizedView::rowsInsertedArtifficial(const QModelIndex &parent,
                                                int start,
                                                int end)
@@ -1438,7 +1438,9 @@ void KCategorizedView::rowsInsertedArtifficial(const QModelIndex &parent,
     Q_UNUSED(start);
     Q_UNUSED(end);
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KCategorizedView::rowsRemoved(const QModelIndex &parent,
                                    int start,
                                    int end)
@@ -1447,6 +1449,7 @@ void KCategorizedView::rowsRemoved(const QModelIndex &parent,
     Q_UNUSED(start);
     Q_UNUSED(end);
 }
+#endif
 
 void KCategorizedView::slotLayoutChanged()
 {
