@@ -92,7 +92,7 @@ public:
   void rowsInserted( const QModelIndex &index, int start, int end )
   {
     Q_Q(KViewStateSaver);
-    QTimer::singleShot(0, q, SLOT(processPendingChanges()));
+    processPendingChanges();
 
     if ( !hasPendingChanges() )
     {
@@ -115,8 +115,9 @@ public:
 };
 
 KViewStateSaver::KViewStateSaver(QObject* parent)
-  : QObject(parent), d_ptr( new KViewStateSaverPrivate(this) )
+  : QObject(0), d_ptr( new KViewStateSaverPrivate(this) )
 {
+  Q_UNUSED(parent);
   qRegisterMetaType<QModelIndex>( "QModelIndex" );
 }
 
@@ -193,8 +194,8 @@ void KViewStateSaver::restoreState(const KConfigGroup& configGroup)
 {
   Q_D(KViewStateSaver);
 
-  // Delete myself if not finished after two seconds.
-  QTimer::singleShot(2000, this, SLOT(deleteLater()));
+  // Delete myself if not finished after ten seconds.
+  QTimer::singleShot(10000, this, SLOT(deleteLater()));
 
 
   d->m_pendingCurrent = configGroup.readEntry( currentKey, QString() );
@@ -203,8 +204,9 @@ void KViewStateSaver::restoreState(const KConfigGroup& configGroup)
   d->m_horizontalScrollBarValue = configGroup.readEntry( scrollStateHorizontalKey, -1 );
   d->m_verticalScrollBarValue = configGroup.readEntry( scrollStateVerticalKey, -1 );
 
-  QTimer::singleShot(0, this, SLOT(processPendingChanges()));
-  d->listenToPendingChanges();
+  d->processPendingChanges();
+  if (d->hasPendingChanges())
+    d->listenToPendingChanges();
 }
 
 QStringList KViewStateSaverPrivate::getExpandedItems(const QModelIndex &index) const
