@@ -30,7 +30,6 @@
 #include <kprotocolinfo.h>
 #include <kcomponentdata.h>
 #include <kcolorscheme.h>
-#include <kapplication.h>
 
 #include <kstyle.h>
 
@@ -78,9 +77,11 @@ static QRgb qt_colorref2qrgb(COLORREF col)
 //static QColor *_buttonBackground = 0;
 static KGlobalSettings::GraphicEffects _graphicEffects = KGlobalSettings::NoEffects;
 
-// KDE5: merge this with KGlobalSettings::Private
-// also think to make all methods static and not expose an object,
+// TODO: merge this with KGlobalSettings::Private
+//
+// F. Kossebau: KDE5: think to make all methods static and not expose an object,
 // making KGlobalSettings rather a namespace
+// D. Faure: how would people connect to signals, then?
 class KGlobalSettingsData
 {
   public:
@@ -148,10 +149,11 @@ class KGlobalSettings::Private
 {
     public:
         Private(KGlobalSettings *q)
-            : q(q), activated(false)
+            : q(q), activated(false), paletteCreated(false)
         {
         }
 
+        QPalette createApplicationPalette(const KSharedConfigPtr &config);
         void _k_slotNotifyChange(int, int);
 
         void propagateQtSettings();
@@ -180,6 +182,8 @@ class KGlobalSettings::Private
 
         KGlobalSettings *q;
         bool activated;
+        bool paletteCreated;
+        QPalette applicationPalette;
 };
 
 KGlobalSettings* KGlobalSettings::self()
@@ -191,8 +195,6 @@ KGlobalSettings* KGlobalSettings::self()
 KGlobalSettings::KGlobalSettings()
     : QObject(0), d(new Private(this))
 {
-    QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
-                                           "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
 }
 
 KGlobalSettings::~KGlobalSettings()
@@ -201,13 +203,25 @@ KGlobalSettings::~KGlobalSettings()
 }
 
 void KGlobalSettings::activate()
+{
+    activate(ApplySettings | ListenForChanges);
+}
+
+void KGlobalSettings::activate(ActivateOptions options)
 {
     if (!d->activated) {
         d->activated = true;
 
-        d->kdisplaySetStyle(); // implies palette setup
-        d->kdisplaySetFont();
-        d->propagateQtSettings();
+        if (options & ListenForChanges) {
+            QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
+                                                   "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
+        }
+
+        if (options & ApplySettings) {
+            d->kdisplaySetStyle(); // implies palette setup
+            d->kdisplaySetFont();
+            d->propagateQtSettings();
+        }
     }
 }
 
@@ -270,6 +284,7 @@ bool KGlobalSettings::showContextMenusOnPress ()
     return g.readEntry("ShowOnPress", true);
 }
 
+#ifndef KDE_NO_DEPRECATED
 int KGlobalSettings::contextMenuKey ()
 {
     KConfigGroup g(KGlobal::config(), "Shortcuts");
@@ -297,6 +312,7 @@ int KGlobalSettings::contextMenuKey ()
 
     return QKeySequence::fromString(s)[0];
 }
+#endif
 
 // NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::inactiveTitleColor()
@@ -578,9 +594,14 @@ KGlobalSettings::KMouseSettings& KGlobalSettingsData::mouseSettings()
     }
 #ifdef Q_WS_WIN
     //not cached
+#ifndef _WIN32_WCE
     mMouseSettings->handed = (GetSystemMetrics(SM_SWAPBUTTON) ?
         KGlobalSettings::KMouseSettings::LeftHanded :
         KGlobalSettings::KMouseSettings::RightHanded);
+#else
+// There is no mice under wince
+    mMouseSettings->handed =KGlobalSettings::KMouseSettings::RightHanded;
+#endif
 #endif
     return *mMouseSettings;
 }
@@ -830,6 +851,7 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
     case PaletteChanged:
         if (activated) {
             KGlobal::config()->reparseConfiguration();
+            paletteCreated = false;
             kdisplaySetPalette();
         }
         break;
@@ -937,44 +959,56 @@ void KGlobalSettings::Private::applyGUIStyle()
 
 QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &config)
 {
-    QPalette palette;
+    return self()->d->createApplicationPalette(config);
+}
 
-    QPalette::ColorGroup states[3] = { QPalette::Active, QPalette::Inactive,
-                                       QPalette::Disabled };
+QPalette KGlobalSettings::Private::createApplicationPalette(const KSharedConfigPtr &config)
+{
+    // This method is typically called once by KQGuiPlatformPlugin::palette and once again
+    // by kdisplaySetPalette(), so we cache the palette to save time.
+    if (!paletteCreated) {
+        paletteCreated = true;
 
-    // TT thinks tooltips shouldn't use active, so we use our active colors for all states
-    KColorScheme schemeTooltip(QPalette::Active, KColorScheme::Tooltip, config);
+        QPalette palette;
 
-    for ( int i = 0; i < 3 ; i++ ) {
-        QPalette::ColorGroup state = states[i];
-        KColorScheme schemeView(state, KColorScheme::View, config);
-        KColorScheme schemeWindow(state, KColorScheme::Window, config);
-        KColorScheme schemeButton(state, KColorScheme::Button, config);
-        KColorScheme schemeSelection(state, KColorScheme::Selection, config);
+        QPalette::ColorGroup states[3] = { QPalette::Active, QPalette::Inactive,
+                                           QPalette::Disabled };
 
-        palette.setBrush( state, QPalette::WindowText, schemeWindow.foreground() );
-        palette.setBrush( state, QPalette::Window, schemeWindow.background() );
-        palette.setBrush( state, QPalette::Base, schemeView.background() );
-        palette.setBrush( state, QPalette::Text, schemeView.foreground() );
-        palette.setBrush( state, QPalette::Button, schemeButton.background() );
-        palette.setBrush( state, QPalette::ButtonText, schemeButton.foreground() );
-        palette.setBrush( state, QPalette::Highlight, schemeSelection.background() );
-        palette.setBrush( state, QPalette::HighlightedText, schemeSelection.foreground() );
-        palette.setBrush( state, QPalette::ToolTipBase, schemeTooltip.background() );
-        palette.setBrush( state, QPalette::ToolTipText, schemeTooltip.foreground() );
+        // TT thinks tooltips shouldn't use active, so we use our active colors for all states
+        KColorScheme schemeTooltip(QPalette::Active, KColorScheme::Tooltip, config);
 
-        palette.setColor( state, QPalette::Light, schemeWindow.shade( KColorScheme::LightShade ) );
-        palette.setColor( state, QPalette::Midlight, schemeWindow.shade( KColorScheme::MidlightShade ) );
-        palette.setColor( state, QPalette::Mid, schemeWindow.shade( KColorScheme::MidShade ) );
-        palette.setColor( state, QPalette::Dark, schemeWindow.shade( KColorScheme::DarkShade ) );
-        palette.setColor( state, QPalette::Shadow, schemeWindow.shade( KColorScheme::ShadowShade ) );
+        for ( int i = 0; i < 3 ; i++ ) {
+            QPalette::ColorGroup state = states[i];
+            KColorScheme schemeView(state, KColorScheme::View, config);
+            KColorScheme schemeWindow(state, KColorScheme::Window, config);
+            KColorScheme schemeButton(state, KColorScheme::Button, config);
+            KColorScheme schemeSelection(state, KColorScheme::Selection, config);
 
-        palette.setBrush( state, QPalette::AlternateBase, schemeView.background( KColorScheme::AlternateBackground) );
-        palette.setBrush( state, QPalette::Link, schemeView.foreground( KColorScheme::LinkText ) );
-        palette.setBrush( state, QPalette::LinkVisited, schemeView.foreground( KColorScheme::VisitedText ) );
+            palette.setBrush( state, QPalette::WindowText, schemeWindow.foreground() );
+            palette.setBrush( state, QPalette::Window, schemeWindow.background() );
+            palette.setBrush( state, QPalette::Base, schemeView.background() );
+            palette.setBrush( state, QPalette::Text, schemeView.foreground() );
+            palette.setBrush( state, QPalette::Button, schemeButton.background() );
+            palette.setBrush( state, QPalette::ButtonText, schemeButton.foreground() );
+            palette.setBrush( state, QPalette::Highlight, schemeSelection.background() );
+            palette.setBrush( state, QPalette::HighlightedText, schemeSelection.foreground() );
+            palette.setBrush( state, QPalette::ToolTipBase, schemeTooltip.background() );
+            palette.setBrush( state, QPalette::ToolTipText, schemeTooltip.foreground() );
+
+            palette.setColor( state, QPalette::Light, schemeWindow.shade( KColorScheme::LightShade ) );
+            palette.setColor( state, QPalette::Midlight, schemeWindow.shade( KColorScheme::MidlightShade ) );
+            palette.setColor( state, QPalette::Mid, schemeWindow.shade( KColorScheme::MidShade ) );
+            palette.setColor( state, QPalette::Dark, schemeWindow.shade( KColorScheme::DarkShade ) );
+            palette.setColor( state, QPalette::Shadow, schemeWindow.shade( KColorScheme::ShadowShade ) );
+
+            palette.setBrush( state, QPalette::AlternateBase, schemeView.background( KColorScheme::AlternateBackground) );
+            palette.setBrush( state, QPalette::Link, schemeView.foreground( KColorScheme::LinkText ) );
+            palette.setBrush( state, QPalette::LinkVisited, schemeView.foreground( KColorScheme::VisitedText ) );
+        }
+        applicationPalette = palette;
     }
 
-    return palette;
+    return applicationPalette;
 }
 
 void KGlobalSettings::Private::kdisplaySetPalette()
@@ -1084,6 +1118,8 @@ void KGlobalSettings::Private::propagateQtSettings()
     QApplication::setStartDragDistance(num);
     num = cg.readEntry("WheelScrollLines", QApplication::wheelScrollLines());
     QApplication::setWheelScrollLines(num);
+    bool showIcons = cg.readEntry("ShowIconsInMenuItems", !QApplication::testAttribute(Qt::AA_DontShowIconsInMenus));
+    QApplication::setAttribute(Qt::AA_DontShowIconsInMenus, !showIcons);
 
     // KDE5: this seems fairly pointless
     emit q->settingsChanged(SETTINGS_QT);
