@@ -65,7 +65,7 @@ protected:
     virtual QString iconNameForAction(QAction *action)
     {
         KIcon icon(action->icon());
-#if QT_VERSION >= 0x040700
+#if QT_VERSION >= 0x040701
         // QIcon::name() is in the 4.7 git branch, but it is not in 4.7 TP.
         // If you get a build error here, you need to update your pre-release
         // of Qt 4.7.
@@ -701,7 +701,9 @@ void KStatusNotifierItemPrivate::registerToDaemon()
     if (!statusNotifierWatcher) {
         statusNotifierWatcher = new org::kde::StatusNotifierWatcher(s_statusNotifierWatcherServiceName, "/StatusNotifierWatcher",
                                                                     QDBusConnection::sessionBus());
-        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierItemUnregistered(const QString&)),
+        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostRegistered()),
+                         q, SLOT(checkForRegisteredHosts()));
+        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostUnregistered()),
                          q, SLOT(checkForRegisteredHosts()));
     }
 
@@ -723,6 +725,8 @@ void KStatusNotifierItemPrivate::serviceChange(const QString &name, const QStrin
         //unregistered
         kDebug(299) << "Connection to the KStatusNotifierWatcher lost";
         setLegacyMode(true);
+        delete statusNotifierWatcher;
+        statusNotifierWatcher = 0;
     } else if (oldOwner.isEmpty()) {
         //registered
        setLegacyMode(false);
@@ -770,9 +774,6 @@ void KStatusNotifierItemPrivate::setLegacySystemTrayEnabled(bool enabled)
     }
 
     if (enabled) {
-        delete statusNotifierWatcher;
-        statusNotifierWatcher = 0;
-
         if (!systemTrayIcon) {
             systemTrayIcon = new KStatusNotifierLegacyIcon(associatedWidget);
             syncLegacySystemTrayIcon();
@@ -945,8 +946,15 @@ KDbusImageVector KStatusNotifierItemPrivate::iconToVector(const QIcon &icon)
 
     QPixmap iconPixmap;
 
+    //availableSizes() won't work on KIcon
+    QList<QSize> allSizes;
+    allSizes << QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall)
+             << QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium)
+             << QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium)
+             << QSize(KIconLoader::SizeLarge, KIconLoader::SizeLarge);
+
     //if an icon exactly that size wasn't found don't add it to the vector
-    foreach (const QSize &size, icon.availableSizes()) {
+    foreach (const QSize &size, allSizes) {
         //hopefully huge and enormous not necessary right now, since it's quite costly
         if (size.width() <= KIconLoader::SizeLarge) {
             iconPixmap = icon.pixmap(size);
