@@ -40,11 +40,14 @@
 #include "kaction_p.h"
 #include "kmessagebox.h"
 #include "kshortcut.h"
-#include "kglobalaccel_component_interface.h"
-#include "kglobalaccel_interface.h"
 
-static org::kde::kglobalaccel::Component *getComponent(const QString &componentUnique)
-    {
+org::kde::kglobalaccel::Component *KGlobalAccelPrivate::getComponent(const QString &componentUnique, bool remember = false)
+{
+    // Check if we already have this component
+    if (components.contains(componentUnique)) {
+        return components[componentUnique];
+    }
+
     // Connect to the kglobalaccel daemon
     org::kde::KGlobalAccel kglobalaccel(
             "org.kde.kglobalaccel",
@@ -74,7 +77,8 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
     org::kde::kglobalaccel::Component *component = new org::kde::kglobalaccel::Component(
         "org.kde.kglobalaccel",
         reply.value().path(),
-        QDBusConnection::sessionBus());
+        QDBusConnection::sessionBus(),
+        q);
 
     // No component no cleaning
     if (!component->isValid()) {
@@ -82,6 +86,15 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
         return NULL;
     }
 
+    if (remember)
+        {
+        // Connect to the signals we are interested in.
+        q->connect(component, SIGNAL(globalShortcutPressed(const QString &, const QString &, qlonglong)),
+                SLOT(_k_invokeAction(const QString &, const QString &, qlonglong)));
+
+        components[componentUnique] = component;
+        }
+
     return component;
 }
 
@@ -89,8 +102,11 @@ static org::kde::kglobalaccel::Component *getComponent(const QString &componentU
 
 KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
      :  isUsingForeignComponentName(false),
+#ifndef KDE_NO_DEPRECATED
         enabled(true),
-        iface("org.kde.kglobalaccel", "/kglobalaccel", QDBusConnection::sessionBus())
+#endif
+        iface("org.kde.kglobalaccel", "/kglobalaccel", QDBusConnection::sessionBus()),
+        q(q)
 {
     // Make sure kded is running. The iface declaration above somehow
     // works anyway.
@@ -106,8 +122,12 @@ KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
             kError() << "Couldn't start kglobalaccel from kglobalaccel.desktop: " << error << endl;
         }
     }
-    QObject::connect(bus, SIGNAL(serviceOwnerChanged(QString, QString, QString)),
-                     q, SLOT(_k_serviceOwnerChanged(QString, QString, QString)));
+    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(iface.service(),
+                                                           QDBusConnection::sessionBus(),
+                                                           QDBusServiceWatcher::WatchForOwnerChange,
+                                                           q);
+    q->connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+                     q, SLOT(_k_serviceOwnerChanged(QString,QString,QString)));
 }
 
 
@@ -130,8 +150,6 @@ KGlobalAccel::KGlobalAccel()
     qDBusRegisterMetaType<KGlobalShortcutInfo>();
     qDBusRegisterMetaType<QList<KGlobalShortcutInfo> >();
 
-    connect(&d->iface, SIGNAL(invokeAction(const QStringList &, qlonglong)),
-            SLOT(_k_invokeAction(const QStringList &, qlonglong)));
     connect(&d->iface, SIGNAL(yourShortcutGotChanged(const QStringList &, const QList<int> &)),
             SLOT(_k_shortcutGotChanged(const QStringList &, const QList<int> &)));
 
@@ -162,8 +180,8 @@ void KGlobalAccel::activateGlobalShortcutContext(
 // static
 bool KGlobalAccel::cleanComponent(const QString &componentUnique)
 {
-    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
-    if (!component.get()) return false;
+    org::kde::kglobalaccel::Component* component = self()->getComponent(componentUnique);
+    if (!component) return false;
 
     return component->cleanUp();
 }
@@ -172,30 +190,42 @@ bool KGlobalAccel::cleanComponent(const QString &componentUnique)
 // static
 bool KGlobalAccel::isComponentActive(const QString &componentUnique)
 {
-    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
-    if (!component.get()) return false;
+    org::kde::kglobalaccel::Component* component = self()->getComponent(componentUnique);
+    if (!component) return false;
 
     return component->isActive();
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 bool KGlobalAccel::isEnabled() const
 {
     return d->enabled;
 }
+#endif
 
 
+org::kde::kglobalaccel::Component *KGlobalAccel::getComponent(const QString &componentUnique)
+{
+    return d->getComponent(componentUnique);
+}
+
+
+#ifndef KDE_NO_DEPRECATED
 void KGlobalAccel::setEnabled(bool enabled)
 {
     d->enabled = enabled;
 }
+#endif
 
 
+#ifndef KDE_NO_DEPRECATED
 void KGlobalAccel::overrideMainComponentData(const KComponentData &kcd)
 {
     d->readComponentData(kcd);
     d->isUsingForeignComponentName = true;
 }
+#endif
 
 
 KGlobalAccel *KGlobalAccel::self()
@@ -253,7 +283,7 @@ void KGlobalAccelPrivate::remove(KAction *action, Removal removal)
         // internal registry. That happened above.
         if (!action->property("isConfigurationAction").toBool()) {
             // If it's a session shortcut unregister it.
-            action->objectName().startsWith("_k_session:")
+            action->objectName().startsWith(QLatin1String("_k_session:"))
                 ? iface.unRegister(actionId)
                 : iface.setInactive(actionId);
         }
@@ -293,6 +323,10 @@ void KGlobalAccelPrivate::updateGlobalShortcut(KAction *action, uint flags)
                 actionId,
                 intListFromShortcut(activeShortcut),
                 activeSetterFlags);
+
+        // Make sure we get informed about changes in the component by kglobalaccel
+        getComponent(componentUniqueForAction(action), true);
+
         // Create a shortcut from the result
         const KShortcut scResult(shortcutFromIntList(result));
 
@@ -330,7 +364,8 @@ QStringList KGlobalAccelPrivate::makeActionId(const KAction *action)
     Q_ASSERT(!action->objectName().isEmpty());
     ret.append(action->objectName());                   // Action Unique Name
     ret.append(componentFriendlyForAction(action));     // Component Friendly name
-    ret.append(action->text());                         // Action Friendly Name
+    const QString actionText = KGlobal::locale()->removeAcceleratorMarker(action->text());
+    ret.append(actionText);                             // Action Friendly Name
     return ret;
 }
 
@@ -371,7 +406,10 @@ QString KGlobalAccelPrivate::componentFriendlyForAction(const KAction *action)
 }
 
 
-void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong timestamp)
+void KGlobalAccelPrivate::_k_invokeAction(
+        const QString &componentUnique,
+        const QString &actionUnique,
+        qlonglong timestamp)
 {
     // If overrideMainComponentData() is active the app can only have
     // configuration actions.
@@ -380,9 +418,9 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     }
 
     KAction *action = 0;
-    QList<KAction *> candidates = nameToAction.values(actionId.at(KGlobalAccel::ActionUnique));
+    QList<KAction *> candidates = nameToAction.values(actionUnique);
     foreach (KAction *const a, candidates) {
-        if (componentUniqueForAction(a) == actionId.at(KGlobalAccel::ComponentUnique)) {
+        if (componentUniqueForAction(a) == componentUnique) {
             action = a;
         }
     }
@@ -391,8 +429,7 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     // - there is no action
     // - the action is not enabled
     // - the action is an configuration action
-    if (!action || !action->isEnabled()
-            || action->property("isConfigurationAction").toBool()) {
+    if (!action || !action->isEnabled() || action->property("isConfigurationAction").toBool()) {
         return;
     }
 
@@ -412,6 +449,7 @@ void KGlobalAccelPrivate::_k_invokeAction(const QStringList &actionId, qlonglong
     action->trigger();
 }
 
+
 void KGlobalAccelPrivate::_k_shortcutGotChanged(const QStringList &actionId,
                                                 const QList<int> &keys)
 {
@@ -427,7 +465,8 @@ void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString &name, const QStr
 {
     Q_UNUSED(oldOwner);
     if (name == QLatin1String("org.kde.kglobalaccel") && !newOwner.isEmpty()) {
-        // kded was restarted (what? you mean it crashes sometimes?)
+        // kglobalaccel was restarted
+        kDebug(123) << "detected kglobalaccel restarting, re-registering all shortcut keys";
         reRegisterAll();
     }
 }
@@ -453,23 +492,29 @@ void KGlobalAccelPrivate::reRegisterAll()
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QList<QStringList> KGlobalAccel::allMainComponents()
 {
     return d->iface.allMainComponents();
 }
+#endif
 
 
+#ifndef KDE_NO_DEPRECATED
 QList<QStringList> KGlobalAccel::allActionsForComponent(const QStringList &actionId)
 {
     return d->iface.allActionsForComponent(actionId);
 }
+#endif
 
 
 //static
+#ifndef KDE_NO_DEPRECATED
 QStringList KGlobalAccel::findActionNameSystemwide(const QKeySequence &seq)
 {
     return self()->d->iface.action(seq[0]);
 }
+#endif
 
 
 QList<KGlobalShortcutInfo> KGlobalAccel::getGlobalShortcutsByKey(const QKeySequence &seq)
@@ -485,6 +530,7 @@ bool KGlobalAccel::isGlobalShortcutAvailable(const QKeySequence &seq, const QStr
 
 
 //static
+#ifndef KDE_NO_DEPRECATED
 bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringList &actionIdentifier,
                                                  const QKeySequence &seq)
 {
@@ -501,6 +547,7 @@ bool KGlobalAccel::promptStealShortcutSystemwide(QWidget *parent, const QStringL
     return KMessageBox::warningContinueCancel(parent, message, title, KGuiItem(i18n("Reassign")))
            == KMessageBox::Continue;
 }
+#endif
 
 
 //static
@@ -545,7 +592,7 @@ bool KGlobalAccel::promptStealShortcutSystemwide(
 //static
 void KGlobalAccel::stealShortcutSystemwide(const QKeySequence &seq)
 {
-    //get the shortcut, remove seq, and set the new shorctut
+    //get the shortcut, remove seq, and set the new shortcut
     const QStringList actionId = self()->d->iface.action(seq[0]);
     if (actionId.size() < 4) // not a global shortcut
         return;
