@@ -49,13 +49,15 @@ Q_DECLARE_METATYPE(StringByteArrayMap)
 
 static QString appid()
 {
-    KComponentData cData = KGlobal::mainComponent();
-    if (cData.isValid()) {
-        const KAboutData* aboutData = cData.aboutData();
-        if (aboutData) {
-            return aboutData->programName();
+    if (KGlobal::hasMainComponent()) {
+        KComponentData cData = KGlobal::mainComponent();
+        if (cData.isValid()) {
+            const KAboutData* aboutData = cData.aboutData();
+            if (aboutData) {
+                return aboutData->programName();
+            }
+            return cData.componentName();
         }
-        return cData.componentName();
     }
     return qApp->applicationName();
 }
@@ -109,9 +111,13 @@ const QString Wallet::FormDataFolder() {
 class Wallet::WalletPrivate
 {
 public:
-    WalletPrivate(int h, const QString &n)
-     : name(n), handle(h)
+    WalletPrivate(Wallet *wallet, int h, const QString &n)
+     : q(wallet), name(n), handle(h)
     {}
+
+    void walletServiceUnregistered();
+
+    Wallet *q;
     QString name;
     QString folder;
     int handle;
@@ -132,13 +138,15 @@ private:
 
 K_GLOBAL_STATIC(KWalletDLauncher, walletLauncher)
 
-Wallet::Wallet(int handle, const QString& name)
-    : QObject(0L), d(new WalletPrivate(handle, name)) {
+static const char s_kwalletdServiceName[] = "org.kde.kwalletd";
 
-    connect(QDBusConnection::sessionBus().interface(),
-            SIGNAL(serviceOwnerChanged(QString,QString,QString)),
-            this,
-            SLOT(slotServiceOwnerChanged(QString,QString,QString)));
+Wallet::Wallet(int handle, const QString& name)
+    : QObject(0L), d(new WalletPrivate(this, handle, name))
+{
+    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(QString::fromLatin1(s_kwalletdServiceName), QDBusConnection::sessionBus(),
+                                                           QDBusServiceWatcher::WatchForUnregistration, this);
+    connect(watcher, SIGNAL(serviceUnregistered(QString)),
+            this, SLOT(walletServiceUnregistered()));
 
     connect(&walletLauncher->getInterface(), SIGNAL(walletClosed(int)), SLOT(slotWalletClosed(int)));
     connect(&walletLauncher->getInterface(), SIGNAL(folderListUpdated(QString)), SLOT(slotFolderListUpdated(QString)));
@@ -173,7 +181,15 @@ Wallet::~Wallet() {
 
 
 QStringList Wallet::walletList() {
-    return walletLauncher->getInterface().wallets();
+    QDBusReply<QStringList> r = walletLauncher->getInterface().wallets();
+
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return QStringList();
+    }
+    else
+        return r;
 }
 
 
@@ -190,26 +206,53 @@ void Wallet::changePassword(const QString& name, WId w) {
 
 bool Wallet::isEnabled() {
     QDBusReply<bool> r = walletLauncher->getInterface().isEnabled();
-    return (r.isValid() && r);
+
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return false;
+    }
+    else
+        return r;
 }
 
 
 bool Wallet::isOpen(const QString& name) {
-    return walletLauncher->getInterface().isOpen(name); // default is false
-}
+    QDBusReply<bool> r = walletLauncher->getInterface().isOpen(name);
 
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return false;
+    }
+    else
+        return r;
+}
 
 int Wallet::closeWallet(const QString& name, bool force) {
     QDBusReply<int> r = walletLauncher->getInterface().close(name, force);
-    return r.isValid() ? r : -1;
+
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return -1;
+    }
+    else
+        return r;
 }
 
 
 int Wallet::deleteWallet(const QString& name) {
     QDBusReply<int> r = walletLauncher->getInterface().deleteWallet(name);
-    return r.isValid() ? r : -1;
-}
 
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return -1;
+    }
+    else
+        return r;
+}
 
 Wallet *Wallet::openWallet(const QString& name, WId w, OpenType ot) {
     if( w == 0 )
@@ -243,6 +286,7 @@ Wallet *Wallet::openWallet(const QString& name, WId w, OpenType ot) {
     }
     // error communicating with the daemon (maybe not running)
     if (!r.isValid()) {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
         delete wallet;
         return 0;
     }
@@ -275,12 +319,27 @@ Wallet *Wallet::openWallet(const QString& name, WId w, OpenType ot) {
 
 
 bool Wallet::disconnectApplication(const QString& wallet, const QString& app) {
-    return walletLauncher->getInterface().disconnectApplication(wallet, app); // default is false
+    QDBusReply<bool> r = walletLauncher->getInterface().disconnectApplication(wallet, app);
+
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return false;
+    }
+    else
+        return r;
 }
 
 
 QStringList Wallet::users(const QString& name) {
-    return walletLauncher->getInterface().users(name); // default is QStringList()
+    QDBusReply<QStringList> r = walletLauncher->getInterface().users(name);
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return QStringList();
+    }
+    else
+        return r;
 }
 
 
@@ -306,7 +365,10 @@ int Wallet::lockWallet() {
     if (r.isValid()) {
         return r;
     }
-    return -1;
+    else {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
+        return -1;
+    }
 }
 
 
@@ -350,7 +412,13 @@ QStringList Wallet::folderList() {
     }
 
     QDBusReply<QStringList> r = walletLauncher->getInterface().folderList(d->handle, appid());
-    return r;
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return QStringList();
+    }
+    else
+        return r;
 }
 
 
@@ -360,7 +428,13 @@ QStringList Wallet::entryList() {
     }
 
     QDBusReply<QStringList> r = walletLauncher->getInterface().entryList(d->handle, d->folder, appid());
-    return r;
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return QStringList();
+    }
+    else
+        return r;
 }
 
 
@@ -370,7 +444,14 @@ bool Wallet::hasFolder(const QString& f) {
     }
 
     QDBusReply<bool> r = walletLauncher->getInterface().hasFolder(d->handle, f, appid());
-    return r; // default is false
+    if (!r.isValid())
+    {
+            kDebug(285) << "Invalid DBus reply: " << r.error();
+            return false;
+    }
+    else
+        return r;
+
 }
 
 
@@ -381,7 +462,14 @@ bool Wallet::createFolder(const QString& f) {
 
     if (!hasFolder(f)) {
         QDBusReply<bool> r = walletLauncher->getInterface().createFolder(d->handle, f, appid());
-        return r;
+
+        if (!r.isValid())
+        {
+                kDebug(285) << "Invalid DBus reply: " << r.error();
+                return false;
+        }
+        else
+            return r;
     }
 
     return true;				// folder already exists
@@ -421,7 +509,13 @@ bool Wallet::removeFolder(const QString& f) {
         setFolder(QString());
     }
 
-    return r;					// default is false
+    if (!r.isValid())
+    {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
+        return false;
+    }
+    else
+        return r;
 }
 
 
@@ -652,7 +746,13 @@ bool Wallet::hasEntry(const QString& key) {
     }
 
     QDBusReply<bool> r = walletLauncher->getInterface().hasEntry(d->handle, d->folder, key, appid());
-    return r;					// default is false
+    if (!r.isValid())
+    {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
+        return false;
+    }
+    else
+        return r;
 }
 
 
@@ -688,18 +788,17 @@ Wallet::EntryType Wallet::entryType(const QString& key) {
 }
 
 
-void Wallet::slotServiceOwnerChanged(const QString& name,const QString& oldOwner,const QString& newOwner) {
-    Q_UNUSED(oldOwner);
-    if (newOwner.isEmpty() && name == "org.kde.kwalletd") {
-        // if openWallet() waits for the DBUS reply, prevent it from waiting forever:
-        if ( d->loop )
-            d->loop->quit();
-        if( d->handle >= 0 )
-            slotWalletClosed(d->handle);
+void Wallet::WalletPrivate::walletServiceUnregistered()
+{
+    if (loop) {
+        loop->quit();
+    }
+
+    if (handle >= 0) {
+        q->slotWalletClosed(handle);
     }
 }
 
-
 void Wallet::slotFolderUpdated(const QString& wallet, const QString& folder) {
     if (d->name == wallet) {
         emit folderUpdated(folder);
@@ -742,14 +841,26 @@ void Wallet::emitWalletAsyncOpenError() {
 bool Wallet::folderDoesNotExist(const QString& wallet, const QString& folder)
 {
     QDBusReply<bool> r = walletLauncher->getInterface().folderDoesNotExist(wallet, folder);
-    return r;
+    if (!r.isValid())
+    {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
+        return false;
+    }
+    else
+        return r;
 }
 
 
 bool Wallet::keyDoesNotExist(const QString& wallet, const QString& folder, const QString& key)
 {
     QDBusReply<bool> r = walletLauncher->getInterface().keyDoesNotExist(wallet, folder, key);
-    return r;
+    if (!r.isValid())
+    {
+        kDebug(285) << "Invalid DBus reply: " << r.error();
+        return false;
+    }
+    else
+        return r;
 }
 
 void Wallet::virtual_hook(int, void*) {
@@ -757,7 +868,7 @@ void Wallet::virtual_hook(int, void*) {
 }
 
 KWalletDLauncher::KWalletDLauncher()
-    : m_wallet("org.kde.kwalletd", "/modules/kwalletd", QDBusConnection::sessionBus()),
+    : m_wallet(QString::fromLatin1(s_kwalletdServiceName), "/modules/kwalletd", QDBusConnection::sessionBus()),
       m_cgroup(KSharedConfig::openConfig("kwalletrc", KConfig::NoGlobals)->group("Wallet"))
 {
 }
@@ -769,7 +880,7 @@ KWalletDLauncher::~KWalletDLauncher()
 org::kde::KWallet &KWalletDLauncher::getInterface()
 {
     // check if kwalletd is already running
-    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kwalletd"))
+    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered(QString::fromLatin1(s_kwalletdServiceName)))
     {
         // not running! check if it is enabled.
         bool walletEnabled = m_cgroup.readEntry("Enabled", true);
@@ -782,7 +893,8 @@ org::kde::KWallet &KWalletDLauncher::getInterface()
                 kError(285) << "Couldn't start kwalletd: " << error << endl;
             }
 
-            if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kwalletd")) {
+            if
+                (!QDBusConnection::sessionBus().interface()->isServiceRegistered(QString::fromLatin1(s_kwalletdServiceName))) {
                 kDebug(285) << "The kwalletd service is still not registered";
             } else {
                 kDebug(285) << "The kwalletd service has been registered";
