@@ -92,6 +92,7 @@ public:
     {
 // causes a weird crash in KWord at least, so let Qt delete it for us.
 //        delete completionBox;
+        delete style.data();
     }
 
     void _k_slotSettingsChanged(int category)
@@ -109,7 +110,9 @@ public:
         // KDE5: remove userTextChanged signal, textEdited does the same...
         if (!completionRunning && (txt != userText)) {
             userText = txt;
+#ifndef KDE_NO_DEPRECATED
             emit q->userTextChanged(txt);
+#endif
         }
     }
 
@@ -120,7 +123,9 @@ public:
         if (!completionRunning && (txt != userText)) {
             userText = txt;
             q->setModified(true);
+#ifndef KDE_NO_DEPRECATED
             emit q->userTextChanged(txt);
+#endif
             emit q->textEdited(txt);
             emit q->textChanged(txt);
         }
@@ -1090,7 +1095,7 @@ void KLineEdit::mousePressEvent( QMouseEvent* e )
     if  ( (e->button() == Qt::LeftButton ||
            e->button() == Qt::MidButton ) &&
           d->clearButton ) {
-        d->clickInClear = d->clearButton->underMouse();
+        d->clickInClear = d->clearButton == childAt( e->pos() );
 
         if ( d->clickInClear ) {
             d->possibleTripleClick = false;
@@ -1109,7 +1114,7 @@ void KLineEdit::mousePressEvent( QMouseEvent* e )
 void KLineEdit::mouseReleaseEvent( QMouseEvent* e )
 {
     if ( d->clickInClear ) {
-        if ( d->clearButton->underMouse() ) {
+        if ( d->clearButton == childAt( e->pos() ) ) {
             QString newText;
             if ( e->button() == Qt::MidButton ) {
                 newText = QApplication::clipboard()->text( QClipboard::Selection );
@@ -1807,15 +1812,19 @@ void KLineEdit::setClickMessage( const QString &msg )
     update();
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KLineEdit::setContextMenuEnabled( bool showMenu )
 {
     QLineEdit::setContextMenuPolicy( showMenu ? Qt::DefaultContextMenu : Qt::NoContextMenu );
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool KLineEdit::isContextMenuEnabled() const
 {
     return  ( contextMenuPolicy() == Qt::DefaultContextMenu );
 }
+#endif
 
 void KLineEdit::setPasswordMode(bool b)
 {
