@@ -95,12 +95,14 @@ KNumInput::KNumInput(QWidget* parent)
     setFocusPolicy(Qt::StrongFocus);
 }
 
+#ifndef KDE_NO_DEPRECATED
 KNumInput::KNumInput(QWidget* parent, KNumInput* below)
     : QWidget(parent), d(new KNumInputPrivate(this, below))
 {
     setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
     setFocusPolicy(Qt::StrongFocus);
 }
+#endif
 
 KNumInput::~KNumInput()
 {
@@ -178,7 +180,9 @@ void KNumInput::layout(bool deep)
         return;
     }
 
-    KNumInput* p = this;
+    w2 = d->column2Width;
+
+    KNumInput* p = d->previousNumInput;
     while (p) {
         p->doLayout();
         w1 = qMax(w1, p->d->column1Width);
@@ -328,12 +332,14 @@ public:
 };
 
 
+#ifndef KDE_NO_DEPRECATED
 KIntNumInput::KIntNumInput(KNumInput* below, int val, QWidget *parent, int _base)
     : KNumInput(parent, below)
     , d(new KIntNumInputPrivate(this, val))
 {
     init(val, _base);
 }
+#endif
 
 KIntNumInput::KIntNumInput(QWidget *parent)
     : KNumInput(parent)
@@ -461,11 +467,13 @@ void KIntNumInput::setRange(int lower, int upper, int singleStep)
     setSliderEnabled(priv->slider);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KIntNumInput::setRange(int lower, int upper, int singleStep, bool slider)
 {
     setRange(lower, upper, singleStep);
     setSliderEnabled(slider);
 }
+#endif
 
 void KIntNumInput::setMinimum(int min)
 {
@@ -550,7 +558,7 @@ QSize KIntNumInput::minimumSizeHint() const
         h = qMax(h, priv->labelSize.height() + 2);
     }
 
-    w = priv->slider ? priv->slider->sizeHint().width() + 8 : 0;
+    w = priv->slider ? priv->slider->sizeHint().width() + KDialog::spacingHint() : 0;
     w += priv->column1Width + priv->column2Width;
 
     if (priv->labelAlignment & (Qt::AlignTop | Qt::AlignBottom)) {
@@ -588,9 +596,9 @@ void KIntNumInput::resizeEvent(QResizeEvent* e)
         priv->label->setGeometry(0, 0, w, d->intSpinBoxSize.height());
     }
 
-    if (qApp->layoutDirection()) {
+    if (qApp->layoutDirection() == Qt::RightToLeft) {
         d->intSpinBox->setGeometry(w, h, priv->slider ? priv->column2Width : qMax(priv->column2Width, e->size().width() - w), d->intSpinBoxSize.height());
-        w += priv->column2Width + 8;
+        w += priv->column2Width + KDialog::spacingHint();
 
         if (priv->slider) {
             priv->slider->setGeometry(w, h, e->size().width() - w, d->intSpinBoxSize.height() + KDialog::spacingHint());
@@ -699,6 +707,7 @@ KDoubleNumInput::KDoubleNumInput(double lower, double upper, double value, QWidg
     init(value, lower, upper, singleStep, precision);
 }
 
+#ifndef KDE_NO_DEPRECATED
 KDoubleNumInput::KDoubleNumInput(KNumInput *below,
                                  double lower, double upper, double value, QWidget *parent,
                                  double singleStep, int precision)
@@ -707,6 +716,7 @@ KDoubleNumInput::KDoubleNumInput(KNumInput *below,
 {
     init(value, lower, upper, singleStep, precision);
 }
+#endif
 
 KDoubleNumInput::~KDoubleNumInput()
 {
@@ -810,7 +820,7 @@ QSize KDoubleNumInput::minimumSizeHint() const
         h = qMax(h, priv->labelSize.height() + 2);
     }
 
-    w = priv->slider ? priv->slider->sizeHint().width() + 8 : 0;
+    w = priv->slider ? priv->slider->sizeHint().width() + KDialog::spacingHint() : 0;
     w += priv->column1Width + priv->column2Width;
 
     if (priv->labelAlignment & (Qt::AlignTop | Qt::AlignBottom)) {
@@ -836,7 +846,7 @@ void KDoubleNumInput::resizeEvent(QResizeEvent* e)
         priv->label->setGeometry(0, 0, w, d->editSize.height());
     }
 
-    if (qApp->layoutDirection()) {
+    if (qApp->layoutDirection() == Qt::RightToLeft) {
         d->spin->setGeometry(w, h, priv->slider ? priv->column2Width
                              : e->size().width() - w, d->editSize.height());
         w += priv->column2Width + KDialog::spacingHint();
