@@ -28,6 +28,7 @@
 #include <kfontsizeaction.h>
 #include <klocale.h>
 #include <ktoggleaction.h>
+#include <kdebug.h>
 
 // Qt includes
 #include <QtGui/QTextList>
@@ -46,8 +47,8 @@ class KRichTextWidget::Private
 public:
     Private(KRichTextWidget *parent)
             :   q(parent),
-            richTextEnabled(false), // It's only enabled when an action makes text rich.
             painterActive(false),
+            richTextEnabled(false), // It's only enabled when an action makes text rich.
             enableRichText(0),
             action_text_foreground_color(0),
             action_text_background_color(0),
@@ -67,13 +68,16 @@ public:
             action_align_left(0),
             action_align_right(0),
             action_align_center(0),
-            action_align_justify(0)
+            action_align_justify(0),
+            action_direction_ltr(0),
+            action_direction_rtl(0),
+            action_text_superscript(0),
+            action_text_subscript(0)
     {
     }
 
     KRichTextWidget *q;
 
-    bool richTextEnabled;
     RichTextSupport richTextSupport;
 
     QTextCharFormat painterFormat;
@@ -81,6 +85,7 @@ public:
 
     QList<KAction*> richTextActionList;
 
+    bool richTextEnabled;
     KToggleAction *enableRichText;
 
     KAction *action_text_foreground_color;
@@ -108,6 +113,12 @@ public:
     KToggleAction *action_align_center;
     KToggleAction *action_align_justify;
 
+    KToggleAction *action_direction_ltr;
+    KToggleAction *action_direction_rtl;
+
+    KToggleAction *action_text_superscript;
+    KToggleAction *action_text_subscript;
+
     //
     // Normal functions
     //
@@ -365,6 +376,32 @@ void KRichTextWidget::createActions(KActionCollection *actionCollection)
         d->action_align_justify = 0;
     }
 
+    if (d->richTextSupport & SupportDirection) {
+        d->action_direction_ltr = new KToggleAction(KIcon("format-text-direction-ltr"), i18nc("@action", "Left-to-Right"), actionCollection);
+        d->action_direction_ltr->setIconText(i18nc("@label left-to-right", "Left-to-Right"));
+        d->richTextActionList.append(d->action_direction_ltr);
+        actionCollection->addAction("direction_ltr", d->action_direction_ltr);
+        connect(d->action_direction_ltr, SIGNAL(triggered()),
+                this, SLOT(makeLeftToRight()));
+
+        d->action_direction_rtl = new KToggleAction(KIcon("format-text-direction-rtl"), i18nc("@action", "Right-to-Left"), actionCollection);
+        d->action_direction_rtl->setIconText(i18nc("@label right-to-left", "Right-to-Left"));
+        d->richTextActionList.append(d->action_direction_rtl);
+        actionCollection->addAction("direction_rtl", d->action_direction_rtl);
+        connect(d->action_direction_rtl, SIGNAL(triggered()),
+                this, SLOT(makeRightToLeft()));
+
+        QActionGroup *directionGroup = new QActionGroup(this);
+        directionGroup->addAction(d->action_direction_ltr);
+        directionGroup->addAction(d->action_direction_rtl);
+    } else {
+        actionCollection->removeAction(d->action_direction_ltr);
+        actionCollection->removeAction(d->action_direction_rtl);
+
+        d->action_direction_ltr = 0;
+        d->action_direction_rtl = 0;
+    }
+
     if (d->richTextSupport & SupportChangeListStyle) {
         d->action_list_style = new KSelectAction(KIcon("format-list-unordered"), i18nc("@title:menu", "List Style"), actionCollection);
         QStringList listStyles;
@@ -459,6 +496,29 @@ void KRichTextWidget::createActions(KActionCollection *actionCollection)
         d->action_to_plain_text = 0;
     }
 
+    if (d->richTextSupport & SupportSuperScriptAndSubScript) {
+        d->action_text_subscript = new KToggleAction(KIcon("format-text-subscript"), i18nc("@action", "Subscript"), actionCollection);
+        d->richTextActionList.append((d->action_text_subscript));
+        actionCollection->addAction("format_text_subscript", d->action_text_subscript);
+
+        connect(d->action_text_subscript, SIGNAL(triggered(bool)),
+                this, SLOT(setTextSubScript(bool)));
+
+        d->action_text_superscript = new KToggleAction(KIcon("format-text-superscript"), i18nc("@action", "Superscript"), actionCollection);
+        d->richTextActionList.append((d->action_text_superscript));
+        actionCollection->addAction("format_text_superscript", d->action_text_superscript);
+
+        connect(d->action_text_superscript, SIGNAL(triggered(bool)),
+                this, SLOT(setTextSuperScript(bool)));
+    } else {
+        actionCollection->removeAction(d->action_text_subscript);
+        d->action_text_subscript = 0;
+
+        actionCollection->removeAction(d->action_text_superscript);
+        d->action_text_superscript = 0;
+    }
+    
+
     disconnect(this, SIGNAL(currentCharFormatChanged(const QTextCharFormat &)),
                this, SLOT(_k_updateCharFormatActions(const QTextCharFormat &)));
     disconnect(this, SIGNAL(cursorPositionChanged()),
@@ -515,6 +575,12 @@ void KRichTextWidget::Private::_k_updateCharFormatActions(const QTextCharFormat
     if (richTextSupport & SupportStrikeOut) {
         action_text_strikeout->setChecked(f.strikeOut());
     }
+
+    if (richTextSupport & SupportSuperScriptAndSubScript) {
+        QTextCharFormat::VerticalAlignment vAlign = format.verticalAlignment();
+        action_text_superscript->setChecked(vAlign == QTextCharFormat::AlignSuperScript);
+        action_text_subscript->setChecked(vAlign == QTextCharFormat::AlignSubScript);
+    }
 }
 
 void KRichTextWidget::Private::_k_updateMiscActions()
@@ -558,6 +624,11 @@ void KRichTextWidget::Private::_k_updateMiscActions()
         }
     }
 
+    if (richTextSupport & SupportDirection) {
+        const Qt::LayoutDirection direction = q->textCursor().blockFormat().layoutDirection();
+        action_direction_ltr->setChecked(direction == Qt::LeftToRight);
+        action_direction_rtl->setChecked(direction == Qt::RightToLeft);
+    }
 }
 
 void KRichTextWidget::Private::_k_setTextForegroundColor()
