@@ -46,14 +46,21 @@
 #include <QtWebKit/QWebView>
 
 #define QL1S(x)  QLatin1String(x)
+#define QL1C(x)  QLatin1Char(x)
 
 static bool excludedMimeType(const QString &type)
 {
-    // Let QtWebKit handle flash and java applets...
-    return (type.startsWith(QL1S("inode/"), Qt::CaseInsensitive) ||
-            type.startsWith(QL1S("application/x-java"), Qt::CaseInsensitive) ||
-            type == QL1S("application/x-shockwave-flash") ||
-            type == QL1S("application/futuresplash"));
+    if (type.startsWith(QL1S("inode/"), Qt::CaseInsensitive))
+        return true;
+
+    if (type.startsWith(QL1S("application/x-java"), Qt::CaseInsensitive))
+        return true;
+
+    if (type == QL1S("application/x-shockwave-flash") ||
+        type == QL1S("application/futuresplash"))
+      return true;
+
+    return false;
 }
 
 class KWebPluginFactory::KWebPluginFactoryPrivate
@@ -84,6 +91,7 @@ KWebPluginFactory::KWebPluginFactory(QObject *parent)
 
 KWebPluginFactory::~KWebPluginFactory()
 {
+    delete d;
 }
 
 QObject* KWebPluginFactory::create(const QString& _mimeType, const QUrl& url, const QStringList& argumentNames, const QStringList& argumentValues) const
@@ -93,7 +101,7 @@ QObject* KWebPluginFactory::create(const QString& _mimeType, const QUrl& url, co
     const int count = argumentNames.count();
 
     for (int i = 0; i < count; ++i) {
-        arguments << argumentNames.at(i) + "=\"" + argumentValues.at(i) + '\"';
+        arguments << QString(argumentNames.at(i) + QL1S("=\"") + argumentValues.at(i) + QL1C('\"'));
     }
 
     QString mimeType (_mimeType.trimmed());
