@@ -90,8 +90,8 @@ void KFileFilterCombo::setFilter(const QString& filter)
 	d->m_filters.append( d->defaultFilter );
 
     QStringList::ConstIterator it;
-    QStringList::ConstIterator end(d->m_filters.end());
-    for (it = d->m_filters.begin(); it != end; ++it) {
+    QStringList::ConstIterator end(d->m_filters.constEnd());
+    for (it = d->m_filters.constBegin(); it != end; ++it) {
 	int tab = (*it).indexOf('|');
 	addItem((tab < 0) ? *it :
 		   (*it).mid(tab + 1));
@@ -214,4 +214,9 @@ QString KFileFilterCombo::defaultFilter() const
     return d->defaultFilter;
 }
 
+bool KFileFilterCombo::isMimeFilter() const
+{
+    return d->isMimeFilter;
+}
+
 #include "kfilefiltercombo.moc"
