@@ -4,7 +4,8 @@
  *                                                                           *
  * This library is free software; you can redistribute it and/or             *
  * modify it under the terms of the GNU Library General Public               *
- * License version 2 as published by the Free Software Foundation.           *
+ * License as published by the Free Software Foundation; either              *
+ * version 2 of the License, or (at your option) any later version.          *
  *                                                                           *
  * This library is distributed in the hope that it will be useful,           *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
@@ -34,25 +35,29 @@
 #include <QtGui/QKeyEvent>
 #include <QtGui/QStyleOption>
 
-QPointer<KUrlNavigatorMenu> KUrlNavigatorButton::m_dirsMenu;
+QPointer<KUrlNavigatorMenu> KUrlNavigatorButton::m_subDirsMenu;
 
-KUrlNavigatorButton::KUrlNavigatorButton(const KUrl& url, KUrlNavigator* parent) :
-    KUrlButton(parent),
+KUrlNavigatorButton::KUrlNavigatorButton(const KUrl& url, QWidget* parent) :
+    KUrlNavigatorButtonBase(parent),
     m_hoverArrow(false),
     m_pendingTextChange(false),
+    m_replaceButton(false),
+    m_wheelSteps(0),
     m_url(url),
-    m_popupDelay(0),
-    m_listJob(0)
+    m_subDir(),
+    m_openSubDirsTimer(0),
+    m_subDirsJob(0)
 {
     setAcceptDrops(true);
     setUrl(url);
     setMouseTracking(true);
-    connect(this, SIGNAL(clicked()), this, SLOT(updateNavigatorUrl()));
 
-    m_popupDelay = new QTimer(this);
-    m_popupDelay->setSingleShot(true);
-    connect(m_popupDelay, SIGNAL(timeout()), this, SLOT(startListJob()));
-    connect(this, SIGNAL(pressed()), this, SLOT(startPopupDelay()));
+    m_openSubDirsTimer = new QTimer(this);
+    m_openSubDirsTimer->setSingleShot(true);
+    m_openSubDirsTimer->setInterval(300);
+    connect(m_openSubDirsTimer, SIGNAL(timeout()), this, SLOT(startSubDirsJob()));
+
+    connect(this, SIGNAL(pressed()), this, SLOT(requestSubDirs()));
 }
 
 KUrlNavigatorButton::~KUrlNavigatorButton()
@@ -70,6 +75,7 @@ void KUrlNavigatorButton::setUrl(const KUrl& url)
         KIO::StatJob* job = KIO::stat(m_url, KIO::HideProgressInfo);
         connect(job, SIGNAL(result(KJob*)),
                 this, SLOT(statFinished(KJob*)));
+        emit startedTextResolving();
     }
 }
 
@@ -78,47 +84,45 @@ KUrl KUrlNavigatorButton::url() const
     return m_url;
 }
 
-void KUrlNavigatorButton::setActive(bool active)
-{
-    QFont adjustedFont(font());
-    if (active) {
-        setDisplayHintEnabled(ActivatedHint, true);
-        adjustedFont.setBold(true);
-    } else {
-        setDisplayHintEnabled(ActivatedHint, false);
-        adjustedFont.setBold(false);
-    }
-
-    setFont(adjustedFont);
-    updateMinimumWidth();
-    update();
-}
-
-bool KUrlNavigatorButton::isActive() const
-{
-    return isDisplayHintEnabled(ActivatedHint);
-}
-
 void KUrlNavigatorButton::setText(const QString& text)
 {
     QString adjustedText = text;
     if (adjustedText.isEmpty()) {
         adjustedText = m_url.protocol();
     }
-    KUrlButton::setText(adjustedText);
+    // Assure that the button always consists of one line
+    adjustedText.remove(QLatin1Char('\n'));
+
+    KUrlNavigatorButtonBase::setText(adjustedText);
     updateMinimumWidth();
 
-    // assure that statFinished() does not overwrite a text that has been
+    // Assure that statFinished() does not overwrite a text that has been
     // set by a client of the URL navigator button
     m_pendingTextChange = false;
 }
 
+void KUrlNavigatorButton::setActiveSubDirectory(const QString& subDir)
+{
+    m_subDir = subDir;
+
+    // We use a different (bold) font on active, so the size hint changes
+    updateGeometry();
+    update();
+}
+
+QString KUrlNavigatorButton::activeSubDirectory() const
+{
+    return m_subDir;
+}
+
 QSize KUrlNavigatorButton::sizeHint() const
 {
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
     // the minimum size is textWidth + arrowWidth() + 2 * BorderWidth; for the
     // preferred size we add the BorderWidth 2 times again for having an uncluttered look
-    const int width = fontMetrics().width(text()) + arrowWidth() + 4 * BorderWidth;
-    return QSize(width, KUrlButton::sizeHint().height());
+    const int width = QFontMetrics(adjustedFont).width(text()) + arrowWidth() + 4 * BorderWidth;
+    return QSize(width, KUrlNavigatorButtonBase::sizeHint().height());
 }
 
 void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
@@ -127,6 +131,10 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
 
     QPainter painter(this);
 
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
+    painter.setFont(adjustedFont);
+
     int buttonWidth  = width();
     int preferredWidth = sizeHint().width();
     if (preferredWidth < minimumWidth()) {
@@ -145,7 +153,7 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
 
     const bool leftToRight = (layoutDirection() == Qt::LeftToRight);
 
-    if (!isDisplayHintEnabled(ActivatedHint)) {
+    if (!m_subDir.isEmpty()) {
         // draw arrow
         const int arrowSize = arrowWidth();
         const int arrowX = leftToRight ? (buttonWidth - arrowSize) - BorderWidth : BorderWidth;
@@ -209,7 +217,7 @@ void KUrlNavigatorButton::paintEvent(QPaintEvent* event)
 
 void KUrlNavigatorButton::enterEvent(QEvent* event)
 {
-    KUrlButton::enterEvent(event);
+    KUrlNavigatorButtonBase::enterEvent(event);
 
     // if the text is clipped due to a small window width, the text should
     // be shown as tooltip
@@ -220,7 +228,7 @@ void KUrlNavigatorButton::enterEvent(QEvent* event)
 
 void KUrlNavigatorButton::leaveEvent(QEvent* event)
 {
-    KUrlButton::leaveEvent(event);
+    KUrlNavigatorButtonBase::leaveEvent(event);
     setToolTip(QString());
 
     if (m_hoverArrow) {
@@ -259,21 +267,21 @@ void KUrlNavigatorButton::dragMoveEvent(QDragMoveEvent* event)
         m_hoverArrow = true;
         update();
 
-        if (m_dirsMenu == 0) {
-            startPopupDelay();
-        } else if (m_dirsMenu->parent() != this) {
-            m_dirsMenu->close();
-            m_dirsMenu->deleteLater();
-            m_dirsMenu = 0;
+        if (m_subDirsMenu == 0) {
+            requestSubDirs();
+        } else if (m_subDirsMenu->parent() != this) {
+            m_subDirsMenu->close();
+            m_subDirsMenu->deleteLater();
+            m_subDirsMenu = 0;
 
-            startPopupDelay();
+            requestSubDirs();
         }
     } else {
-        if (m_popupDelay->isActive()) {
-            stopPopupDelay();
+        if (m_openSubDirsTimer->isActive()) {
+            cancelSubDirsRequest();
         }
-        delete m_dirsMenu;
-        m_dirsMenu = 0;
+        delete m_subDirsMenu;
+        m_subDirsMenu = 0;
         m_hoverArrow = false;
         update();
     }
@@ -281,7 +289,7 @@ void KUrlNavigatorButton::dragMoveEvent(QDragMoveEvent* event)
 
 void KUrlNavigatorButton::dragLeaveEvent(QDragLeaveEvent* event)
 {
-    KUrlButton::dragLeaveEvent(event);
+    KUrlNavigatorButtonBase::dragLeaveEvent(event);
 
     m_hoverArrow = false;
     setDisplayHintEnabled(DraggedHint, false);
@@ -291,25 +299,26 @@ void KUrlNavigatorButton::dragLeaveEvent(QDragLeaveEvent* event)
 void KUrlNavigatorButton::mousePressEvent(QMouseEvent* event)
 {
     if (isAboveArrow(event->x()) && (event->button() == Qt::LeftButton)) {
-        urlNavigator()->requestActivation();
-        startListJob();
-    } else {
-        // the mouse is pressed above the text area
-        KUrlButton::mousePressEvent(event);
+        // the mouse is pressed above the [>] button
+        startSubDirsJob();
     }
+    KUrlNavigatorButtonBase::mousePressEvent(event);
 }
 
 void KUrlNavigatorButton::mouseReleaseEvent(QMouseEvent* event)
 {
     if (!isAboveArrow(event->x()) || (event->button() != Qt::LeftButton)) {
-        // the mouse is released above the text area
-        KUrlButton::mouseReleaseEvent(event);
+        // the mouse has been released above the text area and not
+        // above the [>] button
+        emit clicked(m_url, event->button());
+        cancelSubDirsRequest();
     }
+    KUrlNavigatorButtonBase::mouseReleaseEvent(event);
 }
 
 void KUrlNavigatorButton::mouseMoveEvent(QMouseEvent* event)
 {
-    KUrlButton::mouseMoveEvent(event);
+    KUrlNavigatorButtonBase::mouseMoveEvent(event);
 
     const bool hoverArrow = isAboveArrow(event->x());
     if (hoverArrow != m_hoverArrow) {
@@ -318,57 +327,57 @@ void KUrlNavigatorButton::mouseMoveEvent(QMouseEvent* event)
     }
 }
 
-void KUrlNavigatorButton::updateNavigatorUrl()
+void KUrlNavigatorButton::wheelEvent(QWheelEvent* event)
 {
-    // TODO KDE 4.5: emit signal
-    stopPopupDelay();
-    urlNavigator()->setUrl(m_url);
-}
-
-void KUrlNavigatorButton::startPopupDelay()
-{
-    if (!m_popupDelay->isActive() && (m_listJob == 0)) {
-        m_popupDelay->start(300);
+    if (event->orientation() == Qt::Vertical) {
+        m_wheelSteps = event->delta() / 120;
+        m_replaceButton = true;
+        startSubDirsJob();
+        event->accept();
+    } else {
+        KUrlNavigatorButtonBase::wheelEvent(event);
     }
 }
 
-void KUrlNavigatorButton::stopPopupDelay()
+void KUrlNavigatorButton::requestSubDirs()
 {
-    m_popupDelay->stop();
-    if (m_listJob != 0) {
-        m_listJob->kill();
-        m_listJob = 0;
+    if (!m_openSubDirsTimer->isActive() && (m_subDirsJob == 0)) {
+        m_openSubDirsTimer->start();
     }
 }
 
-void KUrlNavigatorButton::startListJob()
+void KUrlNavigatorButton::startSubDirsJob()
 {
-    if (m_listJob != 0) {
+    if (m_subDirsJob != 0) {
         return;
     }
 
-    m_listJob = KIO::listDir(m_url, KIO::HideProgressInfo, false /*no hidden files*/);
+    const KUrl url = m_replaceButton ? m_url.upUrl() : m_url;
+    m_subDirsJob = KIO::listDir(url, KIO::HideProgressInfo, false /*no hidden files*/);
     m_subDirs.clear(); // just to be ++safe
 
-    connect(m_listJob, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList &)),
-            this, SLOT(entriesList(KIO::Job*, const KIO::UDSEntryList&)));
-    connect(m_listJob, SIGNAL(result(KJob*)), this, SLOT(listJobFinished(KJob*)));
+    connect(m_subDirsJob, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList &)),
+            this, SLOT(addEntriesToSubDirs(KIO::Job*, const KIO::UDSEntryList&)));
+
+    if (m_replaceButton) {
+        connect(m_subDirsJob, SIGNAL(result(KJob*)), this, SLOT(replaceButton(KJob*)));
+    } else {
+        connect(m_subDirsJob, SIGNAL(result(KJob*)), this, SLOT(openSubDirsMenu(KJob*)));
+    }
 }
 
-void KUrlNavigatorButton::entriesList(KIO::Job* job, const KIO::UDSEntryList& entries)
+void KUrlNavigatorButton::addEntriesToSubDirs(KIO::Job* job, const KIO::UDSEntryList& entries)
 {
-    if (job != m_listJob) {
-        return;
-    }
+    Q_ASSERT(job == m_subDirsJob);
 
     foreach (const KIO::UDSEntry& entry, entries) {
         if (entry.isDir()) {
-            QString name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
+            const QString name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
             QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
             if (displayName.isEmpty()) {
                 displayName = name;
             }
-            if ((name != ".") && (name != "..")) {
+            if ((name != QLatin1String(".")) && (name != QLatin1String(".."))) {
                 m_subDirs.append(qMakePair(name, displayName));
             }
         }
@@ -383,6 +392,14 @@ void KUrlNavigatorButton::urlsDropped(QAction* action, QDropEvent* event)
     urlsDropped(url, event);
 }
 
+void KUrlNavigatorButton::slotMenuActionClicked(QAction* action)
+{
+    const int result = action->data().toInt();
+    KUrl url = m_url;
+    url.addPath(m_subDirs.at(result).first);
+    emit clicked(url, Qt::MidButton);
+}
+
 void KUrlNavigatorButton::statFinished(KJob* job)
 {
     if (m_pendingTextChange) {
@@ -394,24 +411,24 @@ void KUrlNavigatorButton::statFinished(KJob* job)
             name = m_url.fileName();
         }
         setText(name);
+
+        emit finishedTextResolving();
     }
 }
 
 /**
- * Helper function for listJobFinished
+ * Helper function for openSubDirsMenu
  */
 static bool naturalLessThan(const QPair<QString, QString>& s1, const QPair<QString, QString>& s2)
 {
     return KStringHandler::naturalCompare(s1.first, s2.first, Qt::CaseInsensitive) < 0;
 }
 
-void KUrlNavigatorButton::listJobFinished(KJob* job)
+void KUrlNavigatorButton::openSubDirsMenu(KJob* job)
 {
-    if (job != m_listJob) {
-        return;
-    }
+    Q_ASSERT(job == m_subDirsJob);
+    m_subDirsJob = 0;
 
-    m_listJob = 0;
     if (job->error() || m_subDirs.isEmpty()) {
         // clear listing
         return;
@@ -421,72 +438,88 @@ void KUrlNavigatorButton::listJobFinished(KJob* job)
     setDisplayHintEnabled(PopupActiveHint, true);
     update(); // ensure the button is drawn highlighted
 
-    if (m_dirsMenu != 0) {
-        m_dirsMenu->close();
-        m_dirsMenu->deleteLater();
-        m_dirsMenu = 0;
+    if (m_subDirsMenu != 0) {
+        m_subDirsMenu->close();
+        m_subDirsMenu->deleteLater();
+        m_subDirsMenu = 0;
     }
 
-    m_dirsMenu = new KUrlNavigatorMenu(this);
-    connect(m_dirsMenu, SIGNAL(urlsDropped(QAction*, QDropEvent*)),
-            this, SLOT(urlsDropped(QAction*, QDropEvent*)));
-
-    m_dirsMenu->setLayoutDirection(Qt::LeftToRight);
-
-    const QString relativeUrl = KUrl::relativeUrl(m_url, urlNavigator()->url());
-    const QString selectedSubdir = relativeUrl.section('/', 1, 1);
-
-    const int subDirsCount = m_subDirs.count();
-    for (int i = 0; i < subDirsCount; ++i) {
-        const QString subDirName = m_subDirs[i].first;
-        const QString subDirDisplayName = m_subDirs[i].second;
-        QString text = KStringHandler::csqueeze(subDirDisplayName, 60);
-        text.replace('&', "&&");
-        QAction* action = new QAction(text, this);
-        if (selectedSubdir == subDirName) {
-            QFont font(action->font());
-            font.setBold(true);
-            action->setFont(font);
-        }
-        action->setData(i);
-        m_dirsMenu->addAction(action);
-
-        if (i > 100) {
-            // Opening a menu with several 100 items makes no sense from
-            // a usability view. Also there are implementation issues in
-            // QMenu if the number of menu items don't fit into the available
-            // screen -> skip remaining items
-            QAction* limitReached = new QAction("...", this);
-            limitReached->setEnabled(false);
-            m_dirsMenu->addAction(limitReached);
-            break;
-        }
-    }
+    m_subDirsMenu = new KUrlNavigatorMenu(this);
+    initMenu(m_subDirsMenu, 0);
 
     const bool leftToRight = (layoutDirection() == Qt::LeftToRight);
     const int popupX = leftToRight ? width() - arrowWidth() - BorderWidth : 0;
-    const QPoint popupPos  = urlNavigator()->mapToGlobal(geometry().bottomLeft() + QPoint(popupX, 0));
+    const QPoint popupPos  = parentWidget()->mapToGlobal(geometry().bottomLeft() + QPoint(popupX, 0));
 
-    const QAction* action = m_dirsMenu->exec(popupPos);
+    const QAction* action = m_subDirsMenu->exec(popupPos);
     if (action != 0) {
         const int result = action->data().toInt();
         KUrl url = m_url;
         url.addPath(m_subDirs[result].first);
-        urlNavigator()->setUrl(url);
+        emit clicked(url, Qt::LeftButton);
     }
 
     m_subDirs.clear();
-    delete m_dirsMenu;
-    m_dirsMenu = 0;
+    delete m_subDirsMenu;
+    m_subDirsMenu = 0;
 
     setDisplayHintEnabled(PopupActiveHint, false);
 }
 
+void KUrlNavigatorButton::replaceButton(KJob* job)
+{
+    Q_ASSERT(job == m_subDirsJob);
+    m_subDirsJob = 0;
+    m_replaceButton = false;
+
+    if (job->error() || m_subDirs.isEmpty()) {
+        return;
+    }
+
+    qSort(m_subDirs.begin(), m_subDirs.end(), naturalLessThan);
+
+    // Get index of the directory that is shown currently in the button
+    const QString currentDir = m_url.fileName();
+    int currentIndex = 0;
+    const int subDirsCount = m_subDirs.count();
+    while (currentIndex < subDirsCount) {
+        if (m_subDirs[currentIndex].first == currentDir) {
+            break;
+        }
+        ++currentIndex;
+    }
+
+    // Adjust the index by respecting the wheel steps and
+    // trigger a replacing of the button content
+    int targetIndex = currentIndex - m_wheelSteps;
+    if (targetIndex < 0) {
+        targetIndex = 0;
+    } else if (targetIndex >= subDirsCount) {
+        targetIndex = subDirsCount - 1;
+    }
+
+    KUrl url = m_url.upUrl();
+    url.addPath(m_subDirs[targetIndex].first);
+
+    emit clicked(url, Qt::LeftButton);
+
+    m_subDirs.clear();
+}
+
+void KUrlNavigatorButton::cancelSubDirsRequest()
+{
+    m_openSubDirsTimer->stop();
+    if (m_subDirsJob != 0) {
+        m_subDirsJob->kill();
+        m_subDirsJob = 0;
+    }
+}
+
 int KUrlNavigatorButton::arrowWidth() const
 {
     // if there isn't arrow then return 0
     int width = 0;
-    if (!isDisplayHintEnabled(ActivatedHint)) {
+    if (!m_subDir.isEmpty()) {
         width = height() / 2;
         if (width < 4) {
             width = 4;
@@ -505,12 +538,13 @@ bool KUrlNavigatorButton::isAboveArrow(int x) const
 bool KUrlNavigatorButton::isTextClipped() const
 {
     int availableWidth = width() - 2 * BorderWidth;
-    if (!isDisplayHintEnabled(ActivatedHint)) {
+    if (!m_subDir.isEmpty()) {
         availableWidth -= arrowWidth() - BorderWidth;
     }
 
-    QFontMetrics fontMetrics(font());
-    return fontMetrics.width(text()) >= availableWidth;
+    QFont adjustedFont(font());
+    adjustedFont.setBold(m_subDir.isEmpty());
+    return QFontMetrics(adjustedFont).width(text()) >= availableWidth;
 }
 
 void KUrlNavigatorButton::updateMinimumWidth()
@@ -530,4 +564,40 @@ void KUrlNavigatorButton::updateMinimumWidth()
     }
 }
 
+void KUrlNavigatorButton::initMenu(KUrlNavigatorMenu* menu, int startIndex)
+{
+    connect(menu, SIGNAL(middleMouseButtonClicked(QAction*)),
+            this, SLOT(slotMenuActionClicked(QAction*)));
+    connect(menu, SIGNAL(urlsDropped(QAction*, QDropEvent*)),
+            this, SLOT(urlsDropped(QAction*, QDropEvent*)));
+
+    menu->setLayoutDirection(Qt::LeftToRight);
+
+    const int maxIndex = startIndex + 30;  // Don't show more than 30 items in a menu
+    const int lastIndex = qMin(m_subDirs.count() - 1, maxIndex);
+    for (int i = startIndex; i <= lastIndex; ++i) {
+        const QString subDirName = m_subDirs[i].first;
+        const QString subDirDisplayName = m_subDirs[i].second;
+        QString text = KStringHandler::csqueeze(subDirDisplayName, 60);
+        text.replace(QLatin1Char('&'), QLatin1String("&&"));
+        QAction* action = new QAction(text, this);
+        if (m_subDir == subDirName) {
+            QFont font(action->font());
+            font.setBold(true);
+            action->setFont(font);
+        }
+        action->setData(i);
+        menu->addAction(action);
+    }
+    if (m_subDirs.count() > maxIndex) {
+        // If too much items are shown, move them into a sub menu
+        menu->addSeparator();
+        KUrlNavigatorMenu* subDirsMenu = new KUrlNavigatorMenu(menu);
+        subDirsMenu->setTitle(i18nc("@action:inmenu", "More"));
+        initMenu(subDirsMenu, maxIndex);
+        menu->addMenu(subDirsMenu);
+    }
+}
+
+
 #include "kurlnavigatorbutton_p.moc"
