@@ -324,6 +324,56 @@ CSSStyleDeclaration Element::style()
     return 0;
 }
 
+Element Element::firstElementChild() const
+{
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    return static_cast<ElementImpl*>(impl)->firstElementChild();
+}
+
+Element Element::lastElementChild() const
+{
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    return static_cast<ElementImpl*>(impl)->lastElementChild();
+}
+
+Element Element::previousElementSibling() const
+{
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    return static_cast<ElementImpl*>(impl)->previousElementSibling();
+}
+
+Element Element::nextElementSibling() const
+{
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    return static_cast<ElementImpl*>(impl)->nextElementSibling();
+}
+
+unsigned long Element::childElementCount() const
+{
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    return static_cast<ElementImpl*>(impl)->childElementCount();
+}
+
+Element Element::querySelector(const DOMString& query) const
+{
+    int ec = 0;
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    Element res = impl->querySelector(query, ec).get();
+    if (ec)
+        throw DOMException(ec);
+    return res;
+}
+
+NodeList Element::querySelectorAll(const DOMString& query) const
+{
+    int ec = 0;
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    NodeList res = impl->querySelectorAll(query, ec).get();
+    if (ec)
+        throw DOMException(ec);
+    return res;
+}
+
 bool Element::khtmlValidAttrName(const DOMString &name)
 {
     // Check if name is valid
