@@ -705,6 +705,9 @@ KJS_IMPLEMENT_PROTOTYPE("CanvasPatternProto", CanvasPatternProto, CanvasPatternF
 
 JSValue *CanvasPatternFunction::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
 {
+    Q_UNUSED(exec);
+    Q_UNUSED(thisObj);
+    Q_UNUSED(args);
     assert(0);
     return NULL;
 }
@@ -734,6 +737,11 @@ void CanvasImageData::mark()
         data->mark();
 }
 
+JSObject* CanvasImageData::valueClone(Interpreter* targetCtx) const
+{
+    return static_cast<JSObject*>(getWrapper<CanvasImageData>(targetCtx->globalExec(), impl()->clone()));
+}
+
 const ClassInfo CanvasImageDataArray::info = { "ImageDataArray", 0, 0, 0 };
 
 CanvasImageDataArray::CanvasImageDataArray(ExecState* exec, CanvasImageData* p) :
@@ -752,6 +760,7 @@ void CanvasImageDataArray::mark()
 
 JSValue* CanvasImageDataArray::indexGetter(ExecState* exec, unsigned index)
 {
+    Q_UNUSED(exec);
     if (index >= size) // paranoia..
         return jsNull();
 
