@@ -39,9 +39,11 @@ namespace KJS {
     DOMNode(JSObject *proto, DOM::NodeImpl* n);
     ~DOMNode();
     virtual bool toBoolean(ExecState *) const;
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
 
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     void putValueProperty(ExecState *exec, int token, JSValue* value, int attr);
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -54,7 +56,7 @@ namespace KJS {
     virtual void pushEventHandlerScope(ExecState *exec, ScopeChain &scope) const;
 
     enum { NodeName, NodeValue, NodeType, ParentNode, ParentElement,
-           ChildNodes, FirstChild, LastChild, PreviousSibling, NextSibling, Item,
+           ChildNodes, FirstChild, LastChild, PreviousSibling, NextSibling, 
            Attributes, NamespaceURI, Prefix, LocalName, OwnerDocument, InsertBefore,
            ReplaceChild, RemoveChild, AppendChild, HasAttributes, HasChildNodes,
            CloneNode, Normalize, IsSupported, AddEventListener, RemoveEventListener,
@@ -83,7 +85,7 @@ namespace KJS {
     ~DOMNodeList();
 
     JSValue* indexGetter(ExecState *exec, unsigned index);
-    
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     virtual JSValue* callAsFunction(ExecState *exec, JSObject* thisObj, const List& args);
     virtual bool implementsCall() const { return true; }
@@ -115,9 +117,11 @@ namespace KJS {
     DOMDocument(ExecState *exec,  DOM::DocumentImpl* d);
     DOMDocument(JSObject *proto, DOM::DocumentImpl* d);
 
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     void putValueProperty(ExecState *exec, int token, JSValue* value, int attr);
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -133,7 +137,9 @@ namespace KJS {
            PreferredStylesheetSet, SelectedStylesheetSet, ReadyState, Async,
            GetElementsByClassName, Title, ExecCommand, QueryCommandEnabled,
            QueryCommandIndeterm, QueryCommandState, QueryCommandSupported,
-           QueryCommandValue };
+           QueryCommandValue, QuerySelector, QuerySelectorAll,
+           CreateExpression, CreateNSResolver, Evaluate
+           };
     DOM::DocumentImpl* impl() { return static_cast<DOM::DocumentImpl*>(m_impl.get()); }
   };
   
@@ -141,12 +147,24 @@ namespace KJS {
 
   DEFINE_PSEUDO_CONSTRUCTOR(DocumentPseudoCtor)
 
+  class DOMDocumentFragment : public DOMNode {
+  public:
+    DOMDocumentFragment(ExecState* exec, DOM::DocumentFragmentImpl* i);
+    virtual const ClassInfo* classInfo() const { return &info; }
+    static const ClassInfo info;
+
+    enum { QuerySelector, QuerySelectorAll };
+  };
+  DEFINE_PSEUDO_CONSTRUCTOR(DocumentFragmentPseudoCtor)
+
   class DOMAttr : public DOMNode {
   public:
     DOMAttr(ExecState *exec, DOM::AttrImpl* a) : DOMNode(exec, a) { }
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     void putValueProperty(ExecState *exec, int token, JSValue* value, int attr);
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -159,6 +177,7 @@ namespace KJS {
     // Build a DOMElement
     DOMElement(ExecState *exec, DOM::ElementImpl* e);
     DOMElement(JSObject *proto, DOM::ElementImpl* e);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
 
@@ -171,7 +190,7 @@ namespace KJS {
            SetAttributeNode, RemoveAttributeNode, GetElementsByTagName,
            GetAttributeNS, SetAttributeNS, RemoveAttributeNS, GetAttributeNodeNS,
            SetAttributeNodeNS, GetElementsByTagNameNS, HasAttribute, HasAttributeNS,
-           GetElementsByClassName, Blur, Focus };
+           GetElementsByClassName, Blur, Focus, QuerySelector, QuerySelectorAll };
   private:
 #if 0
     static JSValue *attributeGetter(ExecState *exec, JSObject*, const Identifier&, const PropertySlot& slot);
@@ -205,6 +224,7 @@ namespace KJS {
     // Build a DOMDocumentType
     DOMDocumentType(ExecState *exec, DOM::DocumentTypeImpl* dt);
 
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
@@ -218,6 +238,7 @@ namespace KJS {
     DOMNamedNodeMap(ExecState *, DOM::NamedNodeMapImpl* m);
     ~DOMNamedNodeMap();
 
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
@@ -239,9 +260,11 @@ namespace KJS {
   public:
     DOMProcessingInstruction(ExecState *exec, DOM::ProcessingInstructionImpl* pi) : DOMNode(exec, pi) { }
 
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -252,6 +275,7 @@ namespace KJS {
   public:
     DOMNotation(ExecState *exec, DOM::NotationImpl* n) : DOMNode(exec, n) { }
 
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
@@ -263,6 +287,7 @@ namespace KJS {
   class DOMEntity : public DOMNode {
   public:
     DOMEntity(ExecState *exec, DOM::EntityImpl* e) : DOMNode(exec, e) { }
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
@@ -293,6 +318,7 @@ namespace KJS {
   class DOMNamedNodesCollection : public DOMObject {
   public:
     DOMNamedNodesCollection(ExecState *exec, const QList<SharedPtr<DOM::NodeImpl> >& nodes );
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -309,8 +335,10 @@ namespace KJS {
   public:
     // Build a DOMCharacterData
     DOMCharacterData(ExecState *exec, DOM::CharacterDataImpl* d);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState *, int token) const;
+    using KJS::JSObject::put;
     virtual void put(ExecState *exec, const Identifier &propertyName, JSValue* value, int attr = None);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -322,6 +350,7 @@ namespace KJS {
   class DOMText : public DOMCharacterData {
   public:
     DOMText(ExecState *exec, DOM::TextImpl* t);
+    using KJS::JSObject::getOwnPropertySlot;
     virtual bool getOwnPropertySlot(ExecState* exec, const Identifier& propertyName, PropertySlot& slot);
     JSValue* getValueProperty(ExecState* exec, int token) const;
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -330,6 +359,14 @@ namespace KJS {
     enum { SplitText, WholeText, ReplaceWholeText };
   };
 
+  class DOMComment : public DOMCharacterData {
+  public:
+    DOMComment(ExecState *exec, DOM::CommentImpl* t);
+    virtual const ClassInfo* classInfo() const { return &info; }
+    static const ClassInfo info;
+    DOM::CommentImpl* impl() const { return static_cast<DOM::CommentImpl*>(m_impl.get()); }
+  };
+
 } // namespace
 
 #endif
