@@ -53,7 +53,7 @@ namespace KJS {
            Images, Applets, Links, Forms, Layers, Anchors, Scripts, All, Clear, Open, Close,
            Write, WriteLn, GetElementsByName, GetSelection, CaptureEvents, ReleaseEvents,
            BgColor, FgColor, AlinkColor, LinkColor, VlinkColor, LastModified,
-           Height, Width, Dir, Frames, CompatMode, DesignMode };
+           Height, Width, Dir, Frames, CompatMode, DesignMode, ActiveElement };
     DOM::HTMLDocumentImpl* impl() const { return static_cast<DOM::HTMLDocumentImpl*>( m_impl.get() ); }
   private:
     static JSValue *nameGetter(ExecState *exec, JSObject*, const Identifier& name, const PropertySlot& slot);
@@ -95,12 +95,12 @@ namespace KJS {
            BaseHref, BaseTarget, IsIndexForm, IsIndexPrompt, StyleDisabled,
            StyleSheet, StyleType, StyleMedia, BodyBackground, BodyVLink, BodyText,
            BodyLink, BodyALink, BodyBgColor, BodyOnLoad, BodyOnBlur, BodyOnFocus,
-           BodyOnError, BodyFocus,
+           BodyOnError, BodyOnMessage, BodyFocus,
            FormAction, FormEncType, FormElements, FormLength, FormAcceptCharset,
            FormReset, FormTarget, FormName, FormMethod, FormSubmit, SelectAdd,
            SelectTabIndex, SelectValue, SelectSelectedIndex, SelectLength,
            SelectRemove, SelectForm, SelectType, SelectOptions,
-           SelectDisabled, SelectMultiple, SelectName, SelectSize, 
+           SelectDisabled, SelectMultiple, SelectName, SelectSize, SelectItem,  
            OptGroupDisabled, OptGroupLabel, OptionIndex, OptionSelected,
            OptionForm, OptionText, OptionDefaultSelected, OptionDisabled,
            OptionLabel, OptionValue, InputReadOnly, InputAccept,
@@ -155,7 +155,7 @@ namespace KJS {
            TableCellWidth, TableCellCellIndex, TableCellChOff, TableCellBgColor,
            TableCellCh, TableCellVAlign, TableCellRowSpan, TableCellHeaders,
            TableCellAlign, TableCellAxis, TableCellScope, FrameSetCols,
-           FrameSetRows, FrameSrc, FrameLocation, FrameFrameBorder, FrameScrolling,
+           FrameSetRows, FrameSetOnMessage, FrameSrc, FrameLocation, FrameFrameBorder, FrameScrolling,
            FrameMarginWidth, FrameLongDesc, FrameMarginHeight, FrameName,
            FrameContentDocument, FrameContentWindow,
            FrameNoResize, FrameWidth, FrameHeight, IFrameLongDesc, IFrameAlign,
