@@ -174,6 +174,10 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
         document()->setHTMLWindowEventListener(EventImpl::SCROLL_EVENT,
             document()->createHTMLEventListener(attr->value().string(), "onscroll", NULL));
         break;
+    case ATTR_ONMESSAGE:
+        document()->setHTMLWindowEventListener(EventImpl::MESSAGE_EVENT,
+            document()->createHTMLEventListener(attr->value().string(), "onmessage", NULL));
+        break;        
     case ATTR_NOSAVE:
 	break;
     default:
@@ -273,6 +277,11 @@ void HTMLFrameElementImpl::ensureUniqueName()
     // Generate synthetic name if there isn't a natural one or
     // if the natural one conflicts
     KHTMLPart* parentPart = document()->part();
+    
+    // If there is no part, we will not load anything, so no 
+    // worry about names being unique or not.
+    if (!parentPart)
+        return;
 
     KHTMLPart* otherFrame = parentPart->findFrame( name.string() );
     if (name.isEmpty() || (otherFrame && otherFrame != contentPart()))
@@ -603,6 +612,10 @@ void HTMLFrameSetElementImpl::parseAttribute(AttributeImpl *attr)
         document()->setHTMLWindowEventListener(EventImpl::UNLOAD_EVENT,
             document()->createHTMLEventListener(attr->value().string(), "onunload", NULL));
         break;
+    case ATTR_ONMESSAGE:
+        document()->setHTMLWindowEventListener(EventImpl::MESSAGE_EVENT,
+            document()->createHTMLEventListener(attr->value().string(), "onmessage", NULL));
+        break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
