@@ -28,7 +28,6 @@
 #include "html/html_imageimpl.h"
 #include "html/html_documentimpl.h"
 
-#include "misc/htmlhashes.h"
 #include "khtmlview.h"
 #include "khtml_part.h"
 #include "css/csshelper.h"
@@ -92,7 +91,7 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
         if ( e && e->button() == 1 )
             utarget = "_blank";
 
-        if ( evt->target()->id() == ID_IMG ) {
+        if ( static_cast<NodeImpl*>(evt->target())->id() == ID_IMG ) {
             HTMLImageElementImpl* img = static_cast<HTMLImageElementImpl*>( evt->target() );
             if ( img && img->isServerMap() )
             {
@@ -150,11 +149,13 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
 	    }
 
 	    if (document()->part() && !isContentEditable()) {
-	      if (k) 
+	      if (k)
 		click();
-	      else
-		document()->part()->
-		    urlSelected( url, button, state, utarget );
+	      else {
+                KParts::OpenUrlArguments args;
+                args.setActionRequestedByUser(true);
+                document()->part()->urlSelected( url, button, state, utarget, args );
+              }
 	    }
         }
         evt->setDefaultHandled();
