@@ -33,6 +33,10 @@
 #include "misc/loader.h"
 #include "dom/html_form.h"
 #include "dom/html_image.h"
+#include "dom/dom_string.h"
+#include "dom/html_document.h"
+#include "dom/dom_element.h"
+#include "xml/dom_elementimpl.h"
 #include <QtGui/QClipboard>
 #include <QtCore/QFileInfo>
 #include <QtGui/QMenu>
@@ -64,9 +68,6 @@
 #include <kactioncollection.h>
 #include <kactionmenu.h>
 
-#include "dom/dom_element.h"
-#include "misc/htmltags.h"
-
 #include "khtmlpart_p.h"
 
 KHTMLPartBrowserExtension::KHTMLPartBrowserExtension( KHTMLPart *parent )
@@ -226,25 +227,25 @@ void KHTMLPartBrowserExtension::copy()
         QClipboard *cb = QApplication::clipboard();
         disconnect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
 #ifndef QT_NO_MIMECLIPBOARD
-	QString htmltext;
-	/*
-	 * When selectionModeEnabled, that means the user has just selected
-	 * the text, not ctrl+c to copy it.  The selection clipboard
-	 * doesn't seem to support mime type, so to save time, don't calculate
-	 * the selected text as html.
-	 * optomisation disabled for now until everything else works.
-	*/
-	//if(!cb->selectionModeEnabled())
-	    htmltext = m_part->selectedTextAsHTML();
-	QMimeData *mimeData = new QMimeData;
-	mimeData->setText(text);
-	if(!htmltext.isEmpty()) {
-	    htmltext.replace( QChar( 0xa0 ), ' ' );
-	    mimeData->setHtml(htmltext);
-	}
+  QString htmltext;
+  /*
+   * When selectionModeEnabled, that means the user has just selected
+   * the text, not ctrl+c to copy it.  The selection clipboard
+   * doesn't seem to support mime type, so to save time, don't calculate
+   * the selected text as html.
+   * optomisation disabled for now until everything else works.
+  */
+  //if(!cb->selectionModeEnabled())
+      htmltext = m_part->selectedTextAsHTML();
+  QMimeData *mimeData = new QMimeData;
+  mimeData->setText(text);
+  if(!htmltext.isEmpty()) {
+      htmltext.replace( QChar( 0xa0 ), ' ' );
+      mimeData->setHtml(htmltext);
+  }
         cb->setMimeData(mimeData);
 #else
-	cb->setText(text);
+  cb->setText(text);
 #endif
 
         connect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
@@ -262,27 +263,19 @@ void KHTMLPartBrowserExtension::copy()
 
 void KHTMLPartBrowserExtension::searchProvider()
 {
-    // action name is of form "previewProvider[<searchproviderprefix>:]"
-    const QString searchProviderPrefix = QString( sender()->objectName() ).mid( 14 );
+    KAction *action = qobject_cast<KAction*>(sender());
+    if (action) {
+        KUrl url = action->data().toUrl();
+        if (url.host().isEmpty()) {
+            KUriFilterData data(action->data().toString());
+            if (KUriFilter::self()->filterSearchUri(data, KUriFilter::WebShortcutFilter))
+                url = data.uri();
+        }
 
-    const QString text = m_part->simplifiedSelectedText();
-    KUriFilterData data;
-    QStringList list;
-    data.setData( searchProviderPrefix + text );
-    list << "kurisearchfilter" << "kuriikwsfilter";
-
-    if( !KUriFilter::self()->filterUri(data, list) )
-    {
-        KDesktopFile file("services", "searchproviders/google.desktop");
-        QString encodedSearchTerm = QUrl::toPercentEncoding(text);
-        KConfigGroup cg(file.desktopGroup());
-        data.setData(cg.readEntry("Query").replace("\\{@}", encodedSearchTerm));
+        KParts::BrowserArguments browserArgs;
+        browserArgs.frameName = "_blank";
+        emit m_part->browserExtension()->openUrlRequest( url, KParts::OpenUrlArguments(), browserArgs );
     }
-
-    KParts::BrowserArguments browserArgs;
-    browserArgs.frameName = "_blank";
-
-    emit m_part->browserExtension()->openUrlRequest( data.uri(), KParts::OpenUrlArguments(), browserArgs );
 }
 
 void KHTMLPartBrowserExtension::paste()
@@ -344,11 +337,11 @@ void KHTMLPartBrowserExtension::updateEditActions()
 }
 
 void KHTMLPartBrowserExtension::extensionProxyEditableWidgetFocused() {
-	editableWidgetFocused();
+  editableWidgetFocused();
 }
 
 void KHTMLPartBrowserExtension::extensionProxyEditableWidgetBlurred() {
-	editableWidgetBlurred();
+  editableWidgetBlurred();
 }
 
 void KHTMLPartBrowserExtension::extensionProxyActionEnabled( const char *action, bool enable )
@@ -430,7 +423,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
         addSearchActions(editActions);
 
-        QString selectedTextURL = selectedTextAsOneLine();
+        QString selectedTextURL = selectedTextAsOneLine(d->m_khtml);
         if ( selectedTextURL.contains("://") && KUrl(selectedTextURL).isValid() ) {
             if (selectedTextURL.length() > 18) {
                 selectedTextURL.truncate(15);
@@ -632,96 +625,48 @@ KHTMLPopupGUIClient::~KHTMLPopupGUIClient()
 
 void KHTMLPopupGUIClient::addSearchActions(QList<QAction *>& editActions)
 {
-    // Fill search provider entries
-    KConfig config("kuriikwsfilterrc");
-    KConfigGroup cg = config.group("General");
-    const QString defaultEngine = cg.readEntry("DefaultSearchEngine", "google");
-    const char keywordDelimiter = cg.readEntry("KeywordDelimiter", static_cast<int>(':'));
-
-    // search text
-    QString selectedText = d->m_khtml->simplifiedSelectedText();
+    const QString selectedText = d->m_khtml->simplifiedSelectedText();
     if (selectedText.isEmpty())
         return;
 
-    selectedText.replace("&", "&&");
-    if (selectedText.length() > 18) {
-        selectedText.truncate(15);
-        selectedText += "...";
-    }
+    KUriFilterData data(selectedText);
+    QStringList alternateProviders;
+    alternateProviders << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
+    data.setAlternateSearchProviders(alternateProviders);
+    data.setAlternateDefaultSearchProvider("google");
 
-    // default search provider
-    KService::Ptr service;
-    if( !defaultEngine.isEmpty())
-        service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(defaultEngine));
+    if (KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter)) {
+        const QString squeezedText = KStringHandler::rsqueeze(selectedText, 21);
+        KAction *action = new KAction(i18n("Search for '%1' with %2",
+                                           squeezedText, data.searchProvider()), this);
+        action->setData(QUrl(data.uri()));
+        action->setIcon(KIcon(data.iconName()));
+        connect(action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(searchProvider()));
+        d->m_actionCollection->addAction("defaultSearchProvider", action);
+        editActions.append(action);
 
-    // search provider icon
-    KIcon icon;
-    KUriFilterData data;
-    QStringList list;
-    data.setData(QString("some keyword"));
-    list << "kurisearchfilter" << "kuriikwsfilter";
-
-    QString name;
-    if (KUriFilter::self()->filterUri(data, list)) {
-        QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-        if (iconPath.isEmpty())
-            icon = KIcon("edit-find");
-        else
-            icon = KIcon(QPixmap(iconPath));
-        name = service->name();
-    } else {
-        icon = KIcon("google");
-        name = "Google";
-    }
-
-    KAction *action = new KAction(i18n("Search for '%1' with %2", selectedText, name), this);
-    d->m_actionCollection->addAction("searchProvider", action);
-    editActions.append(action);
-    action->setIcon(icon);
-    connect(action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(searchProvider()));
-
-    // favorite search providers
-    QStringList favoriteEngines;
-    favoriteEngines << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
-    favoriteEngines = cg.readEntry("FavoriteSearchEngines", favoriteEngines);
-
-    if (!favoriteEngines.isEmpty()) {
-        KActionMenu* providerList = new KActionMenu(i18n("Search for '%1' with", selectedText), this);
-        d->m_actionCollection->addAction("searchProviderList", providerList);
-        editActions.append(providerList);
-
-        QStringList::ConstIterator it = favoriteEngines.constBegin();
-        for (; it != favoriteEngines.constEnd(); ++it) {
-            if (*it==defaultEngine)
-                continue;
-            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(*it));
-            if (!service)
-                continue;
-            const QString searchProviderPrefix = *(service->property("Keys").toStringList().begin()) + keywordDelimiter;
-            data.setData(searchProviderPrefix + "some keyword");
-
-            if (KUriFilter::self()->filterUri(data, list)) {
-                const QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-                if (iconPath.isEmpty())
-                    icon = KIcon("edit-find");
-                else
-                    icon = KIcon(iconPath);
-                name = service->name();
-
-                KAction *action = new KAction(name, this);
-                d->m_actionCollection->addAction(QString("searchProvider" + searchProviderPrefix).toLatin1().constData(), action);
-                action->setIcon(icon);
+        const QStringList preferredSearchProviders = data.preferredSearchProviders();
+        if (!preferredSearchProviders.isEmpty()) {
+            KActionMenu* providerList = new KActionMenu(i18n("Search for '%1' with", squeezedText), this);
+            Q_FOREACH(const QString &searchProvider, preferredSearchProviders) {
+                if (searchProvider == data.searchProvider())
+                    continue;
+                KAction *action = new KAction(searchProvider, this);
+                action->setData(data.queryForPreferredSearchProvider(searchProvider));
+                d->m_actionCollection->addAction(searchProvider, action);
+                action->setIcon(KIcon(data.iconNameForPreferredSearchProvider(searchProvider)));
                 connect(action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(searchProvider()));
-
                 providerList->addAction(action);
             }
+            d->m_actionCollection->addAction("searchProviderList", providerList);
+            editActions.append(providerList);
         }
     }
 }
 
-QString KHTMLPopupGUIClient::selectedTextAsOneLine() const
+QString KHTMLPopupGUIClient::selectedTextAsOneLine(KHTMLPart* part)
 {
-    QString text = d->m_khtml->simplifiedSelectedText();
+    QString text = part->simplifiedSelectedText();
     // in addition to what simplifiedSelectedText does,
     // remove linefeeds and any whitespace surrounding it (#113177),
     // to get it all in a single line.
@@ -734,7 +679,7 @@ void KHTMLPopupGUIClient::openSelection()
     KParts::BrowserArguments browserArgs;
     browserArgs.frameName = "_blank";
 
-    emit d->m_khtml->browserExtension()->openUrlRequest(selectedTextAsOneLine(), KParts::OpenUrlArguments(), browserArgs);
+    emit d->m_khtml->browserExtension()->openUrlRequest(selectedTextAsOneLine(d->m_khtml), KParts::OpenUrlArguments(), browserArgs);
 }
 
 KParts::BrowserExtension::ActionGroupMap KHTMLPopupGUIClient::actionGroups() const
@@ -885,8 +830,9 @@ void KHTMLPopupGUIClient::slotFrameInWindow()
 {
   KParts::OpenUrlArguments args = d->m_khtml->arguments();
   args.metaData()["referrer"] = d->m_khtml->pageReferrer();
-  args.metaData()["forcenewwindow"] = "true";
-  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args );
+  KParts::BrowserArguments browserArgs( d->m_khtml->browserExtension()->browserArguments() );
+  browserArgs.setForcesNewWindow(true);
+  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args, browserArgs );
 }
 
 void KHTMLPopupGUIClient::slotFrameInTop()
@@ -916,14 +862,15 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
   QString name = QLatin1String( "index.html" );
   if ( !suggestedFilename.isEmpty() )
     name = suggestedFilename;
-  else if ( !url.fileName().isEmpty() )
-    name = url.fileName();
+  else if ( !url.fileName(KUrl::ObeyTrailingSlash).isEmpty() )
+    name = url.fileName(KUrl::ObeyTrailingSlash);
 
   KUrl destURL;
   int query;
   do {
     query = KMessageBox::Yes;
-    destURL = KFileDialog::getSaveUrl( name, filter, parent, caption );
+    // convert filename to URL using fromPath to avoid trouble with ':' in filenames (#184202)
+    destURL = KFileDialog::getSaveUrl( KUrl::fromPath(name), filter, parent, caption );
       if( destURL.isLocalFile() )
       {
         QFileInfo info( destURL.toLocalFile() );
@@ -1011,11 +958,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget* parent, const KUrl &url, const KUrl
 
           if ( downloadViaKIO )
           {
-              KIO::Job *job = KIO::file_copy( url, destURL, -1, KIO::Overwrite );
-              job->setMetaData(metadata);
-              job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
-              job->addMetaData("cache", "cache"); // Use entry from cache if available.
-              job->uiDelegate()->setAutoErrorHandlingEnabled( true );
+              KParts::BrowserRun::saveUrlUsingKIO(url, destURL, parent, metadata);
           }
         } //end if(!saved)
     }
@@ -1049,7 +992,7 @@ bool KHTMLPartBrowserHostExtension::openUrlInFrame(const KUrl &url, const KParts
 KParts::BrowserHostExtension* KHTMLPartBrowserHostExtension::findFrameParent( KParts::ReadOnlyPart
       *callingPart, const QString &frame )
 {
-    KHTMLPart *parentPart = m_part->findFrameParent(callingPart, frame);
+    KHTMLPart *parentPart = m_part->d->findFrameParent(callingPart, frame, 0, true /* navigation*/);
     if (parentPart)
        return parentPart->browserHostExtension();
     return 0;
@@ -1111,5 +1054,161 @@ void KHTMLZoomFactorAction::slotTriggered(QAction* action)
     setCurrentAction( 0L );
 }
 
-#include "khtml_ext.moc"
+KHTMLTextExtension::KHTMLTextExtension(KHTMLPart* part)
+    : KParts::TextExtension(part)
+{
+    connect(part, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
+}
 
+KHTMLPart* KHTMLTextExtension::part() const
+{
+    return static_cast<KHTMLPart*>(parent());
+}
+
+bool KHTMLTextExtension::hasSelection() const
+{
+    return part()->hasSelection();
+}
+
+QString KHTMLTextExtension::selectedText(Format format) const
+{
+    switch(format) {
+    case PlainText:
+        return part()->selectedText();
+    case HTML:
+        return part()->selectedTextAsHTML();
+    }
+    return QString();
+}
+
+QString KHTMLTextExtension::completeText(Format format) const
+{
+    switch(format) {
+    case PlainText:
+        return part()->htmlDocument().body().innerText().string();
+    case HTML:
+        return part()->htmlDocument().body().innerHTML().string();
+    }
+    return QString();
+}
+
+////
+
+KHTMLHtmlExtension::KHTMLHtmlExtension(KHTMLPart* part)
+    : KParts::HtmlExtension(part)
+{
+}
+
+KUrl KHTMLHtmlExtension::baseUrl() const
+{
+    return part()->baseURL();
+}
+
+bool KHTMLHtmlExtension::hasSelection() const
+{
+    return part()->hasSelection();
+}
+
+KParts::SelectorInterface::QueryMethods KHTMLHtmlExtension::supportedQueryMethods() const
+{
+    return (KParts::SelectorInterface::SelectedContent | KParts::SelectorInterface::EntireContent);
+}
+
+static KParts::SelectorInterface::Element convertDomElement(const DOM::ElementImpl* domElem)
+{
+    KParts::SelectorInterface::Element elem;
+    elem.setTagName(domElem->tagName().string());
+    const DOM::NamedAttrMapImpl* attrMap = domElem->attributes(true /*readonly*/);
+    if (attrMap) {
+        for (unsigned i = 0; i < attrMap->length(); ++i) {
+            const DOM::AttributeImpl& attr = attrMap->attributeAt(i);
+            elem.setAttribute(attr.localName().string(), attr.value().string());
+            // we could have a setAttributeNS too.
+        }
+    }
+    return elem;
+}
+
+KParts::SelectorInterface::Element KHTMLHtmlExtension::querySelector(const QString& query, KParts::SelectorInterface::QueryMethod method) const
+{
+    KParts::SelectorInterface::Element element;
+
+    // If the specified method is None, return an empty list; similarly
+    // if the document is null, which may be possible in case of an error
+    if (method == KParts::SelectorInterface::None || part()->document().isNull())
+        return element;
+
+    if (!(supportedQueryMethods() & method))
+        return element;
+
+    switch (method) {
+    case KParts::SelectorInterface::EntireContent: {
+        int ec = 0; // exceptions are ignored
+        WTF::RefPtr<DOM::ElementImpl> domElem = part()->document().handle()->querySelector(query, ec);
+        element = convertDomElement(domElem.get());
+        break;
+    }    
+    case KParts::SelectorInterface::SelectedContent:
+        if (part()->hasSelection()) {
+            DOM::Element domElem = part()->selection().cloneContents().querySelector(query);
+            element = convertDomElement(static_cast<DOM::ElementImpl*>(domElem.handle()));
+        }
+        break;        
+    default:
+        break;
+    }
+
+    return element;
+}
+
+QList<KParts::SelectorInterface::Element> KHTMLHtmlExtension::querySelectorAll(const QString& query, KParts::SelectorInterface::QueryMethod method) const
+{
+    QList<KParts::SelectorInterface::Element> elements;
+
+    // If the specified method is None, return an empty list; similarly
+    // if the document is null, which may be possible in case of an error
+    if (method == KParts::SelectorInterface::None || part()->document().isNull())
+        return elements;
+
+    // If the specified method is not supported, return an empty list...
+    if (!(supportedQueryMethods() & method))
+        return elements;
+
+    switch (method) {
+    case KParts::SelectorInterface::EntireContent: {
+        int ec = 0; // exceptions are ignored
+        WTF::RefPtr<DOM::NodeListImpl> nodes = part()->document().handle()->querySelectorAll(query, ec);    
+        const unsigned long len = nodes->length();
+        elements.reserve(len);
+        for (unsigned long i = 0; i < len; ++i) {
+            DOM::NodeImpl* node = nodes->item(i);
+            if (node->isElementNode()) { // should be always true
+                elements.append(convertDomElement(static_cast<DOM::ElementImpl*>(node)));
+            }
+        }
+        break;
+    }
+    case KParts::SelectorInterface::SelectedContent:
+        if (part()->hasSelection()) {
+            DOM::NodeList nodes = part()->selection().cloneContents().querySelectorAll(query);
+            const unsigned long len = nodes.length();
+            for (unsigned long i = 0; i < len; ++i) {
+                DOM::NodeImpl* node = nodes.item(i).handle();
+                if (node->isElementNode())
+                    elements.append(convertDomElement(static_cast<DOM::ElementImpl*>(node)));
+            }
+        }
+        break;
+    default:
+        break;
+    }
+
+    return elements;
+}
+
+KHTMLPart* KHTMLHtmlExtension::part() const
+{
+    return static_cast<KHTMLPart*>(parent());
+}
+
+#include "khtml_ext.moc"
