@@ -55,7 +55,6 @@
 #include "render_form.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
-#include "misc/htmltags.h"
 #include "misc/paintbuffer.h"
 #include "html/html_blockimpl.h"
 #include "xml/dom_restyler.h"
@@ -459,7 +458,7 @@ RenderLayer *RenderLayer::stackingContext() const
     RenderLayer* curr = parent();
     for ( ; curr && !curr->m_object->isCanvas() &&
           curr->m_object->style()->hasAutoZIndex();
-          curr = curr->parent());
+          curr = curr->parent()) {};
     return curr;
 }
 
@@ -468,7 +467,7 @@ RenderLayer* RenderLayer::enclosingPositionedAncestor() const
     RenderLayer* curr = parent();
     for ( ; curr && !curr->m_object->isCanvas() &&
          !curr->m_object->isPositioned() && !curr->m_object->isRelPositioned();
-         curr = curr->parent());
+         curr = curr->parent()) {};
 
     return curr;
 }
@@ -481,7 +480,7 @@ bool RenderLayer::isTransparent() const
 RenderLayer* RenderLayer::transparentAncestor() const
 {
     RenderLayer* curr = parent();
-    for ( ; curr && curr->m_object->style()->opacity() == 1.0f; curr = curr->parent());
+    for ( ; curr && curr->m_object->style()->opacity() == 1.0f; curr = curr->parent()) {};
     return curr;
 }
 
@@ -921,10 +920,17 @@ void RenderLayer::checkScrollbarsAfterLayout()
         m_hBar->setEnabled(needHorizontalBar);
     if (hasOvf && m_object->style()->overflowY() == OSCROLL)
         m_vBar->setEnabled(needVerticalBar);
-
+        
+    // Sometimes we originally had a scrolling overflow, but it got changed to 
+    // hidden/visible. 
+    bool deadScrollX = m_hBar && !m_object->scrollsOverflowX();
+    bool deadScrollY = m_vBar && !m_object->scrollsOverflowY();
+        
     // overflow:auto may need to lay out again if scrollbars got added/removed.
+    // Also remove now useless scrollbars for non-scrollable overflows
     bool scrollbarsChanged = (hasOvf && m_object->style()->overflowX() == OAUTO && haveHorizontalBar != needHorizontalBar)
-                          || (hasOvf && m_object->style()->overflowY() == OAUTO && haveVerticalBar != needVerticalBar);
+                          || (hasOvf && m_object->style()->overflowY() == OAUTO && haveVerticalBar != needVerticalBar)
+                          || deadScrollX || deadScrollY;
     if (scrollbarsChanged && !m_inScrollbarRelayout) {
         if (m_object->style()->overflowX() == OAUTO) {
             showScrollbar(Qt::Horizontal, needHorizontalBar);
@@ -940,6 +946,16 @@ void RenderLayer::checkScrollbarsAfterLayout()
             else
                 resetYOffset();
         }
+        
+        if (deadScrollX) {
+            showScrollbar(Qt::Horizontal, false);
+            resetXOffset();
+        }
+            
+        if (deadScrollY) {
+            showScrollbar(Qt::Vertical, false);
+            resetYOffset();
+        }
 
         m_object->setNeedsLayout(true);
         m_inScrollbarRelayout = true;
@@ -950,7 +966,7 @@ void RenderLayer::checkScrollbarsAfterLayout()
         m_inScrollbarRelayout = false;
 	return;
     }
-
+    
     m_inScrollbarRelayout = false;
 
     // Set up the range (and page step/line step).
@@ -1681,6 +1697,16 @@ bool RenderLayer::shouldBeOverflowOnly() const
 
 void RenderLayer::styleChanged()
 {
+    // If we stopped being a stacking context, make sure to clear our
+    // child lists so we don't end up with dangling references when a kid
+    // is removed (as it wouldn't know to remove from us)
+    if (!isStackingContext() && (m_posZOrderList || m_negZOrderList)) {
+        delete m_posZOrderList;
+        m_posZOrderList = 0;
+        delete m_negZOrderList;
+        m_negZOrderList = 0;
+    }
+
     bool isOverflowOnly = shouldBeOverflowOnly();
     if (isOverflowOnly != m_isOverflowOnly) {
         m_isOverflowOnly = isOverflowOnly;
@@ -2000,3 +2026,4 @@ void Marquee::timerEvent(QTimerEvent* /*evt*/)
 }
 
 #include "render_layer.moc"
+// kate: indent-width 4; replace-tabs on; tab-width 8; space-indent on;
