@@ -45,11 +45,12 @@
 #include "dom/dom_string.h"
 #include "misc/khtmllayout.h"
 #include "misc/shared.h"
+#include "rendering/DataRef.h"
 #include "rendering/font.h"
+#include "rendering/SVGRenderStyle.h"
 
 #include <assert.h>
 
-#include "SVGRenderStyle.h"
 
 #define SET_VAR(group,variable,value) \
     if (!(group->variable == value)) \
@@ -73,78 +74,6 @@ namespace khtml {
     class CachedImage;
     class CachedObject;
 
-template <class DATA>
-class DataRef
-{
-public:
-
-    DataRef()
-    {
-	data=0;
-    }
-    DataRef( const DataRef<DATA> &d )
-    {
-    	data = d.data;
-	data->ref();
-    }
-
-    ~DataRef()
-    {
-    	if(data) data->deref();
-    }
-
-    const DATA* operator->() const
-    {
-    	return data;
-    }
-
-    const DATA* get() const
-    {
-    	return data;
-    }
-
-
-    DATA* access()
-    {
-    	if (!data->hasOneRef())
-	{
-	    data->deref();
-	    data = new DATA(*data);
-	    data->ref();
-	}
-	return data;
-    }
-
-    void init()
-    {
-    	data = new DATA;
-	data->ref();
-    }
-
-    DataRef<DATA>& operator=(const DataRef<DATA>& d)
-    {
-    	if (data==d.data)
-	    return *this;
-    	if (data)
-    	    data->deref();
-    	data = d.data;
-
-	data->ref();
-
-	return *this;
-    }
-
-    bool operator == ( const DataRef<DATA> &o ) const {
-	return (*data == *(o.data) );
-    }
-    bool operator != ( const DataRef<DATA> &o ) const {
-	return (*data != *(o.data) );
-    }
-
-private:
-    DATA* data;
-};
-
 
 //------------------------------------------------
 
@@ -416,7 +345,7 @@ public:
     StyleVisualData(const StyleVisualData& o );
 
     bool operator==( const StyleVisualData &o ) const {
-	return ( clip == o.clip &&
+	return ( clip == o.clip && textDecoration == o.textDecoration &&
 		 palette == o.palette );
     }
     bool operator!=( const StyleVisualData &o ) const {
@@ -439,6 +368,10 @@ enum EBackgroundRepeat {
     REPEAT, REPEAT_X, REPEAT_Y, NO_REPEAT
 };
 
+enum EBackgroundSizeType {
+    BGSLENGTH, BGSCONTAIN, BGSCOVER
+};
+
 enum EBackgroundAttachment {
     BGASCROLL, BGAFIXED, BGALOCAL
 };
@@ -461,6 +394,7 @@ public:
     EBackgroundBox backgroundOrigin() const { return KDE_CAST_BF_ENUM(EBackgroundBox, m_bgOrigin); }
     EBackgroundRepeat backgroundRepeat() const { return KDE_CAST_BF_ENUM(EBackgroundRepeat, m_bgRepeat); }
     LengthSize backgroundSize() const { return m_backgroundSize; }
+    EBackgroundSizeType backgroundSizeType() const { return KDE_CAST_BF_ENUM(EBackgroundSizeType, m_bgSizeType); }
 
     BackgroundLayer* next() const { return m_next; }
     BackgroundLayer* next() { return m_next; }
@@ -482,6 +416,7 @@ public:
     void setBackgroundOrigin(EBackgroundBox b) { m_bgOrigin = b; m_originSet = true; }
     void setBackgroundRepeat(EBackgroundRepeat r) { m_bgRepeat = r; m_repeatSet = true; }
     void setBackgroundSize(const LengthSize& b) { m_backgroundSize = b; m_backgroundSizeSet = true; }
+    void setBackgroundSizeType(EBackgroundSizeType t) { m_bgSizeType = t; m_backgroundSizeSet = true; }
 
     void clearBackgroundImage() { m_imageSet = false; }
     void clearBackgroundXPosition() { m_xPosSet = false; }
@@ -527,6 +462,7 @@ public:
     KDE_BF_ENUM(EBackgroundBox) m_bgClip : 2;
     KDE_BF_ENUM(EBackgroundBox) m_bgOrigin : 2;
     KDE_BF_ENUM(EBackgroundRepeat) m_bgRepeat : 2;
+    KDE_BF_ENUM(EBackgroundSizeType) m_bgSizeType : 2;
 
     LengthSize m_backgroundSize;
 
@@ -983,7 +919,7 @@ protected:
     RenderStyle* pseudoStyle;
 
 // SVG Style
-    DataRef<WebCore::SVGRenderStyle> m_svgStyle;
+    DataRef<khtml::SVGRenderStyle> m_svgStyle;
 
 // !END SYNC!
 
@@ -1047,6 +983,7 @@ public:
     ~RenderStyle();
 
     void inheritFrom(const RenderStyle* inheritParent);
+    void compactWith(const RenderStyle* similarStyle);
 
     PseudoId styleType() const { return KDE_CAST_BF_ENUM(PseudoId, noninherited_flags.f._styleType); }
     void setStyleType(PseudoId pi) { noninherited_flags.f._styleType = pi; }
@@ -1489,6 +1426,7 @@ public:
     static EBackgroundBox initialBackgroundOrigin() { return BGPADDING; }
     static EBackgroundRepeat initialBackgroundRepeat() { return REPEAT; }
     static LengthSize initialBackgroundSize() { return LengthSize(); }
+    static EBackgroundSizeType initialBackgroundSizeType() { return BGSLENGTH; }
     static bool initialBorderCollapse() { return false; }
     static EBorderStyle initialBorderStyle() { return BNONE; }
     static ECaptionSide initialCaptionSide() { return CAPTOP; }
@@ -1546,8 +1484,8 @@ public:
     static bool initialTextOverflow() { return false; }
 
     // SVG
-    const WebCore::SVGRenderStyle* svgStyle() const { return m_svgStyle.get(); }
-    WebCore::SVGRenderStyle* accessSVGStyle() { return m_svgStyle.access(); }
+    const khtml::SVGRenderStyle* svgStyle() const { return m_svgStyle.get(); }
+    khtml::SVGRenderStyle* accessSVGStyle() { return m_svgStyle.access(); }
 };
 
 class RenderPageStyle {
