@@ -59,8 +59,8 @@ void KFileMetaPropsPlugin::KFileMetaPropsPluginPrivate::configureShownMetaData()
     dialog->setDefaultButton(KDialog::Ok);
 
     QLabel* descriptionLabel  = new QLabel(i18nc("@label::textbox",
-                                                 "Configure which data should "
-                                                 "be shown"));
+                                                 "Select which data should "
+                                                 "be shown:"));
     descriptionLabel->setWordWrap(true);
 
     KFileMetaDataConfigurationWidget* configWidget = new KFileMetaDataConfigurationWidget();
