@@ -30,8 +30,11 @@
 #include <kio/scheduler.h>
 #include <kconfiggroup.h>
 #include <ksharedconfig.h>
+#include <kprotocolinfo.h>
+#include <klocalizedstring.h>
 
 #include <QtCore/QUrl>
+#include <QtCore/QWeakPointer>
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusReply>
@@ -44,16 +47,25 @@
 #define QL1S(x)   QLatin1String(x)
 #define QL1C(x)   QLatin1Char(x)
 
+static bool isLocalRequest(const KUrl& url)
+{
+    const QString scheme (url.protocol());
+    return (KProtocolInfo::isKnownProtocol(scheme) && 
+            KProtocolInfo::protocolClass(scheme).compare(QL1S(":local"), Qt::CaseInsensitive) == 0);
+}
+
 namespace KIO {
 
 class AccessManager::AccessManagerPrivate
 {
 public:
-    AccessManagerPrivate():externalContentAllowed(true) {}
-    KIO::MetaData metaDataForRequest(QNetworkRequest request);
-    bool isRequestAllowed(const QUrl& url) const;
+    AccessManagerPrivate() 
+    : externalContentAllowed(true)
+    {}
 
-    bool externalContentAllowed;
+    void setMetaDataForRequest(QNetworkRequest request, KIO::MetaData& metaData);
+    
+    bool externalContentAllowed;    
     KIO::MetaData requestMetaData;
     KIO::MetaData sessionMetaData;
 };
@@ -63,10 +75,15 @@ namespace Integration {
 class CookieJar::CookieJarPrivate
 {
 public:
-  CookieJarPrivate(): windowId((WId)-1), enabled(true) {}
+  CookieJarPrivate()
+    : windowId((WId)-1), 
+      isEnabled(true),
+      isStorageDisabled(false)
+  {}
 
   WId windowId;
-  bool enabled;
+  bool isEnabled;
+  bool isStorageDisabled;
 };
 
 }
@@ -125,15 +142,25 @@ KIO::MetaData& AccessManager::sessionMetaData()
     return d->sessionMetaData;
 }
 
+void AccessManager::putReplyOnHold(QNetworkReply* reply)
+{
+    KDEPrivate::AccessManagerReply* r = qobject_cast<KDEPrivate::AccessManagerReply*>(reply);
+    if (!r)
+      return;
+
+    r->putOnHold();
+}
+
 QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest &req, QIODevice *outgoingData)
 {
     KIO::SimpleJob *kioJob = 0;
-    const QUrl reqUrl (req.url());
-
-    if ( !d->isRequestAllowed(reqUrl) ) {
+    const KUrl reqUrl (req.url());
+    
+    if (!d->externalContentAllowed && !isLocalRequest(reqUrl)) {
         kDebug( 7044 ) << "Blocked: " << reqUrl;
-        /* if kioJob equals zero, the AccessManagerReply will block the request */
-        return new KDEPrivate::AccessManagerReply(op, req, kioJob, this);
+        KDEPrivate::AccessManagerReply* reply = new KDEPrivate::AccessManagerReply(op, req, kioJob, this);
+        reply->setStatus(i18n("Blocked request."),QNetworkReply::ContentAccessDenied);
+        return reply;
     }
 
     switch (op) {
@@ -152,7 +179,10 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
         }
         case PutOperation: {
             //kDebug( 7044 ) << "PutOperation:" << reqUrl;
-            kioJob = KIO::put(reqUrl, -1, KIO::HideProgressInfo);
+            if (outgoingData)
+                kioJob = KIO::storedPut(outgoingData->readAll(), reqUrl, -1, KIO::HideProgressInfo);
+            else
+                kioJob = KIO::put(reqUrl, -1, KIO::HideProgressInfo);
             break;
         }
         case PostOperation: {
@@ -160,22 +190,60 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
             kioJob = KIO::http_post(reqUrl, outgoingData->readAll(), KIO::HideProgressInfo);
             break;
         }
-        default:
-            //kDebug( 7044 ) << "Unknown operation";
-            return 0;
+        case DeleteOperation: {
+            //kDebug(7044) << "DeleteOperation:" << reqUrl;
+            kioJob = KIO::file_delete(reqUrl, KIO::HideProgressInfo);
+            break;
+        }
+        case CustomOperation: {
+            const QByteArray& method = req.attribute(QNetworkRequest::CustomVerbAttribute).toByteArray();
+            //kDebug(7044) << "CustomOperation:" << reqUrl << "method:" << method << "outgoing data:" << outgoingData;
+            if (method.isEmpty()) {
+                KDEPrivate::AccessManagerReply* reply = new KDEPrivate::AccessManagerReply(op, req, kioJob, this);
+                reply->setStatus(i18n("Unknown HTTP verb."), QNetworkReply::ProtocolUnknownError);
+                return reply;
+            }
+            if (outgoingData)
+                kioJob = KIO::http_post(reqUrl, outgoingData->readAll(), KIO::HideProgressInfo);
+            else
+                kioJob = KIO::get(reqUrl, KIO::NoReload, KIO::HideProgressInfo);
+            kioJob->metaData().insert(QL1S("CustomHTTPMethod"), method);
+            break;
+        }
+        default: {
+            kWarning(7044) << "Unsupported KIO operation requested! Defering to QNetworkAccessManager...";
+            return QNetworkAccessManager::createRequest(op, req, outgoingData);
+        }
     }
 
     kioJob->setRedirectionHandlingEnabled(false);
-    KIO::Scheduler::scheduleJob(kioJob);
+    
+    switch (req.priority()) {
+    case QNetworkRequest::HighPriority:
+        KIO::Scheduler::setJobPriority(kioJob, -5);
+        break;
+    case QNetworkRequest::LowPriority:
+        KIO::Scheduler::setJobPriority(kioJob, 5);
+        break;
+    default:
+        break;
+    }
+
     KDEPrivate::AccessManagerReply *reply = new KDEPrivate::AccessManagerReply(op, req, kioJob, this);
-
-    kioJob->addMetaData(d->metaDataForRequest(req));
-
-    if ( op == PostOperation && !kioJob->metaData().contains("content-type"))  {
-        QVariant header = req.header(QNetworkRequest::ContentTypeHeader);
+    if (req.hasRawHeader("x-kdewebkit-ignore-disposition")) {
+        kDebug(7044) << "Content-Disposition WILL BE IGNORED!";
+        reply->setIgnoreContentDisposition(true);
+    }
+    
+    KIO::MetaData metaData;
+    d->setMetaDataForRequest(req, metaData);
+    kioJob->addMetaData(metaData);
+    
+    if ( op == PostOperation && !kioJob->metaData().contains(QL1S("content-type")))  {
+        const QVariant header = req.header(QNetworkRequest::ContentTypeHeader);
         if (header.isValid())
           kioJob->addMetaData(QL1S("content-type"),
-                              QString::fromLatin1("Content-Type: %1").arg(header.toString()));
+                              (QL1S("Content-Type: ") + header.toString()));
         else
           kioJob->addMetaData(QL1S("content-type"),
                               QL1S("Content-Type: application/x-www-form-urlencoded"));
@@ -184,10 +252,8 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
     return reply;
 }
 
-KIO::MetaData AccessManager::AccessManagerPrivate::metaDataForRequest(QNetworkRequest request)
+void AccessManager::AccessManagerPrivate::setMetaDataForRequest(QNetworkRequest request, KIO::MetaData& metaData)
 {
-    KIO::MetaData metaData;
-
     // Add any meta data specified within request...
     QVariant userMetaData = request.attribute (static_cast<QNetworkRequest::Attribute>(MetaData));
     if (userMetaData.isValid() && userMetaData.type() == QVariant::Map) {
@@ -220,12 +286,13 @@ KIO::MetaData AccessManager::AccessManagerPrivate::metaDataForRequest(QNetworkRe
     request.setRawHeader("Connection", QByteArray());
     request.setRawHeader("If-None-Match", QByteArray());
     request.setRawHeader("If-Modified-Since", QByteArray());
+    request.setRawHeader("x-kdewebkit-ignore-disposition", QByteArray());
 
     QStringList customHeaders;
     Q_FOREACH(const QByteArray &key, request.rawHeaderList()) {
         const QByteArray value = request.rawHeader(key);
         if (value.length())
-            customHeaders << (key + ": " + value);
+            customHeaders << (key + QL1S(": ") + value);
     }
 
     if (!customHeaders.isEmpty())
@@ -241,15 +308,6 @@ KIO::MetaData AccessManager::AccessManagerPrivate::metaDataForRequest(QNetworkRe
     // Append per session meta data, if any...
     if (!sessionMetaData.isEmpty())
         metaData += sessionMetaData;
-
-    return metaData;
-}
-
-bool AccessManager::AccessManagerPrivate::isRequestAllowed(const QUrl& url) const
-{
-    const QString scheme (url.scheme());
-
-    return (externalContentAllowed || scheme == QL1S("file")  || scheme == QL1S("data"));
 }
 
 
@@ -299,12 +357,16 @@ WId CookieJar::windowId() const {
     return d->windowId;
 }
 
+bool CookieJar::isCookieStorageDisabled() const {
+   return d->isStorageDisabled;
+}
+
 QList<QNetworkCookie> CookieJar::cookiesForUrl(const QUrl &url) const {
     QList<QNetworkCookie> cookieList;
 
-    if (d->enabled) {
+    if (d->isEnabled) {
         QDBusInterface kcookiejar("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer");
-        QDBusReply<QString> reply = kcookiejar.call("findDOMCookies", url.toString(), (qlonglong)d->windowId);
+        QDBusReply<QString> reply = kcookiejar.call("findDOMCookies", url.toString(QUrl::RemoveUserInfo), (qlonglong)d->windowId);
 
         if (reply.isValid()) {
             const QString cookieStr = reply.value();
@@ -326,13 +388,17 @@ QList<QNetworkCookie> CookieJar::cookiesForUrl(const QUrl &url) const {
 }
 
 bool CookieJar::setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const QUrl &url) {
-    if (d->enabled) {
+    if (d->isEnabled) {
         QDBusInterface kcookiejar("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer");
-
         Q_FOREACH(const QNetworkCookie &cookie, cookieList) {
             QByteArray cookieHeader ("Set-Cookie: ");
-            cookieHeader += cookie.toRawForm();
-            kcookiejar.call("addCookies", url.toString(), cookieHeader, (qlonglong)d->windowId);
+            if (d->isStorageDisabled && !cookie.isSessionCookie()) {
+                QNetworkCookie sessionCookie(cookie);
+                sessionCookie.setExpirationDate(QDateTime());
+                cookieHeader += sessionCookie.toRawForm();
+            } else
+                cookieHeader += cookie.toRawForm();
+            kcookiejar.call("addCookies", url.toString(QUrl::RemoveUserInfo), cookieHeader, (qlonglong)d->windowId);
             //kDebug(7044) << "[" << d->windowId << "]" << cookieHeader << " from " << url;
         }
 
@@ -342,14 +408,17 @@ bool CookieJar::setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const
     return false;
 }
 
+void CookieJar::setDisableCookieStorage(bool disable) {
+    d->isStorageDisabled = disable;
+}
+
 void CookieJar::setWindowId(WId id) {
     d->windowId = id;
 }
 
 void CookieJar::reparseConfiguration() {
     KConfigGroup cfg = KSharedConfig::openConfig("kcookiejarrc", KConfig::NoGlobals)->group("Cookie Policy");
-    d->enabled = cfg.readEntry("Cookies", true);
+    d->isEnabled = cfg.readEntry("Cookies", true);
 }
 
-
 #include "accessmanager.moc"
