@@ -18,42 +18,116 @@
  *  Boston, MA 02110-1301, USA.
  */
 
+#include "authinfo.h"
+
 #include <config.h>
 
+#include <sys/stat.h> // don't move it down the include order, it breaks compilation on MSVC
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <sys/stat.h>
 #include <sys/types.h>
 
-#include <qdir.h>
-#include <qfile.h>
+#include <QtCore/QByteArray>
+#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtDBus/QDBusArgument>
+#include <QtDBus/QDBusMetaType>
+#include <kde_file.h>
 
 #include <kdebug.h>
 #include <kstandarddirs.h>
 #include <ksavefile.h>
-#include <kstaticdeleter.h>
-#include <kde_file.h>
-
-#include "kio/authinfo.h"
 
 #define NETRC_READ_BUF_SIZE 4096
 
 using namespace KIO;
 
-AuthInfo::AuthInfo()
+//////
+
+class ExtraField
+{
+public:    
+    QString customTitle; // reserved for future use
+    AuthInfo::FieldFlags flags;
+    QVariant value;
+
+    ExtraField() : flags(AuthInfo::ExtraFieldNoFlags) {}
+};
+Q_DECLARE_METATYPE(ExtraField)
+
+QDataStream& operator<< (QDataStream& s, const ExtraField& extraField)
+{
+    s << extraField.customTitle;
+    s << (int)extraField.flags;
+    s << extraField.value;
+    return s;
+}
+
+QDataStream& operator>> (QDataStream& s, ExtraField& extraField)
+{
+    s >> extraField.customTitle ;
+    int i;
+    s >> i;
+    extraField.flags = (AuthInfo::FieldFlags)i;
+    s >> extraField.value ;
+    return s;
+}
+
+QDBusArgument &operator<<(QDBusArgument &argument, const ExtraField &extraField)
+{
+    argument.beginStructure();
+    argument << extraField.customTitle << (int)extraField.flags
+             << QDBusVariant(extraField.value);
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, ExtraField &extraField)
+{
+    QDBusVariant value;
+    int flag;
+    
+    argument.beginStructure();
+    argument >> extraField.customTitle >> flag >> value;
+    argument.endStructure();
+
+    extraField.value = value.variant();
+    extraField.flags = (KIO::AuthInfo::FieldFlags)flag;
+    return argument;
+}
+
+class KIO::AuthInfoPrivate  
+{
+public:
+    AuthInfoPrivate() 
+    {}
+    
+    QMap<QString, ExtraField> extraFields;
+};
+
+
+//////
+
+AuthInfo::AuthInfo() : d(new AuthInfoPrivate())
 {
     modified = false;
     readOnly = false;
     verifyPath = false;
     keepPassword = false;
+    AuthInfo::registerMetaTypes();
 }
 
-AuthInfo::AuthInfo( const AuthInfo& info )
+AuthInfo::AuthInfo( const AuthInfo& info ) : d(new AuthInfoPrivate())
 {
     (*this) = info;
 }
 
+AuthInfo::~AuthInfo()
+{
+    delete d;
+}
+
 AuthInfo& AuthInfo::operator= ( const AuthInfo& info )
 {
     url = info.url;
@@ -69,76 +143,159 @@ AuthInfo& AuthInfo::operator= ( const AuthInfo& info )
     readOnly = info.readOnly;
     keepPassword = info.keepPassword;
     modified = info.modified;
+    d->extraFields = info.d->extraFields; 
     return *this;
 }
 
+bool AuthInfo::isModified() const
+{
+    return modified;
+}
+
+void AuthInfo::setModified( bool flag )
+{
+    modified = flag;
+}
+
+/////
+
+void AuthInfo::setExtraField(const QString &fieldName, const QVariant & value)
+{
+    d->extraFields[fieldName].value = value;
+}
+ 
+void AuthInfo::setExtraFieldFlags(const QString &fieldName, const FieldFlags flags)
+{
+    d->extraFields[fieldName].flags = flags;
+}
+ 
+QVariant AuthInfo::getExtraField(const QString &fieldName) const
+{
+    if (!d->extraFields.contains(fieldName)) return QVariant();
+    return d->extraFields[fieldName].value; 
+}
+ 
+AuthInfo::FieldFlags AuthInfo::getExtraFieldFlags(const QString &fieldName) const
+{
+    if (!d->extraFields.contains(fieldName)) return AuthInfo::ExtraFieldNoFlags;
+    return d->extraFields[fieldName].flags; 
+}
+
+void AuthInfo::registerMetaTypes()
+{
+    qRegisterMetaType<ExtraField>();
+    qRegisterMetaType<KIO::AuthInfo>();
+    qDBusRegisterMetaType<ExtraField>();
+    qDBusRegisterMetaType<KIO::AuthInfo>();
+}
+
+/////
+
 QDataStream& KIO::operator<< (QDataStream& s, const AuthInfo& a)
 {
-    s << a.url << a.username << a.password << a.prompt << a.caption
+    s << (quint8)1
+      << a.url << a.username << a.password << a.prompt << a.caption
       << a.comment << a.commentLabel << a.realmValue << a.digestInfo
-      << Q_UINT8(a.verifyPath ? 1:0) << Q_UINT8(a.readOnly ? 1:0)
-      << Q_UINT8(a.keepPassword ? 1:0) << Q_UINT8(a.modified ? 1:0);
+      << a.verifyPath << a.readOnly << a.keepPassword << a.modified
+      << a.d->extraFields;
     return s;
 }
 
 QDataStream& KIO::operator>> (QDataStream& s, AuthInfo& a)
 {
-    Q_UINT8 verify = 0;
-    Q_UINT8 ro = 0;
-    Q_UINT8 keep = 0;
-    Q_UINT8 mod  = 0;
-
-    s >> a.url >> a.username >> a.password >> a.prompt >> a.caption
+    quint8 version;
+    s >> version
+      >> a.url >> a.username >> a.password >> a.prompt >> a.caption
       >> a.comment >> a.commentLabel >> a.realmValue >> a.digestInfo
-      >> verify >> ro >> keep >> mod;
-    a.verifyPath = (verify != 0);
-    a.readOnly = (ro != 0);
-    a.keepPassword = (keep != 0);
-    a.modified = (mod != 0);
+      >> a.verifyPath >> a.readOnly >> a.keepPassword >> a.modified
+      >> a.d->extraFields;
     return s;
 }
 
+QDBusArgument &KIO::operator<<(QDBusArgument &argument, const AuthInfo &a)
+{
+    argument.beginStructure();
+    argument << (quint8)1
+             << a.url.url() << a.username << a.password << a.prompt << a.caption
+             << a.comment << a.commentLabel << a.realmValue << a.digestInfo
+             << a.verifyPath << a.readOnly << a.keepPassword << a.modified
+             << a.d->extraFields;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &KIO::operator>>(const QDBusArgument &argument, AuthInfo &a)
+{
+    QString url;
+    quint8 version;
+    
+    argument.beginStructure();
+    argument >> version
+             >> url >> a.username >> a.password >> a.prompt >> a.caption
+             >> a.comment >> a.commentLabel >> a.realmValue >> a.digestInfo
+             >> a.verifyPath >> a.readOnly >> a.keepPassword >> a.modified
+             >> a.d->extraFields;
+    argument.endStructure();
+
+    a.url = url;
+    return argument;
+}
+
+typedef QList<NetRC::AutoLogin> LoginList;
+typedef QMap<QString, LoginList> LoginMap;
+
+class NetRC::NetRCPrivate
+{
+public:
+    NetRCPrivate()
+        : isDirty(false)
+    {}
+    bool isDirty;
+    LoginMap loginMap;
+};
 
 NetRC* NetRC::instance = 0L;
 
 NetRC::NetRC()
+    : d( new NetRCPrivate )
 {
-    isDirty = false;
 }
 
 NetRC::~NetRC()
 {
     delete instance;
     instance = 0L;
+    delete d;
 }
 
 NetRC* NetRC::self()
 {
     if ( !instance )
-        instance = new NetRC();
+        instance = new NetRC;
     return instance;
 }
 
-bool NetRC::lookup( const KURL& url, AutoLogin& login, bool userealnetrc,
-                    QString type, int mode )
+bool NetRC::lookup( const KUrl& url, AutoLogin& login, bool userealnetrc,
+                    const QString &_type, LookUpMode mode )
 {
-  // kdDebug() << "AutoLogin lookup for: " << url.host() << endl;
+  // kDebug() << "AutoLogin lookup for: " << url.host();
   if ( !url.isValid() )
     return false;
 
+  QString type = _type;
   if ( type.isEmpty() )
     type = url.protocol();
 
-  if ( loginMap.isEmpty() || isDirty )
+  if ( d->loginMap.isEmpty() || d->isDirty )
   {
-    loginMap.clear();
+    d->loginMap.clear();
 
-    QString filename = locateLocal("config", "kionetrc");
+    QString filename = KStandardDirs::locateLocal("config", QLatin1String("kionetrc"));
     bool status = parse (openf (filename));
 
     if ( userealnetrc )
     {
-      filename =  QDir::homeDirPath()+ QDir::separator() + ".netrc";
+      filename =  QDir::homePath() + QLatin1String("/.netrc");
       status |= parse (openf(filename));
     }
 
@@ -146,19 +303,19 @@ bool NetRC::lookup( const KURL& url, AutoLogin& login, bool userealnetrc,
       return false;
   }
 
-  if ( !loginMap.contains( type ) )
+  if ( !d->loginMap.contains( type ) )
     return false;
 
-  LoginList l = loginMap[type];
+  const LoginList& l = d->loginMap[type];
   if ( l.isEmpty() )
     return false;
 
-  for (LoginList::Iterator it = l.begin(); it != l.end(); ++it)
+  for (LoginList::ConstIterator it = l.begin(); it != l.end(); ++it)
   {
-    AutoLogin &log = *it;
+    const AutoLogin &log = *it;
 
     if ( (mode & defaultOnly) == defaultOnly &&
-          log.machine == QString::fromLatin1("default") &&
+          log.machine == QLatin1String("default") &&
           (login.login.isEmpty() || login.login == log.login) )
     {
       login.type = log.type;
@@ -169,7 +326,7 @@ bool NetRC::lookup( const KURL& url, AutoLogin& login, bool userealnetrc,
     }
 
     if ( (mode & presetOnly) == presetOnly &&
-          log.machine == QString::fromLatin1("preset") &&
+          log.machine == QLatin1String("preset") &&
           (login.login.isEmpty() || login.login == log.login) )
     {
       login.type = log.type;
@@ -195,11 +352,15 @@ bool NetRC::lookup( const KURL& url, AutoLogin& login, bool userealnetrc,
   return true;
 }
 
+void NetRC::reload()
+{
+    d->isDirty = true;
+}
+
 int NetRC::openf( const QString& f )
 {
   KDE_struct_stat sbuff;
-  QCString ef = QFile::encodeName(f);
-  if ( KDE_stat(ef, &sbuff) != 0 )
+  if ( KDE::stat(f, &sbuff) != 0 )
     return -1;
 
   // Security check!!
@@ -207,7 +368,7 @@ int NetRC::openf( const QString& f )
        sbuff.st_uid != geteuid() )
     return -1;
 
-  return KDE_open( ef, O_RDONLY );
+  return KDE::open( f, O_RDONLY );
 }
 
 QString NetRC::extract( const char* buf, const char* key, int& pos )
@@ -242,7 +403,7 @@ QString NetRC::extract( const char* buf, const char* key, int& pos )
     }
   }
 
-  return QString::null;
+  return QString();
 }
 
 bool NetRC::parse( int fd )
@@ -280,9 +441,9 @@ bool NetRC::parse( int fd )
       while( buf[tail-1] == '\n' || buf[tail-1] =='\r' )
         tail--;
 
-      QString mac = QString::fromLatin1(buf, tail).stripWhiteSpace();
+      QString mac = QString::fromLatin1(buf, tail).trimmed();
       if ( !mac.isEmpty() )
-        loginMap[type][index].macdef[macro].append( mac );
+        d->loginMap[type][index].macdef[macro].append( mac );
 
       continue;
     }
@@ -294,35 +455,35 @@ bool NetRC::parse( int fd )
       if (strncasecmp(buf+pos, "default", 7) == 0 )
       {
         pos += 7;
-        l.machine = QString::fromLatin1("default");
+        l.machine = QLatin1String("default");
       }
       else if (strncasecmp(buf+pos, "preset", 6) == 0 )
       {
         pos += 6;
-        l.machine = QString::fromLatin1("preset");
+        l.machine = QLatin1String("preset");
       }
     }
-    // kdDebug() << "Machine: " << l.machine << endl;
+    // kDebug() << "Machine: " << l.machine;
 
     l.login = extract( buf, "login", pos );
-    // kdDebug() << "Login: " << l.login << endl;
+    // kDebug() << "Login: " << l.login;
 
     l.password = extract( buf, "password", pos );
     if ( l.password.isEmpty() )
       l.password = extract( buf, "account", pos );
-    // kdDebug() << "Password: " << l.password << endl;
+    // kDebug() << "Password: " << l.password;
 
     type = l.type = extract( buf, "type", pos );
     if ( l.type.isEmpty() && !l.machine.isEmpty() )
-      type = l.type = QString::fromLatin1("ftp");
-    // kdDebug() << "Type: " << l.type << endl;
+      type = l.type = QLatin1String("ftp");
+    // kDebug() << "Type: " << l.type;
 
     macro = extract( buf, "macdef", pos );
     isMacro = !macro.isEmpty();
-    // kdDebug() << "Macro: " << macro << endl;
+    // kDebug() << "Macro: " << macro;
 
-    loginMap[l.type].append(l);
-    index = loginMap[l.type].count()-1;
+    d->loginMap[l.type].append(l);
+    index = d->loginMap[l.type].count()-1;
   }
 
   delete [] buf;
