@@ -125,15 +125,15 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
       // (the non-unicity of the stringlist gives us the refcounting, basically).
   }
 
-  if (!validUrl(lister, _url)) {
+    if (!validUrl(lister, _url)) {
         kDebug(7004) << lister << "url=" << _url << "not a valid url";
         return false;
-  }
+    }
 
+    //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 #ifdef DEBUG_CACHE
     printDebug();
 #endif
-  //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 
     if (!_keep) {
         // stop any running jobs for lister
@@ -145,7 +145,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
         lister->d->rootFileItem = KFileItem();
     } else if (lister->d->lstDirs.contains(_url)) {
         // stop the job listing _url for this lister
-        stop(lister, _url, true /*silent*/);
+        stopListingUrl(lister, _url, true /*silent*/);
 
         // remove the _url as well, it will be added in a couple of lines again!
         // forgetDirs with three args does not do this
@@ -271,18 +271,50 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
     return true;
 }
 
+KDirLister::Private::CachedItemsJob* KDirLister::Private::cachedItemsJobForUrl(const KUrl& url) const
+{
+    Q_FOREACH(CachedItemsJob* job, m_cachedItemsJobs) {
+	if (job->url() == url)
+	    return job;
+    }
+    return 0;
+}
+
+KDirLister::Private::CachedItemsJob::CachedItemsJob(KDirLister* lister, const KFileItemList& items, 
+                                                    const KFileItem& rootItem, const KUrl& url, bool reload)
+  : KJob(lister),
+    m_lister(lister), m_url(url),
+    m_items(items), m_rootItem(rootItem),
+    m_reload(reload), m_emitCompleted(true)
+{
+    //kDebug() << "Creating CachedItemsJob" << this << "for lister" << lister << url;
+    if (lister->d->cachedItemsJobForUrl(url)) {
+      kWarning(7004) << "Lister" << lister << "has a cached items job already for" << url;
+    }
+    lister->d->m_cachedItemsJobs.append(this);
+    setAutoDelete(true);
+    start();
+}
+
+// Called by start() via QueuedConnection
 void KDirLister::Private::CachedItemsJob::done()
 {
-    //kDebug() << "lister" << m_lister << "says" << m_lister->d->m_cachedItemsJob << "this=" << this;
-    Q_ASSERT(m_lister->d->m_cachedItemsJob == this);
-    kDirListerCache->emitItemsFromCache(m_lister, m_items, m_rootItem, m_url, m_reload, m_emitCompleted);
+    if (!m_lister) // job was already killed, but waiting deletion due to deleteLater
+        return;
+    kDirListerCache->emitItemsFromCache(this, m_lister, m_items, m_rootItem, m_url, m_reload, m_emitCompleted);
     emitResult();
 }
 
-void KDirListerCache::emitItemsFromCache(KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem, const KUrl& _url, bool _reload, bool _emitCompleted)
+bool KDirLister::Private::CachedItemsJob::doKill()
 {
-    lister->d->m_cachedItemsJob = 0;
+    //kDebug() << this;
+    kDirListerCache->emitItemsFromCache(this, m_lister, KFileItemList(), KFileItem(), m_url, false, false);
+    m_lister = 0;
+    return true;
+}
 
+void KDirListerCache::emitItemsFromCache(KDirLister::Private::CachedItemsJob* cachedItemsJob, KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem, const KUrl& _url, bool _reload, bool _emitCompleted)
+{
     const QString urlStr = _url.url();
     DirItem *itemU = kDirListerCache->itemsInUse.value(urlStr);
     Q_ASSERT(itemU); // hey we're listing that dir, so this can't be 0, right?
@@ -291,26 +323,38 @@ void KDirListerCache::emitItemsFromCache(KDirLister* lister, const KFileItemList
 
     kdl->complete = false;
 
-    if ( kdl->rootFileItem.isNull() && kdl->url == _url )
+    if (kdl->rootFileItem.isNull() && !rootItem.isNull() && kdl->url == _url) {
         kdl->rootFileItem = rootItem;
+    }
+    if (!items.isEmpty()) {
+        //kDebug(7004) << "emitting" << items.count() << "for lister" << lister;
+        kdl->addNewItems(_url, items);
+        kdl->emitItems();
+    }
 
-    //kDebug(7004) << "emitting" << items.count() << "for lister" << lister;
-    kdl->addNewItems(_url, items);
-    kdl->emitItems();
+    // Modifications to data structures only below this point;
+    // so that addNewItems is called with a consistent state
+
+    lister->d->m_cachedItemsJobs.removeAll(cachedItemsJob);
 
     KDirListerCacheDirectoryData& dirData = directoryData[urlStr];
     Q_ASSERT(dirData.listersCurrentlyListing.contains(lister));
 
+    KIO::ListJob *listJob = jobForUrl(urlStr);
+    if (!listJob) {
+        Q_ASSERT(!dirData.listersCurrentlyHolding.contains(lister));
+        kDebug(7004) << "Moving from listing to holding, because no more job" << lister << urlStr;
+        dirData.listersCurrentlyHolding.append( lister );
+        dirData.listersCurrentlyListing.removeAll( lister );
+    } else {
+        //kDebug(7004) << "Still having a listjob" << listJob << ", so not moving to currently-holding.";
+    }
+
     // Emit completed, unless we were told not to,
     // or if listDir() was called while another directory listing for this dir was happening,
     // so we "joined" it. We detect that using jobForUrl to ensure it's a real ListJob,
     // not just a lister-specific CachedItemsJob (which wouldn't emit completed for us).
-    if (_emitCompleted && jobForUrl( urlStr ) == 0) {
-
-        Q_ASSERT(!dirData.listersCurrentlyHolding.contains(lister));
-        //kDebug(7004) << "Moving from listing to holding, because emitCompleted is true and no job" << lister << urlStr;
-        dirData.listersCurrentlyHolding.append( lister );
-        dirData.listersCurrentlyListing.removeAll( lister );
+    if (_emitCompleted) {
 
         kdl->complete = true;
         emit lister->completed( _url );
@@ -353,48 +397,39 @@ void KDirListerCache::stop( KDirLister *lister, bool silent )
     //printDebug();
 #endif
     //kDebug(7004) << "lister: " << lister;
-    bool stopped = false;
 
+    const KUrl::List urls = lister->d->lstDirs;
+    Q_FOREACH(const KUrl& url, urls) {
+        //kDebug() << "Stopping any listjob for" << url.url();
+        stopListJob(url.url(), silent);
+    }
+
+    Q_FOREACH(KDirLister::Private::CachedItemsJob* job, lister->d->m_cachedItemsJobs) {
+        //kDebug() << "Killing cached items job";
+        job->kill(); // removes job from list, too
+    }
+
+#if 0 // test code
     QHash<QString,KDirListerCacheDirectoryData>::iterator dirit = directoryData.begin();
     const QHash<QString,KDirListerCacheDirectoryData>::iterator dirend = directoryData.end();
     for( ; dirit != dirend ; ++dirit ) {
         KDirListerCacheDirectoryData& dirData = dirit.value();
-        if ( dirData.listersCurrentlyListing.removeAll(lister) ) { // contains + removeAll in one go
-            // lister is listing url
-            const QString url = dirit.key();
-
-            //kDebug(7004) << " found lister" << lister << "in list - for" << url;
-            stopLister(lister, url, dirData, silent);
-            stopped = true;
+        if (dirData.listersCurrentlyListing.contains(lister)) {
+            kDebug(7004) << "ERROR: found lister" << lister << "in list - for" << dirit.key();
         }
     }
-
-    if (lister->d->m_cachedItemsJob) {
-        delete lister->d->m_cachedItemsJob;
-        lister->d->m_cachedItemsJob = 0;
-        stopped = true;
-    }
-
-    if ( stopped ) {
-        if (!silent) {
-            emit lister->canceled();
-        }
-        lister->d->complete = true;
-    }
-
-    // this is wrong if there is still an update running!
-    //Q_ASSERT( lister->d->complete );
+#endif
 }
 
-void KDirListerCache::stop(KDirLister *lister, const KUrl& _u, bool silent)
+void KDirListerCache::stopListingUrl(KDirLister *lister, const KUrl& _u, bool silent)
 {
     KUrl url(_u);
     url.adjustPath( KUrl::RemoveTrailingSlash );
     const QString urlStr = url.url();
 
-    if (lister->d->m_cachedItemsJob && lister->d->m_cachedItemsJob->url() == url) {
-        delete lister->d->m_cachedItemsJob;
-        lister->d->m_cachedItemsJob = 0;
+    KDirLister::Private::CachedItemsJob* cachedItemsJob = lister->d->cachedItemsJobForUrl(url);
+    if (cachedItemsJob) {
+        cachedItemsJob->kill(); // removes job from list, too
     }
 
     // TODO: consider to stop all the "child jobs" of url as well
@@ -404,34 +439,32 @@ void KDirListerCache::stop(KDirLister *lister, const KUrl& _u, bool silent)
     if (dirit == directoryData.end())
         return;
     KDirListerCacheDirectoryData& dirData = dirit.value();
-    if ( dirData.listersCurrentlyListing.removeAll(lister) ) { // contains + removeAll in one go
+    if (dirData.listersCurrentlyListing.contains(lister)) {
 
         //kDebug(7004) << " found lister" << lister << "in list - for" << urlStr;
-        stopLister(lister, urlStr, dirData, silent);
-
-        if ( lister->d->numJobs() == 0 ) {
-            lister->d->complete = true;
-            // we killed the last job for lister
-            if (!silent) {
-                emit lister->canceled();
-            }
-            //kDebug(7004) << "Entry now being listed by" << dirData.listersCurrentlyListing;
-        }
+        stopListJob(urlStr, silent);
     }
 }
 
-// Helper for both stop() methods
-void KDirListerCache::stopLister(KDirLister* lister, const QString& url, KDirListerCacheDirectoryData& dirData, bool silent)
+// Helper for stop() and stopListingUrl()
+void KDirListerCache::stopListJob(const QString& url, bool silent)
 {
-    // Let's just leave the job running.
+    // Old idea: if it's an update job, let's just leave the job running.
     // After all, update jobs do run for "listersCurrentlyHolding",
     // so there's no reason to kill them just because @p lister is now a holder.
 
-    // Move lister to listersCurrentlyHolding
-    dirData.listersCurrentlyHolding.append(lister);
+    // However it could be a long-running non-local job (e.g. filenamesearch), which
+    // the user wants to abort, and which will never be used for updating...
+    // And in any case slotEntries/slotResult is not meant to be called by update jobs.
+    // So, change of plan, let's kill it after all, in a way that triggers slotResult/slotUpdateResult.
 
-    if (!silent)
-        emit lister->canceled(KUrl(url));
+    KIO::ListJob *job = jobForUrl(url);
+    if (job) {
+        //kDebug() << "Killing list job" << job;
+        if (silent)
+            job->setProperty("_kdlc_silent", true);
+        job->kill(KJob::EmitResult);
+    }
 }
 
 void KDirListerCache::setAutoUpdate( KDirLister *lister, bool enable )
@@ -461,6 +494,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister )
     const KUrl::List lstDirsCopy = lister->d->lstDirs;
     lister->d->lstDirs.clear();
 
+    //kDebug() << "Iterating over dirs" << lstDirsCopy;
     for ( KUrl::List::const_iterator it = lstDirsCopy.begin();
           it != lstDirsCopy.end(); ++it ) {
         forgetDirs( lister, *it, false );
@@ -612,11 +646,11 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
         // Emit any cached items.
         // updateDirectory() is about the diff compared to the cached items...
         Q_FOREACH(KDirLister *kdl, listers) {
-            if (kdl->d->m_cachedItemsJob) {
-                KDirLister::Private::CachedItemsJob* job = kdl->d->m_cachedItemsJob;
-                job->setEmitCompleted(false);
-                job->done(); // sets kdl->d->m_cachedItemsJob to 0
-                delete job;
+	    KDirLister::Private::CachedItemsJob* cachedItemsJob = kdl->d->cachedItemsJobForUrl(_dir);
+            if (cachedItemsJob) {
+                cachedItemsJob->setEmitCompleted(false);
+                cachedItemsJob->done(); // removes from cachedItemsJobs list
+                delete cachedItemsJob;
                 killed = true;
             }
         }
@@ -631,7 +665,7 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
         kWarning() << "listers for" << _dir << "=" << listers;
         kWarning() << "job=" << job;
         Q_FOREACH(KDirLister *kdl, listers) {
-            kDebug() << "lister" << kdl << "m_cachedItemsJob=" << kdl->d->m_cachedItemsJob;
+            kDebug() << "lister" << kdl << "m_cachedItemsJobs=" << kdl->d->m_cachedItemsJobs;
         }
 #ifndef NDEBUG
         printDebug();
@@ -1112,6 +1146,9 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
     KDirListerCacheDirectoryData& dirData = *dit;
     if (dirData.listersCurrentlyListing.isEmpty()) {
         kError(7004) << "Internal error: job is listing" << url << "but directoryData says no listers are currently listing " << urlStr;
+#ifndef NDEBUG
+        printDebug();
+#endif
         Q_ASSERT( !dirData.listersCurrentlyListing.isEmpty() );
         return;
     }
@@ -1168,7 +1205,7 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
 void KDirListerCache::slotResult( KJob *j )
 {
 #ifdef DEBUG_CACHE
-  printDebug();
+    //printDebug();
 #endif
 
   Q_ASSERT( j );
@@ -1212,12 +1249,19 @@ void KDirListerCache::slotResult( KJob *j )
     foreach ( KDirLister *kdl, listers )
     {
       kdl->d->jobDone( job );
-      kdl->handleError( job );
-      emit kdl->canceled( jobUrl );
-      if ( kdl->d->numJobs() == 0 )
-      {
+      if (job->error() != KJob::KilledJobError) {
+          kdl->handleError( job );
+      }
+      const bool silent = job->property("_kdlc_silent").toBool();
+      if (!silent) {
+          emit kdl->canceled( jobUrl );
+      }
+
+      if (kdl->d->numJobs() == 0) {
         kdl->d->complete = true;
-        emit kdl->canceled();
+        if (!silent) {
+            emit kdl->canceled();
+        }
       }
     }
   }
@@ -1604,10 +1648,15 @@ void KDirListerCache::slotUpdateResult( KJob * j )
             //don't bother the user
             //kdl->handleError( job );
 
-            emit kdl->canceled( jobUrl );
+            const bool silent = job->property("_kdlc_silent").toBool();
+            if (!silent) {
+                emit kdl->canceled( jobUrl );
+            }
             if ( kdl->d->numJobs() == 0 ) {
                 kdl->d->complete = true;
-                emit kdl->canceled();
+                if (!silent) {
+                    emit kdl->canceled();
+                }
             }
         }
 
@@ -1819,7 +1868,7 @@ void KDirListerCache::deleteDir( const KUrl& dirUrl )
             // we need a copy because stop modifies the list
             QList<KDirLister *> listers = (*dit).listersCurrentlyListing;
             foreach ( KDirLister *kdl, listers )
-                stop( kdl, deletedUrl );
+                stopListingUrl( kdl, deletedUrl );
             // tell listers holding deletedUrl to forget about it
             // this will stop running updates for deletedUrl as well
 
@@ -1909,8 +1958,8 @@ void KDirListerCache::printDebug()
             list += " 0x" + QString::number( (qlonglong)listit, 16 );
         kDebug(7004) << "  " << dit.key() << (*dit).listersCurrentlyListing.count() << "listers:" << list;
         foreach ( KDirLister* listit, (*dit).listersCurrentlyListing ) {
-            if (listit->d->m_cachedItemsJob) {
-                kDebug(7004) << "  Lister" << listit << "has CachedItemsJob" << listit->d->m_cachedItemsJob;
+            if (!listit->d->m_cachedItemsJobs.isEmpty()) {
+                kDebug(7004) << "  Lister" << listit << "has CachedItemsJobs" << listit->d->m_cachedItemsJobs;
             } else if (KIO::ListJob* listJob = jobForUrl(dit.key())) {
                 kDebug(7004) << "  Lister" << listit << "has ListJob" << listJob;
             } else {
@@ -1962,9 +2011,9 @@ KDirLister::KDirLister( QObject* parent )
 
 KDirLister::~KDirLister()
 {
-    //kDebug(7003) << "-KDirLister";
+    //kDebug(7003) << "~KDirLister" << this;
 
-    // Stop all running jobs
+    // Stop all running jobs, remove lister from lists
     if (!kDirListerCache.isDestroyed()) {
         stop();
         kDirListerCache->forgetDirs( this );
@@ -1991,7 +2040,7 @@ void KDirLister::stop()
 
 void KDirLister::stop( const KUrl& _url )
 {
-    kDirListerCache->stop( this, _url );
+    kDirListerCache->stopListingUrl( this, _url );
 }
 
 bool KDirLister::autoUpdate() const
@@ -2647,7 +2696,7 @@ void KDirListerCacheDirectoryData::moveListersWithoutCachedItemsJob(const KUrl&
     QMutableListIterator<KDirLister *> lister_it(listersCurrentlyListing);
     while (lister_it.hasNext()) {
         KDirLister* kdl = lister_it.next();
-        if (!kdl->d->m_cachedItemsJob || kdl->d->m_cachedItemsJob->url() != url) {
+        if (!kdl->d->cachedItemsJobForUrl(url)) {
             // OK, move this lister from "currently listing" to "currently holding".
 
             // Huh? The KDirLister was present twice in listersCurrentlyListing, or was in both lists?
@@ -2657,7 +2706,7 @@ void KDirListerCacheDirectoryData::moveListersWithoutCachedItemsJob(const KUrl&
             }
             lister_it.remove();
         } else {
-            //kDebug(7004) << "Not moving" << kdl << "to listersCurrentlyHolding because it still has job" << kdl->d->m_cachedItemsJob;
+            //kDebug(7004) << "Not moving" << kdl << "to listersCurrentlyHolding because it still has job" << kdl->d->m_cachedItemsJobs;
         }
     }
 }
