@@ -30,6 +30,7 @@
 #include <QtGui/QWidget>
 
 #include <kurl.h>
+#include <kdebug.h>
 #include <kio/global.h>
 #include <kdirwatch.h>
 
@@ -63,7 +64,6 @@ public:
     hasPendingChanges = false;
 
     window = 0;
-    m_cachedItemsJob = 0;
   }
 
     void _k_emitCachedItems(const KUrl&, bool, bool);
@@ -105,6 +105,9 @@ public:
 
     void emitChanges();
 
+    class CachedItemsJob;
+    CachedItemsJob* cachedItemsJobForUrl(const KUrl& url) const;
+
 
   KDirLister *m_parent;
 
@@ -144,8 +147,7 @@ public:
   KFileItemList *lstMimeFilteredItems, *lstRemoveItems;
 
     QWidget *window; // Main window this lister is associated with
-    class CachedItemsJob;
-    CachedItemsJob* m_cachedItemsJob;
+    QList<CachedItemsJob*> m_cachedItemsJobs;
 
     QString nameFilter; // parsed into lstFilters
 
@@ -193,7 +195,7 @@ public:
     // stop all running jobs for lister
     void stop( KDirLister *lister, bool silent = false );
     // stop just the job listing url for lister
-    void stop( KDirLister *lister, const KUrl &_url, bool silent = false );
+    void stopListingUrl( KDirLister *lister, const KUrl &_url, bool silent = false );
 
   void setAutoUpdate( KDirLister *lister, bool enable );
 
@@ -208,7 +210,8 @@ public:
 
     // Called by CachedItemsJob:
     // Emits those items, for this lister and this url
-    void emitItemsFromCache(KDirLister* lister, const KFileItemList& lst, const KFileItem& rootItem,
+    void emitItemsFromCache(KDirLister::Private::CachedItemsJob* job, KDirLister* lister,
+                            const KFileItemList& lst, const KFileItem& rootItem,
                             const KUrl& _url, bool _reload, bool _emitCompleted);
 
 public Q_SLOTS:
@@ -257,8 +260,7 @@ private:
 
     bool validUrl( const KDirLister *lister, const KUrl& _url ) const;
 
-    // helper for both stop methods
-    void stopLister(KDirLister* lister, const QString& url, KDirListerCacheDirectoryData& dirData, bool silent);
+    void stopListJob(const QString& url, bool silent);
 
     KIO::ListJob *jobForUrl( const QString& url, KIO::ListJob *not_job = 0 );
     const KUrl& joburl( KIO::ListJob *job );
@@ -419,7 +421,7 @@ private:
 
     // Set of local files that we have changed recently (according to KDirWatch)
     // We temporize the notifications by keeping them 500ms in this list.
-    QSet<QString /*url*/> pendingUpdates;
+    QSet<QString /*path*/> pendingUpdates;
     // The timer for doing the delayed updates
     QTimer pendingUpdateTimer;
 
@@ -450,7 +452,7 @@ struct KDirListerCacheDirectoryData
     // Listers that are currently holding this url
     QList<KDirLister *> listersCurrentlyHolding;
 
-    void moveListersWithoutCachedItemsJob();
+    void moveListersWithoutCachedItemsJob(const KUrl& url);
 };
 
 //const unsigned short KDirListerCache::MAX_JOBS_PER_LISTER = 5;
@@ -463,16 +465,7 @@ class KDirLister::Private::CachedItemsJob : public KJob {
     Q_OBJECT
 public:
     CachedItemsJob(KDirLister* lister, const KFileItemList& items, const KFileItem& rootItem,
-                   const KUrl& url, bool reload)
-        : KJob(lister),
-          m_lister(lister), m_url(url),
-          m_items(items), m_rootItem(rootItem),
-          m_reload(reload), m_emitCompleted(true) {
-        Q_ASSERT(lister->d->m_cachedItemsJob == 0);
-        lister->d->m_cachedItemsJob = this;
-        setAutoDelete(true);
-        start();
-    }
+                   const KUrl& url, bool reload);
 
     /*reimp*/ void start() { QMetaObject::invokeMethod(this, "done", Qt::QueuedConnection); }
 
@@ -481,6 +474,9 @@ public:
 
     KUrl url() const { return m_url; }
 
+protected:
+    virtual bool doKill();
+
 public Q_SLOTS:
     void done();
 
