@@ -21,6 +21,7 @@
 #include "kfileitem.h"
 
 #include <config.h>
+#include <config-kio.h>
 
 #include <sys/time.h>
 #include <pwd.h>
@@ -189,7 +190,9 @@ public:
     // For special case like link to dirs over FTP
     QString m_guessedMimeType;
     mutable QString m_access;
+#ifndef KDE_NO_DEPRECATED
     QMap<const void*, void*> m_extra; // DEPRECATED
+#endif
     mutable KFileMetaInfo m_metaInfo;
 
     enum { NumFlags = KFileItem::CreationTime + 1 };
@@ -606,6 +609,7 @@ KDateTime KFileItem::time( FileTimes which ) const
     return d->time(which);
 }
 
+#ifndef KDE_NO_DEPRECATED
 time_t KFileItem::time( unsigned int which ) const
 {
     switch (which) {
@@ -618,6 +622,7 @@ time_t KFileItem::time( unsigned int which ) const
         return d->time(ModificationTime).toTime_t();
     }
 }
+#endif
 
 QString KFileItem::user() const
 {
@@ -742,21 +747,10 @@ QString KFileItem::mimeComment() const
 static QString iconFromDesktopFile(const QString& path)
 {
     KDesktopFile cfg( path );
-    const KConfigGroup group = cfg.desktopGroup();
     const QString icon = cfg.readIcon();
-    const QString type = cfg.readPath();
-
-    if ( cfg.hasDeviceType() )
-    {
-        const QString unmount_icon = group.readEntry( "UnmountIcon" );
-        const QString dev = cfg.readDevice();
-        if ( !icon.isEmpty() && !unmount_icon.isEmpty() && !dev.isEmpty() )
-        {
-            KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByDevice(dev);
-            if (!mountPoint) // not mounted?
-                return unmount_icon;
-        }
-    } else if ( cfg.hasLinkType() ) {
+    if ( cfg.hasLinkType() ) {
+        const KConfigGroup group = cfg.desktopGroup();
+        const QString type = cfg.readPath();
         const QString emptyIcon = group.readEntry( "EmptyIcon" );
         if ( !emptyIcon.isEmpty() ) {
             const QString u = cfg.readUrl();
@@ -853,6 +847,15 @@ QStringList KFileItem::overlays() const
         if ( group.hasKey( "Exec" ) && !KDesktopFile::isAuthorizedDesktopFile( localPath() ) ) {
             names.append( "emblem-important" );
         }
+
+        if (cfg.hasDeviceType()) {
+            const QString dev = cfg.readDevice();
+            if (!dev.isEmpty()) {
+                KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByDevice(dev);
+                if (mountPoint) // mounted?
+                    names.append("emblem-mounted");
+            }
+        }
     }
 
     if ( isHidden() ) {
@@ -879,6 +882,11 @@ QStringList KFileItem::overlays() const
     return names;
 }
 
+QString KFileItem::comment() const
+{
+    return d->m_entry.stringValue( KIO::UDSEntry::UDS_COMMENT );
+}
+
 // ## where is this used?
 QPixmap KFileItem::pixmap( int _size, int _state ) const
 {
@@ -1010,6 +1018,7 @@ bool KFileItem::isFile() const
     return !isDir();
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool KFileItem::acceptsDrops() const
 {
     // A directory ?
@@ -1030,6 +1039,7 @@ bool KFileItem::acceptsDrops() const
 
     return false;
 }
+#endif
 
 QString KFileItem::getStatusBarInfo() const
 {
@@ -1114,11 +1124,6 @@ QString KFileItem::getToolTipText(int maxcount) const
             if ( item.isValid() )
             {
                 QString s = item.value().toString();
-                if ( ( item.properties().attributes() & PredicateProperties::SqueezeText )
-                     && s.length() > 50) {
-                    s.truncate(47);
-                    s.append("...");
-                }
                 if ( !s.isEmpty() )
                 {
                     count++;
@@ -1161,6 +1166,7 @@ bool KFileItem::operator!=(const KFileItem& other) const
     return d != other.d;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
                              bool _delayedMimeTypes, bool _urlIsDirectory )
 {
@@ -1186,12 +1192,14 @@ void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
     d->readUDSEntry( _urlIsDirectory );
     d->init();
 }
+#endif
 
 KFileItem::operator QVariant() const
 {
     return qVariantFromValue(*this);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::setExtraData( const void *key, void *value )
 {
     if ( !key )
@@ -1199,16 +1207,21 @@ void KFileItem::setExtraData( const void *key, void *value )
 
     d->m_extra.insert( key, value ); // replaces the value of key if already there
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 const void * KFileItem::extraData( const void *key ) const
 {
     return d->m_extra.value( key, 0 );
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::removeExtraData( const void *key )
 {
     d->m_extra.remove( key );
 }
+#endif
 
 QString KFileItem::permissionsString() const
 {
@@ -1224,6 +1237,7 @@ QString KFileItem::timeString( FileTimes which ) const
     return KGlobal::locale()->formatDateTime( d->time(which) );
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KFileItem::timeString( unsigned int which ) const
 {
     switch (which) {
@@ -1236,6 +1250,7 @@ QString KFileItem::timeString( unsigned int which ) const
         return timeString(ModificationTime);
     }
 }
+#endif
 
 void KFileItem::setMetaInfo( const KFileMetaInfo & info ) const
 {
@@ -1253,10 +1268,12 @@ KFileMetaInfo KFileItem::metaInfo(bool autoget, int what) const
     return d->m_metaInfo;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::assign( const KFileItem & item )
 {
     *this = item;
 }
+#endif
 
 KUrl KFileItem::mostLocalUrl(bool &local) const
 {
@@ -1276,6 +1293,12 @@ KUrl KFileItem::mostLocalUrl(bool &local) const
     }
 }
 
+KUrl KFileItem::mostLocalUrl() const
+{
+    bool local = false;
+    return mostLocalUrl(local);
+}
+
 QDataStream & operator<< ( QDataStream & s, const KFileItem & a )
 {
     // We don't need to save/restore anything that refresh() invalidates,
@@ -1348,6 +1371,7 @@ KUrl KFileItem::targetUrl() const
 
 KUrl KFileItem::nepomukUri() const
 {
+#ifndef KIO_NO_NEPOMUK
     const QString nepomukUriStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_NEPOMUK_URI );
     if(!nepomukUriStr.isEmpty()) {
         return KUrl(nepomukUriStr);
@@ -1358,6 +1382,9 @@ KUrl KFileItem::nepomukUri() const
     else {
         return KUrl();
     }
+#else
+    return KUrl();
+#endif
 }
 
 /*
