@@ -37,44 +37,131 @@
 #undef ERROR
 #endif
 
+class KUriFilterPrivate;
 class KUriFilterDataPrivate;
-
 class KCModule;
 
 /**
-* A basic message object used for exchanging filtering information between the
-* filter plugins and the application requesting the filtering service.
+ * Class that holds information about a search provider.
+ *
+ * @since 4.6
+ */
+class KIO_EXPORT KUriFilterSearchProvider
+{
+public:
+    /**
+     * Default constructor.
+     */
+    KUriFilterSearchProvider();
+
+    /**
+     * Copy constructor.
+     */
+    KUriFilterSearchProvider(const KUriFilterSearchProvider&);
+
+    /**
+     * Destructor.
+     */
+    virtual ~KUriFilterSearchProvider();
+
+    /**
+     * Returns the desktop filename of the search provider without any extension.
+     *
+     * For example, if the desktop filename of the search provider was
+     * "foobar.desktop", this function will return "foobar".
+     */
+    QString desktopEntryName() const;
+
+    /**
+     * Returns the descriptive name of the search provider, e.g. "Google News".
+     *
+     * This name comes from the "Name=" property entry in the desktop file that
+     * contains the search provider's information.
+     */
+    QString name() const;
+
+    /**
+     * Returns the icon name associated with the search provider when available.
+     */
+    QString iconName() const;
+
+    /**
+     * Returns all the web shortcut keys associated with this search provider.
+     *
+     * @see defaultKey
+     */
+    QStringList keys() const;
+
+    /**
+     * Returns the default web shortcut key for this search provider.
+     *
+     * Right now this is the same as doing keys().first(), it might however
+     * change based on what the backend plugins do.
+     *
+     * @see keys
+     */
+    QString defaultKey() const;
+
+    /**
+     * Assignment operator.
+     */
+    KUriFilterSearchProvider& operator=(const KUriFilterSearchProvider&);
+
+protected:
+    void setDesktopEntryName(const QString&);
+    void setIconName(const QString&);
+    void setKeys(const QStringList&);
+    void setName(const QString&);
+
+private:
+    friend class KUriFilterPlugin;
+    class KUriFilterSearchProviderPrivate;
+    KUriFilterSearchProviderPrivate * const d;
+};
+
+/**
+* This class is a basic messaging class used to exchange filtering information
+* between the filter plugins and the application requesting the filtering
+* service.
 *
 * Use this object if you require a more detailed information about the URI you
 * want to filter. Any application can create an instance of this class and send
 * it to KUriFilter to have the plugins fill out all possible information about
 * the URI.
 *
-* The member functions provided by this class are thread safe and reentrant. However,
-* since the class itself is a Singleton, you have to take care when calling @ref self()
-* in multi-threaded applications. It is highly adviced that you hold a member variable
+* On successful filtering you can use @ref uriType() to determine what type
+* of resource the request was filtered into. See @ref KUriFilter::UriTypes for
+* details. If an error is encountered, then @ref KUriFilter::Error is returned.
+* You can use @ref errorMsg to obtain the error information.
+*
+* The functions in this class are not reentrant.
 *
 * \b Example
 *
+* Here is a basic example of how this class is used with @ref KUriFilter:
 * \code
-*   QString text = "kde.org";
-*   KUriFilterData d = text;
-*   bool filtered = KUriFilter::self()->filter( d );
-*   cout << "URL: " << text.toLatin1() << endl
-*        << "Filtered URL: " << d.uri().url().toLatin1() << endl
-*        << "URI Type: " << d.uriType() << endl
-*        << "Was Filtered: " << filtered << endl;
+*   KUriFilterData filterData (QLatin1String("kde.org"));
+*   bool filtered = KUriFilter::self()->filterUri(filterData);
 * \endcode
 *
-* The above code should yield the following output:
+* If you are only interested in getting the list of preferred search providers,
+* then you can do the following (requires KDE >= 4.5):
+*
 * \code
-*   URI: kde.org
-*   Filtered URI: http://kde.org
-*   URI Type: 0            <== means NET_PROTOCOL
-*   Was Filtered: 1        <== means the URL was successfully filtered
+* KDE 4.5:
+* KUriFilterData data;
+* data.setData("<text-to-search-for>");
+* data.setAlternateDefaultSearchProvider("google");
+* bool filtered = KUriFilter::self()->filterUri(data, "kuriikwsfilter");
+*
+* KDE >= 4.6:
+* KUriFilterData data;
+* data.setData("<text-to-search-for>");
+* data.setSearchFilteringOption(KUriFilterData::RetrievePreferredSearchProvidersOnly);
+* bool filtered = KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter);
 * \endcode
 *
-* @short A message object for exchanging filtering URI info.
+* @short A class for exchanging filtering information.
 * @author Dawit Alemayehu <adawit at kde.org>
 */
 
@@ -99,6 +186,38 @@ public:
      */
     enum UriTypes { NetProtocol=0, LocalFile, LocalDir, Executable, Help, Shell, Blocked, Error, Unknown };
 
+    /**
+      * This enum describes the search filtering options to be used.
+      *
+      * @li SearchFilterOptionNone
+      *     No search filter options are set and normal filtering is performed
+      *     on the input data.
+      * @li RetrieveSearchProvidersOnly
+      *     If set, the list of all available search providers are returned without
+      *     any input filtering. This flag only applies when used in conjunction
+      *     with the @ref KUriFilter::NormalTextFilter flag.
+      * @li RetrievePreferredSearchProvidersOnly
+      *     If set, the list of preferred search providers are returned without
+      *     any input filtering. This flag only applies when used in conjunction
+      *     with the @ref KUriFilter::NormalTextFilter flag.
+      * @li RetrieveAvailableSearchProvidersOnly
+      *     Same as doing RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly,
+      *     where all available search providers are returned if no preferred ones
+      *     ones are available. No input filtering will be performed.
+      *
+      * @see setSearchFilteringOptions
+      * @see KUriFilter::filterSearchUri
+      * @since 4.6
+      */
+    enum SearchFilterOption
+    {
+        SearchFilterOptionNone = 0x0,
+        RetrieveSearchProvidersOnly = 0x01,
+        RetrievePreferredSearchProvidersOnly = 0x02,
+        RetrieveAvailableSearchProvidersOnly = (RetrievePreferredSearchProvidersOnly | RetrieveSearchProvidersOnly)
+    };
+    Q_DECLARE_FLAGS(SearchFilterOptions, SearchFilterOption)
+
     /**
      * Default constructor.
      *
@@ -245,41 +364,72 @@ public:
     QString searchProvider() const;
 
     /**
-     * Returns a list of the names of the preferred search providers.
+     * Returns a list of the names of preferred or available search providers.
      *
-     * This function returns the list of providers that are marked as favorite
-     * or preferred by the user whenever the data it contains is filtered by the
-     * 'kuriikwsfilter' search uri filter plugin.
+     * This function returns the list of providers marked as preferred whenever
+     * the input data, i.e. @ref typedString, is successfully filtered.
      *
-     * If no prior search providers were marked preferred or no default search
-     * engine was selected, then this function will return an empty list unless
-     * you set alternate default or search providers using the appropriate function.
-     * See @ref setAlternateDefaultSearchProvider and @ref setAlternateSearchProviders.
+     * If no default search provider has been selected prior to a filter request,
+     * this function will return an empty list. To avoid this problem you must
+     * either set an alternate default search provider using @ref setAlternateDefaultSearchProvider
+     * or set one of the @ref SearchFilterOption flags if you are only interested
+     * in getting the list of providers and not filtering the input.
      *
-     * You can use @ref queryForPreferredServiceProvider to obtain the queries
-     * associated with the returned search providers.
+     * Additionally, you can also provide alternate search providers in case
+     * there are no preferred ones already selected.
      *
-     * @see searchProvider
-     * @see alternateSearchProviders
-     * @see alternateDefaultSearchProvider
+     * You can use @ref queryForPreferredServiceProvider to obtain the query
+     * associated with the list of search providers returned by this function.
+     *
+     * @see setAlternateSearchProviders
+     * @see setAlternateDefaultSearchProvider
+     * @see setSearchFilteringOption
      * @see queryForPreferredServiceProvider
      * @since 4.5
      */
     QStringList preferredSearchProviders() const;
 
     /**
-     * Returns the query url for the given preferred search provider.
+     * Returns information about @p provider.
+     *
+     * You can use this function to obtain the more information about the search
+     * providers returned by @ref preferredSearchProviders.
+     *
+     * @see preferredSearchProviders
+     * @see KUriFilterSearchProvider
+     * @since 4.6
+     */
+     KUriFilterSearchProvider queryForSearchProvider(const QString& provider) const;
+
+    /**
+     * Returns the web shortcut url for the given preferred search provider.
      *
      * You can use this function to obtain the query for the preferred search
-     * providers returned by @ref preferredSearchProviders. Note that this
-     * function actually returns a query, e.g. "gg:foo bar", that must be
-     * processed using the KUriFilter class.
+     * providers returned by @ref preferredSearchProviders.
+     *
+     * The query returned by this function is in web shortcut format, i.e.
+     * "gg:foo bar", and must be re-filtered through KUriFilter to obtain a
+     * valid url.
      *
      * @see preferredSearchProviders
      * @since 4.5
      */
     QString queryForPreferredSearchProvider(const QString &provider) const;
 
+    /**
+     * Returns all the query urls for the given search provider.
+     *
+     * Use this function to obtain all the different queries that can be used
+     * for the given provider. For example, if a search engine provider named
+     * "foobar" has web shortcuts named "foobar", "foo" and "bar", then this
+     * function, unlike @ref queryForPreferredSearchProvider, will return a
+     * a query for each and every web shortcut.
+     *
+     * @see queryForPreferredSearchProvider
+     * @since 4.6
+     */
+    QStringList allQueriesForSearchProvider(const QString& provider) const;
+
     /**
      * Returns the icon associated with the given preferred search provider.
      *
@@ -317,6 +467,26 @@ public:
      */
     QString alternateDefaultSearchProvider() const;
 
+    /**
+     * Returns the default protocol to use when filtering potentially valid url inputs.
+     *
+     * By default this function will return an empty string.
+     *
+     * @see setDefaultUrlScheme
+     * @since 4.6
+     */
+    QString defaultUrlScheme() const;
+
+    /**
+     * Returns the specified search filter options.
+     *
+     * By default this function returns @ref SearchFilterOptionNone.
+     *
+     * @see setSearchFilteringOptions
+     * @since 4.6
+     */
+    SearchFilterOptions searchFilteringOptions() const;
+
     /**
      * The name of the icon that matches the current filtered URL.
      *
@@ -400,6 +570,39 @@ public:
      */
     void setAlternateDefaultSearchProvider(const QString &provider);
 
+    /**
+     * Sets the default scheme used when filtering potentially valid url inputs.
+     *
+     * Use this function to change the default protocol used when filtering
+     * potentially valid url inputs. The default protocol is http.
+     *
+     * If the scheme is specified without a separator, then  "://" will be used
+     * as the separator by default. For example, if the default url scheme was
+     * simply set to "ftp", then a potentially valid url input such as "kde.org"
+     * will be filtered to "ftp://kde.org".
+     *
+     * @see defaultUrlScheme
+     * @since 4.6
+     */
+    void setDefaultUrlScheme(const QString&);
+
+    /**
+      * Sets the options used by search filter plugins to filter requests.
+      *
+      * The default search filter option is @ref SearchFilterOptionNone. See
+      * @ref SearchFilterOption for the description of the other flags.
+      *
+      * It is important to note that the options set through this function can
+      * prevent any filtering from being performed by search filter plugins.
+      * As such, @ref uriTypes can return KUriFilterData::Unknown and @ref uri
+      * can return an invalid url eventhough the filtering request returned
+      * a successful response.
+      *
+      * @see searchFilteringOptions
+      * @since 4.6
+      */
+    void setSearchFilteringOptions(SearchFilterOptions options);
+
     /**
      * Overloaded assigenment operator.
      *
@@ -448,8 +651,11 @@ public:
      * ([search provider name], [search query, search query icon name])
      *
      * @since 4.5
+     * @deprecated Use @ref KUriFilterSearchProvider instead. See @ref setSearchProviders;
      */
-    typedef QHash<QString, QPair<QString, QString> > ProviderInfoList;
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED typedef QHash<QString, QPair<QString, QString> > ProviderInfoList;
+#endif
 
     /**
      * Constructs a filter plugin with a given name
@@ -512,15 +718,25 @@ protected:
      *
      * @since 4.5
      */
-    void setSearchProvider( KUriFilterData &data, const QString& provider,
-                            const QString &term, const QChar &separator) const;
+    void setSearchProvider( KUriFilterData& data, const QString& provider,
+                            const QString& term, const QChar& separator) const;
 
     /**
      * Sets the name of the preferred search providers in @p data.
      *
      * @since 4.5
+     * @deprecated Use @ref setSearchProviders instead.
      */
-    void setPreferredSearchProviders(KUriFilterData &data, const ProviderInfoList &providers) const;
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED void setPreferredSearchProviders(KUriFilterData& data, const ProviderInfoList& providers) const;
+#endif
+
+    /**
+     * Sets the information about the search @p providers in @p data.
+     *
+     * @since 4.6
+     */
+    void setSearchProviders(KUriFilterData& data, const QList<KUriFilterSearchProvider*>& providers) const;
 
     /**
      * Returns the icon name for the given @p url and URI @p type.
@@ -533,39 +749,30 @@ private:
     class KUriFilterPluginPrivate * const d;
 };
 
-
-class KUriFilterPrivate;
 /**
- * Manages the filtering of URIs.
+ * KUriFilter applies a number of filters to a URI and returns a filtered version if any
+ * filter matches.
+ * A simple example is "kde.org" to "http://www.kde.org", which is commonplace in web browsers.
  *
- * The intention of this plugin class is to allow people to extend the
- * functionality of KUrl without modifying it directly. This way KUrl will
- * remain a generic parser capable of parsing any generic URL that adheres to
- * specifications.
+ * The filters are implemented as plugins in @ref KUriFilterPlugin subclasses.
  *
- * The KUriFilter class applies a number of filters to a URI and returns the
- * filtered version whenever possible. The filters are implemented using
- * plugins to provide easy extensibility of the filtering mechanism. New
- * filters can be added in the future by simply inheriting from the
- * @ref KUriFilterPlugin class.
- *
- * Use of this plugin-manager class is straight forward.  Since it is a
- * singleton object, all you have to do is obtain an instance by doing
- * @p KUriFilter::self() and use any of the public member functions to
- * preform the filtering.
+ * KUriFilter is a singleton object: obtain the instance by calling
+ * @p KUriFilter::self() and use the public member functions to
+ * perform the filtering.
  *
  * \b Example
  *
  * To simply filter a given string:
  *
  * \code
- * bool filtered = KUriFilter::self()->filterUri( "kde.org" );
+ * QString url("kde.org");
+ * bool filtered = KUriFilter::self()->filteredUri( url );
  * \endcode
  *
  * You can alternatively use a KUrl:
  *
  * \code
- * KUrl url = "kde.org";
+ * KUrl url("kde.org");
  * bool filtered = KUriFilter::self()->filterUri( url );
  * \endcode
  *
@@ -574,40 +781,85 @@ class KUriFilterPrivate;
  * of a boolean flag:
  *
  * \code
- * QString u = KUriFilter::self()->filteredUri( "kde.org" );
+ * QString filteredText = KUriFilter::self()->filteredUri( "kde.org" );
  * \endcode
  *
- * You can also restrict the filter(s) to be used by supplying
- * the name of the filter(s) to use.  By defualt all available
- * filters will be used. To use specific filters, add the names
- * of the filters you want to use to a QStringList and invoke
- * the appropriate filtering function. The examples below show
- * the use of specific filters. The first one uses a single
- * filter called kshorturifilter while the second example uses
- * multiple filters:
+ * All of the above examples should result in "kde.org" being filtered into
+ * "http://kde.org".
+ *
+ * You can also restrict the filters to be used by supplying the name of the
+ * filters you want to use. By defualt all available filters are used.
+ *
+ * To use specific filters, add the names of the filters you want to use to a
+ * QStringList and invoke the appropriate filtering function.
+ *
+ * The examples below show the use of specific filters. KDE ships with the
+ * following filter plugins by default:
+ *
+ * kshorturifilter:
+ * This is used for filtering potentially valid url inputs such as "kde.org"
+ * Additionally it filters shell variables and shortcuts such as $HOME and
+ * ~ as well as man and info page shortcuts, # and ## respectively.
+ *
+ * kuriikwsfilter:
+ * This is used for filtering normal input text into a web search url using the
+ * configured fallback search engine selected by the user.
+ *
+ * kurisearchfilter:
+ * This is used for filtering KDE webshortcuts. For example "gg:KDE" will be
+ * converted to a url for searching the work "KDE" using the Google search
+ * engine.
+ *
+ * localdomainfilter:
+ * This is used for doing a DNS lookup to determine whether the input is a valid
+ * local address.
  *
  * \code
- * QString text = "kde.org";
- * bool filtered = KUriFilter::self()->filterUri( text, "kshorturifilter" );
+ * QString text ("kde.org");
+ * bool filtered = KUriFilter::self()->filterUri(text, QLatin1String("kshorturifilter"));
  * \endcode
  *
+ * The above code should result in "kde.org" being filtered into "http://kde.org".
+ *
  * \code
  * QStringList list;
- * list << "kshorturifilter" << "localdomainfilter";
+ * list << QLatin1String("kshorturifilter") << QLatin1String("localdomainfilter");
  * bool filtered = KUriFilter::self()->filterUri( text, list );
  * \endcode
  *
- * KUriFilter also allows richer data exchange through a simple
- * meta-object called @p KUriFilterData. Using this meta-object
- * you can find out more information about the URL you want to
- * filter. See KUriFilterData for examples and details.
+ * Additionally if you only want to do search related filtering, you can use the
+ * search specific function, @ref filterSearchUri, that is available in KDE
+ * 4.5 and higher. For example, to search for a given input on the web you
+ * can do the following:
  *
- * @short Filters a given URL into its proper format whenever possible.
+ * KUriFilterData filterData ("foo");
+ * bool filtered = KUriFilter::self()->filterSearchUri(filterData, KUriFilterData::NormalTextFilter);
+ *
+ * KUriFilter converts all filtering requests to use @ref KUriFilterData
+ * internally. The use of this bi-directional class allows you to send specific
+ * instructions to the filter plugins as well as receive detailed information
+ * about the filtered request from them. See the documentation of KUriFilterData
+ * class for more examples and details.
+ *
+ * All functions in this class are thread safe and reentrant.
+ *
+ * @short Filters the given input into a valid url whenever possible.
  */
-
 class KIO_EXPORT KUriFilter
 {
 public:
+    /**
+    * This enum describes the types of search plugin filters available.
+    *
+    * @li NormalTextFilter      The plugin used to filter normal text, e.g. "some term to search".
+    * @li WebShortcutFilter     The plugin used to filter web shortcuts, e.g. gg:KDE.
+    */
+    enum SearchFilterType {
+        NormalTextFilter = 0x01,
+        WebShortcutFilter = 0x02
+    };
+    Q_DECLARE_FLAGS(SearchFilterTypes, SearchFilterType)
+
     /**
      *  Destructor
      */
@@ -684,17 +936,32 @@ public:
     QString filteredUri( const QString &uri, const QStringList& filters = QStringList() );
 
     /**
-     * Filters @p data using only the default search uri filter plugins.
-     *
-     * Only use this function if you are sure that the input you want to
-     * filter is a search term.
-     *
-     * @param data object that contains the URI to be filtered.
-     * @return true if the the data specified by @p data was successfully filtered.
+     * See @ref filterSearchUri(KUriFilterData&, SearchFilterTypes)
      *
      * @since 4.5
+     * @deprecated Use filterSearchUri(KUriFilterData&, SearchFilterTypes) instead.
      */
-    bool filterSearchUri(KUriFilterData &data);
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED bool filterSearchUri(KUriFilterData &data);
+#endif
+
+    /**
+     * Filter @p data using the criteria specified by @p types.
+     *
+     * The search filter type can be individual value of @ref SearchFilterTypes
+     * or a combination of those types using the bitwise OR operator.
+     *
+     * You can also use the flags from @ref KUriFilterData::SearchFilterOption
+     * to alter the filtering mechanisms of the search filter providers.
+     *
+     * @param data object that contains the URI to be filtered.
+     * @param types the search filters used to filter the request.
+     * @return true if the specified @p data was successfully filtered.
+     *
+     * @see KUriFilterData::setSearchFilteringOptions
+     * @since 4.6
+     */
+    bool filterSearchUri(KUriFilterData &data, SearchFilterTypes types);
 
     /**
      * Return a list of the names of all loaded plugins.
@@ -724,4 +991,7 @@ private:
     KUriFilterPrivate * const d;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(KUriFilterData::SearchFilterOptions)
+Q_DECLARE_OPERATORS_FOR_FLAGS(KUriFilter::SearchFilterTypes)
+
 #endif
