@@ -121,12 +121,23 @@ namespace KIO {
         static void doJob(SimpleJob *job);
 
         /**
-         * Calling ths function makes that @p job gets scheduled for later
-         * execution, if multiple jobs are registered it might wait for
-         * other jobs to finish.
+         * Schedules @p job scheduled for later
+         * execution. This just sets the job's priority to 1 now.
          * @param job the job to schedule
          */
-        static void scheduleJob(SimpleJob *job);
+#ifndef KDE_NO_DEPRECATED
+        KDE_DEPRECATED static void scheduleJob(SimpleJob *job);
+#endif
+
+        /**
+         * Changes the priority of @p job; jobs of the same priority run in the order in which
+         * they were created. Jobs of lower numeric priority always run before any
+         * waiting jobs of higher numeric priority. The range of priority is -10 to 10,
+         * the default priority of jobs is 0.
+         * @param job the job to change
+         * @param priority new priority of @p job, lower runs earlier
+         */
+        static void setJobPriority(SimpleJob *job, int priority);
 
         /**
          * Stop the execution of a job.
@@ -233,6 +244,7 @@ namespace KIO {
          * @see slaveConnected()
          * @see slaveError()
          */
+        // KDE5: those methods should probably be removed, ugly and only marginally useful
         static bool connect( const char *signal, const QObject *receiver,
                              const char *member);
 
@@ -273,17 +285,14 @@ namespace KIO {
                                                const QString &host, bool connected))
 
         // connected to D-Bus signal:
-        Q_PRIVATE_SLOT(schedulerPrivate, void slotReparseSlaveConfiguration(const QString &))
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotReparseSlaveConfiguration(const QString &, const QDBusMessage&))
 
-        Q_PRIVATE_SLOT(schedulerPrivate, void startStep())
-        Q_PRIVATE_SLOT(schedulerPrivate, void slotCleanIdleSlaves())
         Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveConnected())
         Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveError(int error, const QString &errorMsg))
-        Q_PRIVATE_SLOT(schedulerPrivate, void slotScheduleCoSlave())
         Q_PRIVATE_SLOT(schedulerPrivate, void slotUnregisterWindow(QObject *))
     private:
         friend class SchedulerPrivate;
-        SchedulerPrivate * const d;
+        SchedulerPrivate *const d;
 };
 
 }
