@@ -28,11 +28,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define SAVE_DELAY 3 // Save after 3 minutes
 
-#include <unistd.h>
-
 #include <QtCore/QTimer>
 #include <QtCore/QFile>
-
 #include <QtDBus/QtDBus>
 
 #include <kconfig.h>
@@ -46,6 +43,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kpluginfactory.h>
 #include <kpluginloader.h>
 
+
+#define QL1S(x)  QLatin1String(x)
+#define QL1C(x)  QLatin1Char(x)
+
 K_PLUGIN_FACTORY(KdedCookieServerFactory,
                  registerPlugin<KCookieServer>();
     )
@@ -82,20 +83,20 @@ KCookieServer::KCookieServer(QObject* parent, const QList<QVariant>&)
    mAdvicePending = false;
    mTimer = new QTimer();
    mTimer->setSingleShot(true);
-   connect(mTimer, SIGNAL( timeout()), SLOT( slotSave()));
+   connect(mTimer, SIGNAL(timeout()), SLOT(slotSave()));
    mConfig = new KConfig("kcookiejarrc");
    mCookieJar->loadConfig( mConfig );
 
-   QString filename = KStandardDirs::locateLocal("data", "kcookiejar/cookies");
+   const QString filename = KStandardDirs::locateLocal("data", "kcookiejar/cookies");
 
    // Stay backwards compatible!
-   QString filenameOld = KStandardDirs::locate("data", "kfm/cookies");
+   const QString filenameOld = KStandardDirs::locate("data", "kfm/cookies");
    if (!filenameOld.isEmpty())
    {
       mCookieJar->loadCookies( filenameOld );
       if (mCookieJar->saveCookies( filename))
       {
-         unlink(QFile::encodeName(filenameOld)); // Remove old kfm cookie file
+         QFile::remove(filenameOld); // Remove old kfm cookie file
       }
    }
    else
@@ -163,7 +164,7 @@ void KCookieServer::addCookies( const QString &url, const QByteArray &cookieHead
     }
 }
 
-void KCookieServer::checkCookies( KHttpCookieList *cookieList)
+void KCookieServer::checkCookies(KHttpCookieList *cookieList)
 {
     KHttpCookieList *list;
 
@@ -181,11 +182,9 @@ void KCookieServer::checkCookies( KHttpCookieList *cookieList)
             mCookieJar->addCookie(cookie);
             cookieIterator.remove();
             break;
-
         case KCookieReject:
             cookieIterator.remove();
             break;
-
         default:
             break;
         }
@@ -207,7 +206,7 @@ void KCookieServer::checkCookies( KHttpCookieList *cookieList)
             shownCookies << i;
         }
     }
-    kDebug() << shownCookies;
+    //kDebug() << shownCookies;
 
     KCookieWin *kw = new KCookieWin( 0L, currentList,
                                      mCookieJar->preferredDefaultPolicy(),
@@ -321,12 +320,9 @@ bool KCookieServer::cookieMatches(const KHttpCookie& c,
                                   const QString &path, const QString &name)
 {
     const bool hasDomain = !domain.isEmpty();
-    return
-        ((hasDomain && c.domain() == domain) ||
-         fqdn == c.host()) &&
-        (c.path()   == path) &&
-        (c.name()   == name) &&
-        (!c.isExpired(time(0)));
+    return (((hasDomain && c.domain() == domain) || fqdn == c.host()) &&
+            (c.path() == path) && (c.name() == name) &&
+            (!c.isExpired()));
 }
 
 
@@ -361,15 +357,15 @@ QStringList
 KCookieServer::findDomains()
 {
    QStringList result;
-   const QStringList domains = mCookieJar->getDomainList();
-   for ( QStringList::ConstIterator domIt = domains.begin();
-         domIt != domains.end(); ++domIt )
+   QStringListIterator it (mCookieJar->getDomainList());
+   while (it.hasNext())
    {
        // Ignore domains that have policy set for but contain
        // no cookies whatsoever...
-       const KHttpCookieList* list =  mCookieJar->getCookieList(*domIt, "");
+       const QString domain = it.next();
+       const KHttpCookieList* list =  mCookieJar->getCookieList(domain, "");
        if ( list && !list->isEmpty() )
-          result << *domIt;
+          result << domain;
    }
    return result;
 }
@@ -377,27 +373,50 @@ KCookieServer::findDomains()
 // DBUS function
 QStringList
 KCookieServer::findCookies(const QList<int> &fields,
-                           const QString &domain,
+                           const QString &_domain,
                            const QString &fqdn,
                            const QString &path,
                            const QString &name)
 {
     QStringList result;
-    const bool allDomCookies = name.isEmpty();
-
-    const KHttpCookieList* list =  mCookieJar->getCookieList(domain, fqdn);
-    if (list && !list->isEmpty()) {
-        Q_FOREACH(const KHttpCookie& cookie, *list) {
-            if (!allDomCookies) {
-                if (cookieMatches(cookie, domain, fqdn, path, name)) {
-                    putCookie(result, cookie, fields);
-                    break;
-                }
-            } else {
-                putCookie(result, cookie, fields);
+    const bool allCookies = name.isEmpty();
+    const QStringList domainList = _domain.split(QLatin1Char(' '));
+    
+    if (allCookies)
+    {
+        Q_FOREACH(const QString& domain, domainList)
+        {
+            const KHttpCookieList* list =  mCookieJar->getCookieList(domain, fqdn);
+            if (!list) 
+                continue;
+            Q_FOREACH(const KHttpCookie& cookie, *list)
+            {
+                if (cookie.isExpired())
+                    continue;
+                putCookie(result, cookie, fields);              
             }
         }
     }
+    else
+    {
+        Q_FOREACH(const QString& domain, domainList)
+        {
+            const KHttpCookieList* list =  mCookieJar->getCookieList(domain, fqdn);
+            if (!list) 
+                continue;
+            Q_FOREACH(const KHttpCookie& cookie, *list)
+            {
+                if (cookie.isExpired())
+                    continue;
+                if (cookieMatches(cookie, domain, fqdn, path, name)) 
+                {
+                    putCookie(result, cookie, fields);
+                    break;
+                }
+            }
+        }
+    }
+    
     return result;
 }
 
@@ -435,12 +454,11 @@ KCookieServer::deleteCookie(const QString &domain, const QString &fqdn,
 {
     KHttpCookieList* cookieList = mCookieJar->getCookieList( domain, fqdn );
     if (cookieList && !cookieList->isEmpty()) {
-        for (KHttpCookieList::iterator cookieIterator = cookieList->begin();
-             cookieIterator != cookieList->end();
-             ++cookieIterator ) {
-            KHttpCookie& cookie = *cookieIterator;
-            if (cookieMatches(cookie, domain, fqdn, path, name)) {
-                mCookieJar->eatCookie(cookieIterator);
+        KHttpCookieList::Iterator itEnd = cookieList->end();
+        for (KHttpCookieList::Iterator it = cookieList->begin(); it != itEnd; ++it)
+        {
+            if (cookieMatches(*it, domain, fqdn, path, name)) {
+                mCookieJar->eatCookie(it);
                 saveCookieJar();
                 break;
             }
@@ -495,7 +513,7 @@ KCookieServer::addDOMCookies(const QString &url, const QByteArray &cookieHeader,
 }
 
 // DBUS function
-void
+bool
 KCookieServer::setDomainAdvice(const QString &url, const QString &advice)
 {
    QString fqdn;
@@ -509,7 +527,9 @@ KCookieServer::setDomainAdvice(const QString &url, const QString &advice)
                                   KCookieJar::strToAdvice(advice));
       // Save the cookie config if it has changed
       mCookieJar->saveConfig( mConfig );
+      return true;
    }
+   return false;
 }
 
 // DBUS function
@@ -524,15 +544,15 @@ KCookieServer::getDomainAdvice(const QString &url)
       QStringList domains;
       mCookieJar->extractDomains(fqdn, domains);
 
-      QStringList::ConstIterator it = domains.constBegin();
-      while ( (advice == KCookieDunno) && (it != domains.constEnd()) )
+      QStringListIterator it (domains);
+      while ( (advice == KCookieDunno) && it.hasNext() )
       {
          // Always check advice in both ".domain" and "domain". Note
          // that we only want to check "domain" if it matches the
          // fqdn of the requested URL.
-         if ( (*it)[0] == '.' || (*it) == fqdn )
-            advice = mCookieJar->getDomainAdvice(*it);
-         ++it;
+        const QString& domain = it.next();
+         if ( domain.at(0) == '.' || domain == fqdn )
+            advice = mCookieJar->getDomainAdvice(domain);
       }
       if (advice == KCookieDunno)
          advice = mCookieJar->getGlobalAdvice();
