@@ -282,7 +282,7 @@ void CoreEngine::downloadPreview(Entry *entry)
         return;
     }
 
-    KUrl destination = KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10);
+    KUrl destination = QString(KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10));
     //kDebug() << "Downloading preview '" << source << "' to '" << destination << "'";
 
     // FIXME: check for validity
@@ -322,7 +322,7 @@ void CoreEngine::downloadPayload(Entry *entry)
         return;
     }
 
-    KUrl destination = KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10);
+    KUrl destination = QString(KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10));
     kDebug() << "Downloading payload '" << source << "' to '" << destination << "'";
 
     // FIXME: check for validity
@@ -354,7 +354,7 @@ bool CoreEngine::uploadEntry(Provider *provider, Entry *entry)
     }
 
     // FIXME: validate files etc.
-
+    m_uploadprovider = provider;
     m_uploadedentry = entry;
 
     KUrl sourcepayload = KUrl(entry->payload().representation());
@@ -425,7 +425,7 @@ void CoreEngine::slotProgress(KJob *job, unsigned long percent)
         url = transferJob->url().fileName();
     }
 
-    QString message = QString("loading %1").arg(url);
+    QString message = i18n("loading %1",url);
     emit signalProgress(message, percent);
 }
 
@@ -490,7 +490,7 @@ void CoreEngine::slotUploadPayloadResult(KJob *job)
         return;
     }
 
-    if (m_uploadedentry->preview().isEmpty()) {
+    if (m_uploadedentry->preview().representation().isEmpty()) {
         // FIXME: we abuse 'job' here for the shortcut if there's no preview
         slotUploadPreviewResult(job);
         return;
@@ -499,6 +499,8 @@ void CoreEngine::slotUploadPayloadResult(KJob *job)
     KUrl sourcepreview = KUrl(m_uploadedentry->preview().representation());
     KUrl destfolder = m_uploadprovider->uploadUrl();
 
+    destfolder.setFileName(sourcepreview.fileName());
+
     KIO::FileCopyJob *fcjob = KIO::file_copy(sourcepreview, destfolder, -1, KIO::Overwrite | KIO::HideProgressInfo);
     connect(fcjob,
             SIGNAL(result(KJob*)),
@@ -522,12 +524,18 @@ void CoreEngine::slotUploadPreviewResult(KJob *job)
     // when we upload, the entry should probably be cached!
 
     // FIXME: adhere to meta naming rules as discussed
-    KUrl sourcemeta = KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10) + ".meta";
+    KUrl sourcemeta = QString(KGlobal::dirs()->saveLocation("tmp") + KRandom::randomString(10) + ".meta");
     KUrl destfolder = m_uploadprovider->uploadUrl();
 
+    destfolder.setFileName(sourcemeta.fileName());
+
     EntryHandler eh(*m_uploadedentry);
     QDomElement exml = eh.entryXML();
 
+    QDomDocument doc;
+    QDomElement root = doc.createElement("ghnsupload");
+    root.appendChild(exml);
+
     QFile f(sourcemeta.path());
     if (!f.open(QIODevice::WriteOnly | QIODevice::Text)) {
         kError() << "Cannot write meta information to '" << sourcemeta << "'." << endl;
@@ -539,7 +547,7 @@ void CoreEngine::slotUploadPreviewResult(KJob *job)
         return;
     }
     QTextStream metastream(&f);
-    metastream << exml;
+    metastream << root;
     f.close();
 
     KIO::FileCopyJob *fcjob = KIO::file_copy(sourcemeta, destfolder, -1, KIO::Overwrite | KIO::HideProgressInfo);
@@ -1399,12 +1407,16 @@ bool CoreEngine::install(const QString &payloadfile)
         }
         if (!m_installation->installPath().isEmpty()) {
 #if defined(Q_WS_WIN)
+#ifndef _WIN32_WCE
             WCHAR wPath[MAX_PATH+1];
             if ( SHGetFolderPathW(NULL, CSIDL_APPDATA, NULL, SHGFP_TYPE_CURRENT, wPath) == S_OK) {
                 installdir = QString::fromUtf16((const ushort *) wPath) + QLatin1Char('/') + m_installation->installPath() + QLatin1Char('/');
             } else {
+#endif
                 installdir =  QDir::home().path() + QLatin1Char('/') + m_installation->installPath() + QLatin1Char('/');
+#ifndef _WIN32_WCE
             }
+#endif
 #else
             installdir = QDir::home().path() + '/' + m_installation->installPath() + '/';
 #endif
