@@ -247,7 +247,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
                         setSelectedPart( part, w );
                     else {
 #ifdef DEBUG_PARTMANAGER
-                        kDebug(1000) << "Part " << part << " (non-selectable) made active because " << w->className() << " got event" << " " << evType;
+                        kDebug(1000) << "Part " << part << " (non-selectable) made active because " << w->metaObject()->className() << " got event" << " " << evType;
 #endif
                         d->setReason( ev );
                         setActivePart( part, w );
@@ -258,7 +258,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
                 else if ( d->m_selectedWidget == w && d->m_selectedPart == part )
                 {
 #ifdef DEBUG_PARTMANAGER
-                    kDebug(1000) << "Part " << part << " made active (from selected) because " << w->className() << " got event" << " " << evType;
+                    kDebug(1000) << "Part " << part << " made active (from selected) because " << w->metaObject()->className() << " got event" << " " << evType;
 #endif
                     d->setReason( ev );
                     setActivePart( part, w );
@@ -276,7 +276,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
             else if ( part != d->m_activePart )
             {
 #ifdef DEBUG_PARTMANAGER
-                kDebug(1000) << "Part " << part << " made active because " << w->className() << " got event" << " " << evType;
+                kDebug(1000) << "Part " << part << " made active because " << w->metaObject()->className() << " got event" << " " << evType;
 #endif
                 d->setReason( ev );
                 setActivePart( part, w );
@@ -292,7 +292,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
                     ( w->windowFlags() & Qt::Popup ) || ( w->windowFlags() & Qt::Tool ) ) )
         {
 #ifdef DEBUG_PARTMANAGER
-            kDebug(1000) << QString("No part made active although %1/%2 got event - loop aborted").arg(obj->name()).arg(obj->className());
+            kDebug(1000) << QString("No part made active although %1/%2 got event - loop aborted").arg(obj->objectName()).arg(obj->metaObject()->className());
 #endif
             return false;
         }
@@ -300,7 +300,7 @@ bool PartManager::eventFilter( QObject *obj, QEvent *ev )
     }
 
 #ifdef DEBUG_PARTMANAGER
-    kDebug(1000) << QString("No part made active although %1/%2 got event").arg(obj->name()).arg(obj->className());
+    kDebug(1000) << QString("No part made active although %1/%2 got event").arg(obj->objectName()).arg(obj->metaObject()->className());
 #endif
     return false;
 }
@@ -366,18 +366,13 @@ void PartManager::addPart( Part *part, bool setActive )
 
 void PartManager::removePart( Part *part )
 {
-    if ( !d->m_parts.contains( part ) )
-    {
-        kFatal(1000) << QString("Can't remove part %1, not in KPartManager's list.").arg(part->objectName());
+    if (!d->m_parts.contains(part)) {
         return;
     }
 
-    //Warning. The part could be already deleted
-    //kDebug(1000) << QString("Part %1 removed").arg(part->name());
-    int nb = d->m_parts.count();
-    bool ok = d->m_parts.removeAll( part );
-    Q_ASSERT( ok );
-    Q_ASSERT( (int)d->m_parts.count() == nb-1 );
+    const int nb = d->m_parts.removeAll(part);
+    Q_ASSERT(nb == 1);
+    Q_UNUSED(nb); // no warning in release mode
     part->setManager(0);
 
     emit partRemoved( part );
@@ -410,7 +405,7 @@ void PartManager::setActivePart( Part *part, QWidget *widget )
 {
     if ( part && !d->m_parts.contains( part ) )
     {
-        kWarning( 1000 ) << "PartManager::setActivePart : trying to activate a non-registered part! " << part->objectName();
+        kWarning(1000) << "trying to activate a non-registered part!" << part->objectName();
         return; // don't allow someone call setActivePart with a part we don't know about
     }
 
@@ -480,7 +475,9 @@ void PartManager::setActivePart( Part *part, QWidget *widget )
     // Set the new active instance in KGlobal
     setActiveComponent(d->m_activePart ? d->m_activePart->componentData() : KGlobal::mainComponent());
 
+#ifdef DEBUG_PARTMANAGER
     kDebug(1000) << this << " emitting activePartChanged " << d->m_activePart;
+#endif
     emit activePartChanged( d->m_activePart );
 }
 
@@ -541,13 +538,13 @@ QWidget *PartManager::selectedWidget() const
 
 void PartManager::slotObjectDestroyed()
 {
-    kDebug(1000) << "KPartManager::slotObjectDestroyed()";
+    kDebug(1000);
     removePart( const_cast<Part *>( static_cast<const Part *>( sender() ) ) );
 }
 
 void PartManager::slotWidgetDestroyed()
 {
-    kDebug(1000) << "KPartsManager::slotWidgetDestroyed()";
+    kDebug(1000);
     if ( static_cast<const QWidget *>( sender() ) == d->m_activeWidget )
         setActivePart( 0 ); //do not remove the part because if the part's widget dies, then the
     //part will delete itself anyway, invoking removePart() in its destructor
