@@ -170,13 +170,17 @@ protected:
      * @internal
      * @return The plugin created from the library @p libname
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED static Plugin* loadPlugin( QObject * parent, const char* libname );
+#endif
 
     /**
      * @internal, added only for source compatibility
      * @return The plugin created from the library @p libname
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED static Plugin* loadPlugin( QObject * parent, const QByteArray &libname );
+#endif
 
     /**
      * @internal
@@ -184,6 +188,8 @@ protected:
      */
     static Plugin* loadPlugin( QObject * parent, const QString &libname );
 
+    static Plugin* loadPlugin( QObject * parent, const QString &libname, const QString &keyword  );
+
     virtual void setComponentData(const KComponentData &instance);
 
 private:
