@@ -65,13 +65,15 @@ class KCModuleContainer::KCModuleContainerPrivate
 // KComponentData objects when needed.
 /***********************************************************************/
 KCModuleContainer::KCModuleContainer( QWidget* parent, const QString& mods )
-	: KCModule( KGlobal::mainComponent(), parent ),d(new KCModuleContainerPrivate( QString(mods).remove( ' ' ).split( ',' ) ))
+    : KCModule( KGlobal::mainComponent(), parent ),
+      d(new KCModuleContainerPrivate( QString(mods).remove( ' ' ).split( ',', QString::SkipEmptyParts ) ))
 {
 	init();
 }
 
 KCModuleContainer::KCModuleContainer( QWidget* parent, const QStringList& mods )
-	: KCModule( KGlobal::mainComponent(), parent ), d( new KCModuleContainerPrivate( mods ) )
+    : KCModule( KGlobal::mainComponent(), parent ),
+      d( new KCModuleContainerPrivate( mods ) )
 {
 	init();
 }
@@ -83,7 +85,7 @@ void KCModuleContainer::init()
 	d->topLayout->setObjectName( "topLayout" );
 	d->tabWidget = new KTabWidget(this);
 	d->tabWidget->setObjectName( "tabWidget");
-	connect( d->tabWidget, SIGNAL( currentChanged( QWidget* ) ), SLOT( tabSwitched( QWidget* ) ));
+	connect( d->tabWidget, SIGNAL(currentChanged(int)), SLOT(tabSwitched(int)));
 	d->topLayout->addWidget( d->tabWidget );
 
 	if ( !d->modules.isEmpty() )
@@ -128,9 +130,9 @@ void KCModuleContainer::addModule( const QString& module )
 	setButtons( buttons() | proxy->realModule()->buttons() );
 }
 
-void KCModuleContainer::tabSwitched( QWidget * module )
+void KCModuleContainer::tabSwitched(int index)
 {
-	KCModuleProxy* mod = (KCModuleProxy *) module;
+	KCModuleProxy* mod = static_cast<KCModuleProxy *>(d->tabWidget->widget(index));
 	setQuickHelp( mod->quickHelp() );
 	setAboutData( mod->aboutData() );
 }
