@@ -46,7 +46,7 @@ class KCModuleProxy;
  *
  * @author Frans Englich <frans.englich@telia.com>
  */
-class KUTILS_EXPORT KCModuleContainer : public KCModule
+class KCMUTILS_EXPORT KCModuleContainer : public KCModule
 {
 	Q_OBJECT
 	public:
@@ -131,7 +131,7 @@ class KUTILS_EXPORT KCModuleContainer : public KCModule
 		/**
 		 * Enables/disables the Admin Mode button, as appropriate.
 		 */
-		void tabSwitched( QWidget * module );
+		void tabSwitched(int);
 
 		void moduleChanged(KCModuleProxy *proxy);
 
