@@ -46,8 +46,6 @@
 #include <QtDBus/QDBusConnectionInterface>
 
 
-// FIXME: connect to some NepomukServer signal which emit enabled/disabled information
-//        when the server shuts down and is started again
 // FIXME: disconnect localSocketClient after n seconds of idling (but take care of not
 //        disconnecting when iterators are open)
 
@@ -63,7 +61,8 @@ public:
           dbusModel( 0 ),
           localSocketModel( 0 ),
           dummyModel( 0 ),
-          m_socketConnectFailed( false ) {
+          m_socketConnectFailed( false ),
+          m_initMutex( QMutex::Recursive ) {
     }
 
     ~GlobalModelContainer() {
@@ -84,6 +83,8 @@ public:
 
         if( forced ) {
             m_socketConnectFailed = false;
+            delete dbusModel;
+            dbusModel = 0;
         }
 
         // TODO: check if the service is also initialized
@@ -96,7 +97,7 @@ public:
         if ( !m_socketConnectFailed && !localSocketClient.isConnected() ) {
             delete localSocketModel;
             localSocketModel = 0;
-            QString socketName = KGlobal::dirs()->locateLocal( "data", "nepomuk/socket" );
+            QString socketName = KGlobal::dirs()->locateLocal( "socket", "nepomuk-socket" );
             kDebug() << "Connecting to local socket" << socketName;
             if ( localSocketClient.connect( socketName ) ) {
                 localSocketModel = localSocketClient.createModel( "main" );
@@ -109,10 +110,10 @@ public:
     }
 
     Soprano::Model* model() {
-        init( false );
-
         QMutexLocker lock( &m_initMutex );
 
+        init( false );
+
         // we always prefer the faster local socket client
         if ( localSocketModel ) {
             return localSocketModel;
@@ -154,20 +155,7 @@ Nepomuk::MainModel::MainModel( QObject* parent )
       d( new Private(this) )
 {
     setParent( parent );
-
     init();
-
-    if ( s_modelContainer->dbusModel ) {
-        // we have to use the dbus model for signals in any case
-        connect( s_modelContainer->dbusModel, SIGNAL( statementsAdded() ),
-                 this, SIGNAL( statementsAdded() ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementsRemoved() ),
-                 this, SIGNAL( statementsRemoved() ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementAdded(const Soprano::Statement&) ),
-                 this, SIGNAL( statementAdded(const Soprano::Statement&) ) );
-        connect( s_modelContainer->dbusModel, SIGNAL( statementRemoved(const Soprano::Statement&) ),
-                 this, SIGNAL( statementRemoved(const Soprano::Statement&) ) );
-    }
 }
 
 
@@ -186,6 +174,17 @@ bool Nepomuk::MainModel::isValid() const
 bool Nepomuk::MainModel::init()
 {
     s_modelContainer->init( true );
+    if ( s_modelContainer->dbusModel ) {
+        // we have to use the dbus model for signals in any case
+        connect( s_modelContainer->dbusModel, SIGNAL( statementsAdded() ),
+                 this, SIGNAL( statementsAdded() ) );
+        connect( s_modelContainer->dbusModel, SIGNAL( statementsRemoved() ),
+                 this, SIGNAL( statementsRemoved() ) );
+        connect( s_modelContainer->dbusModel, SIGNAL( statementAdded(const Soprano::Statement&) ),
+                 this, SIGNAL( statementAdded(const Soprano::Statement&) ) );
+        connect( s_modelContainer->dbusModel, SIGNAL( statementRemoved(const Soprano::Statement&) ),
+                 this, SIGNAL( statementRemoved(const Soprano::Statement&) ) );
+    }
     return isValid();
 }
 
