@@ -23,7 +23,10 @@
 
 #include <QtCore/QHash>
 #include <QtCore/QStringList>
-#include <QtCore/QUrl>
+
+#include <kurl.h>
+
+#include "class.h"
 
 #include "nepomuk_export.h"
 
@@ -34,6 +37,10 @@ namespace Nepomuk {
     class Variant;
     class Tag;
     class Thing;
+    class File;
+    namespace Types {
+        class Property;
+    }
 
     enum ErrorCode {
         NoError = 0,
@@ -65,6 +72,23 @@ namespace Nepomuk {
      *
      * \see ResourceManager
      *
+     * \section nepomuk_resource_file_uris Special case: file URLs
+     *
+     * \p file:/ URLs are handled as a special case in Nepomuk. Starting with KDE 4.4
+     * they are no longer used to identify the Nepomuk resource but only stored as
+     * nie:url property. All resources have \p nepomuk:/res/<UUID> URIs. The Resource
+     * constructors handle this automatically. Thus, one can still use file URLs to
+     * construct the objects. But be aware of the following example:
+     *
+     * \code
+     * KUrl fileUrl("file:///home/foobar/example.txt");
+     * Nepomuk::Resource fileRes(fileUrl);
+     * QUrl fileResUri = fileRes.resourceUri();
+     * \endcode
+     *
+     * Here \p fileUrl and \p fileResUri are NOT equal. The latter is the resource URI
+     * of the form \p nepomuk:/res/<UUID>.
+     *
      * \author Sebastian Trueg <trueg@kde.org>
      */
     class NEPOMUK_EXPORT Resource
@@ -106,10 +130,6 @@ namespace Nepomuk {
          * \li A URI which already exist in Nepomuk results in loading of that particular resource.
          * \li A string which already exists as the nao:identifier of a resource results in loading
          * of that particular resource.
-         * \li A URI which does not exist yet is used to create a new resource (Caution: due to
-         * encoding weirdness using KUrl::url or QUrl::toString here might result in unwanted
-         * behaviour. It is recommended to always use the Resource(QUrl,QUrl) constructor if
-         * possible)
          * \li Any other string is used as nao:identifier for a new resource. This resource can
          * later be loaded again by using the same identifier with this constructor.
          *
@@ -146,7 +166,9 @@ namespace Nepomuk {
         /**
          * \deprecated use Resource( const QString&, const QUrl& )
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED Resource( const QString& pathOrIdentifier, const QString& type );
+#endif
 
         /**
          * Creates a new Resource object.
@@ -155,6 +177,8 @@ namespace Nepomuk {
          * created. Using an empty QUrl will result in a new resource with a random URI being created
          * on the first call to setProperty.
          *
+         * See the \ref nepomuk_resource_file_uris Special file URL handling.
+         *
          * \param type The URI identifying the type of the resource. If it is empty
          *             Resource falls back to http://www.w3.org/2000/01/rdf-schema\#Resource or
          *             in case the resource already exists the type will be read from the
@@ -212,11 +236,13 @@ namespace Nepomuk {
          * For historical reasons the method does return a URI as QString instead
          * of QUrl. The value equals resourceUri().toString().
          *
-         * \sa resourceUri, getIdentifiers
+         * \sa resourceUri(), getIdentifiers()
          *
-         * \deprecated use resourceUri instead
+         * \deprecated use resourceUri() instead
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED QString uri() const;
+#endif
 
         /**
          * The URI of the resource, uniquely identifying it. This URI in most
@@ -225,10 +251,11 @@ namespace Nepomuk {
          *
          * The most important thing to remember is that the URI of for example
          * a file does not necessarily have a relation to its local path.
-         * (Although Nepomuk tries to keep the URI of file resources in sync
-         * with the file URL for convenience.)
          *
-         * \sa uri, getIdentifiers
+         * \return The resource URI of the resource or an empty url if the
+         * resource does not exist() yet.
+         *
+         * \sa getIdentifiers()
          */
         QUrl resourceUri() const;
 
@@ -246,7 +273,9 @@ namespace Nepomuk {
          *
          * \deprecated use resourceType instead
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED QString type() const;
+#endif
 
         /**
          * The main type of the resource. Nepomuk tries hard to make this
@@ -300,7 +329,9 @@ namespace Nepomuk {
         /**
          * \deprecated Use properties()
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED QHash<QString, Variant> allProperties() const;
+#endif
 
         /**
          * \return A list of all defined properties
@@ -317,10 +348,19 @@ namespace Nepomuk {
          */
         bool hasProperty( const QUrl& uri ) const;
 
+        /**
+         * Check if the resource has a property \p p with value \p v.
+         *
+         * \since 4.5
+         */
+        bool hasProperty( const Types::Property& p, const Variant& v ) const;
+
         /**
          * \deprecated use hasProperty( const QUrl& ) const
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED bool hasProperty( const QString& uri ) const;
+#endif
 
         /**
          * Retrieve the value of property \a uri. If the property is not defined for
@@ -333,7 +373,9 @@ namespace Nepomuk {
         /**
          * \deprecated use property( const QUrl& ) const
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED Variant property( const QString& uri ) const;
+#endif
 
         /**
          * Set a property of the resource.
@@ -356,7 +398,9 @@ namespace Nepomuk {
         /**
          * \deprecated use setProperty( const QUrl& )
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED void setProperty( const QString& uri, const Variant& value );
+#endif
 
         /**
          * Remove property \a uri from this resource object.
@@ -378,7 +422,9 @@ namespace Nepomuk {
         /**
          * \deprecated use removeProperty( const QUrl& )
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED void removeProperty( const QString& uri );
+#endif
 
         /**
          * Remove this resource completely.
@@ -439,255 +485,317 @@ namespace Nepomuk {
         Thing pimoThing();
 
         /**
-         * Operator to compare two Resource objects. Normally one does not need this. It is
-         * mainly intended for testing and debugging purposes.
+         * Operator to compare two Resource objects.
          */
         bool operator==( const Resource& ) const;
 
         /**
-         * Get property 'description'. Everything can be annotated with 
-         * a simple string comment. 
+         * Operator to compare two Resource objects.
+         *
+         * \since 4.4
+         */
+        bool operator!=( const Resource& ) const;
+
+        /**
+         * Get property 'description'. Everything can be annotated with
+         * a simple string comment.
          */
         QString description() const;
 
         /**
-         * Set property 'description'. Everything can be annotated with 
-         * a simple string comment. 
+         * Set property 'description'. Everything can be annotated with
+         * a simple string comment.
          */
         void setDescription( const QString& value );
 
         /**
-         * \return The URI of the property 'description'. 
+         * \return The URI of the property 'description'.
          */
         static QString descriptionUri();
 
         /**
-         * Get property 'identifier'. 
+         * Get property 'identifier'.
          */
         QStringList identifiers() const;
 
         /**
-         * Set property 'identifier'. 
+         * Set property 'identifier'.
          */
         void setIdentifiers( const QStringList& value );
 
         /**
-         * Add a value to property 'identifier'. 
+         * Add a value to property 'identifier'.
          */
         void addIdentifier( const QString& value );
 
         /**
-         * \return The URI of the property 'identifier'. 
+         * \return The URI of the property 'identifier'.
          */
         static QString identifierUri();
 
         /**
-         * Get property 'altLabel'. 
+         * Get property 'altLabel'.
          */
         QStringList altLabels() const;
 
         /**
-         * Set property 'altLabel'. 
+         * Set property 'altLabel'.
          */
         void setAltLabels( const QStringList& value );
 
         /**
-         * Add a value to property 'altLabel'. 
+         * Add a value to property 'altLabel'.
          */
         void addAltLabel( const QString& value );
 
         /**
-         * \return The URI of the property 'altLabel'. 
+         * \return The URI of the property 'altLabel'.
          */
         static QString altLabelUri();
 
         /**
-         * Get property 'annotation'. 
+         * Get property 'annotation'.
          */
         QList<Resource> annotations() const;
 
         /**
-         * Set property 'annotation'. 
+         * Set property 'annotation'.
          */
         void setAnnotations( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'annotation'. 
+         * Add a value to property 'annotation'.
          */
         void addAnnotation( const Resource& value );
 
         /**
-         * \return The URI of the property 'annotation'. 
+         * \return The URI of the property 'annotation'.
          */
         static QString annotationUri();
 
         /**
-         * Get property 'Tag'. Each Resource can be tagged with an arbitrary 
-         * number of Tags. This allows a simple grouping of resources. 
+         * Get property 'Tag'. Each Resource can be tagged with an arbitrary
+         * number of Tags. This allows a simple grouping of resources.
          */
         QList<Tag> tags() const;
 
         /**
-         * Set property 'Tag'. Each Resource can be tagged with an arbitrary 
-         * number of Tags. This allows a simple grouping of resources. 
+         * Set property 'Tag'. Each Resource can be tagged with an arbitrary
+         * number of Tags. This allows a simple grouping of resources.
          */
         void setTags( const QList<Tag>& value );
 
         /**
-         * Add a value to property 'Tag'. Each Resource can be tagged with 
-         * an arbitrary number of Tags. This allows a simple grouping of 
-         * resources. 
+         * Add a value to property 'Tag'. Each Resource can be tagged with
+         * an arbitrary number of Tags. This allows a simple grouping of
+         * resources.
          */
         void addTag( const Tag& value );
 
         /**
-         * \return The URI of the property 'Tag'. 
+         * \return The URI of the property 'Tag'.
          */
         static QString tagUri();
 
         /**
-         * Get property 'Topic'. 
+         * Get property 'Topic'.
          */
         QList<Resource> topics() const;
 
         /**
-         * Set property 'Topic'. 
+         * Set property 'Topic'.
          */
         void setTopics( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'Topic'. 
+         * Add a value to property 'Topic'.
          */
         void addTopic( const Resource& value );
 
         /**
-         * \return The URI of the property 'Topic'. 
+         * \return The URI of the property 'Topic'.
          */
         static QString topicUri();
 
         /**
-         * Get property 'isTopicOf'. 
+         * Get property 'isTopicOf'.
          */
         QList<Resource> isTopicOfs() const;
 
         /**
-         * Set property 'isTopicOf'. 
+         * Set property 'isTopicOf'.
          */
         void setIsTopicOfs( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'isTopicOf'. 
+         * Add a value to property 'isTopicOf'.
          */
         void addIsTopicOf( const Resource& value );
 
         /**
-         * \return The URI of the property 'isTopicOf'. 
+         * \return The URI of the property 'isTopicOf'.
          */
         static QString isTopicOfUri();
 
         /**
-         * Get property 'isRelated'. 
+         * Get property 'isRelated'.
          */
         QList<Resource> isRelateds() const;
 
         /**
-         * Set property 'isRelated'. 
+         * Set property 'isRelated'.
          */
         void setIsRelateds( const QList<Resource>& value );
 
         /**
-         * Add a value to property 'isRelated'. 
+         * Add a value to property 'isRelated'.
          */
         void addIsRelated( const Resource& value );
 
         /**
-         * \return The URI of the property 'isRelated'. 
+         * \return The URI of the property 'isRelated'.
          */
         static QString isRelatedUri();
 
         /**
-         * Get property 'label'. 
+         * Get property 'label'.
          */
         QString label() const;
 
         /**
-         * Set property 'label'. 
+         * Set property 'label'.
          */
         void setLabel( const QString& value );
 
         /**
-         * \return The URI of the property 'label'. 
+         * \return The URI of the property 'label'.
          */
         static QString labelUri();
 
         /**
-         * Get property 'Rating'. 
+         * Get property 'Rating'.
          */
         quint32 rating() const;
 
         /**
-         * Set property 'Rating'. 
+         * Set property 'Rating'.
          */
         void setRating( const quint32& value );
 
         /**
-         * \return The URI of the property 'Rating'. 
+         * \return The URI of the property 'Rating'.
          */
         static QString ratingUri();
 
         /**
-         * Get property 'Symbol'. Each resource can have a symbol assigned. 
-         * For now this is a simple string which can either be the patch to 
-         * an actual pixmap file or just the name of an icon as defined by 
-         * the freedesktop.org standard. 
+         * Get property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
          */
         QStringList symbols() const;
 
         /**
-         * Set property 'Symbol'. Each resource can have a symbol assigned. 
-         * For now this is a simple string which can either be the patch to 
-         * an actual pixmap file or just the name of an icon as defined by 
-         * the freedesktop.org standard. 
+         * Set property 'Symbol'. Each resource can have a symbol assigned.
+         * For now this is a simple string which can either be the path to
+         * an actual pixmap file or just the name of an icon as defined by
+         * the freedesktop.org standard.
          */
         void setSymbols( const QStringList& value );
 
         /**
-         * Add a value to property 'Symbol'. Each resource can have a symbol 
-         * assigned. For now this is a simple string which can either be 
-         * the patch to an actual pixmap file or just the name of an icon as 
-         * defined by the freedesktop.org standard. 
+         * Add a value to property 'Symbol'. Each resource can have a symbol
+         * assigned. For now this is a simple string which can either be
+         * the path to an actual pixmap file or just the name of an icon as
+         * defined by the freedesktop.org standard.
          */
         void addSymbol( const QString& value );
 
         /**
-         * \return The URI of the property 'Symbol'. 
+         * \return The URI of the property 'Symbol'.
          */
         static QString symbolUri();
 
         /**
-         * Get all resources that have this resource set as property 'annotation'. 
-         * \sa ResourceManager::allResourcesWithProperty 
+         * Get all resources that have this resource set as property 'annotation'.
+         * \sa ResourceManager::allResourcesWithProperty
          */
         QList<Resource> annotationOf() const;
 
         /**
-         * Get all resources that have this resource set as property 'isRelated'. 
-         * \sa ResourceManager::allResourcesWithProperty 
+         * Get all resources that have this resource set as property 'isRelated'.
+         * \sa ResourceManager::allResourcesWithProperty
          */
         QList<Resource> isRelatedOf() const;
 
         /**
-         * Retrieve a list of all available Resource resources. This list 
-         * consists of all resource of type Resource that are stored in 
-         * the local Nepomuk meta data storage and any changes made locally. 
-         * Be aware that in some cases this list can get very big. Then it 
-         * might be better to use libKNep directly. 
+         * Retrieve a list of all available Resource resources. This list
+         * consists of all resource of type Resource that are stored in
+         * the local Nepomuk meta data storage and any changes made locally.
+         * Be aware that in some cases this list can get very big. Then it
+         * might be better to use libKNep directly.
          *
          * \sa ResourceManager::allResources
+         *
+         * \warning This list will be very big. Usage of this method is
+         * discouraged. Use Query::QueryServiceClient in combination with an
+         * empty Query::Query instead.
          */
         static QList<Resource> allResources();
 
+        /**
+         * \return The usage count stored for this resource.
+         *
+         * \sa increaseUsageCount()
+         *
+         * \since 4.5
+         */
+        int usageCount() const;
+
+        /**
+         * Increase the usage count of this resource and also
+         * update the last used date to the current date and time.
+         *
+         * \since 4.5
+         */
+        void increaseUsageCount();
+
+        /**
+         * \return \p true if this resource represents a file. Use toFile() to retrieve the
+         * corresponding file resource which provides convinience methods to handle file
+         * resources.
+         *
+         * \since 4.6
+         */
+        bool isFile() const;
+
+        /**
+         * Convert this resource into a File resource to have access to the convinience methods
+         * provided by the File class.
+         *
+         * \since 4.6
+         */
+        File toFile() const;
+
+        /**
+         * Allows to quickly load a resource from its resource URI without any
+         * additional checks. This is mostly used for optimized code within Nepomuk.
+         *
+         * In most situations the construtor Resource( QUrl, QUrl ) is better suited.
+         *
+         * \since 4.5
+         */
+        static Resource fromResourceUri( const KUrl& uri, const Nepomuk::Types::Class& type = Nepomuk::Types::Class(), ResourceManager* manager = 0 );
+
     private:
+        /**
+         * Determines the final ResourceData and updates m_data if
+         * necessary. This will call ResourceData::determineUri()
+         * and optionally merge with already loaded ResourceData
+         * instances representing the same resource.
+         */
+        void determineFinalResourceData() const;
+
         ResourceData* m_data;
 
         class Private;
@@ -695,6 +803,8 @@ namespace Nepomuk {
 
         friend class ResourceData;
     };
+
+    NEPOMUK_EXPORT uint qHash( const Resource& res );
 }
 
 #endif
