@@ -1,6 +1,7 @@
 /*
  * This file is part of the Nepomuk KDE project.
  * Copyright (C) 2006-2010 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2010 Vishesh Handa <handa.vish@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -71,7 +72,6 @@ Nepomuk::ResourceData* Nepomuk::ResourceManagerPrivate::data( const QUrl& uri, c
         return data;
     }
     else {
-        QMutexLocker lock( &mutex );
         return new ResourceData( QUrl(), uri, type, this );
     }
 }
@@ -99,7 +99,6 @@ Nepomuk::ResourceData* Nepomuk::ResourceManagerPrivate::dataForResourceUri( cons
         return data;
     }
     else {
-        QMutexLocker lock( &mutex );
         return new ResourceData( uri, QUrl(), type, this );
     }
 }
@@ -181,7 +180,7 @@ void Nepomuk::ResourceManagerPrivate::cleanupCache( int num )
 {
     QMutexLocker lock( &mutex );
 
-    QSet<ResourceData*> rdl = m_uriKickoffData.values().toSet();
+    QSet<ResourceData*> rdl = m_uriKickoffData.values().toSet() + m_initializedData.values().toSet();
     for( QSet<ResourceData*>::iterator rdIt = rdl.begin();
          rdIt != rdl.end(); ++rdIt ) {
         ResourceData* data = *rdIt;
@@ -339,10 +338,12 @@ bool Nepomuk::ResourceManager::initialized() const
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 Nepomuk::Resource Nepomuk::ResourceManager::createResourceFromUri( const QString& uri )
 {
     return Resource( uri, QUrl() );
 }
+#endif
 
 void Nepomuk::ResourceManager::removeResource( const QString& uri )
 {
@@ -357,10 +358,12 @@ void Nepomuk::ResourceManager::notifyError( const QString& uri, int errorCode )
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QString& type )
 {
     return allResourcesOfType( QUrl(type) );
 }
+#endif
 
 
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QUrl& type )
@@ -412,10 +415,12 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResources()
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( const QString& uri, const Variant& v )
 {
     return allResourcesWithProperty( QUrl(uri), v );
 }
+#endif
 
 
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( const QUrl& uri, const Variant& v )
@@ -434,16 +439,8 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( con
         }
 
         // check remote data
-        Soprano::Node n;
-        if( v.isResource() ) {
-            n = v.toResource().resourceUri();
-        }
-        else {
-            n = valueToRDFNode(v);
-        }
-
         Soprano::Model* model = mainModel();
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), uri, n ) );
+        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), uri, v.toNode() ) );
 
         while( it.next() ) {
             Statement s = *it;
@@ -462,10 +459,12 @@ void Nepomuk::ResourceManager::clearCache()
 }
 
 
+#ifndef KDE_NO_DEPRECATED
 QString Nepomuk::ResourceManager::generateUniqueUri()
 {
     return generateUniqueUri( QString() ).toString();
 }
+#endif
 
 
 QUrl Nepomuk::ResourceManager::generateUniqueUri( const QString& name )
