@@ -96,7 +96,7 @@ public:
     Containment *addContainment(const QString &name, const QVariantList &args = QVariantList());
 
     /**
-     * Returns the Containment, if any, for a given physical screen
+     * Returns the Containment, if any, for a given physical screen and desktop
      *
      * @param screen number of the physical screen to locate
      * @param desktop the virtual desktop) to locate; if < 0 then it will
@@ -104,6 +104,21 @@ public:
      */
     Containment *containmentForScreen(int screen, int desktop = -1) const;
 
+    /**
+     * Returns the Containment for a given physical screen and desktop, creating one
+     * if none exists
+     *
+     * @param screen number of the physical screen to locate
+     * @param desktop the virtual desktop) to locate; if < 0 then it will
+     *        simply return the first Containment associated with screen
+     * @param defaultPluginIfNonExistent the plugin to load by default; "null" is an empty
+     * Containment and "default" creates the default plugin
+     * @param defaultArgs optional arguments to pass in when creating a Containment if needed
+     * @since 4.6
+     */
+    Containment *containmentForScreen(int screen, int desktop,
+                                      const QString &defaultPluginIfNonExistent,
+                                      const QVariantList &defaultArgs = QVariantList());
     /**
      * Adds a widget in the topleft quadrant in the scene. Widgets in the topleft quadrant are
      * normally never shown unless you specifically aim a view at it, which makes it ideal for
@@ -254,24 +269,46 @@ public:
     /**
      * Imports an applet layout from a config file. The results will be added to the
      * current set of Containments.
+     * @deprecated Use the 4.6 version that takes a KConfigGroup
      *
      * @param config the name of the config file to load from,
      *               or the default config file if QString()
      * @return the list of containments that were loaded
      * @since 4.5
      */
-    QList<Plasma::Containment *> importLayout(const KConfigBase &config);
+#ifndef KDE_NO_DEPRECATED
+    KDE_DEPRECATED QList<Plasma::Containment *> importLayout(const KConfigBase &config);
+#endif
+
+    /**
+     * Returns the name of the preferred plugin to be used as containment toolboxes.
+     * CustomContainments and CustomPanelContainments can still override it as their liking. It's also not guaranteed that the plugin will actually exist.
+     *
+     * @param type the containment type of which we want to know the associated toolbox plugin
+     * @since 4.6
+     */
+    QString preferredToolBoxPlugin(const Containment::Type type) const;
+
+    /**
+     * Imports an applet layout from a config file. The results will be added to the
+     * current set of Containments.
+     *
+     * @param config the name of the config file to load from,
+     *               or the default config file if QString()
+     * @return the list of containments that were loaded
+     * @since 4.6
+     */
+    QList<Plasma::Containment *> importLayout(const KConfigGroup &config);
 
     /**
      * Exports a set of containments to a config file.
      *
      * @param config the config group to save to
      * @param containments the list of containments to save
-     * @since 4.5.4
+     * @since 4.6
      */
     void exportLayout(KConfigGroup &config, QList<Containment*> containments);
 
-
 public Q_SLOTS:
     /**
      * Initializes the layout from a config file. This will first clear any existing
@@ -427,6 +464,14 @@ protected:
      */
     void mapAnimation(Animator::Animation from, const QString &to);
 
+    /**
+     * @return The preferred toolbox plugin name for a given containment type.
+     * @param type the containment type of which we want to know the preferred toolbox plugin.
+     * @param plugin the toolbox plugin name
+     * @since 4.6
+     */
+    void setPreferredToolBoxPlugin(const Containment::Type type, const QString &plugin);
+
     //Reimplemented from QGraphicsScene
     void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
     void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
