@@ -22,7 +22,9 @@
 
 #include <QtCore/QHash>
 #include <QtCore/QObject>
+#include <QtCore/QTimer>
 
+#include <kjob.h>
 #include <plasma/plasma_export.h>
 #include <plasma/dataengine.h>
 
@@ -122,6 +124,36 @@ class PLASMA_EXPORT DataContainer : public QObject
         void connectVisualization(QObject *visualization, uint pollingInterval,
                                   Plasma::IntervalAlignment alignment);
 
+        /**
+         * sets this data container to be automatically stored.
+         * @param whether this data container should be stored
+         * @since 4.6
+         */
+        void setStorageEnabled(bool store);
+
+        /**
+         * @return true if the data container has been marked for storage
+         * @since 4.6
+         */
+        bool isStorageEnabled() const;
+
+        /**
+         * @return true if the data container has been updated, but not stored
+         */
+        bool needsToBeStored() const;
+
+        /**
+         * sets that the data container needs to be stored or not.
+         * @param whether the data container needs to be stored
+         */
+        void setNeedsToBeStored(bool store);
+
+        /**
+         * @return the DataEngine that the DataContainer is
+         * a child of.
+         */
+        DataEngine* getDataEngine();
+
     public Q_SLOTS:
         /**
          * Disconnects an object from this DataContainer.
@@ -215,7 +247,14 @@ class PLASMA_EXPORT DataContainer : public QObject
 
     private:
         friend class SignalRelay;
+        friend class DataContainerPrivate;
+        friend class DataEngineManager;
         DataContainerPrivate *const d;
+
+        Q_PRIVATE_SLOT(d, void storeJobFinished(KJob *job))
+        Q_PRIVATE_SLOT(d, void populateFromStoredData(KJob *job))
+        Q_PRIVATE_SLOT(d, void store())
+        Q_PRIVATE_SLOT(d, void retrieve())
 };
 
 } // Plasma namespace
