@@ -51,7 +51,6 @@ class DialogPrivate;
 class PLASMA_EXPORT Dialog : public QWidget
 {
     Q_OBJECT
-
     public:
         /**
          * Use these flags to choose the active resize corners.
@@ -73,7 +72,18 @@ class PLASMA_EXPORT Dialog : public QWidget
         explicit Dialog(QWidget * parent = 0, Qt::WindowFlags f =  Qt::Window);
         virtual ~Dialog();
 
+        /**
+         * Sets a QGraphicsWidget to be shown as the content in this dialog.
+         * The dialog will then set up a QGraphicsView and coordinate geometry with
+         * the widget automatically.
+         *
+         * @arg widget the QGraphicsWidget to display in this dialog
+         */
         void setGraphicsWidget(QGraphicsWidget *widget);
+
+        /**
+         * @return the graphics widget shown in this dialog
+         */
         QGraphicsWidget *graphicsWidget();
 
         /**
@@ -90,15 +100,29 @@ class PLASMA_EXPORT Dialog : public QWidget
         /**
          * Causes an animated hide; requires compositing to work, otherwise
          * the dialog will simply hide.
+         * @since 4.3
          */
         void animatedHide(Plasma::Direction direction);
 
         /**
-         * Causes an animated hide; requires compositing to work, otherwise
-         * the dialog will simply hide.
+         * Causes an animated show; requires compositing to work, otherwise
+         * the dialog will simply show.
+         * @since 4.3
          */
         void animatedShow(Plasma::Direction direction);
 
+        /**
+         * @return the preferred aspect ratio mode for placement and resizing
+         * @since 4.4
+         */
+        Plasma::AspectRatioMode aspectRatioMode() const;
+
+        /**
+         * Sets the preferred aspect ratio mode for placement and resizing
+         * @since 4.4
+        */
+        void setAspectRatioMode(Plasma::AspectRatioMode mode);
+
     Q_SIGNALS:
         /**
          * Fires when the dialog automatically resizes.
@@ -110,6 +134,17 @@ class PLASMA_EXPORT Dialog : public QWidget
          */
         void dialogVisible(bool status);
 
+    public Q_SLOTS:
+        /**
+         * Adjusts the dialog to the associated QGraphicsWidget's geometry
+         * Should not normally need to be called by users of Dialog as Dialog
+         * does it automatically. Event compression may cause unwanted delays,
+         * however, and so this method may be called to immediately cause a
+         * synchronization.
+         * @since 4.5
+         */
+        void syncToGraphicsWidget();
+
     protected:
         /**
          * Reimplemented from QWidget
@@ -120,6 +155,7 @@ class PLASMA_EXPORT Dialog : public QWidget
         bool eventFilter(QObject *watched, QEvent *event);
         void hideEvent(QHideEvent *event);
         void showEvent(QShowEvent *event);
+        void focusInEvent(QFocusEvent *event);
         void mouseMoveEvent(QMouseEvent *event);
         void mousePressEvent(QMouseEvent *event);
         void mouseReleaseEvent(QMouseEvent *event);
@@ -141,8 +177,10 @@ class PLASMA_EXPORT Dialog : public QWidget
          * React to theme changes
          */
         Q_PRIVATE_SLOT(d, void themeChanged())
-        Q_PRIVATE_SLOT(d, void progressHide(qreal))
-        Q_PRIVATE_SLOT(d, void progressShow(qreal))
+        Q_PRIVATE_SLOT(d, void checkBorders())
+        Q_PRIVATE_SLOT(d, void delayedAdjustSize())
+
+        friend class PopupAppletPrivate;
 };
 
 } // Plasma namespace
