@@ -596,6 +596,10 @@ void ExtenderItem::destroy()
 
 void ExtenderItem::setCollapsed(bool collapsed)
 {
+    if (extender()->d->destroying) {
+        return;
+    }
+
     config().writeEntry("isCollapsed", collapsed);
     d->collapsed = collapsed;
     d->collapseIcon->setToolTip(collapsed ? i18n("Expand this widget") : i18n("Collapse this widget"));
@@ -1001,7 +1005,7 @@ void ExtenderItemPrivate::themeChanged()
 {
     kDebug();
     if (dragStarted) {
-        background->setImagePath("opaque/widgets/extender-background");
+        background->setImagePath("opaque/dialogs/background");
         background->setEnabledBorders(FrameSvg::AllBorders);
     } else {
         background->setImagePath("widgets/extender-background");
