@@ -33,10 +33,12 @@
 #include <karchive.h>
 #include <kcomponentdata.h>
 #include <kdesktopfile.h>
+#ifndef PLASMA_NO_KIO
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
 #include <kio/jobclasses.h>
 #include <kio/job.h>
+#endif
 #include <kmimetype.h>
 #include <kplugininfo.h>
 #include <kstandarddirs.h>
@@ -56,6 +58,74 @@
 namespace Plasma
 {
 
+#ifdef PLASMA_NO_KIO // Provide some convenience for dealing with folders
+
+bool copyFolder(QString sourcePath, QString targetPath)
+{
+    QDir source(sourcePath);
+    if(!source.exists())
+        return false;
+
+    QDir target(targetPath);
+    if(!target.exists()) {
+        QString targetName = target.dirName();
+        target.cdUp();
+        target.mkdir(targetName);
+        target = QDir(targetPath);
+    }
+
+    foreach (const QString &fileName, source.entryList(QDir::Files)) {
+        QString sourceFilePath = sourcePath + QDir::separator() + fileName;
+        QString targetFilePath = targetPath + QDir::separator() + fileName;
+
+        if (!QFile::copy(sourceFilePath, targetFilePath)) {
+            return false;
+        }
+    }
+
+    foreach (const QString &subFolderName, source.entryList(QDir::AllDirs | QDir::NoDotAndDotDot)) {
+        QString sourceSubFolderPath = sourcePath + QDir::separator() + subFolderName;
+        QString targetSubFolderPath = targetPath + QDir::separator() + subFolderName;
+
+        if (!copyFolder(sourceSubFolderPath, targetSubFolderPath)) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+bool removeFolder(QString folderPath)
+{
+    QDir folder(folderPath);
+    if(!folder.exists())
+        return false;
+
+    foreach (const QString &fileName, folder.entryList(QDir::Files)) {
+        if (!QFile::remove(folderPath + QDir::separator() + fileName)) {
+            return false;
+        }
+    }
+
+    foreach (const QString &subFolderName, folder.entryList(QDir::AllDirs | QDir::NoDotAndDotDot)) {
+        if (!removeFolder(folderPath + QDir::separator() + subFolderName)) {
+            return false;
+        }
+    }
+
+    QString folderName = folder.dirName();
+    folder.cdUp();
+    return folder.rmdir(folderName);
+}
+
+#endif // PLASMA_NO_KIO
+
+
+Package::Package()
+    : d(new PackagePrivate(PackageStructure::Ptr(0), QString()))
+{
+}
+
 Package::Package(const QString &packageRoot, const QString &package,
                  PackageStructure::Ptr structure)
     : d(new PackagePrivate(structure, packageRoot + '/' + package))
@@ -67,19 +137,54 @@ Package::Package(const QString &packagePath, PackageStructure::Ptr structure)
 {
 }
 
+Package::Package(const Package &other)
+    : d(new PackagePrivate(*other.d))
+{
+}
+
 Package::~Package()
 {
     delete d;
 }
 
+Package &Package::operator=(const Package &rhs)
+{
+    if (&rhs != this) {
+        *d = *rhs.d;
+    }
+
+    return *this;
+}
+
 bool Package::isValid() const
 {
     if (!d->valid) {
         return false;
     }
 
+    //search for the file in all prefixes and in all possible paths for each prefix
+    //even if it's a big nested loop, usually there is one prefix and one location
+    //so shouldn't cause too much disk access
+    QStringList prefixes = d->structure->contentsPrefixPaths();
+    if (prefixes.isEmpty()) {
+        prefixes << QString();
+    }
+
     foreach (const char *dir, d->structure->requiredDirectories()) {
-        if (!QFile::exists(d->structure->path() + d->structure->contentsPrefix() + d->structure->path(dir))) {
+        bool failed = true;
+        foreach (const QString &path, d->structure->searchPath(dir)) {
+            foreach (const QString &prefix, prefixes) {
+                if (QFile::exists(d->structure->path() + prefix + path)) {
+                    failed = false;
+                    break;
+                }
+            }
+            if (!failed) {
+                break;
+            }
+        }
+
+        if (failed) {
             kWarning() << "Could not find required directory" << dir;
             d->valid = false;
             return false;
@@ -87,9 +192,21 @@ bool Package::isValid() const
     }
 
     foreach (const char *file, d->structure->requiredFiles()) {
-        if (!QFile::exists(d->structure->path() + d->structure->contentsPrefix() + d->structure->path(file))) {
-            kWarning() << "Could not find required file" << file << ", look in"
-                       << d->structure->path() + d->structure->contentsPrefix() + d->structure->path(file) << endl;
+        bool failed = true;
+        foreach (const QString &path, d->structure->searchPath(file)) {
+            foreach (const QString &prefix, prefixes) {
+                if (QFile::exists(d->structure->path() + prefix + path)) {
+                    failed = false;
+                    break;
+                }
+            }
+            if (!failed) {
+                break;
+            }
+        }
+
+        if (failed) {
+            kWarning() << "Could not find required file" << file;
             d->valid = false;
             return false;
         }
@@ -101,38 +218,63 @@ bool Package::isValid() const
 QString Package::filePath(const char *fileType, const QString &filename) const
 {
     if (!d->valid) {
-        kDebug() << "package is not valid";
+        //kDebug() << "package is not valid";
         return QString();
     }
 
-    QString path = d->structure->path(fileType);
+    QStringList paths;
 
-    if (path.isEmpty()) {
-        kDebug() << "no matching path came of it, while looking for" << fileType << filename;
-        return QString();
-    }
+    if (qstrlen(fileType) != 0) {
+        paths = d->structure->searchPath(fileType);
 
-    path.prepend(d->structure->path() + d->structure->contentsPrefix());
-
-    if (!filename.isEmpty()) {
-        path.append("/").append(filename);
-    }
-
-    if (QFile::exists(path)) {
-        if (d->structure->allowExternalPaths()) {
-            return path;
+        if (paths.isEmpty()) {
+            //kDebug() << "no matching path came of it, while looking for" << fileType << filename;
+            return QString();
         }
+    } else {
+        //when filetype is empty paths is always empty, so try with an empty string
+        paths << QString();
+    }
 
-        // ensure that we don't return files outside of our base path
-        // due to symlink or ../ games
-        QDir dir(path);
-        QString canonicalized = dir.canonicalPath() + QDir::separator();
-        if (canonicalized.startsWith(d->structure->path())) {
-            return path;
+    //Nested loop, but in the medium case resolves to just one iteration
+    QStringList prefixes = d->structure->contentsPrefixPaths();
+    if (prefixes.isEmpty()) {
+        prefixes << QString();
+    }
+
+    //kDebug() << "prefixes:" << prefixes.count() << prefixes;
+    foreach (const QString &contentsPrefix, prefixes) {
+        const QString prefix(d->structure->path() + contentsPrefix);
+
+        foreach (const QString &path, paths) {
+            QString file = prefix + path;
+
+            if (!filename.isEmpty()) {
+                file.append("/").append(filename);
+            }
+
+            //kDebug() << "testing" << file << QFile::exists("/bin/ls") << QFile::exists(file);
+            if (QFile::exists(file)) {
+                if (d->structure->allowExternalPaths()) {
+                    //kDebug() << "found" << file;
+                    return file;
+                }
+
+                // ensure that we don't return files outside of our base path
+                // due to symlink or ../ games
+                QDir dir(file);
+                QString canonicalized = dir.canonicalPath() + QDir::separator();
+
+                //kDebug() << "testing that" << canonicalized << "is in" << d->structure->path();
+                if (canonicalized.startsWith(d->structure->path())) {
+                    //kDebug() << "found" << file;
+                    return file;
+                }
+            }
         }
     }
 
-    kDebug() << path << "does not exist";
+    //kDebug() << fileType << filename << "does not exist in" << prefixes << "at root" << d->structure->path();
     return QString();
 }
 
@@ -152,18 +294,24 @@ QStringList Package::entryList(const char *fileType) const
 
 PackageMetadata Package::metadata() const
 {
-    return d->structure->metadata();
+    if (d->structure) {
+        return d->structure->metadata();
+    }
+
+    return PackageMetadata();
 }
 
 void Package::setPath(const QString &path)
 {
-    d->structure->setPath(path);
-    d->valid = !d->structure->path().isEmpty();
+    if (d->structure) {
+        d->structure->setPath(path);
+        d->valid = !d->structure->path().isEmpty();
+    }
 }
 
 const QString Package::path() const
 {
-    return d->structure->path();
+    return d->structure ? d->structure->path() : QString();
 }
 
 const PackageStructure::Ptr Package::structure() const
@@ -228,15 +376,13 @@ void PackagePrivate::updateHash(const QString &basePath, const QString &subPath,
 QString Package::contentsHash() const
 {
 #ifdef QCA2_FOUND
-    if (!QCA::isSupported("sha1")) {
-        kWarning() << "can not create hash for" << path() << "due to no SHA1 support in QCA2";
+    if (!d->valid) {
+        kWarning() << "can not create hash due to Package being invalid";
         return QString();
     }
 
-    const QString basePath = d->structure->path() + d->structure->contentsPrefix();
-    QDir dir(basePath);
-
-    if (!dir.exists()) {
+    if (!QCA::isSupported("sha1")) {
+        kWarning() << "can not create hash for" << path() << "due to no SHA1 support in QCA2";
         return QString();
     }
 
@@ -255,7 +401,21 @@ QString Package::contentsHash() const
         kWarning() << "no metadata at" << metadataPath;
     }
 
-    d->updateHash(basePath, QString(), dir, hash);
+    QStringList prefixes = d->structure->contentsPrefixPaths();
+    if (prefixes.isEmpty()) {
+        prefixes << QString();
+    }
+
+    foreach (QString prefix, prefixes) {
+        const QString basePath = d->structure->path() + prefix;
+        QDir dir(basePath);
+
+        if (!dir.exists()) {
+            return QString();
+        }
+
+        d->updateHash(basePath, QString(), dir, hash);
+    }
     return QCA::arrayToHex(hash.final().toByteArray());
 #else
     // no QCA2!
@@ -347,7 +507,7 @@ bool Package::installPackage(const QString &package,
         if (mimetype->is("application/zip")) {
             archive = new KZip(package);
         } else if (mimetype->is("application/x-compressed-tar") ||
-                   mimetype->is("application/x-tar") || mimetype->is("application/x-bzip-compressed-tar")) {
+                   mimetype->is("application/x-tar")|| mimetype->is("application/x-bzip-compressed-tar")) {
             archive = new KTar(package);
         } else {
             kWarning() << "Could not open package file, unsupported archive format:" << package << mimetype->name();
@@ -406,17 +566,32 @@ bool Package::installPackage(const QString &package,
 
     if (archivedPackage) {
         // it's in a temp dir, so just move it over.
+#ifndef PLASMA_NO_KIO
         KIO::CopyJob *job = KIO::move(KUrl(path), KUrl(targetName), KIO::HideProgressInfo);
-        if (!job->exec()) {
-            kWarning() << "Could not move package to destination:" << targetName << " : " << job->errorString();
+        const bool ok = job->exec();
+        const QString errorString = job->errorString();
+#else
+        const bool ok = copyFolder(path, targetName);
+        removeFolder(path);
+        const QString errorString("unknown");
+#endif
+        if (!ok) {
+            kWarning() << "Could not move package to destination:" << targetName << " : " << errorString;
             return false;
         }
     } else {
         // it's a directory containing the stuff, so copy the contents rather
         // than move them
+#ifndef PLASMA_NO_KIO
         KIO::CopyJob *job = KIO::copy(KUrl(path), KUrl(targetName), KIO::HideProgressInfo);
-        if (!job->exec()) {
-            kWarning() << "Could not copy package to destination:" << targetName << " : " << job->errorString();
+        const bool ok = job->exec();
+        const QString errorString = job->errorString();
+#else
+        const bool ok = copyFolder(path, targetName);
+        const QString errorString("unknown");
+#endif
+        if (!ok) {
+            kWarning() << "Could not copy package to destination:" << targetName << " : " << errorString;
             return false;
         }
     }
@@ -442,8 +617,15 @@ bool Package::installPackage(const QString &package,
         QString serviceName = servicePrefix + meta.pluginName();
 
         QString service = KStandardDirs::locateLocal("services", serviceName + ".desktop");
+#ifndef PLASMA_NO_KIO
         KIO::FileCopyJob *job = KIO::file_copy(metaPath, service, -1, KIO::HideProgressInfo);
-        if (job->exec()) {
+        const bool ok = job->exec();
+        const QString errorString = job->errorString();
+#else
+        const bool ok = QFile::copy(metaPath, service);
+        const QString errorString("unknown");
+#endif
+        if (ok) {
             // the icon in the installed file needs to point to the icon in the
             // installation dir!
             QString iconPath = targetName + '/' + cg.readEntry("Icon");
@@ -454,7 +636,7 @@ bool Package::installPackage(const QString &package,
                 cg.writeEntry("Icon", iconPath);
             }
         } else {
-            kWarning() << "Could not register package as service (this is not necessarily fatal):" << serviceName << " : " << job->errorString();
+            kWarning() << "Could not register package as service (this is not necessarily fatal):" << serviceName << " : " << errorString;
         }
     }
 
@@ -482,12 +664,18 @@ bool Package::uninstallPackage(const QString &pluginName,
 
     if (!ok) {
         kWarning() << "Unable to remove " << service;
-        return ok;
     }
 
+#ifndef PLASMA_NO_KIO
     KIO::DeleteJob *job = KIO::del(KUrl(targetName));
-    if (!job->exec()) {
-        kWarning() << "Could not delete package from:" << targetName << " : " << job->errorString();
+    ok = job->exec();
+    const QString errorString = job->errorString();
+#else
+    ok = removeFolder(targetName);
+    const QString errorString("unknown");
+#endif
+    if (!ok) {
+        kWarning() << "Could not delete package from:" << targetName << " : " << errorString;
         return false;
     }
 
@@ -520,8 +708,12 @@ bool Package::registerPackage(const PackageMetadata &data, const QString &iconPa
                               iconPath.right(iconPath.length() - iconPath.lastIndexOf("/")));
         cg.writeEntry("Icon", installedIcon);
         installedIcon = KStandardDirs::locateLocal("icon", installedIcon);
+#ifndef PLASMA_NO_KIO
         KIO::FileCopyJob *job = KIO::file_copy(iconPath, installedIcon, -1, KIO::HideProgressInfo);
         job->exec();
+#else
+        QFile::copy(iconPath, installedIcon);
+#endif
     }
 
     return true;
@@ -562,16 +754,38 @@ PackagePrivate::PackagePrivate(const PackageStructure::Ptr st, const QString &p)
         : structure(st),
           service(0)
 {
-    structure->setPath(p);
-    valid = !structure->path().isEmpty();
+    if (structure) {
+        structure->setPath(p);
+    }
+
+    valid = structure && !structure->path().isEmpty();
+}
+
+PackagePrivate::PackagePrivate(const PackagePrivate &other)
+        : structure(other.structure),
+          service(other.service),
+          valid(other.valid)
+{
 }
 
 PackagePrivate::~PackagePrivate()
 {
 }
 
+PackagePrivate &PackagePrivate::operator=(const PackagePrivate &rhs)
+{
+    structure = rhs.structure;
+    service = rhs.service;
+    valid = rhs.valid;
+    return *this;
+}
+
 void PackagePrivate::publish(AnnouncementMethods methods)
 {
+    if (!structure) {
+        return;
+    }
+
     if (!service) {
         service = new PlasmoidService(structure->path());
     }
