@@ -102,9 +102,11 @@ AppletHandle::AppletHandle(Containment *parent, Applet *applet, const QPointF &h
     m_configureIcons = new Svg(this);
     m_configureIcons->setImagePath("widgets/configuration-icons");
     m_configureIcons->setContainsMultipleImages(true);
+    connect(m_configureIcons, SIGNAL(repaintNeeded()), this, SLOT(scheduleUpdate()));
 
     m_background = new FrameSvg(this);
     m_background->setImagePath("widgets/background");
+    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(scheduleUpdate()));
     m_applet->installSceneEventFilter(this);
 }
 
@@ -236,7 +238,7 @@ void AppletHandle::paint(QPainter *painter, const QStyleOptionGraphicsItem *opti
                 //kDebug() << "opaquePoint" << opaquePoint
                 //         << m_background->marginSize(LeftMargin) << m_decorationRect.width();
                 g.setColorAt(0.0, Qt::transparent);
-                g.setColorAt(qMax(0.0, opaquePoint - 0.05), Qt::transparent);
+                g.setColorAt(qMax(0.0, opaquePoint - 0.05), Qt::transparent); //krazy:exclude=qminmax
                 g.setColorAt(opaquePoint, transparencyColor);
                 g.setColorAt(1.0, transparencyColor);
             } else {
@@ -382,6 +384,11 @@ void AppletHandle::emitDisappear()
     emit disappearDone(this);
 }
 
+void AppletHandle::scheduleUpdate()
+{
+    update();
+}
+
 AppletHandle::ButtonType AppletHandle::mapToButton(const QPointF &point) const
 {
     int iconMargin = m_iconSize / 2;
