@@ -22,13 +22,16 @@
 #define PLASMA_FRAMESVG_P_H
 
 #include <QHash>
+#include <QStringBuilder>
+
+#include <kdebug.h>
 
 namespace Plasma
 {
 class FrameData
 {
 public:
-    FrameData()
+    FrameData(FrameSvg *svg)
       : enabledBorders(FrameSvg::AllBorders),
         frameSize(-1,-1),
         topHeight(0),
@@ -43,9 +46,10 @@ public:
         stretchBorders(false),
         tileCenter(false)
     {
+        ref(svg);
     }
 
-    FrameData(const FrameData &other)
+    FrameData(const FrameData &other, FrameSvg *svg)
       : enabledBorders(other.enabledBorders),
         frameSize(other.frameSize),
         topHeight(0),
@@ -58,18 +62,27 @@ public:
         bottomMargin(0),
         noBorderPadding(false),
         stretchBorders(false),
-        tileCenter(false)
+        tileCenter(false),
+        composeOverBorder(false)
     {
+        ref(svg);
     }
 
     ~FrameData()
     {
     }
 
+    void ref(FrameSvg *svg);
+    bool deref(FrameSvg *svg);
+    bool removeRefs(FrameSvg *svg);
+    bool isUsed() const;
+    int refcount() const;
+
     FrameSvg::EnabledBorders enabledBorders;
     QPixmap cachedBackground;
-    QPixmap cachedOverlay;
-    QRegion cachedMask;
+    QHash<QString, QRegion> cachedMasks;
+    static const int MAX_CACHED_MASKS = 10;
+
     QSize frameSize;
 
     //measures
@@ -89,6 +102,9 @@ public:
     bool noBorderPadding : 1;
     bool stretchBorders : 1;
     bool tileCenter : 1;
+    bool composeOverBorder : 1;
+
+    QHash<FrameSvg *, int> references;
 };
 
 class FrameSvgPrivate
@@ -101,18 +117,18 @@ public:
     {
     }
 
-    ~FrameSvgPrivate()
-    {
-        qDeleteAll(frames);
-        frames.clear();
-    }
+    ~FrameSvgPrivate();
+
+    QPixmap alphaMask();
 
     void generateBackground(FrameData *frame);
-    void cacheFrame(const QString &prefix);
-    void updateSizes();
+    void generateFrameBackground(FrameData *frame);
+    QString cacheId(FrameData *frame, const QString &prefixToUse) const;
+    void cacheFrame(const QString &prefixToSave, const QPixmap &background, const QPixmap &overlay);
+    void updateSizes() const;
     void updateNeeded();
     void updateAndSignalSizes();
-    QSizeF frameSize(FrameData *frame);
+    QSizeF frameSize(FrameData *frame) const;
 
     Location location;
     QString prefix;
@@ -123,6 +139,8 @@ public:
     QPoint overlayPos;
 
     QHash<QString, FrameData*> frames;
+
+    static QHash<QString, FrameData *> s_sharedFrames;
 };
 
 }
