@@ -72,7 +72,7 @@ int WallpaperRenderThread::render(const QString &file,
     }
 
     if (!isRunning()) {
-        start();
+        start(QThread::LowPriority);
     } else {
         m_condition.wakeOne();
     }
@@ -116,13 +116,13 @@ void WallpaperRenderThread::run()
         result.fill(color.rgba());
 
         if (file.isEmpty() || !QFile::exists(file)) {
-            emit done(token, result, file, size, method, color);
+            emit done(this, token, result, file, size, method, color);
             break;
         }
 
         QPoint pos(0, 0);
         bool tiled = false;
-        bool scalable = file.endsWith("svg") || file.endsWith("svgz");
+        bool scalable = file.endsWith(QLatin1String("svg")) || file.endsWith(QLatin1String("svgz"));
         QSize scaledSize;
         QImage img;
 
@@ -155,7 +155,6 @@ void WallpaperRenderThread::run()
         switch (method)
         {
         case Wallpaper::ScaledResize:
-            imgSize *= ratio;
             scaledSize = size;
             break;
         case Wallpaper::CenteredResize:
@@ -180,7 +179,6 @@ void WallpaperRenderThread::run()
 
             break;
         case Wallpaper::MaxpectResize: {
-            imgSize *= ratio;
             float xratio = (float) size.width() / imgSize.width();
             float yratio = (float) size.height() / imgSize.height();
             if (xratio > yratio) {
@@ -197,7 +195,6 @@ void WallpaperRenderThread::run()
             break;
         }
         case Wallpaper::ScaledAndCroppedResize: {
-            imgSize *= ratio;
             float xratio = (float) size.width() / imgSize.width();
             float yratio = (float) size.height() / imgSize.height();
             if (xratio > yratio) {
@@ -261,11 +258,16 @@ void WallpaperRenderThread::run()
         }
 
         // signal we're done
-        emit done(token, result, file, size, method, color);
+        emit done(this, token, result, file, size, method, color);
         break;
     }
 }
 
+int WallpaperRenderThread::currentToken()
+{
+    return m_currentToken;
+}
+
 } // namespace Plasma
 
 #include "wallpaperrenderthread_p.moc"
