@@ -73,17 +73,47 @@ class ServicePrivate;
  * Plasma::ServiceJob *job = service->startOperationCall(op);
  * connect(job, SIGNAL(finished(KJob*)), this, SLOT(jobCompeted()));
  * @endcode
+ *
+ * Please remember, the service needs to be deleted when it will no longer be
+ * used. This can be done manually or by these (perhaps easier) alternatives:
+ *
+ * If it is needed throughout the lifetime of the object:
+ * @code
+ * service->setParent(this);
+ * @endcode
+ *
+ * If the service will not be used after just one operation call, use:
+ * @code
+ * connect(job, SIGNAL(finished(KJob*)), service, SLOT(deleteLater()));
+ * @endcode
+ *
  */
 class PLASMA_EXPORT Service : public QObject
 {
     Q_OBJECT
     Q_DECLARE_PRIVATE(Service)
+    Q_PROPERTY(QString destination READ destination WRITE setDestination)
+    Q_PROPERTY(QStringList operationNames READ operationNames)
+    Q_PROPERTY(QString name READ name)
+
 public:
     /**
      * Destructor
      */
     ~Service();
 
+    /**
+     * Used to load a given service from a plugin.
+     *
+     * @param name the plugin name of the service to load
+     * @param args a list of arguments to supply to the service plugin when loading it
+     * @param parent the parent object, if any, for the service
+     *
+     * @return a Service object, guaranteed to be not null.
+     * @since 4.5
+     */
+    static Service *load(const QString &name, const QVariantList &args, QObject *parent = 0);
+
     /**
      * Used to load a given service from a plugin.
      *
@@ -283,6 +313,7 @@ private:
     friend class PackagePrivate;
     friend class ServiceProvider;
     friend class RemoveService;
+    friend class PluginLoader;
 };
 
 } // namespace Plasma
