@@ -20,27 +20,24 @@
 
 #include "pushbutton.h"
 
-#include <QStyleOptionGraphicsItem>
-#include <QPainter>
 #include <QDir>
-#include <QApplication>
-
+#include <QPainter>
+#include <QStyleOptionGraphicsItem>
 #include <QWeakPointer>
-#include <QPropertyAnimation>
 
 #include <kicon.h>
 #include <kiconeffect.h>
 #include <kmimetype.h>
 #include <kpushbutton.h>
 
-#include "theme.h"
-#include "svg.h"
-#include "framesvg.h"
 #include "animator.h"
+#include "animations/animation.h"
+#include "framesvg.h"
 #include "paintutils.h"
 #include "private/actionwidgetinterface_p.h"
-#include <plasma/private/focusindicator_p.h>
-#include "animations/animation.h"
+#include "private/focusindicator_p.h"
+#include "private/themedwidgetinterface_p.h"
+#include "theme.h"
 
 namespace Plasma
 {
@@ -50,11 +47,9 @@ class PushButtonPrivate : public ActionWidgetInterface<PushButton>
 public:
     PushButtonPrivate(PushButton *pushButton)
         : ActionWidgetInterface<PushButton>(pushButton),
-          q(pushButton),
           background(0),
           fadeIn(false),
-          svg(0),
-          customFont(false)
+          svg(0)
     {
     }
 
@@ -107,11 +102,18 @@ public:
         static_cast<KPushButton*>(q->widget())->setIcon(KIcon(pm));
     }
 
+    void pressedChanged()
+    {
+        if (q->nativeWidget()->isDown() || q->nativeWidget()->isChecked()) {
+            focusIndicator->animateVisibility(false);
+        } else {
+            focusIndicator->animateVisibility(true);
+        }
+    }
+
     void syncActiveRect();
     void syncBorders();
 
-    PushButton *q;
-
     FrameSvg *background;
     bool fadeIn;
     qreal opacity;
@@ -119,11 +121,11 @@ public:
 
     Animation *hoverAnimation;
 
+    FocusIndicator *focusIndicator;
     QString imagePath;
     QString absImagePath;
     Svg *svg;
     QString svgElement;
-    bool customFont;
 };
 
 void PushButtonPrivate::syncActiveRect()
@@ -170,11 +172,12 @@ PushButton::PushButton(QGraphicsWidget *parent)
 
     d->hoverAnimation = Animator::create(Animator::PixmapTransitionAnimation);
     d->hoverAnimation->setTargetWidget(this);
-    d->hoverAnimation->setProperty("startPixmap", d->background->framePixmap());
 
     KPushButton *native = new KPushButton;
     connect(native, SIGNAL(pressed()), this, SIGNAL(pressed()));
+    connect(native, SIGNAL(pressed()), this, SLOT(pressedChanged()));
     connect(native, SIGNAL(released()), this, SIGNAL(released()));
+    connect(native, SIGNAL(released()), this, SLOT(pressedChanged()));
     connect(native, SIGNAL(clicked()), this, SIGNAL(clicked()));
     connect(native, SIGNAL(toggled(bool)), this, SIGNAL(toggled(bool)));
     setWidget(native);
@@ -183,12 +186,13 @@ PushButton::PushButton(QGraphicsWidget *parent)
 
     setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
 
-    FocusIndicator *focusIndicator = new FocusIndicator(this, "widgets/button");
+    d->focusIndicator = new FocusIndicator(this, d->background);
 
     d->syncBorders();
     setAcceptHoverEvents(true);
 
     connect(d->background, SIGNAL(repaintNeeded()), SLOT(syncBorders()));
+    d->initTheming();
 }
 
 PushButton::~PushButton()
@@ -248,6 +252,7 @@ QString PushButton::image() const
 
 void PushButton::setStyleSheet(const QString &stylesheet)
 {
+    d->focusIndicator->setVisible(stylesheet.isEmpty());
     widget()->setStyleSheet(stylesheet);
 }
 
@@ -296,6 +301,11 @@ void PushButton::setChecked(bool checked)
     nativeWidget()->setChecked(checked);
 }
 
+void PushButton::click()
+{
+    nativeWidget()->animateClick();
+}
+
 bool PushButton::isChecked() const
 {
     return nativeWidget()->isChecked();
@@ -316,21 +326,19 @@ void PushButton::resizeEvent(QGraphicsSceneResizeEvent *event)
     d->setPixmap();
 
    if (d->background) {
-        //resize all four panels
-        d->background->setElementPrefix("pressed");
-        d->background->resizeFrame(size());
+        //resize all panels
+       d->background->setElementPrefix("pressed");
+       d->background->resizeFrame(size());
 
-        d->syncActiveRect();
+       d->syncActiveRect();
 
-        d->background->setElementPrefix("active");
-        d->background->resizeFrame(d->activeRect.size());
-        d->hoverAnimation->setProperty("targetPixmap", d->background->framePixmap());
-        d->background->setElementPrefix("focus");
-        d->background->resizeFrame(d->activeRect.size());
+       d->background->setElementPrefix("normal");
+       d->background->resizeFrame(size());
+       d->hoverAnimation->setProperty("startPixmap", d->background->framePixmap());
 
-        d->background->setElementPrefix("normal");
-        d->background->resizeFrame(size());
-        d->hoverAnimation->setProperty("startPixmap", d->background->framePixmap());
+       d->background->setElementPrefix("active");
+       d->background->resizeFrame(d->activeRect.size());
+       d->hoverAnimation->setProperty("targetPixmap", d->background->framePixmap());
    }
 
    QGraphicsProxyWidget::resizeEvent(event);
@@ -414,14 +422,7 @@ void PushButton::paint(QPainter *painter,
         }
     }
 
-    QFont widgetFont;
-    if (d->customFont) {
-        widgetFont = font();
-    } else {
-        widgetFont = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-    }
-
-    QFontMetricsF fm(widgetFont);
+    QFontMetricsF fm(font());
     // If the height is too small increase the Height of the button to shall the whole text #192988
     if (rect.height() < fm.height()) {
         rect.setHeight(fm.height());
@@ -437,7 +438,7 @@ void PushButton::paint(QPainter *painter,
 
         QPainter p(&bufferPixmap);
         p.setPen(painter->pen());
-        p.setFont(widgetFont);
+        p.setFont(font());
 
         // Create the alpha gradient for the fade out effect
         QLinearGradient alphaGradient(0, 0, 1, 0);
@@ -445,12 +446,12 @@ void PushButton::paint(QPainter *painter,
         if (option->direction == Qt::LeftToRight) {
             alphaGradient.setColorAt(0, QColor(0, 0, 0, 255));
             alphaGradient.setColorAt(1, QColor(0, 0, 0, 0));
-            p.drawText(bufferPixmap.rect(), Qt::AlignLeft|Qt::AlignVCenter,
+            p.drawText(bufferPixmap.rect(), Qt::AlignLeft|Qt::AlignVCenter|Qt::TextShowMnemonic,
                        nativeWidget()->text());
         } else {
             alphaGradient.setColorAt(0, QColor(0, 0, 0, 0));
             alphaGradient.setColorAt(1, QColor(0, 0, 0, 255));
-            p.drawText(bufferPixmap.rect(), Qt::AlignRight|Qt::AlignVCenter,
+            p.drawText(bufferPixmap.rect(), Qt::AlignRight|Qt::AlignVCenter|Qt::TextShowMnemonic,
                        nativeWidget()->text());
         }
 
@@ -459,8 +460,8 @@ void PushButton::paint(QPainter *painter,
 
         painter->drawPixmap(rect.topLeft(), bufferPixmap);
     } else {
-        painter->setFont(widgetFont);
-        painter->drawText(rect, Qt::AlignCenter, nativeWidget()->text());
+        painter->setFont(font());
+        painter->drawText(rect, Qt::AlignCenter|Qt::TextShowMnemonic, nativeWidget()->text());
     }
 }
 
@@ -485,10 +486,7 @@ void PushButton::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
 
 void PushButton::changeEvent(QEvent *event)
 {
-    if (event->type() == QEvent::FontChange) {
-        d->customFont = true;
-    }
-
+    d->changeEvent(event);
     QGraphicsProxyWidget::changeEvent(event);
 }
 
@@ -508,7 +506,6 @@ void PushButton::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 
     d->hoverAnimation->start();
 
-
     QGraphicsProxyWidget::hoverLeaveEvent(event);
 }
 
