@@ -607,8 +607,10 @@ public:
 
     void handleKeyPressEvent(QKeyEvent *event)
     {
-        if (!widget.data())
+        if (!widget.data()) {
+            event->ignore();
             return;
+        }
 
         QPointF start = q->scrollPosition();
         QPointF end = start;
@@ -637,7 +639,8 @@ public:
             }
             break;
         default:
-            break;
+            event->ignore();
+            return;
         }
 
         fixupAnimation.groupX->stop();
@@ -1422,7 +1425,13 @@ bool ScrollWidget::eventFilter(QObject *watched, QEvent *event)
         d->stopAnimations();
         d->adjustScrollbarsTimer->start(200);
         updateGeometry();
-        ensureItemVisible(d->widget.data());
+        if (d->widget.data()->size().width() < viewportGeometry().width() ||
+            d->widget.data()->size().height() < viewportGeometry().height()) {
+            d->widget.data()->setPos(d->minXExtent(),
+                                     d->minYExtent());
+        } else {
+            ensureItemVisible(d->widget.data());
+        }
     } else if (watched == d->widget.data() && event->type() == QEvent::GraphicsSceneMove) {
         d->horizontalScrollBar->blockSignals(true);
         d->verticalScrollBar->blockSignals(true);
