@@ -1,20 +1,21 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2006 Michaël Larouche <michael.larouche@kdemail.net>
+/*
+    Copyright 2006 Michaël Larouche <michael.larouche@kdemail.net>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    Lesser General Public License for more details.
 
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fakedevice.h"
 
@@ -42,18 +43,9 @@
 
 #include <solid/genericinterface.h>
 
-using namespace Solid::Backends::Fake;
+#include "fakedevice_p.h"
 
-class FakeDevice::Private
-{
-public:
-    QString udi;
-    QMap<QString, QVariant> propertyMap;
-    QStringList interfaceList;
-    bool locked;
-    QString lockReason;
-    bool broken;
-};
+using namespace Solid::Backends::Fake;
 
 FakeDevice::FakeDevice(const QString &udi, const QMap<QString, QVariant> &propertyMap)
     : Solid::Ifaces::Device(), d(new Private)
@@ -76,11 +68,24 @@ FakeDevice::FakeDevice(const QString &udi, const QMap<QString, QVariant> &proper
         Solid::DeviceInterface::Type type = Solid::DeviceInterface::stringToType(interface);
         createDeviceInterface(type);
     }
+
+    connect(d.data(), SIGNAL(propertyChanged(const QMap<QString,int> &)),
+            this, SIGNAL(propertyChanged(const QMap<QString,int> &)));
+    connect(d.data(), SIGNAL(conditionRaised(const QString &, const QString &)),
+            this, SIGNAL(conditionRaised(const QString &, const QString &)));
+}
+
+FakeDevice::FakeDevice(const FakeDevice& dev)
+    : Solid::Ifaces::Device(), d(dev.d)
+{
+    connect(d.data(), SIGNAL(propertyChanged(const QMap<QString,int> &)),
+            this, SIGNAL(propertyChanged(const QMap<QString,int> &)));
+    connect(d.data(), SIGNAL(conditionRaised(const QString &, const QString &)),
+            this, SIGNAL(conditionRaised(const QString &, const QString &)));
 }
 
 FakeDevice::~FakeDevice()
 {
-    delete d;
 }
 
 QString FakeDevice::udi() const
@@ -124,6 +129,26 @@ QString FakeDevice::icon() const
     }
 }
 
+QStringList FakeDevice::emblems() const
+{
+    QStringList res;
+
+    if (queryDeviceInterface(Solid::DeviceInterface::StorageAccess)) {
+        if (property("isMounted").toBool()) {
+            res << "emblem-mounted";
+        } else {
+            res << "emblem-unmounted";
+        }
+    }
+
+    return res;
+}
+
+QString FakeDevice::description() const
+{
+    return product();
+}
+
 QVariant FakeDevice::property(const QString &key) const
 {
     return d->propertyMap[key];
@@ -155,7 +180,7 @@ bool FakeDevice::setProperty(const QString &key, const QVariant &value)
     QMap<QString,int> change;
     change[key] = change_type;
 
-    emit propertyChanged(change);
+    emit d->propertyChanged(change);
 
     return true;
 }
@@ -169,7 +194,7 @@ bool FakeDevice::removeProperty(const QString &key)
     QMap<QString,int> change;
     change[key] = Solid::GenericInterface::PropertyRemoved;
 
-    emit propertyChanged(change);
+    emit d->propertyChanged(change);
 
     return true;
 }
@@ -216,7 +241,7 @@ QString FakeDevice::lockReason() const
 
 void FakeDevice::raiseCondition(const QString &condition, const QString &reason)
 {
-    emit conditionRaised(condition, reason);
+    emit d->conditionRaised(condition, reason);
 }
 
 bool FakeDevice::queryDeviceInterface(const Solid::DeviceInterface::Type &type) const
@@ -287,6 +312,10 @@ QObject *FakeDevice::createDeviceInterface(const Solid::DeviceInterface::Type &t
     case Solid::DeviceInterface::SmartCardReader:
         iface = new FakeSmartCardReader(this);
         break;
+    case Solid::DeviceInterface::SerialInterface:
+        break;
+    case Solid::DeviceInterface::InternetGateway:
+        break;
     case Solid::DeviceInterface::Unknown:
         break;
     case Solid::DeviceInterface::Last:
@@ -303,3 +332,4 @@ QObject *FakeDevice::createDeviceInterface(const Solid::DeviceInterface::Type &t
 }
 
 #include "backends/fakehw/fakedevice.moc"
+#include "backends/fakehw/fakedevice_p.moc"
