@@ -1,20 +1,21 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2005-2007 Kevin Ottens <ervin@kde.org>
+/*
+    Copyright 2005-2007 Kevin Ottens <ervin@kde.org>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    Lesser General Public License for more details.
 
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "device.h"
@@ -65,6 +66,8 @@
 #include <solid/ifaces/serialinterface.h>
 #include <solid/smartcardreader.h>
 #include <solid/ifaces/smartcardreader.h>
+#include <solid/internetgateway.h>
+#include <solid/ifaces/internetgateway.h>
 
 
 Solid::Device::Device(const QString &udi)
@@ -232,6 +235,8 @@ const Solid::DeviceInterface *Solid::Device::asDeviceInterface(const DeviceInter
             case DeviceInterface::SmartCardReader:
                 iface = deviceinterface_cast(Ifaces::SmartCardReader, SmartCardReader, dev_iface);
                 break;
+            case DeviceInterface::InternetGateway:
+                iface = deviceinterface_cast(Ifaces::InternetGateway, InternetGateway, dev_iface);
             case DeviceInterface::Unknown:
             case DeviceInterface::Last:
                 break;
@@ -242,6 +247,7 @@ const Solid::DeviceInterface *Solid::Device::asDeviceInterface(const DeviceInter
         {
             // Lie on the constness since we're simply doing caching here
             const_cast<Device *>(this)->d->setInterface(type, iface);
+            iface->d_ptr->setDevicePrivate(d.data());
         }
 
         return iface;
@@ -266,7 +272,7 @@ Solid::DevicePrivate::~DevicePrivate()
     foreach (DeviceInterface *iface, m_ifaces) {
         delete iface->d_ptr->backendObject();
     }
-    qDeleteAll(m_ifaces);
+    setBackendObject(0);
 }
 
 void Solid::DevicePrivate::_k_destroyed(QObject *object)
@@ -277,6 +283,12 @@ void Solid::DevicePrivate::_k_destroyed(QObject *object)
 
 void Solid::DevicePrivate::setBackendObject(Ifaces::Device *object)
 {
+
+    if (m_backendObject) {
+        m_backendObject.data()->disconnect(this);
+    }
+
+    delete m_backendObject.data();
     m_backendObject = object;
 
     if (object) {
@@ -286,12 +298,11 @@ void Solid::DevicePrivate::setBackendObject(Ifaces::Device *object)
 
     if (!m_ifaces.isEmpty()) {
         foreach (DeviceInterface *iface, m_ifaces) {
-            delete iface->d_ptr->backendObject();
             delete iface;
-            if (!ref.deref()) deleteLater();
         }
 
         m_ifaces.clear();
+        if (!ref.deref()) deleteLater();
     }
 }
 
@@ -302,7 +313,8 @@ Solid::DeviceInterface *Solid::DevicePrivate::interface(const DeviceInterface::T
 
 void Solid::DevicePrivate::setInterface(const DeviceInterface::Type &type, DeviceInterface *interface)
 {
-    ref.ref();
+    if(m_ifaces.isEmpty())
+        ref.ref();
     m_ifaces[type] = interface;
 }
 
